package com.brilliance.ccbank.common;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.brilliance.eibs.core.model.IAbsDef;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.IFilterDef;
import com.brilliance.eibs.core.model.IInterfaceDef;
import com.brilliance.eibs.core.model.IStepDef;
import com.brilliance.eibs.core.model.ISubfieldDef;
import com.brilliance.eibs.core.model.ITransactionDef;
import com.brilliance.eibs.core.model.impl.FieldDef;
import com.brilliance.eibs.core.model.impl.FilterDef;
import com.brilliance.eibs.core.model.impl.InterfaceDef;
import com.brilliance.eibs.core.model.impl.SubfieldDef;
import com.brilliance.eibs.core.model.impl.TransactionDef;
import com.brilliance.eibs.util.StringUtil;

public class CommonUtil
{

  /**
   * 获取Filter类
   * @param id
   * @param cls
   * @param type
   * @param parentDef
   * @return
   */
  public static IFilterDef getFilter(String id, String cls, String type, IAbsDef parentDef)
  {
    IFilterDef filterDef = new FilterDef();
    filterDef.setRef(id);
    filterDef.setType(type);
    filterDef.setId(id);
    filterDef.setExecuteClass(cls);
    filterDef.setParentDef(parentDef);
    return filterDef;
  }

  /**
   * 返回field对象
   * @param tag
   * @param etag
   * @param value
   * @param type
   * @param scope
   * @return
   */
  public static IFieldDef getFieldDef(String tag, String etag, String value, String type, String scope)
  {
    IFieldDef fieldDef = new FieldDef();
    fieldDef.setTag(dealStr(tag));
    fieldDef.setEtag(dealStr(etag));
    fieldDef.setType(dealStr(type));
    fieldDef.setValue(dealStr(value));
    scope = dealStr(scope);
    if (StringUtil.isEmpty(scope))
      scope = "";
    fieldDef.setScope(scope);
    return fieldDef;
  }
  
  public static ISubfieldDef getSubFieldDef(String tag, String etag, String value, String type, String scope)
  {
    ISubfieldDef fieldDef = new SubfieldDef();
    fieldDef.setTag(dealStr(tag));
    fieldDef.setEtag(dealStr(etag));
    fieldDef.setType(dealStr(type));
    fieldDef.setValue(dealStr(value));
    scope = dealStr(scope);
    if (StringUtil.isEmpty(scope))
      scope = "";
    fieldDef.setScope(scope);
    return fieldDef;
  }

   
  /**
   * @param interfaceid 接口名称，和交易名称一样
   * @param stepDef  
   * @return
   */
  public static IInterfaceDef getInterfaceDef(String interfaceid, IStepDef stepDef)
  {
    IInterfaceDef interfaceDef = makeInterfaceDef(interfaceid);
    ITransactionDef transactionDef = makeTransactionDef(interfaceid, interfaceDef);
    List<ITransactionDef> trnlist = new LinkedList<ITransactionDef>();
    trnlist.add(transactionDef);
    interfaceDef.setTransactionDefs(trnlist);
    List<IStepDef> steplist = new LinkedList<IStepDef>();
    
    stepDef.setId(interfaceid);
    stepDef.setTransaction(transactionDef);
    steplist.add(stepDef);
    transactionDef.setSteps(steplist);
    
    return interfaceDef;
  }
  
  
  public static IInterfaceDef makeInterfaceDef(String interfaceId)
  {
    IInterfaceDef interfaceDef = new InterfaceDef();
    interfaceDef.setId(interfaceId);
    interfaceDef.setType("server");
    interfaceDef.setVersion("2.0");

    String namespace = "ccbank";
    String cls = "com.brilliance.ccbank.common.CommonAction";
    String stc = "true";
    interfaceDef.addFunc(namespace, cls, stc);

    return interfaceDef;
  }

  public IInterfaceDef makeInterfaceDef(Map par)
  {
    IInterfaceDef interfaceDef = new InterfaceDef();
    interfaceDef.setId(CommonUtil.getAttribute(par, "id"));
    interfaceDef.setType("server");
    interfaceDef.setVersion("2.0");
    interfaceDef.setDescription(CommonUtil.getAttribute(par, "dsp"));

    String namespace = "ccbank";
    String cls = "com.brilliance.ccbank.common.CommonAction";
    String stc = "true";
    interfaceDef.addFunc(namespace, cls, stc);

    /*  IFieldDef fieldDef1 = new FieldDef();
      fieldDef1.setValue("${ccbank:getErrorCode('CCBANK','00000')}");
      fieldDef1.setParentDef(filterDef);
      filterDef.addSub(fieldDef1);

      fieldDef1 = new FieldDef();
      fieldDef1.setValue("${ccbank:codeTable('cur','USA')}");
      fieldDef1.setParentDef(filterDef);
      filterDef.addSub(fieldDef1);

      fieldDef1 = new FieldDef();
      fieldDef1.setValue("${ccbank:getErrorDsp('CCBANK','S00001','HHH')}");*/

    return interfaceDef;
  }

  public static ITransactionDef makeTransactionDef(String id, IInterfaceDef interfaceDef)
  {
    ITransactionDef transactionDef = new TransactionDef();
    transactionDef.setId(id);
    transactionDef.setVersion("2.0");
    transactionDef.setInterfaceDef(interfaceDef);
    return transactionDef;
  }

 

  public static String dealStr(String str)
  {
    if (StringUtil.isEmpty(str))
      return null;
    return str;
  }
  
  public static String elStr(String val)
  {
    if (StringUtil.isEmpty(val))
      return "";
    else
      return "${" + val + "}";
  }
  
  public static String getAttribute(Map m, String key)
  {
    return m.get(key) == null ? "" : m.get(key).toString();
  }
}
