package com.brilliance.ccbank.srcipt.core;

public class FlwMapping
{
  private String  flwId;           //流程ID
  private String  flwNam;          //流程名
  private Integer seqId;           //执行顺序号
  private Integer retVal;          //判断值
  private Integer nextSeqId;       //下一步执行顺序号
  private String  compId;          //组件ID
  private String  compTyp;         //组件类型
  final String    WHITESPACE = " ";

  public FlwMapping( String flwId, String flwNam, Integer seqId, Integer retVal, Integer nextSeqId, String compId, String compTyp )
  {
    if (flwId == null)
      throw new IllegalArgumentException("参数 'flwId' 不能为空");
    this.flwId = flwId;
    this.flwNam = flwNam;
    this.seqId = seqId;
    this.retVal = retVal;
    this.nextSeqId = nextSeqId;
    this.compId = compId;
    this.compTyp = compTyp;
  }

  public String getFlwId()
  {
    return flwId;
  }

  public String getFlwNam()
  {
    return flwNam;
  }

  public Integer getSeqId()
  {
    return seqId;
  }

  public Integer getRetVal()
  {
    return retVal;
  }

  public Integer getNextSeqId()
  {
    return nextSeqId;
  }

  public String getCompId()
  {
    return compId;
  }

  public String getCompTyp()
  {
    return compTyp;
  }

  public String getBegin()
  {
    return "BEGIN" + WHITESPACE + flwId + WHITESPACE + seqId;
  }

  public String getEnd()
  {
    return "END" + WHITESPACE + flwId + WHITESPACE + seqId;
  }

  public String getCall()
  {
    return ScriptType.CALL.TYPE_CODE + WHITESPACE + compId + WHITESPACE + seqId + WHITESPACE + compTyp;
  }

  public String getIf()
  {
    return ScriptType.IF.TYPE_CODE + WHITESPACE + compId + WHITESPACE + seqId + WHITESPACE + compTyp + WHITESPACE + retVal;
  }

  public String getElse()
  {
    return ScriptType.ELSE.TYPE_CODE;
  }

  public String getElseIf()
  {
    return ScriptType.ELSEIF.TYPE_CODE + WHITESPACE + compId + WHITESPACE + seqId + WHITESPACE + compTyp + WHITESPACE + retVal;
  }

  public static class Builder
  {
    private String  flwId;
    private String  flwNam;
    private Integer seqId;
    private Integer retVal;
    private Integer nextSeqId;
    private String  compId;
    private String  compTyp;

    public Builder( String flwId, Integer seqId, Integer nextSeqId )
    {
      this.flwId = flwId;
      this.seqId = seqId;
      this.nextSeqId = nextSeqId;
    }

    public Builder( String flwId, Integer seqId )
    {
      this.flwId = flwId;
      this.seqId = seqId;
    }

    public Builder flwNam(String flwNam)
    {
      this.flwNam = flwNam;
      return this;
    }

    public Builder retVal(int retVal)
    {
      this.retVal = retVal;
      return this;
    }

    public Builder compId(String compId)
    {
      this.compId = compId;
      return this;
    }

    public Builder compTyp(String compTyp)
    {
      this.compTyp = compTyp.toUpperCase();
      return this;
    }

    public FlwMapping build()
    {
      return new FlwMapping(this.flwId, this.flwNam, this.seqId, this.retVal, this.nextSeqId, this.compId, this.compTyp);
    }
  }
}
