package com.brilliance.ccbank.srcipt.reader;

import java.nio.charset.Charset;

import com.brilliance.ccbank.srcipt.core.ScriptType;
import com.brilliance.ccbank.srcipt.model.impl.BeginDef;
import com.brilliance.ccbank.srcipt.parser.TokenParser;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.StringUtil;

/**
 * 脚本阅读器
 * @author weicong
 *
 */
public abstract class ScriptReader
{
  protected BeginDef beginDef;
  private int        skipnum = 0;

  protected String   scriptline;
  protected String   charset = PropertyUtil.getProperty("sys.properties", "SYS_ENCODING");

  public abstract String readScriptLine()
      throws Exception;

  public boolean isBegin()
      throws Exception
  {
    boolean isbegin = false;
    while ((scriptline = readLine()) != null)
    {
      if (StringUtil.isEmpty(scriptline))
      {
        continue;
      }
      String[] args=TokenParser.elementsOfLine(scriptline);
      if(!ScriptType.BEGIN.equals(ScriptType.valueOf(args[0].toUpperCase())))
        continue;
      beginDef = new BeginDef(args[1]);
      isbegin = true;
      break;
    }
    return isbegin;
  }

  public BeginDef getBeginDef()
  {
    return beginDef;
  }

  public boolean hasNext()
      throws Exception
  {
    try{
      if (skipnum == 0)
        return readScriptLine() != null;
      skipnum--;
      return true;
    }catch (NullPointerException e) {
      return false;
    }
  }

  /**
   * 回退skipnum=skipnum+num行
   * @param num 此次设置的num行
   */
  public void backWard(int num)
  {
    this.skipnum += num;
  }

  public String next()
  {
    return scriptline;
  }

  public void setLine(String s)
  {
    this.scriptline = s;
  }

  public String getCharset()
  {
    return charset;
  }

  public void setCharset(String charset)
  {
    if (Charset.isSupported(charset))
      this.charset = charset;
    else
    {
      throw new InterfaceException("00403", "不支持的编码格式转换 [" + charset.toUpperCase() + "]");
    }
  }
  
  public abstract String readLine()throws Exception;
}
