package com.brilliance.eibs.factory.parser.impl;

import com.brilliance.ccbank.common.CommonUtil;
import com.brilliance.ccbank.common.XmlUnLoader;
import com.brilliance.ccbank.dao.TCodtblMapper;
import com.brilliance.ccbank.dao.TErrcodMapper;
import com.brilliance.ccbank.model.TCodtbl;
import com.brilliance.ccbank.model.TErrcod;
import com.brilliance.ccbank.service.CiticbankServer;
import com.brilliance.ccbank.service.ManagerService;
import com.brilliance.ccbank.service.impl.FlwEngineService;
import com.brilliance.ccbank.srcipt.core.FlwMapping;
import com.brilliance.ccbank.srcipt.core.ScriptParser;
import com.brilliance.eibs.core.debug.DebugMode;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.core.model.impl.StepDef;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.parser.ConfigParser;
import com.brilliance.eibs.factory.parser.IHotDeployParser;
import com.brilliance.eibs.factory.parser.ParserFactory;
import com.brilliance.eibs.factory.parser.ParserFactory.ParserName;
import com.brilliance.eibs.listener.helpers.Watchdog;
import org.apache.ibatis.session.SqlSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据库配置解析器
 *
 * @author weicong
 */
public class DbParser
        implements ConfigParser, IHotDeployParser {
    protected static Logger logger = LoggerFactory.getLogger("dbparser");

    /**
     * 验证数据库连接是否正常
     **/
    public void validateConnected()
            throws Throwable {
        SessionFactory dbSource = new SessionFactory(logger);
        dbSource.validateConnected();
    }

    /**
     * 从数据库加载交易到内存
     */
    @Override
    public boolean mainFileToMemo(BeanFactory beanFactory) {
        return mainFileToMemo(beanFactory, null, null);
    }

    public boolean mainFileToMemo(BeanFactory beanFactory, String interfaceId, String transactionId) {
        init(beanFactory);

        //1.查找服务注册信息表，得到所有业务应用APPID  List<AppID>
        //2.通过业务应用ID,查找场景组件实例表，接口名为业务应用四要素+内部交易码
        //3.查找场景组件，得到流程实例表T_FLW_INST
        //4.通过流程实例查找场景组件的解包、基础组件CALL、场景组件的打包

        FlwEngineService flw = new FlwEngineService();
        List<FlwMapping> flwmapper = flw.makeScript();
        try {
            List<IAbsDef> flwlist = ScriptParser.dbParse(flwmapper, null);
            StepDef stepDef = new StepDef();
            IInterfaceDef interfaceDef = CommonUtil.getInterfaceDef("testA", stepDef);
            flw.handleFlwList(flwlist, stepDef);
            new XmlUnLoader().unloader(interfaceDef);

        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    @Override
    public ITransactionDef getTransactionDef(String interfaceid, String transid) {
        return null;
    }

    @Override
    public ITransactionDef getTransactionDef(IInterfaceDef interfaceDef, String transid) {
        List<ITransactionDef> transactionDefs = interfaceDef.getTransactionDefs();
        for (ITransactionDef transactionDef : transactionDefs) {
            if (transactionDef.getId().equals(transid))
                return transactionDef;
        }
        throw new InterfaceException("00003", "can not find transaction " + transid + " in interface " + interfaceDef.getId());
    }

    @Override
    public List<IServiceDef> getServiceDefs(String filename) {
        return null;
    }

    public List<IServiceDef> getServiceDefs(String filename, String[] args, BeanFactory beanFactory) {
        List<IServiceDef> serviceDefs = new ManagerService(logger).getServiceDefs(beanFactory);
        for (IServiceDef serviceDef : serviceDefs) {
            IInterfaceDef interfaceDef = new CiticbankServer().getccbankServer(serviceDef);
            add2BeanFactory(interfaceDef, beanFactory);
        }
        return serviceDefs;
    }

    public void add2BeanFactory(IInterfaceDef interfaceDef, BeanFactory beanFactory) {
        if (interfaceDef.isInner())
            return;
        new XmlUnLoader().unloader(interfaceDef);
        String id = interfaceDef.getId();
        IInterfaceDef retinterfaceDef = null;
        if (DebugMode.singleInstance().isDebug()) {
            retinterfaceDef = ((XmlParser) ParserFactory.getInstance(ParserName.XMLPARSER, logger)).getInterfaceDef(id);
            retinterfaceDef.setFuncMap(interfaceDef.getFuncMap());
            retinterfaceDef.setCodetableMap(interfaceDef.getCodetableMap());
            retinterfaceDef.setDynamicCodetableMap(interfaceDef.getDynamicCodetableMap());
            retinterfaceDef.setCountermap(interfaceDef.getCountermap());
        }

        beanFactory.addInterface(id, retinterfaceDef);
        logger.info("init interface [" + id + "] success.");
    }

    public List<ITaskDef> getTaskDefs(String filename, String[] args) {
        // TODO Auto-generated method stub
        return new ArrayList<ITaskDef>();
    }

    @Override
    public List<ITaskDef> getTaskDefs(String filename) {
        // TODO Auto-generated method stub
        return new ArrayList<ITaskDef>();
    }

    @Override
    public IServiceDef getServiceDef(String id) {
        List<IServiceDef> serviceDefs = getServiceDefs(null);
        for (IServiceDef serviceDef : serviceDefs) {
            if (serviceDef.getId().equals(id)) {
                return serviceDef;
            }
        }
        return null;
    }

    @Override
    public void setLogger(Logger log) {
        // TODO Auto-generated method stub
        logger = log;
    }

    @Override
    public Logger getLogger() {
        // TODO Auto-generated method stub
        return logger;
    }

    @Override
    public Watchdog createHotDeployListener() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Watchdog createBDHotDeployListener() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String getHotDeployState() {
        return "off";
    }

    /***
     * 服务启动做一些初始化操作，初始化内存的时候使用
     * **/
    public void init(BeanFactory beanFactory) {
        SqlSession session = SessionFactory.getSession();
        //初始化错误码
        TCodtblMapper codtblMapper = session.getMapper(TCodtblMapper.class);
        List<Map<String, Object>> codtblList = new ArrayList<Map<String, Object>>();
        List<TCodtbl> codtbls = codtblMapper.selAll();
        for (TCodtbl tCodtbl : codtbls) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("id", tCodtbl.getTyp());
            map.put("srccod", tCodtbl.getSrccod());
            map.put("dstcod", tCodtbl.getDstcod());
            codtblList.add(map);
        }
        beanFactory.addCodeTable(codtblList);
        //初始化码表

        TErrcodMapper errcodMapper = session.getMapper(TErrcodMapper.class);
        List<Map<String, Object>> errList = new ArrayList<Map<String, Object>>();
        List<TErrcod> errcods = errcodMapper.selAll();
        for (TErrcod errCodtbl : errcods) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("sysnam", errCodtbl.getSysnam());
            map.put("extcod", errCodtbl.getExtcod());
            map.put("inncod", errCodtbl.getInncod());
            map.put("chndsp", errCodtbl.getChndsp());
            errList.add(map);
        }
        beanFactory.addErrorCodes(errList);
        session.close();

    /*    DbSource dbAction = new DbSource(new SessionFactory(logger));
        beanFactory.addErrorCodes(dbAction.getErroCode(new Object[]{}));
        beanFactory.addCodeTable(dbAction.getCodeTables(new Object[]{}));
        dbAction.close();*/
    }

}
