package com.brilliance.eibs.factory.parser.impl;

import java.net.URL;
import java.sql.Connection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.session.SqlSession;
import org.slf4j.Logger;
import org.dom4j.Document;

import com.brilliance.ccbank.dao.TSvrRgstMapper;
import com.brilliance.ccbank.model.TSvrRgst;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IConnectionDef;
import com.brilliance.eibs.core.model.modelbeans.CommonsDef;
import com.brilliance.eibs.core.model.modelbeans.ConnectionsDef;
import com.brilliance.eibs.core.service.instance.plugin.DBConnPool;
import com.brilliance.eibs.core.service.instance.plugin.JdbcExecutor;
import com.brilliance.eibs.factory.parser.ParserFactory;
import com.brilliance.eibs.factory.parser.ParserFactory.ParserName;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.XmlUtil;

/**
 * DB数据源层，包括配置中的XML数据源和JdbcExcutor执行器
 * ***/

public class SessionFactory {
	public Logger logger;
	public static String CONNECTIONID = PropertyUtil.getProperty("sys.properties", "CONNECTIONID");

	Connection connection = null;

	public SessionFactory(Logger logger) {
		this.logger = logger;
		try {
			this.connection = DBConnPool.getConnection(getCommonsConnectionDef(CONNECTIONID), logger);
		} catch (Exception e) {
			throw new InterfaceException("01502", e);
		}

	}

	private final CommonsDef mainCommonsDef = getCommonsDef(XmlParser.COMMON_PATH);

	public IConnectionDef getCommonsConnectionDef(String connectionId) {
		ConnectionsDef connectionsDef = mainCommonsDef.getConnectionsDef();
		if (connectionsDef == null) {
			throw new InterfaceException("00009", "commons connections node not exist!");
		}
		List<IConnectionDef> connectionDefs = connectionsDef.getConnections();
		for (IConnectionDef connectionDef : connectionDefs) {
			if (connectionDef.getId().equals(connectionId)) {
				return connectionDef;
			}
		}
		throw new InterfaceException("00010", "commons connection [" + connectionId + "] not exist!");
	}

	public CommonsDef getCommonsDef(String path) {
		URL commonURL = XmlUtil.getUrl(path);
		Document doc = XmlUtil.getDocument(commonURL);
		WriteXMLFlag wxf = new WriteXMLFlag();
		XmlParser xmlParser = ((XmlParser) ParserFactory.getInstance(ParserName.XMLPARSER, logger));
		xmlParser.initEnvironment(doc);
		return xmlParser.getCommonsDef(doc, wxf);
	}

	public void validateConnected() throws Throwable {
		IConnectionDef connectionDef = getCommonsConnectionDef(CONNECTIONID);
		// 接着验证是否能连接到指定数据库
		DBConnPool.getConnection(connectionDef, logger).close();
	}

	/** 获取JDBC执行器 **/
	public JdbcExecutor getJdbcExecutor() throws Exception {
		// Connection connection =
		// DBConnPool.getConnection(getCommonsConnectionDef(CONNECTIONID),
		// logger);
		if (connection == null) {
			connection = DBConnPool.getConnection(getCommonsConnectionDef(CONNECTIONID), logger);
		}
		// return new JdbcExecutor(connection, BeaLogFactory.getBeaLog(logger));
		return null;
	}

	public static SqlSession getSession() {
		/*
		 * SessionFactory dbSource = new SessionFactory(LoggerFactory.getLogger("aa"));
		 * SqlSession sqlSession =
		 * DBConnPool.getSessionFactory(dbSource.getCommonsConnectionDef
		 * (CONNECTIONID)).openSession(); return sqlSession;
		 */
		return null;
	}

	public static void main(String[] args) {
		SqlSession session = SessionFactory.getSession();
		// 查找服务信息
		TSvrRgstMapper svrRgstMapper = session.getMapper(TSvrRgstMapper.class);
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("ip", "127.0.0.1");
		map.put("port", 8899);
		List<TSvrRgst> list = svrRgstMapper.selList1("127.0.0.1", 8899);
		System.out.println(list.get(0).getAppId());

		TSvrRgst t = svrRgstMapper.sel("127.0.0.1", 8899);
		System.out.println(t.getAppId());
		session.close();
		// TAppInst inTAppInst=(TAppInst)
		// sqlSession.selectOne("selectByPrimaryKey","00000001");
		// TAppInst inTAppInst=(TAppInst)
		// sqlSession.selectOne("selectByPrimaryKey","00000001");
		// System.out.println(inTAppInst.getAppId());
		// TSvrRgstMapper svrRgstMapper =
		// sqlSession.getMapper(TSvrRgstMapper.class);
		// List<TSvrRgst> list=svrRgstMapper.selSvrRgstInfo("1");
	}
}
