package com.brilliance.ext.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;

/**
 * @author ge
 *
 */
public class ExcelUtil {

	private static Logger logger = LogUtil.getLogger("excel");

	private static int THREE = 3;
	private static int FOUR = 4;

	/**
	 * @param srcDataPath
	 * @param srcDataEncoding
	 * @param destPath
	 * @param templatePath
	 * @param sheetName
	 * @param exchangePlace
	 * @return
	 * @throws IOException
	 *             结算量报表函数
	 */
	public static boolean generateJSLExcel(String srcDataPath, String srcDataEncoding, String destPath, String templatePath, String sheetName,
			String exchangePlace) throws IOException {

		try {
			List<String> srcData = FileUtils.readLines(new File(srcDataPath), srcDataEncoding);
			List<String[]> dataList = new ArrayList<String[]>();
			String[] realHeads = null;
			String unit = "";
			String date = "";
			for (int i = 0; i < srcData.size(); i++) {
				// 处理币种单位
				if (i == 0) {
					String first = srcData.get(0);
					if (!StringUtil.isEmpty(first)) {
						String[] firstArrays = first.split(",");
						unit = firstArrays[firstArrays.length - 1];
					}
				}
				// 去除文本第一行
				if (i == 0 || i == 2) {
					continue;
				}
				String str = srcData.get(i);
				// 日期处理
				if (str.trim().startsWith("日期")) {
					date = str.replace(",", "");
					continue;
				}
				String[] dataArray = str.trim().split(",");
				if (str.trim().startsWith("序号")) {
					realHeads = new String[dataArray.length];
					for (int j = 0; j < dataArray.length; j++) {
						realHeads[j] = dataArray[j].trim();
					}
					continue;
				}
				dataArray = str.trim().split(",=\"");
				// 添加业务数据
				String realData[] = new String[realHeads.length];
				for (int j = 0; j < dataArray.length; j++) {
					realData[j] = dataArray[j].replace("'", "").replace("\"", "");
					if (j == 0) {
						realData[j] = realData[j].replace("=", "");
					}
				}
				dataList.add(realData);
			}

			realHeads = handleHeaders(realHeads);

			return generateExcel(FOUR, unit, templatePath, destPath, sheetName, date, dataList, realHeads, exchangePlace, srcDataPath);
		} catch (Exception e) {
			logger.error("error:", e);
			e.printStackTrace();
			return false;
		}
	}

	public static String[] handleHeaders(String[] realHeads) {
		String otherCur = "其他费用币种";
		int otherCurCnt = 1;
		String otherAmt = "其他费用金额";
		int otherAmtCnt = 1;
		String ruzhang = "入账账户性质";
		int ruzhangCnt = 1;
		String chuzhang = "出账账户性质";
		int chuzhangCnt = 1;

		for (int i = 0; i < realHeads.length; i++) {
			String currentStr = realHeads[i];
			if (currentStr.equals(otherCur)) {
				realHeads[i] = otherCur + otherCurCnt;
				otherCurCnt++;

			}
			if (currentStr.equals(otherAmt)) {
				realHeads[i] = otherAmt + otherAmtCnt;
				otherAmtCnt++;
			}
			if (currentStr.equals(ruzhang)) {
				realHeads[i] = ruzhang + ruzhangCnt;
				ruzhangCnt++;
			}
			if (currentStr.equals(chuzhang)) {
				realHeads[i] = chuzhang + chuzhangCnt;
				chuzhangCnt++;
			}
		}
		return realHeads;
	}

	/**
	 * @param srcDataPath
	 * @param srcDataEncoding
	 * @param destPath
	 * @param templatePath
	 * @param sheetName
	 * @param exchangePlace
	 * @return
	 * @throws IOException
	 *             台帐Excel生成函数
	 */
	public static boolean generateExcel(String srcDataPath, String srcDataEncoding, String destPath, String templatePath, String sheetName,
			String exchangePlace) throws IOException {
		// 读取配置文件
		String path = Thread.currentThread().getContextClassLoader().getResource("cfg/sheet1.txt").getPath();
		FileInputStream fis = new FileInputStream(new File(path));
		List<String> sheetList = IOUtils.readLines(fis, "GBK");

		try {
			List<String> srcData1 = new ArrayList<String>();

			List<String> srcData = FileUtils.readLines(new File(srcDataPath), srcDataEncoding);

			if (sheetName.equals("手续费明细台账")) {
				srcData.add(0, "日期, ");
				srcData.add(0, "大表头");
				for (int i = 0; i < srcData.size(); i++) {
					if (i == 2) {
						String strsxf = srcData.get(i).replace("\"", "");
						String sss = strsxf.replace(",", "\t");
						srcData1.add(sss);
					} else {
						srcData1.add(srcData.get(i));
					}

				}
			} else {

				if ((!StringUtil.isEmpty(srcData.get(0)) && srcData.get(0).trim().startsWith("\"序号"))
						|| (!StringUtil.isEmpty(srcData.get(0)) && srcData.get(0).trim().startsWith("序号"))) {
					srcData.add(0, "日期, ");
					srcData.add(0, "大表头");
					for (int i = 0; i < srcData.size(); i++) {
						String sss = srcData.get(i).replace("\"", "").replace("\t", "").replace("\'", "");
						srcData1.add(sss);
					}
				}
				if (!StringUtil.isEmpty(srcData.get(0)) && srcData.get(0).trim().startsWith("日期")) {
					srcData.add(0, "大表头");
					for (int i = 0; i < srcData.size(); i++) {
						String sss = srcData.get(i).replace("\"", "").replace("\t", "").replace("\'", "");
						srcData1.add(sss);
					}
				}
			}
			if (srcData1.size() == 0)
				srcData1 = srcData;

			List<String[]> dataList = new ArrayList<String[]>();
			String[] realHeads = null;
			String date = "";
			String unit = "";
			for (int i = 0; i < srcData1.size(); i++) {
				// 去除文本第一行
				if (i == 0) {
					continue;
				}
				if (sheetList.contains(sheetName) && (i == 2)) {
					continue;
				}
				String str = srcData1.get(i);
				String[] dataArray = str.trim().split("\t");
				if (dataArray.length == 1)
					dataArray = str.split(",");
				// 日期处理
				/*
				 * if (str.trim().startsWith("日期") && dataArray.length == 2) {
				 * date = dataArray[0].trim() + ":" + dataArray[1]; continue; }
				 */
				if (str.trim().startsWith("日期") && (i == 1)) {
					date = dataArray[0].trim() + ":" + dataArray[1];
					if (dataArray.length > 2) {
						unit = dataArray[dataArray.length - 1];
					}
					continue;
				}

				if (sheetList.contains(sheetName)) {
					if (i == 3) {
						realHeads = new String[dataArray.length];
						for (int j = 0; j < dataArray.length; j++) {
							realHeads[j] = dataArray[j].trim();
						}
						continue;
					}
				} else {
					if (str.trim().startsWith("序号")) {
						realHeads = new String[dataArray.length];
						for (int j = 0; j < dataArray.length; j++) {
							realHeads[j] = dataArray[j].trim();
						}
						continue;
					}
				}

				// 添加业务数据
				String realData[] = new String[realHeads.length];

				if (sheetName.equals("手续费明细台账")) {

					for (int j = 0; j < dataArray.length; j++) {
						// System.out.println("j=" + j);
						// System.out.println(realHeads.length + "," +
						// dataArray.length);
						// System.out.println(j);
						String strxxf = dataArray[j];
						strxxf = strxxf.replace("'", "").replace("\"", "");
						if (strxxf.startsWith(","))
							strxxf = strxxf.substring(1);
						if (strxxf.trim().startsWith(","))
							strxxf = strxxf.substring(1);
						realData[j] = strxxf;
					}

				} else {
					for (int j = 0; j < dataArray.length; j++) {
						realData[j] = dataArray[j].replace("'", "").replace("\"", "");
					}
				}
				dataList.add(realData);
			}

			realHeads = handleHeaders(realHeads);

			if (sheetName.equals("付汇业务监测日报表")) {
				dataList = handleDataList(dataList);
			}

			int dataRow = THREE;
			if (sheetList.contains(sheetName)) {
				dataRow = FOUR;
			}

			return generateExcel(dataRow, unit, templatePath, destPath, sheetName, date, dataList, realHeads, exchangePlace, srcDataPath);
		} catch (Exception e) {
			logger.error("error:", e);
			e.printStackTrace();
			return false;
		}

	}

	public static List<String[]> handleDataList(List<String[]> dataList) {

		Collections.sort(dataList, new Comparator<String[]>() {

			@Override
			public int compare(String[] arg0, String[] arg1) {

				String hits0 = arg0[arg0.length - 3];
				if (StringUtil.isEmpty(hits0))
					hits0 = "00000000";
				String hits1 = arg1[arg1.length - 3];
				if (StringUtil.isEmpty(hits1))
					hits1 = "00000000";
				int hits00 = Integer.parseInt(hits0.replace(".", "").replace("-", "").replace("/", ""));
				int hits11 = Integer.parseInt(hits1.replace(".", "").replace("-", "").replace("/", ""));
				if (hits11 > hits00)
					return -1;
				else if (hits11 == hits00)
					return 0;
				else
					return 1;
			}
		});
		return dataList;
	}

	/** */
	/**
	 * 文件重命名
	 * 
	 * @param path
	 *            文件目录
	 * @param oldname
	 *            原来的文件名
	 * @param newname
	 *            新文件名
	 */
	public static void renameFile(String oldPath, String newPath) {
		File oldfile = new File(oldPath);
		File newfile = new File(newPath);
		if (!oldfile.exists()) {
			return;// 重命名文件不存在
		}
		if (!newfile.exists())// 若在该目录下已经有一个文件和新文件名相同，则不允许重命名
			oldfile.renameTo(newfile);
	}

	/**
	 * @param templatePath
	 *            模板路径
	 * @param targetPath
	 *            目标路径
	 * @param sheetName
	 *            页签名
	 * @param beginDate
	 *            起始日期
	 * @param endDate
	 *            结束日期
	 * @param dataList
	 *            数据
	 * @param exchangePlace
	 *            位置交换 0,2#2,0
	 * @throws Exception
	 */

	public static int location(String[] arrays, String str) {
		for (int i = 0; i < arrays.length; i++) {
			if (arrays[i].equals(str)) {
				return i;
			}
		}
		return 0;
	}

	@SuppressWarnings("resource")
	public static boolean generateExcel(int templateRowNumber, String unit, String templatePath, String targetPath, String sheetName, String date,
			List<String[]> dataList, String[] realHeads, String exchangePlace, String srcDataPath) {
		FileOutputStream fos = null;
		FileInputStream f = null;
		Workbook workbook = null;
		boolean rig = true;
		try {
			Map<Integer, Integer> map = new HashMap<Integer, Integer>();
			// 模板所在行号
			// int templateRowNumber = 3;
			fos = new FileOutputStream(targetPath);
			f = new FileInputStream(templatePath);
			// workbook = WorkbookFactory.create(f);
			workbook = new XSSFWorkbook(f);
			// 去除模板中无用的标签
			while (workbook.getNumberOfSheets() > 1) {
				for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
					if (!workbook.getSheetAt(i).getSheetName().equals(sheetName)) {
						workbook.removeSheetAt(i);
						i = 0;
					}
				}
			}

			Sheet sheet = workbook.getSheet(sheetName);
			Row templateRow = sheet.getRow(templateRowNumber);
			Row headtemplateRow = sheet.getRow(templateRowNumber - 1);

			String templateCellNames[] = new String[headtemplateRow.getPhysicalNumberOfCells()];

			if (!StringUtil.isEmpty(exchangePlace)) {
				try {
					String[] headtemplateHeads = new String[headtemplateRow.getPhysicalNumberOfCells()];
					for (int i = 0; i < headtemplateRow.getPhysicalNumberOfCells(); i++) {
						headtemplateHeads[i] = headtemplateRow.getCell(i).getStringCellValue().trim();
					}
					headtemplateHeads = handleHeaders(headtemplateHeads);

					String[] excArray = exchangePlace.split("#");
					for (int k = 0; k < excArray.length; k++) {
						String[] places = excArray[k].split(",");
						// map.put(Integer.parseInt(places[0].trim()),
						// Integer.parseInt(places[1].trim()));
						map.put(location(headtemplateHeads, places[0]), location(realHeads, places[1]));
					}
				} catch (Exception e) {
					e.printStackTrace();
					logger.error("error:", e);
					System.out.println("数据交换数据格式【" + exchangePlace + "】不符合规则，例如正确格式为【***,***#***,***】!");
					return false;
				}
			}

			String[] headtemplateHeads = new String[headtemplateRow.getPhysicalNumberOfCells()];
			for (int i = 0; i < headtemplateRow.getPhysicalNumberOfCells(); i++) {
				headtemplateHeads[i] = headtemplateRow.getCell(i).getStringCellValue().trim();
			}
			headtemplateHeads = handleHeaders(headtemplateHeads);

			if (templateRowNumber == THREE) {
				for (int i = 0; i < headtemplateHeads.length; i++) {
					templateCellNames[i] = headtemplateHeads[i].trim();
					for (int j = 0; j < realHeads.length; j++) {

						if (templateCellNames[i].equals(realHeads[j])) {
							map.put(i, j);
							break;
						}
					}
				}
			} else if (templateRowNumber == FOUR) {
				for (int i = 0; i < headtemplateHeads.length; i++) {
					map.put(i, i);
				}
			}

			Row dateRow = sheet.getRow(templateRowNumber - (templateRowNumber - 1));
			dateRow.getCell(0).setCellValue(date);
			if (!StringUtil.isEmpty(unit)) {
				sheet.getRow(1).createCell(realHeads.length - 1).setCellValue(unit);
			}

			// 填充每一行数据
			for (int i = 0; i < dataList.size(); i++) {
				Row newRow = sheet.createRow(templateRowNumber + i + 1);
				String[] arrays = dataList.get(i);
				for (int j = 0; j < templateRow.getPhysicalNumberOfCells(); j++) {
					// System.out.println(j);
					Cell newCell = newRow.createCell(j);
					if (templateRow.getCell(j) == null) {
						System.out.println("请检查模板【" + templatePath + "】是否缺少边框，列名称为:" + headtemplateRow.getCell(j).getStringCellValue());
						templateRow.createCell(j);
					}
					newCell.setCellStyle(templateRow.getCell(j).getCellStyle());
					short align = CellStyle.ALIGN_RIGHT;
					// templateRow.getCell(j).getCellType()==Cell.CELL_TYPE_NUMERIC
					newCell.getCellStyle().setWrapText(false);
					int tempCellType = templateRow.getCell(j).getCellType();
					if (tempCellType == Cell.CELL_TYPE_NUMERIC) {
						align = CellStyle.ALIGN_RIGHT;
					} else {
						align = CellStyle.ALIGN_LEFT;
					}

					newCell.getCellStyle().setAlignment(align);
					/*
					 * if (j >= arrays.length) { newCell.setCellValue("");
					 * continue; }
					 */

					if (map.containsKey(j)) {

						CellStyle style = templateRow.getCell(j).getCellStyle();
						if (tempCellType == Cell.CELL_TYPE_NUMERIC) {
							try {

								// 日期处理
								if (style.getDataFormatString().startsWith("yyyy/")) {
									newCell.getCellStyle().setDataFormat(templateRow.getCell(j).getCellStyle().getDataFormat());
									String newValue = arrays[map.get(j)];
									if (StringUtil.isEmpty(newValue))
										newCell.setCellValue("");
									else {
										newValue = newValue.replace("-", "/").replace(".", "/");
										if (newValue.trim().length() == 8) {
											newValue = newValue.substring(0, 4) + "/" + newValue.substring(4, 6) + "/" + newValue.substring(6, 8);
										}
										newCell.setCellValue(CommonFunctionUtils.parseDate(newValue, "yyy/MM/dd"));

									}
								}
								// 金额处理
								else {
									if (StringUtil.isEmpty(arrays[map.get(j)])) {
										newCell.setCellValue("");
									} else {
										String str = arrays[map.get(j)].replace(",", "");
										/*
										 * if (str.equals("1.140801"))
										 * System.out.println(111);
										 */
										if (str.contains("%")) {
											str = str.replace("%", "");
											newCell.setCellValue(Double.parseDouble(str) / 100);
										} else {
											newCell.setCellValue(Double.parseDouble(str));
										}

									}

								}
							} catch (Exception e) {
								e.printStackTrace();
								logger.error("error:", e);
								System.out.println("warn：请检查数据项,程序暂时冗余处理,国结文件名是:" + srcDataPath + ",sheetName=" + sheetName + ",表头是:"
										+ headtemplateRow.getCell(j).getStringCellValue());
								newCell.setCellValue(arrays[map.get(j)]);
							}

						} else {
							newCell.setCellValue(arrays[map.get(j)]);
						}

					} else
						newCell.setCellValue("");
				}
			}
			if (dataList.size() > 0)
				sheet.shiftRows(templateRowNumber + 1, templateRowNumber + dataList.size(), -1);
			else
				sheet.removeRow(templateRow);

		} catch (Exception e) {
			e.printStackTrace();
			rig = false;
		} finally {
			try {
				if (workbook != null && fos != null) {
					workbook.write(fos);
					fos.flush();
					fos.close();
				}
				IOUtils.closeQuietly(f);
			} catch (Exception e) {
				rig = false;
			}
		}
		return rig;
	}

	public static String hanleDate(String date) {
		try {
			String newdate = date.replace("/", "").replace("-", "");
			return newdate.substring(0, 4) + "年" + newdate.substring(4, 6) + "月" + newdate.substring(6, 8) + "日";
		} catch (Exception e) {
			System.out.println("日期格式【" + date + "】不符合规则，例如正确格式为【yyyyMMdd或者yyyy/MM/dd或者/yyyy-MM-dd】!");
		}
		return "";

	}
}
