package com.brilliance.eibs.bean;

import java.util.ArrayList;
import java.util.List;

/**
 * 装载javabean原型的源类
 * 
 * @author yangrui
 * 
 */
public class JavaBean
    extends Bean
{

  private static final long serialVersionUID = 1L;
  private List<Property>    props            = new ArrayList<Property>();

  public List<Property> getProps()
  {
    return props;
  }

  public void setProps(List<Property> props)
  {
    this.props = props;
  }

  /**
   * 写出javabean的开始部分。 package import class
   */
  private void start()
  {
    if (packageName.length() != 0)
      sb.append("package " + packageName + ";\n\n");
    sb.append("import java.io.Serializable;" + "\n\n");
    sb.append("public class " + className + " implements Serializable { \n");
  }

  /**
   * 写javabean的属性部分
   */
  private void generateProperties()
  {
    Property pro = null;
    for (int i = 0; i < props.size(); i++)
    {
      pro = props.get(i);
      sb.append("\tprivate " + pro.getType() + " " + pro.getName() + ";\n");
    }
    sb.append("\n");
  }

  /**
   * 写每个属性的set() and get()
   */
  private void generateMethods()
  {
    Property pro = null;
    String name = null;
    String type = null;
    for (int i = 0; i < props.size(); i++)
    {
      pro = props.get(i);
      name = pro.getName();
      type = pro.getType();

      // set()
      sb.append("\tpublic void set" + name.substring(0, 1).toUpperCase() + name.substring(1, name.length()) + "(" + type + " " + name
          + ") { \n");
      sb.append("\t\tthis." + name + " = " + name + ";\n");
      sb.append("\t}\n");

      // get() 处理boolean类型
      if (!"boolean".equals(type))
      {
        sb.append("\tpublic " + type + " get" + name.substring(0, 1).toUpperCase() + name.substring(1, name.length()) + "() { \n");
        sb.append("\t\treturn " + name + ";\n");
        sb.append("\t}\n");
      }
      else
      {
        sb.append("\tpublic " + type + " is" + name.substring(0, 1).toUpperCase() + name.substring(1, name.length()) + "() { \n");
        sb.append("\t\treturn " + name + ";\n");
        sb.append("\t}\n");
      }
    }
  }

  /**
   * 添加结尾“}”
   */
  private void end()
  {
    sb.append("}\n");
  }

  /**
   * 执行javabean的组装过程
   */
  @Override
  public void execute()
  {
    start();
    generateProperties();
    generateMethods();
    end();
  }

  @Override
  public boolean equals(Object obj)
  {
    if (obj == this)
      return true;
    if (!(obj instanceof JavaBean))
      return false;
    JavaBean bean = (JavaBean) obj;
    return super.equals(obj) && bean.getProps().equals(props);
  }

}
