package com.brilliance.eibs.coder.impl;

import com.brilliance.eibs.coder.AsymmetryCoder;
import com.brilliance.eibs.coder.SignCoder;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.util.Map;

public class DSACoder
    extends AsymmetryCoder
{

  private static final String KEY_ALGORITHM       = "DSA";
  private static final String SIGNATURE_ALGORITHM = "SHA1withDSA";

  //512-1024 64的倍数 默认1024
  private static final int    KEY_SIZE            = 1024;

  public static Map<String, Object> initKey(int keySize)
      throws NoSuchAlgorithmException
  {
    return generateKey(null, keySize);
  }

  public static Map<String, Object> initKey()
      throws NoSuchAlgorithmException
  {
    return initKey(KEY_SIZE);
  }

  public static Map<String, Object> generateKey(String strKey)
      throws NoSuchAlgorithmException
  {
    return generateKey(strKey, KEY_SIZE);
  }

  public static Map<String, Object> generateKey(String strKey, int keySize)
      throws NoSuchAlgorithmException
  {
    return generateKey(strKey, keySize, KEY_ALGORITHM);
  }

  public static byte[] sign(byte[] data, byte[] privateKey)
      throws NoSuchAlgorithmException, InvalidKeySpecException, InvalidKeyException, SignatureException
  {
    return SignCoder.sign(data, privateKey, KEY_ALGORITHM, SIGNATURE_ALGORITHM);
  }

  public static boolean verify(byte[] data, byte[] publicKey, byte[] sign)
      throws NoSuchAlgorithmException, InvalidKeySpecException, InvalidKeyException, SignatureException
  {
    return SignCoder.verify(data, publicKey, sign, KEY_ALGORITHM, SIGNATURE_ALGORITHM);
  }
}
