package com.brilliance.eibs.coder.impl;

import javax.crypto.*;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import java.security.*;
import java.security.spec.InvalidKeySpecException;

/**
 * @author hujun
 *
 */
public class PBECoder
{

  public static final String PBEWITHMD5ANDDES       = "PBEWithMD5andDES";
  public static final String PBEWITHMD5ANDTRIPLEDES = "PBEWithMD5andTripleDES";
  public static final String PBEWITHSHA1ANDDESEDE   = "PBEWithSHA1andDESede";
  public static final String PBEWITHSHA1ANDRC2_40   = "PBEWithSHA1andRC2_40";
  public static final String PBKDF2WITHHMACSHA1     = "PBKDF2WithHmacSHA1";

  //public static final String KEY_ALGORITHM   = "PBEWithMD5andDES";

  public static final int    ITERATION_COUNT        = 100;

  public static byte[] initSalt()
  {
    SecureRandom random = new SecureRandom();
    return random.generateSeed(8);
  }

  private static Key toKey(String password, String algorithm)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException
  {
    PBEKeySpec keySpec = new PBEKeySpec(password.toCharArray());
    SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(algorithm);
    return keyFactory.generateSecret(keySpec);
  }

  public static byte[] decrypt(byte[] data, String password, byte[] salt)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return decrypt(data, password, salt, PBEWITHMD5ANDDES);
  }

  public static byte[] decrypt(byte[] data, String password, byte[] salt, String algorithm)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return decrypt(data, password, salt, algorithm, ITERATION_COUNT);
  }

  public static byte[] decrypt(byte[] data, String password, byte[] salt, int iteration)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return decrypt(data, password, salt, PBEWITHMD5ANDDES, iteration);
  }

  public static byte[] decrypt(byte[] data, String password, byte[] salt, String algorithm, int iteration)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    Key k = toKey(password, algorithm);
    PBEParameterSpec parameterSpec = new PBEParameterSpec(salt, iteration);
    Cipher cipher = Cipher.getInstance(k.getAlgorithm());
    cipher.init(Cipher.DECRYPT_MODE, k, parameterSpec);
    return cipher.doFinal(data);
  }

  public static byte[] encrypt(byte[] data, String password, byte[] salt)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return encrypt(data, password, salt, PBEWITHMD5ANDDES);
  }

  public static byte[] encrypt(byte[] data, String password, byte[] salt, String algorithm)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return encrypt(data, password, salt, algorithm, ITERATION_COUNT);
  }

  public static byte[] encrypt(byte[] data, String password, byte[] salt, int iteration)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    return encrypt(data, password, salt, PBEWITHMD5ANDDES, iteration);
  }

  public static byte[] encrypt(byte[] data, String password, byte[] salt, String algorithm, int iteration)
      throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException,
      BadPaddingException, InvalidAlgorithmParameterException
  {
    Key k = toKey(password, algorithm);
    PBEParameterSpec parameterSpec = new PBEParameterSpec(salt, iteration);
    Cipher cipher = Cipher.getInstance(k.getAlgorithm());
    cipher.init(Cipher.ENCRYPT_MODE, k, parameterSpec);
    return cipher.doFinal(data);
  }

}
