package com.brilliance.eibs.core.debug;

import com.brilliance.eibs.core.debug.model.Debug;
import com.brilliance.eibs.core.debug.model.DebugIdentity;
import com.brilliance.eibs.core.frame.IIdentity;
import com.brilliance.eibs.core.model.impl.AbsDef;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author hujun
 *
 *         debug工厂
 */
public class AbstractDebugFactory {


    private static Map<DebugIdentity, IDebug<? extends AbsDef>> debugs =
            new ConcurrentHashMap<DebugIdentity, IDebug<? extends AbsDef>>();

    @SuppressWarnings("unchecked")
    public static <E extends AbsDef> IDebug<E> createDebug(String hostname, String interfaceName,
            String transactionId) {
        DebugIdentity identity = new DebugIdentity(hostname, interfaceName, transactionId);
        if (debugs.containsKey(identity)) return (IDebug<E>) debugs.get(identity);
        IDebug<E> debug = new Debug<E>();
        debugs.put(identity, debug);
        return debug;
    }

    @SuppressWarnings("unchecked")
    public static <E extends AbsDef> IDebug<E> getDebbug(IIdentity identity) {
        if (null == identity) return null;
        if (debugs.containsKey(identity)) return (IDebug<E>) debugs.get(identity);
        return null;
    }

    public static void clear(String hostname) {
        for (Map.Entry<DebugIdentity, IDebug<? extends AbsDef>> entry : debugs.entrySet()) {
            DebugIdentity identity = entry.getKey();
            if (identity.getHostname().equals(hostname)) entry.getValue().turnOffAll();
        }
    }

}
