package com.brilliance.eibs.core.debug.model;

import com.brilliance.eibs.core.debug.IDebug;
import com.brilliance.eibs.core.debug.IDebugAdaptor;
import com.brilliance.eibs.core.debug.adapor.AbstractDebugAdaptor;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author hujun
 *
 *         debug类，用于获取调试服务中的
 */
public class Debug<E> implements IDebug<E> {

    private Lock lock = new ReentrantLock();
    private Condition condition = lock.newCondition();

    private Map<E, IDebugAdaptor<E>> adaptors = new ConcurrentHashMap<E, IDebugAdaptor<E>>();


    public IDebugAdaptor<E> getAdaptor(E e) {
        if (adaptors.containsKey(e)) return adaptors.get(e);
        return null;
    }

    public IDebugAdaptor<E> addAdaptor(E e) {
        IDebugAdaptor<E> adaptor = getAdaptor(e);
        if (null != adaptor) return adaptor;
        adaptor = new AbstractDebugAdaptor<E>(e, new Breakpoint<E>(this));
        adaptors.put(e, adaptor);
        return adaptor;
    }

    @Override
    public void suspend() {
        lock.lock();
        try {
            condition.await();
        } catch (InterruptedException e) {} finally {
            lock.unlock();
        }
    }

    @Override
    public void resume() {
        lock.lock();
        try {
            condition.signal();
        } finally {
            lock.unlock();
        }
    }

    @Override
    public void turnOffAll() {
        for (Map.Entry<E, IDebugAdaptor<E>> entry : adaptors.entrySet()) {
            entry.getValue().turnOff();
        }
    }

}
