package com.brilliance.eibs.core.frame.instance.impl;

import com.brilliance.eibs.core.debug.AbsDebugInstance;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.ICallDef;
import com.brilliance.eibs.core.model.impl.AbsDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.core.service.TransactionThread;
import com.brilliance.eibs.core.service.instance.impl.AbsFilter;
import com.brilliance.eibs.core.threadpool.ThreadPoolFactory;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.MDC;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;

public class CallInstance extends AbsDebugInstance {

    private ICallDef callDef;
    private AbsFilter filter;

    public CallInstance(ICallDef callDef, AbsFilter filter) {
        super((AbsDef) callDef);
        this.callDef = callDef;
        this.filter = filter;
    }

    @Override
    public void normalProcess(Context context) {
        ResultMsg resultMsg = null;
        AbstractEL elParser = context.getElParser();
        Logger logger = LogUtil.getLogger(context);
        String iName = elParser.getExPression(context, callDef.getInterfaceName(), null).toString();
        String tName = elParser.getExPression(context, callDef.getTransactionName(), null).toString();
        String type = callDef.getType();
        String mode = callDef.getMode();

        String pre = elParser.getExPression(context, callDef.getPre(), null).toString();
        boolean megre = Boolean.valueOf(callDef.getMerge());

        Object[] args = filter.dealArguments(callDef.getArguments());

        Context subcontext = new Context();
        subcontext.setInputMap(context.getInputMap());

        if (StringUtil.isEmpty(mode) || mode.equals("T")) {
            String parentName = (String) MDC.get(LogUtil.LOG_FILE_NAME_SYMBOL);
            if (type.equals("subx")) {

                subcontext.addVariable("transaction", Constants.Log_VAR, LogUtil.getTrnLog(iName, tName));
                resultMsg = new Client().call(subcontext, iName, tName, args);

                /*
                 * Map<String, Object> map = context.getOutputMap(); for (String
                 * key : map.keySet()) { context.addVariable("transaction", key,
                 * map.get(key)); }
                 */

                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
                if (!resultMsg.isSuccess()) {
                    // context.getResource().setErrorcode(resultMsg.getContent().toString());
                    context.getResource().setErrorcode(resultMsg.getReturnCode());
                    context.setObject(resultMsg.getContent());
                    context.getResource().setDescrption(resultMsg.getDescription());
                    context.getResource().setDescription(resultMsg.getDescription());
                    Exception e = resultMsg.getCauseException();
                    if (e instanceof InterfaceException)
                        throw (InterfaceException) e;
                    else
                        throw new InterfaceException("10000", "call sub-transaction exception", resultMsg.getCauseException());
                }
            } else if (type.equals("sub")) {

                Logger log = (Logger) context.getVariable(Constants.Log_VAR);
                resultMsg = new Client().call(log, iName, tName, args, subcontext);

                /*
                 * Map<String, Object> map = context.getOutputMap(); for (String
                 * key : map.keySet()) { context.addVariable("transaction", key,
                 * map.get(key)); }
                 */

                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
                if (!resultMsg.isSuccess()) {
                    // context.getResource().setErrorcode(resultMsg.getContent().toString());
                    context.getResource().setErrorcode(resultMsg.getReturnCode());
                    context.setObject(resultMsg.getContent());
                    context.getResource().setDescrption(resultMsg.getDescription());
                    context.getResource().setDescription(resultMsg.getDescription());
                    Exception e = resultMsg.getCauseException();
                    if (e instanceof InterfaceException) {
                        throw (InterfaceException) e;
                    } else {
                        if (!StringUtil.isEmpty(resultMsg.getReturnCode()) || !StringUtil.isEmpty(resultMsg.getDescription())) {
                            throw new InterfaceException(resultMsg.getReturnCode(), resultMsg.getDescription(), resultMsg.getCauseException());
                        } else {
                            throw new InterfaceException("10000", "call sub-transaction exception", resultMsg.getCauseException());
                        }

                    }

                }
            } else if (type.equals("sub_conn")) {
                resultMsg = new Client().call(iName, tName, args, null, context);
                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
                if (!resultMsg.isSuccess()) {
                    context.getResource().setErrorcode(resultMsg.getContent().toString());
                    Exception e = resultMsg.getCauseException();
                    if (e instanceof InterfaceException)
                        throw (InterfaceException) e;
                    else
                        throw new InterfaceException("10000", "call sub-transaction exception", resultMsg.getCauseException());
                }
            } else if (type.equals("chain")) {
                resultMsg = new Client().call(iName, tName, args, null, subcontext, pre, megre);
                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
            } else if (type.equals("sub_chain")) {
                Logger log = (Logger) context.getVariable(Constants.Log_VAR);
                resultMsg = new Client().call(iName, tName, args, log, subcontext);
                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
            } else if (type.equals("chain_context")) {
                Logger log = (Logger) context.getVariable(Constants.Log_VAR);
                resultMsg = new Client().call(log, iName, tName, args, subcontext);
                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
                context.copy(subcontext);
            } else if (type.equals("sub_context")) {

                Logger log = (Logger) context.getVariable(Constants.Log_VAR);
                resultMsg = new Client().call(log, iName, tName, args, subcontext);

                /*
                 * Map<String, Object> map = context.getOutputMap(); for (String
                 * key : map.keySet()) { context.addVariable("transaction", key,
                 * map.get(key)); }
                 */

                filter.saveToContext(callDef.getScope(), callDef.getTag(), resultMsg);
                if (!resultMsg.isSuccess()) {
                    // context.getResource().setErrorcode(resultMsg.getContent().toString());
                    context.getResource().setErrorcode(resultMsg.getReturnCode());
                    context.setObject(resultMsg.getContent());
                    context.getResource().setDescrption(resultMsg.getDescription());
                    context.getResource().setDescription(resultMsg.getDescription());
                    Exception e = resultMsg.getCauseException();
                    if (e instanceof InterfaceException)
                        throw (InterfaceException) e;
                    else
                        throw new InterfaceException("10000", "call sub-transaction exception", resultMsg.getCauseException());
                }
                context.copy(subcontext);
            }
            MDC.put(LogUtil.LOG_FILE_NAME_SYMBOL, parentName);
        } else if (mode.equals("S")) {

            TransactionThread run = new TransactionThread(iName, tName, args);
            Thread t = new Thread(run);
            t.start();
            try {
                t.join();
            } catch (InterruptedException e) {
                logger.error(
                        "Thread of call [ InterfaceName=" + callDef.getInterfaceName() + ", TransactionName=" + callDef.getTransactionName()
                                + " ] is error.");
            }
        } else if (mode.equals("AS")) {
            Object[] nwargs = (Object[]) cloneBySer(args);
            TransactionThread run = new TransactionThread(iName, tName, nwargs, pre);
            Future<?> f = ThreadPoolFactory.getFixedExecutor(40, "callTrn_" + iName + "_" + tName).submit(run);
            filter.saveToContext(callDef.getScope(), callDef.getTag(), f);
        } else if (mode.equals("single")) {
            // 特殊需求
            if (args != null && args.length > 0) {
                String key = "" + args[args.length - 1];
                logger.info("execute single call key = " + key);
                Executor singleExecutor = ThreadPoolFactory.getSingleExecutor(key);
                TransactionThread run = new TransactionThread(iName, tName, args);
                logger.info("queue size = " + ((ThreadPoolExecutor) singleExecutor).getQueue().size());
                singleExecutor.execute(run);
            }
        }

        // modified by gechengyang 20160515
        Map<String, Object> outMap = new HashMap<String, Object>(subcontext.getOutputMap());
        context.setOutputMap(outMap);
        subcontext.getOutputMap().clear();

    }

    public Object cloneBySer(Object baseObj) {
        Object o = null;
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(baos);
            oos.writeObject(baseObj);
            oos.close();
            ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
            ObjectInputStream ois = new ObjectInputStream(bais);
            o = ois.readObject();
            ois.close();
        } catch (IOException e) {
            throw new InterfaceException("10000", "call clone IOException" + e.getMessage());
        } catch (ClassNotFoundException e) {
            throw new InterfaceException("10000", "call clone ClassNotFoundException" + e.getMessage());
        }
        return o;
    }
}
