package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.model.IConnectionDef;
import com.brilliance.eibs.core.model.IPropertyDef;
import com.brilliance.eibs.core.model.converter.ConnectionDefConverter;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.StringUtil;
import com.thoughtworks.xstream.annotations.*;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @see IConnectionDef
 */
@XStreamAlias("connection")
@XStreamConverter(ConnectionDefConverter.class)
public class ConnectionDef
    extends AbsDef
    implements IConnectionDef
{
  private static final long         serialVersionUID = 1L;
  @XStreamOmitField
  private String                    id;
  @XStreamAsAttribute
  private String                    joinedid;
  @XStreamAsAttribute
  private String                    ref;
  @XStreamOmitField
  private String                    via;
  @XStreamAsAttribute
  private String                    type;                                                  // 类型
  @XStreamOmitField
  private String                    executeClass;
  @XStreamOmitField
  private boolean                   pooled;                                                // 是否池化
  @XStreamOmitField
  private boolean                   keepalive;                                             //是否为长连接
  @XStreamOmitField
  private PoolSetting               poolSetting;                                           // 池设置

  @XStreamImplicit
  private Map<String, IPropertyDef> propertyMap      = new HashMap<String, IPropertyDef>();

  // added by hujun on 2014/6/17
  @XStreamOmitField
  private Map<String, String>       encryptPropMap   = new HashMap<String, String>();
  @XStreamImplicit
  private List<Object>              subList          = new ArrayList<Object>();
  @XStreamAsAttribute
  private String                    encoding;
  @XStreamAsAttribute
  private String                    times;                                                 //允许重复执行次数
  @XStreamOmitField
  private String                    errorDescription;

  public String getRef()
  {
    return ref;
  }

  public void setRef(String ref)
  {
    this.ref = ref;
  }

  public String getVia()
  {
    return via;
  }

  public void setVia(String via)
  {
    this.via = via;
  }

  public PoolSetting getPoolSetting()
  {
    if (poolSetting == null)
      return new PoolSetting();

    return poolSetting;
  }

  public void setPoolSetting(PoolSetting poolSetting)
  {
    this.poolSetting = poolSetting;
  }

  public boolean isPooled()
  {
    return pooled;
  }

  public void setPooled(boolean pooled)
  {
    this.pooled = pooled;
  }

  public String getType()
  {
    return type;
  }

  public void setType(String type)
  {
    this.type = type;
  }

  public String getEncoding()
  {
    if (StringUtil.isEmpty(encoding))
      encoding = PropertyUtil.getProperty("sys.properties", "SYS_ENCODING");
    return encoding;
  }

  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  public String getTimes()
  {
    if (StringUtil.isEmpty(times))
      times = "1";
    return times;
  }

  public void setTimes(String times)
  {
    this.times = times;
  }

  public String getExecuteClass()
  {
    return executeClass;
  }

  public void setExecuteClass(String executeClass)
  {
    this.executeClass = executeClass;
  }

  public class PoolSetting
      implements Serializable
  {
    private static final long serialVersionUID = 1L;
    final Map<String, String> properties       = new HashMap<String, String>();

    public Map<String, String> getProperties()
    {
      return this.properties;
    }

    public String getProperty(String name)
    {
      return (String) properties.get(name);
    }

    public void addProperty(String name, String value)
    {
      if (!properties.containsKey(name))
        properties.put(name, value);
    }

    public boolean equals(Object obj)
    {
      boolean b = EqualsBuilder.reflectionEquals(this, obj);
      return b;
    }

    public int hashCode()
    {
      return HashCodeBuilder.reflectionHashCode(this);
    }
  }

  public boolean getKeepalive()
  {
    return keepalive;
  }

  public void setKeepalive(boolean keepalive)
  {
    this.keepalive = keepalive;
  }

  public String getId()
  {
    return id;
  }

  public void setId(String id)
  {
	this.id = id;
  }

  public String getJoinedid()
  {
    return joinedid;
  }

  public void setJoinedid(String joinedid)
  {
    this.joinedid = joinedid;
  }

  public List<Object> getSubList()
  {
    return subList;
  }

  public void setSubList(List<Object> subList)
  {
    this.subList = subList;
  }

  public Map<String, IPropertyDef> getPropertyMap()
  {
    return propertyMap;
  }

  public Map<String, String> getEncryptPropMap()
  {
    return encryptPropMap;
  }

  public IPropertyDef getProperty(String key)
  {
    if (key.equals("password"))
    {
      IPropertyDef def = propertyMap.get(key);
      if (null == def)
      {
        String xp = IPropertyDef.ENCRYPT_PRE + key;
        return propertyMap.get(xp);
      }
      return def;
    }
    return encryptPropMap.get(key) == null ? propertyMap.get(key) : propertyMap.get(encryptPropMap.get(key));
  }

  public void addProperty(String key, IPropertyDef propertyDef)
  {
    this.propertyMap.put(key, propertyDef);
  }

  public void addEncryptProperty(String key, String xkey)
  {
    this.encryptPropMap.put(key, xkey);
  }

  public void addSub(Object obj)
  {
    if (obj != null)
      this.subList.add(obj);
  }

  @Override
  public String getErrorDescription()
  {
    return errorDescription;
  }

  @Override
  public void setErrorDescription(String description)
  {
    this.errorDescription = description;
  }

  public void combine(IConnectionDef connectionDef)
  {
    setId(connectionDef.getId());
    setExecuteClass(connectionDef.getExecuteClass());
    setKeepalive(connectionDef.getKeepalive());
    setPooled(connectionDef.isPooled());
    setPoolSetting(connectionDef.getPoolSetting());
    setEncoding(connectionDef.getEncoding());
    setErrorDescription(connectionDef.getErrorDescription());
    setJoinedid(connectionDef.getJoinedid());
    setRef(connectionDef.getRef());
    setSubList(connectionDef.getSubList());
    setTimes(connectionDef.getTimes());
    setType(connectionDef.getType());
    setVia(connectionDef.getVia());
    propertyMap.putAll(connectionDef.getPropertyMap());
    encryptPropMap.putAll(connectionDef.getEncryptPropMap());
  }

  public boolean equals(Object obj)
  {
    boolean b = EqualsBuilder.reflectionEquals(this, obj);
    return b;
  }

  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

  @Override
  public String getUnique()
  {
    return "connection:" + ref;
  }

  @Override
  public void addProperty(IPropertyDef propertyDef)
  {
    this.propertyMap.put(propertyDef.getName(), propertyDef);
  }

  @Override
  public void addProperty(String key, String value)
  {
    this.propertyMap.put(key, new PropertyDef(key, value));
  }

}
