package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.exception.ErrorCodes;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.factory.BeanFactory.FunctionKey;
import com.brilliance.eibs.factory.FactoryManager;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.util.*;

/**
 * @see com.brilliance.interfaceWork.core.model.IInterfaceDef
 */
@XStreamAlias("interface")
public class InterfaceDef
    implements IInterfaceDef
{

  private static final long                serialVersionUID    = 1L;
  @XStreamAsAttribute
  private String                           id;
  @XStreamOmitField
  private boolean                          singleton;                                                       // 是否单例
  @XStreamAsAttribute
  private String                           basePackage;                                                     // 基础包名
  @XStreamAsAttribute
  private String                           version;                                                         //接口版本
  @XStreamOmitField
  private String                           logLevel;                                                        //接口logger级别（可以适当减少log输出）
  @XStreamOmitField
  private String                           description;                                                     // 描述
  @XStreamOmitField
  private String                           encoding;
  @XStreamAsAttribute
  private String                           language;
  @XStreamAsAttribute
  private String                           type;
  @XStreamAsAttribute
  private String                           defaultScope;
  @XStreamImplicit
  private List<ITransactionDef>            transactionDefs     = new ArrayList<ITransactionDef>();
  @XStreamOmitField
  private ErrorCodes                       errorCodes;
  @XStreamOmitField
  private Map<String, Map<String, String>> codetableMap        = new HashMap<String, Map<String, String>>();
  @XStreamOmitField
  private Map<String, String>              dynamicCodetableMap = new HashMap<String, String>();
  @XStreamOmitField
  private Map<String, IModuleDef>          moduleMap           = new HashMap<String, IModuleDef>();
  @XStreamOmitField
  private Map<FunctionKey, IFunctionDef>   functionMap         = new HashMap<FunctionKey, IFunctionDef>();
  @XStreamOmitField
  private Map<FunctionKey, IFuncDef>       funcMap             = new HashMap<FunctionKey, IFuncDef>();
  @XStreamOmitField
  private Map<String, ICounterDef>         countermap          = new HashMap<String, ICounterDef>();
  @XStreamOmitField
  private ResourceBundle                   resb;

  public boolean isInner()
  {
    if (id.startsWith("__"))
      return true;
    return false;
  }

  /**
   * modified by weicong 2016-04-05
   * 保存该interface对象所用到的connections和filters集合
   */
  @XStreamOmitField
  private List<IConnectionDef> connections = new ArrayList<IConnectionDef>();

  @XStreamOmitField
  private List<IFilterDef>     filters     = new ArrayList<IFilterDef>();

  public void addFilters(IFilterDef filterDef)
  {
    if (filters.size() == 0)
      filters.add(filterDef);
    else
    {
      boolean flag = true;
      for (IFilterDef fl : filters)
      {
        if (filterDef.getId().equals(fl.getId()))
        {
          flag = false;
          break;
        }
      }
      if (flag)
        filters.add(filterDef);
    }
  }

  public void addConnection(IConnectionDef connectionDef)
  {
    if (connections.size() == 0)
      connections.add(connectionDef);
    else
    {
      boolean flag = true;
      for (IConnectionDef cn : connections)
      {
        if (connectionDef.getId().equals(cn.getId()))
        {
          flag = false;
          break;
        }
      }
      if (flag)
        connections.add(connectionDef);
    }
  }

  public List<IConnectionDef> getConnections()
  {
    return connections;
  }

  public void setConnections(List<IConnectionDef> connections)
  {
    this.connections = connections;
  }

  public Map<String, IModuleDef> getModuleMap()
  {
    return moduleMap;
  }

  public void setModuleMap(Map<String, IModuleDef> moduleMap)
  {
    this.moduleMap = moduleMap;
  }

  public Map<FunctionKey, IFunctionDef> getFunctionMap()
  {
    return functionMap;
  }

  public Map<String, ICounterDef> getCountermap()
  {
    return countermap;
  }

  public void setCountermap(Map<String, ICounterDef> countermap)
  {
    this.countermap = countermap;
  }

  public void setFunctionMap(Map<FunctionKey, IFunctionDef> functionMap)
  {
    this.functionMap = functionMap;
  }

  public ResourceBundle getResb()
  {
    return resb;
  }

  public void setResb(ResourceBundle resb)
  {
    this.resb = resb;
  }

  public Map<String, Map<String, String>> getCodetableMap()
  {
    return codetableMap;
  }

  public void setCodetableMap(Map<String, Map<String, String>> codetableMap)
  {
    this.codetableMap = codetableMap;
  }

  public Map<String, String> getDynamicCodetableMap()
  {
    return dynamicCodetableMap;
  }

  public void setDynamicCodetableMap(Map<String, String> dynamicCodetableMap)
  {
    this.dynamicCodetableMap = dynamicCodetableMap;
  }

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public String getId()
  {
    return id;
  }

  public void setId(String id)
  {
    this.id = id;
  }

  public boolean isSingleton()
  {
    return singleton;
  }

  public void setSingleton(boolean singleton)
  {
    this.singleton = singleton;
  }

  public String getBasePackage()
  {
    return basePackage;
  }

  public void setBasePackage(String basePackage)
  {
    this.basePackage = basePackage;
  }

  public String getEncoding()
  {
    return encoding;
  }

  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  public String getDefaultScope()
  {
    return defaultScope;
  }

  public void setDefaultScope(String defaultScope)
  {
    this.defaultScope = defaultScope;
  }

  public String getLanguage()
  {
    return language;
  }

  public void setLanguage(String language)
  {
    this.language = language;
  }

  public String getType()
  {
    return type;
  }

  public void setType(String type)
  {
    this.type = type;
  }

  public String getVersion()
  {
    return version;
  }

  public void setVersion(String version)
  {
    this.version = version;
  }

  public String getLogLevel()
  {
    return logLevel;
  }

  public void setLogLevel(String logLevel)
  {
    this.logLevel = logLevel;
  }

  public List<ITransactionDef> getTransactionDefs()
  {
    return transactionDefs;
  }

  public void setTransactionDefs(List<ITransactionDef> transactionDefs)
  {
    this.transactionDefs = transactionDefs;
  }

  public ErrorCodes getErrorCodes()
  {
    return errorCodes;
  }

  public void setErrorCodes(ErrorCodes errorCodes)
  {
    this.errorCodes = errorCodes;
  }

  public boolean equals(Object obj)
  {
    return EqualsBuilder.reflectionEquals(this, obj);
  }

  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

  public Map<FunctionKey, IFuncDef> getFuncMap()
  {
    return funcMap;
  }

  public void setFuncMap(Map<FunctionKey, IFuncDef> funcMap)
  {
    this.funcMap = funcMap;
  }

  public void addFunc(String namespace, String cls, String stc)
  {
    IFuncDef functionDef = new FuncDef();
    functionDef.setNamespace(namespace);
    functionDef.setCls(cls);
    functionDef.setStc(stc);
    funcMap.put(FactoryManager.getBeanFactory().new FunctionKey(namespace + cls + stc, null), functionDef);
  }

  public List<IFilterDef> getFilters()
  {
    return filters;
  }

  public void setFilters(List<IFilterDef> filters)
  {
    this.filters = filters;
  }

}
