package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.model.IArgumentDef;
import com.brilliance.eibs.core.model.IPropertyDef;
import com.brilliance.eibs.factory.parser.impl.XPwdCoder;
import com.brilliance.eibs.util.StringUtil;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@XStreamAlias("property")
public class PropertyDef implements IPropertyDef, Serializable {
	private static final long serialVersionUID = 1L;
	@XStreamAsAttribute
	private String name;
	@XStreamAsAttribute
	private String value;
	@XStreamOmitField
	private String scope;
	@XStreamOmitField
	private String encrypt;
	@XStreamOmitField
	private String flg;
	@XStreamImplicit
	private List<IArgumentDef> arguments = new ArrayList<IArgumentDef>();

	public PropertyDef() {

	}

	public PropertyDef(String name, String value) {
		this.name = name;
		this.value = value;
	}

	public String getEncrypt() {
		return encrypt;
	}

	public void setEncrypt(String encrypt) {
		this.encrypt = encrypt;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getValue() {

		if (this.name.equals(IPropertyDef.ENCRYPT_PRE + "password")) {
			return new XPwdCoder().decrypt(this.value, StringUtil.isEmpty(encrypt) ? XPwdCoder.DES_ALGORITHM : encrypt);
		}
		String s = !StringUtil.isEmpty(this.encrypt) && this.name.startsWith(IPropertyDef.ENCRYPT_PRE) ? new XPwdCoder().decrypt(this.value, encrypt)
				: this.value;
		return s;

	}

	public void setValue(String value) {
		this.value = value;
	}

	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	public List<IArgumentDef> getArguments() {
		return arguments;
	}

	public void setArguments(List<IArgumentDef> arguments) {
		this.arguments = arguments;
	}

	public boolean equals(Object obj) {
		boolean b = EqualsBuilder.reflectionEquals(this, obj);
		return b;
	}

	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	public String getFlg() {
		return flg;
	}

	public void setFlg(String flg) {
		this.flg = flg;
	}
}
