package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.model.IDefinition;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.model.converter.ServiceDefConverter;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

@XStreamAlias("service")
@XStreamConverter(ServiceDefConverter.class)
public class ServiceDef implements IServiceDef, IDefinition {
	private static final long serialVersionUID = 1L;
	@XStreamAsAttribute
	private String id;
	@XStreamAsAttribute
	private String type;
	@XStreamAsAttribute
	private String language;
	@XStreamAsAttribute
	private String encoding;
	@XStreamAsAttribute
	private String state;
	@XStreamAsAttribute
	private String expression;
	@XStreamAsAttribute
	private String description;
	@XStreamImplicit
	private Map<String, String> propertyMap = new HashMap<String, String>();

	private Map<String, Map<String, String>> propertyArguments = new HashMap<String, Map<String, String>>();

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getLanguage() {
		return language;
	}

	public void setLanguage(String language) {
		this.language = language;
	}

	public String getEncoding() {
		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getExpression() {
		return expression;
	}

	public void setExpression(String expression) {
		this.expression = expression;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Map<String, String> getPropertyMap() {
		return propertyMap;
	}

	public void setPropertyMap(Map<String, String> propertyMap) {
		this.propertyMap = propertyMap;
	}

	public void addProperty(String key, String value) {
		this.propertyMap.put(key, value);
	}

	public void addPropertyArgument(String propertyKey,String argumentKey,String argumentVal){
		Map<String, String> arguments=this.propertyArguments.get(propertyKey);
		if(arguments==null){
			arguments=new HashMap<String, String>();
			this.propertyArguments.put(propertyKey, arguments);
		}
		arguments.put(argumentKey, argumentVal);
	}
	
	public Map<String, String> getPropertyArguments(String propertyKey){
		return this.propertyArguments.get(propertyKey);
	}
	
	public boolean equals(Object obj) {
		boolean b = EqualsBuilder.reflectionEquals(this, obj);
		return b;
	}

	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	public IServiceDef deepClone() throws IOException, ClassNotFoundException {
		// 将对象写到流里
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(bos);
		oos.writeObject(this);
		// 从流里读回来
		ByteArrayInputStream bis = new ByteArrayInputStream(bos.toByteArray());
		ObjectInputStream ois = new ObjectInputStream(bis);
		return (ServiceDef) ois.readObject();
	}
}
