package com.brilliance.eibs.core.module;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.IModuleDef;
import com.brilliance.eibs.util.StringUtil;

import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.Map;

public class FixedModule
    extends AbsModule
{
  private Object[] fields;
  private int[]    length;
  private String[] type;
  private int      fieldCnt;
  private boolean  hasSignal;
  private String   numberFillChar = "0";
  private String   numberFillPos  = "left";
  private String   stringFillChar = " ";
  private String   stringFillPos  = "right";

  public FixedModule( IModuleDef moduleDef )
  {
    super(moduleDef);
    fieldCnt = moduleDef.getMap().size();
    fields = new String[fieldCnt];
    length = new int[fieldCnt];
    type = new String[fieldCnt];
    init();
  }

  public FixedModule( String encoding, Object obj, IModuleDef moduleDef )
  {
    super(encoding, obj, moduleDef);
    fieldCnt = moduleDef.getMap().size();
    fields = new String[fieldCnt];
    length = new int[fieldCnt];
    unpack();
  }

  public void setNumberFillStrategy(String pos, String fillChar)
  {
    numberFillPos = pos;
    numberFillChar = fillChar;
  }

  public void setStringFillStrategy(String pos, String fillChar)
  {
    stringFillPos = pos;
    stringFillChar = fillChar;
  }

  private void init()
  {
    Map<String, IFieldDef> fields = moduleDef.getMap();
    Iterator<String> it = fields.keySet().iterator();
    while (it.hasNext())
    {
      IFieldDef fieldDef = fields.get(it.next());
      int index = Integer.valueOf(fieldDef.getEtag());
      length[index] = Integer.valueOf(fieldDef.getSize());
      type[index] = fieldDef.getType();
    }
  }

  /**
   * 解析模型
   */
  private void unpack()
  {
    init();
    int startIndex = 0;
    if (obj instanceof byte[])
    {
      byte[] bytes = (byte[]) obj;
      for (int i = 0; i < this.fields.length; i++)
      {
        int len = length[i];
        byte[] retBytes = new byte[len];
        byte[] newbytes = null;
        newbytes = new byte[bytes.length - len];
        System.arraycopy(bytes, 0, retBytes, 0, len);
        System.arraycopy(bytes, len, newbytes, 0, bytes.length - len);
        bytes = newbytes;
      }
    }
    else if (obj instanceof String)
    {
      String value = obj.toString();
      for (int i = 0; i < this.fields.length; i++)
      {
        this.fields[i] = value.substring(startIndex, startIndex + this.length[i]);
        startIndex += this.length[i];
      }
    }
  }

  private String fillToSize(String value, String type, int size)
  {
    int length = value.length();
    if (length < size)
    {
      if (hasSignal)
      {
        if (value.startsWith("-") || value.startsWith("+"))
        {
          if (numberFillPos.equals("left"))
            value = value.substring(0, 1) + StringUtil.repeat(numberFillChar, size - length) + value.substring(1);
          else if (numberFillPos.equals("right"))
            value = value + StringUtil.repeat(numberFillChar, size - length);
        }
        else
        {
          if (numberFillPos.equals("left"))
            value = "+" + StringUtil.repeat(numberFillChar, size - length - 1) + value;
          else if (numberFillPos.equals("right"))
            value = "+" + value + StringUtil.repeat(numberFillChar, size - length);
        }
      }
      else if (type.startsWith("decimal") || type.startsWith("number"))
      {
        if (numberFillPos.equals("left"))
          value = StringUtil.repeat(numberFillChar, size - length) + value;
        else if (numberFillPos.equals("right"))
          value = value + StringUtil.repeat(numberFillChar, size - length);
      }
      else if (type.equals("string"))
        if (stringFillPos.equals("left"))
          value = StringUtil.repeat(stringFillChar, size - length) + value;
        else if (stringFillPos.equals("right"))
          value = value + StringUtil.repeat(stringFillChar, size - length);
    }
    else if (size < length)
    {
      throw new InterfaceException("01003");
    }
    return value;
  }

  /**
   * 添加数值类型域
   * @param index
   * @param value
   * @param type
   */
  private void addNumerField(int index, Object value, String type)
  {
    int start = type.indexOf('(');
    String subType = type.substring(start + 1, type.length() - 1);
    String[] number = subType.split(",");
    int num0 = Integer.parseInt(number[0]);
    int size = num0;
    int num1 = 0;
    if (number.length > 1)
    {
      num1 = Integer.parseInt(number[1]);
      if (num1 > 0)
        size += 1;//有小数点,加1位
    }
    if (number.length > 2)
    {
      hasSignal = Boolean.valueOf(number[2]);
      if (hasSignal)
        size += 1;//有符号位,加1位
    }
    String realValue = null;
    if (value == null || StringUtil.isEmpty(value.toString()))//如果当前值为空，按照类型初始化
    {
      realValue = "0";
      if (num1 != 0)//有小数点
      {
        realValue += "." + StringUtil.repeat("0", num1);
      }
    }
    else
    {
      realValue = value.toString();
      //value小数点后缺多少位补多少
      if (num1 > 0)
      {
        if (realValue.contains("e") || realValue.contains("E"))
        {
          String pattern = "0." + StringUtil.repeat("0", num1);
          realValue = StringUtil.formatDouble(Double.valueOf(realValue), pattern);
        }
        int less;
        if (-1 == realValue.indexOf('.'))//找不到小数点
        {
          less = num1;
          realValue += '.';
        }
        else
        {
          less = num1 - (realValue.length() - realValue.indexOf('.') - 1);
        }
        if (less > 0)
        {
          realValue += StringUtil.repeat("0", less);
        }
        else if (less < 0)
        {
          realValue = realValue.substring(0, realValue.length() + less);
        }
      }
      //判断value值是否符合type类型
      String regex = "[+-]?\\d{0," + (num0 - num1) + "}";
      if (num1 > 0)
        regex += "\\.\\d{" + num1 + "}";
      if (!realValue.matches(regex))
      {
        throw new InterfaceException("01001", "value [" + value + "] does not match type: " + type);
      }
    }
    fields[index] = fillToSize(realValue, type, length[index]);
  }

  private void addStringField(int index, Object value, String type)
  {
    String realValue = null;
    if (value == null)
      realValue = "";
    else
      realValue = value.toString();
    fields[index] = fillToSize(realValue, type, length[index]);
  }

  private void addBytesField(int index, Object value, String fType)
  {
    if (value instanceof Integer)
    {
      if (length[index] > 4)
      {
        throw new InterfaceException("01002", "size can not be larger than 4 when convert a Integer to byte[]");
      }
      else
      {
        byte[] bytes = new byte[length[index]];
        int src = (Integer) value;
        for (int i = 0; i < length[index]; i++)
        {
          bytes[i] = (byte) ((byte) ((src >> ((length[index] - i - 1) * 8)) & 0xFF));
        }
        fields[index] = bytes;
      }
    }
    else
    {
      byte[] bytes = null;
      if (value instanceof byte[])
      {
        bytes = (byte[]) value;
      }
      else
      {
        try
        {
          bytes = value.toString().getBytes(encoding);
        }
        catch (UnsupportedEncodingException e)
        {
          throw new InterfaceException("00403", "unsupported encoding: " + encoding, e);
        }
      }
      byte[] newbytes = new byte[length[index]];
      if (bytes.length > length[index])
      {
        throw new InterfaceException("01003", "value is too long,larger than [size:" + length[index] + "]");
      }
      System.arraycopy(bytes, 0, newbytes, length[index] - bytes.length, bytes.length);
      //      if (bytes.length < length[index])
      //      {
      //        for (int i = 0; i < length[index] - bytes.length; i++)
      //        {
      //          newbytes[i] = 0x00;
      //        }
      //      }
      fields[index] = newbytes;
    }
  }

  /**
   * 组装添加数据域
   * @param index
   * @param value
   */
  public void addField(int index, Object value)
  {
    String fType = this.type[index];
    if (fType.startsWith("number") || fType.startsWith("decimal"))
    {
      addNumerField(index, value, fType);
    }
    else if (fType.equals("byte[]"))
    {
      addBytesField(index, value, fType);
    }
    else
    {
      addStringField(index, value, fType);
    }

  }

  /**
   * 获取某个域值
   * @param index
   * @return
   */
  public Object getField(int index)
  {
    return fields[index];
  }

  private boolean hasBytes()
  {
    for (int i = 0; i < fieldCnt; i++)
    {
      if (fields[i] instanceof byte[])
        return true;
    }
    return false;
  }

  public int getTotalLength()
  {
    int totalLen = 0;
    for (int i = 0; i < fieldCnt; i++)
      totalLen += length[i];
    return totalLen;
  }

  /**
   * 获取module组装值
   * @return
   */
  public Object getModuleValue()
  {
    if (hasBytes())
    {
      byte[] bytes = new byte[getTotalLength()];
      int startIndex = 0;
      for (Object obj : fields)
      {
        byte[] objbytes = null;
        if (obj instanceof String)
        {
          try
          {
            objbytes = ((String) obj).getBytes(encoding);
          }
          catch (UnsupportedEncodingException e)
          {
            throw new InterfaceException("00403", "unsupported encoding:" + encoding, e);
          }
        }
        else if (obj instanceof byte[])
        {
          objbytes = ((byte[]) obj);
        }
        System.arraycopy(objbytes, 0, bytes, startIndex, objbytes.length);
        startIndex += objbytes.length;
      }
      return bytes;
    }
    else
    {
      StringBuffer sb = new StringBuffer();
      for (int i = 0; i < fieldCnt; i++)
      {
        if (fields[i] == null)
          fields[i] = StringUtil.repeat(" ", length[i]);
        sb.append(fields[i]);
      }
      return sb.toString();
    }
  }

  @Override
  public Map<String, Object> packModule()
  {
    return null;
  }

  @Override
  public Map<String, Object> unpackModule()
  {
    return null;
  }
}
