package com.brilliance.eibs.core.module;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.IModuleDef;
import com.brilliance.eibs.core.model.ISubfieldDef;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ISO8583Module extends AbsModule {
    private byte[] bitmap;
    private byte[] body;
    private int len;

    public ISO8583Module(String encoding, Object obj, IModuleDef moduleDef) {
        super(encoding, obj, moduleDef);
    }

    /**
     * bitmap转换成二进制字符串
     * **/
    public String toBinaryString(byte[] bytes) {
        String str = "";
        for (byte b : bytes) {
            String hex = Integer.toBinaryString(b & 0xff);
            hex = String.format("%08d", Integer.parseInt(hex));
            str += hex;
        }
        return str;
    }

    public void init() {
        byte[] bytes = (byte[]) obj;
        byte fisrtbtye = bytes[0];
        fisrtbtye = (byte) (bytes[0] & 0x80);
        int bitmapSize = 0;
        if (fisrtbtye == (byte) 0x80) {
            bitmapSize = 16;
            len = 128;
        } else {
            bitmapSize = 8;
            len = 64;
        }
        bitmap = new byte[bitmapSize];
        body = new byte[bytes.length - bitmapSize];
        System.arraycopy(bytes, 0, bitmap, 0, bitmapSize);
        System.arraycopy(bytes, bitmapSize, body, 0, bytes.length - bitmapSize);
    }

    public Map<String, Object> packModule() {
        return null;
    }

    public Map<String, Object> unpackModule() {
        Map<String, Object> map = new HashMap<String, Object>();
        IFieldDef dstFieldDef = null;
        try {
            init();
            String str = toBinaryString(bitmap);
            for (int j = 1; j < len; j++) {
                int index = 0;
                dstFieldDef = getModuleFieldDefbyEtag(String.valueOf(j));
                if (dstFieldDef != null) {
                    String etag = dstFieldDef.getEtag();
                    for (int i = 1; i < str.length(); i++) {
                        if ("1".equals(String.valueOf(str.charAt(i)))) {
                            String temp = String.valueOf(i);
                            // 当前i和etag不相等，则index需要变化
                            if (!temp.equals(etag)) {
                                if (i > Integer.parseInt(etag)) break;
                                index = caculateIndex(getModuleFieldDefbyEtag(temp), index, body);
                            } else {
                                // 解析当前filed对应的值
                                parseFiledValue(map, dstFieldDef, index, body);
                                break;
                            }
                        }
                    }
                }
            }
        } catch (UnsupportedEncodingException e) {
            throw new InterfaceException("00403", "unsupported encoding:" + encoding, e);
        } catch (Exception e) {
            throw new InterfaceException("00403", "parse 【" + dstFieldDef.getEtag()
                    + "】 module filed exception:", e);
        }
        return map;
    }

    public Object getValuebyetag(String etag) {
        Map<String, Object> map = unpackModule();
        return map.get(etag);
    }

    private void parseFiledValue(Map<String, Object> map, IFieldDef dstFieldDef, int index,
            byte[] body) throws UnsupportedEncodingException {
        byte bytp[] = getValueByte(dstFieldDef, index, body);
        // 处理子域
        if (!dstFieldDef.getSubfields().isEmpty()) {
            handleSubfileds(map, dstFieldDef.getTag(), dstFieldDef.getSubfields(), bytp);
        } else {
            map.put(dstFieldDef.getTag(), new String(bytp, encoding));
        }
    }

    public void handleSubfiledsByNested(Map<String, Object> map, String parentnam,
            List<ISubfieldDef> subfields, byte[] body) throws UnsupportedEncodingException {
        int index = 0;
        Map<String, Object> childmap=new HashMap<String, Object>();
        childmap.put(parentnam, new String(body, encoding));
        map.put(parentnam, childmap);
        for (ISubfieldDef subfield : subfields) {
            byte bytp[] = getValueByte(subfield, index, body);
            if (!subfield.getSubfields().isEmpty()) {
                handleSubfiledsByNested(map, subfield.getTag(), subfield.getSubfields(), bytp);
            } else {
                childmap.put(subfield.getTag(), new String(bytp, encoding));
            }
            index = caculateIndex(subfield, index, body);
        }
    }
    public void handleSubfileds(Map<String, Object> map, String parentnam,
                                 List<ISubfieldDef> subfields, byte[] body) throws UnsupportedEncodingException {
        int index = 0;
        map.put(parentnam, new String(body, encoding));
        for (ISubfieldDef subfield : subfields) {
            byte bytp[] = getValueByte(subfield, index, body);
            if (!subfield.getSubfields().isEmpty()) {
                handleSubfileds(map, subfield.getTag(), subfield.getSubfields(), bytp);
            } else {
                map.put(subfield.getTag(), new String(bytp, encoding));
            }
            index = caculateIndex(subfield, index, body);
        }
    }

    private int caculateIndex(IFieldDef field, int index, byte[] bytp)
            throws UnsupportedEncodingException {
        if (!"vrString".equals(field.getType())) {
            // 定长处理
            int size = Integer.parseInt(field.getSize());
            index += size;
        } else {
            // 变长处理
            int varsize = Integer.parseInt(field.getVarSize());
            String bodystr = new String(bytp, encoding);
            bodystr = bodystr.substring(index);
            int strlen = Integer.parseInt(bodystr.substring(0, varsize));
            index += varsize;
            index += strlen;
        }
        return index;
    }

    private byte[] getValueByte(IFieldDef dstFieldDef, int index, byte[] body)
            throws UnsupportedEncodingException {
        byte bytp[] = null;
        if (!"vrString".equals(dstFieldDef.getType())) {
            // 定长处理
            int size = Integer.parseInt(dstFieldDef.getSize());
            bytp = new byte[size];
            System.arraycopy(body, index, bytp, 0, size);
        } else {
            // 变长处理
            int varsize = Integer.parseInt(dstFieldDef.getVarSize());
            String bodystr = null;
            bodystr = new String(body, encoding);
            bodystr = bodystr.substring(index);
            int strlen = Integer.parseInt(bodystr.substring(0, varsize));
            bodystr = bodystr.substring(varsize);
            bytp = new byte[strlen];
            System.arraycopy(bodystr.getBytes(), 0, bytp, 0, strlen);
        }
        return bytp;
    }
}
