package com.brilliance.eibs.core.service.instance;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.exception.SysCode;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.core.model.impl.ContinueDef;
import com.brilliance.eibs.core.model.impl.ProcessDef.TaskDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.impl.AbsFilter;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.util.ExceptionUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;

import java.net.SocketTimeoutException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 交易接口实例实现类
 *
 * @author hujun
 */
public class TransactionInstance implements ITransactionInstance {
    private String defaultBasePackage = "com.brilliance.eibs.core.service.instance.impl";
    private ITransactionDef def; // 要执行的交易实例
    private IProcessDef processDef; // 流程定义
    private String interfaceId; // 接口名
    private String transactionId; // 交易名
    protected Context context; // 上下文
    protected Logger logger; // 日志打印器
    protected AbstractEL elParser; // 表达式解析器
    protected String basePackage; // 调用的处理类所在的包
    private boolean exceptionHandled = false; // 标识异常出现后是否被处理
    // private Level origallog; // 源日志级别
    // private boolean subStepErrorPrinted = false;
    // //step下节点出错，日志是否已打印（step下节点递归执行，使用此标识保证错误只打印一次）
    // private boolean breakDone = true;
    // private boolean continueDone = true;
    private Object lock = new Object();
    private static Map<String, TransactionInstance> map = new HashMap<String, TransactionInstance>();
    private static Map<String, Object> lockMap = new HashMap<String, Object>();

    public static synchronized Object getLock(ITransactionDef def) {
        Object lock = null;
        String key = def.getInterfaceDef().getId() + "_" + def.getId();
        if (!lockMap.containsKey(key)) {
            lock = new Object();
            lockMap.put(key, lock);
        }
        return lockMap.get(key);
    }

    /**
     * @param
     **/
    public static synchronized void getInstance(ITransactionDef def, Context context) {
        String key = def.getInterfaceDef().getId() + "_" + def.getId();
        TransactionInstance transinstance = null;
        if (!map.containsKey(key)) {
            transinstance = new TransactionInstance(def, context);
            map.put(key, transinstance);
        } else {
            transinstance = map.get(key);
        }

        transinstance.setDef(def);
        transinstance.setContext(context);
        // return transinstance;
        transinstance.execute();

    }

    public TransactionInstance() {
    }

    public TransactionInstance(ITransactionDef def, Context context) {
        this.def = def;
        this.context = context;
        this.logger = LogUtil.getLogger(context);
        this.elParser = context.getElParser();
        this.interfaceId = def.getInterfaceDef().getId();
        this.transactionId = def.getId();
        this.basePackage = def.getInterfaceDef().getBasePackage();
        if (StringUtil.isEmpty(this.basePackage))
            this.basePackage = defaultBasePackage;
    }

    @Override
    public ITransactionDef getDef() {
        return def;
    }

    @Override
    public void setDef(ITransactionDef def) {
        this.def = def;
    }

    public void setBasePackage(String basePackage) {
        this.basePackage = basePackage;
    }

    @Override
    public String getInterfaceId() {
        return interfaceId;
    }

    @Override
    public void setInterfaceId(String interfaceId) {
        this.interfaceId = interfaceId;
    }

    @Override
    public String getTransactionId() {
        return transactionId;
    }

    @Override
    public void setTransactionId(String transactionId) {
        this.transactionId = transactionId;
    }

    @Override
    public Context getContext() {
        return context;
    }

    @Override
    public void setContext(Context context) {
        this.context = context;
    }

    public void setElParser(ApacheEL elParser) {
        this.elParser = elParser;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * com.brilliance.eibs.core.service.instance.ITransactionInstance#execute()
     *
     * update by hujun 2013/8/8
     */
    @Override
    public void execute() {

        if (def.isSingleton() == true) {
            // excuteWithLock();
            Object lock = getLock(def);
            synchronized (lock) {
                excuteWithoutLock();
            }
        } else {
            excuteWithoutLock();
        }
    }

    public void excuteWithoutLock() {
        excuteTransaction();
    }

    public void excuteWithLock() {
        synchronized (lock) {
            logger.info("<--------  Interface of [" + interfaceId + "_" + transactionId + "] is singleton !!!  -------->");
            excuteTransaction();
        }
    }

    public void excuteTransaction() {
        logger.info(
                IExecutableInstance.LOG_FLAG + "Execution transaction [" + transactionId + "] in interface [" + context.getVariable("interfaceName")
                        + "]." + IExecutableInstance.LOG_FLAG);
        context.addVariable("transaction", "socket_timeout_occurs", false);
        // 初始化
        context.addVariable("transaction", "____subStepErrorPrinted", false);
        context.addVariable("transaction", "____breakDone", true);
        context.addVariable("transaction", "____continueDone", true);

        // 获取流程定义
        processDef = def.getProcessDef();
        // 判断是否存在process,存在process
        if (null != processDef)
            executeProcess(processDef);
        else
            executeDefault();// 执行默认流程,顺序执行step
        // TODO
        context.getResultMsg().setReturnCode(context.getResource().getErrorcode());
        context.getResultMsg().setDescription(context.getResource().getDescrption());
        context.getResultMsg().setContent(context.getObject());
        logger.info(
                IExecutableInstance.LOG_FLAG + "transaction [" + transactionId + "] in interface [" + context.getVariable("interfaceName")
                        + "] is done." + IExecutableInstance.LOG_FLAG);
    }

    /**
     * 执行默认流程，在不配置流程定义的前提下，按照step的顺序执行
     */
    private void executeDefault() {
        // logger.debug( "Perform the default process.");
        IStepDef curStep = null;
        IStepDef finallyCurStep = null;
        String curStepName = null;
        for (Iterator<IStepDef> it = def.getSteps().iterator(); it.hasNext(); ) {
            IStepDef tmp = it.next();
            if ("__finally".equals(tmp.getId())) {
                finallyCurStep = tmp;
                break;
            }
        }
        for (Iterator<IStepDef> it = def.getSteps().iterator(); it.hasNext(); ) {
            try {
                if (!context.getResource().getErrorcode().equals(IResource.ERRORCODE_NORMAL)) {
                    // 添加数据库回滚操作
                    try {
                        context.rollbackConnectionAdaptors();
                    } catch (SQLException e) {
                        logger.error(e.getMessage());
                        return;
                    }
                    // 设置异常处理
                    context.addVariable("transaction", "__cause", context.getResultMsg().getCauseException());
                    executeStep(EXCEPTION_STEP_SYMBOL);
                    context.getResultMsg().setContent(context.getObject());
                    break;
                }
                curStep = it.next();
                curStepName = curStep.getId();
                if (EXCEPTION_STEP_SYMBOL.equals(curStepName) || "__finally".equals(curStepName))
                    continue;
                logger.info("The current step is " + curStepName);
                traverseExecutable(curStep);
            } finally {
                // 清空step的上下文
                context.getStepMap().clear();
            }
        }
        try {
            if (finallyCurStep != null) {
                traverseExecutable(finallyCurStep);
            }
        } finally {
            context.getStepMap().clear();
        }
        logger.debug("The default process is done.");
    }

    /**
     * 执行step
     *
     * @param curStepName
     */
    private void executeStep(String curStepName) {
        // FIXME
        logger.info(IExecutableInstance.LOG_FLAG + "The current step [" + curStepName + "] begin to run");
        IStepDef curStep = getStepDef(curStepName);
        traverseExecutable(curStep);
        if ((processDef == null && curStepName.equals(EXCEPTION_STEP_SYMBOL))
                || (processDef != null && curStepName.equals(processDef.getExceptionTaskName())))
            context.getResultMsg().setToException(true);
        context.getStepMap().clear();
        logger.info(IExecutableInstance.LOG_FLAG + "The current step [" + curStepName + "] finished to run");
    }

    /**
     * 执行step
     *
     * @param curStepName
     * @param clear       是否清空当前step内存
     */
    private void executeStep(String curStepName, boolean clear) {
        logger.info("The current step is [" + curStepName + "]");
        IStepDef curStep = getStepDef(curStepName);
        traverseExecutable(curStep, false);
        if (clear)
            context.getStepMap().clear();
    }

    /**
     * 执行process流程
     *
     * @param sub
     */
    private void executeProcessSubList(Object sub) {
        // 检测流程的上一个子定义是否出错且被处理，如果是，直接结束流程，返回错误
        if (exceptionHandled)
            return;
        // 流程的上一个子定义是否出错,如果出错，处理异常
        if (!context.getResource().getErrorcode().equals(IResource.ERRORCODE_NORMAL)) {
            if (!StringUtil.isEmpty(processDef.getExceptionTaskName())) {
                executeStep(processDef.getExceptionTaskName());
                context.getResultMsg().setSuccess(false);
                context.getResultMsg().setContent(context.getObject());
            }
            exceptionHandled = true;
            return;
        }
        if (sub instanceof TaskDef) {
            TaskDef curTask = (TaskDef) sub;
            executeStep(curTask.getName());
        } else if (sub instanceof IForeachDef) {
            IForeachDef foreachDef = (IForeachDef) sub;
            List<IAbsDef> list = foreachDef.getSubList();
            // 循环变量
            String tag = foreachDef.getTag();
            String contextKey = (String) elParser.getExPression(context, tag, null);
            // 循环开始点，默认为0
            String begin = foreachDef.getBegin();
            int beginValue = StringUtil.isEmpty(begin) ? 0 : Integer.parseInt(elParser.getExPression(context, begin, null).toString());
            // 循环步长
            String step = foreachDef.getStep();
            int stepValue = StringUtil.isEmpty(begin) ? 1 : Integer.parseInt(elParser.getExPression(context, step, null).toString());
            // 循环结束点
            String end = foreachDef.getEnd();
            int endValue = (int) Double.parseDouble(elParser.getExPression(context, end, null).toString());
            // 循环变量的生命周期
            String scope = foreachDef.getScope();
            String condition = foreachDef.getCondition();
            context.addVariable(scope, contextKey + "_size", endValue);
            for (; beginValue < endValue; beginValue += stepValue) {
                context.addVariable(scope, contextKey, beginValue);
                foreachDef.setCurrentCount(beginValue);
                if (StringUtil.isEmpty(condition) || (!StringUtil.isEmpty(condition) && (Boolean) elParser.getExPression(context, condition, null))) {
                    for (Object object : list) {
                        executeProcessSubList(object);
                    }
                }
            }
        } else if (sub instanceof IIfDef) {
            IIfDef ifDef = (IIfDef) sub;
            Map<String, Object> map = ifDef.getConditionMap();
            Iterator<String> it = map.keySet().iterator();
            boolean ifValue = false;
            while (it.hasNext()) {
                String key = it.next();
                Object object = elParser.getExPression(context, key, null);
                if ((ifValue = ((Boolean) object).booleanValue()) == true) {
                    Object value = map.get(key);
                    List<?> instances = (List<?>) value;
                    for (Object instance : instances) {
                        executeProcessSubList(instance);
                    }
                    break;
                }
            }
            if (ifValue == false) {
                Object elseCondition = ifDef.getElseCondition();
                if (elseCondition == null)
                    return;
                List<?> instances = (List<?>) ifDef.getElseCondition();
                for (Object instance : instances) {
                    executeProcessSubList(instance);
                }
            }
        }
    }

    /**
     * 执行流程定义
     *
     * @param process
     */
    private void executeProcess(IProcessDef process) {
        logger.info(IExecutableInstance.LOG_FLAG + "Perform a custom process.");
        // 获取流程子定义
        List<IAbsDef> subList = process.getSubList();
        int listSize = subList.size();
        for (int i = 0; i < listSize; i++) {
            Object sub = subList.get(i);
            executeProcessSubList(sub);
        }
        // 流程执行完之后，如果过程出错且没有被处理
        if (!context.getResource().getErrorcode().equals(IResource.ERRORCODE_NORMAL)) {
            if (!exceptionHandled && !StringUtil.isEmpty(processDef.getExceptionTaskName())) {
                executeStep(processDef.getExceptionTaskName());
                context.getResultMsg().setSuccess(false);
                context.getResultMsg().setContent(context.getObject());
                exceptionHandled = true;
            }
        }
        logger.debug("The custom process is done.");
    }

    /**
     * 遍历Step下所有可执行的配置项
     *
     * @param curStep
     * @param process
     * @param curTaskName
     * @return
     */
    private void traverseExecutable(IStepDef curStep) {
        traverseExecutable(curStep, true);
    }

    private boolean isCausedBySocketTimeout(Exception e) {
        Throwable t = e;
        while (t != null) {
            if (t instanceof SocketTimeoutException)
                return true;
            t = t.getCause();
        }
        return false;
    }

    private void traverseExecutable(IStepDef curStep, boolean clear) {
        // 遍历step下的filter，connection
        for (Iterator<IAbsDef> it = curStep.getExecuteList().iterator(); it.hasNext(); ) {
            Object obj = it.next();
            try {
                runStepSubDef(obj);
            } catch (Exception e) {
                handleExpcetion(e);
                context.getResultMsg().setSuccess(false);
                if (e instanceof InterfaceException && null != ((InterfaceException) e).getException())
                    context.getResultMsg().setCauseException(((InterfaceException) e).getException());
                else
                    context.getResultMsg().setCauseException(e);
                if (!clear)
                    context.setObject(context.getResource().getErrorcode());
                break;
            } finally {
                // 清除transition的上下文
                if (clear)
                    context.getTransitionMap().clear();
            }
        }
    }

    /**
     * 根据TaskName获取相应的StepDef，如果返回为空，直接抛出异常
     *
     * @param stepName
     * @return
     */
    private IStepDef getStepDef(String stepName) {
        IStepDef curStep = null;
        for (Iterator<IStepDef> it = def.getSteps().iterator(); it.hasNext(); ) {
            IStepDef step = it.next();
            // 获取第一个step节点
            if (step.getId().equals(stepName)) {
                curStep = step;
                break;
            }
        }
        if (curStep == null) {
            logger.error("step [" + stepName + "] not exist");
            throw new InterfaceException("00019");
        }
        return curStep;
    }

    private void runForeachDef(IForeachDef foreachDef) throws Exception {
        List<IAbsDef> list = foreachDef.getSubList();
        String tag = foreachDef.getTag();
        String contextKey = (String) elParser.getExPression(context, tag, null);
        String begin = foreachDef.getBegin();
        int beginValue = StringUtil.isEmpty(begin) ? 0 : Integer.parseInt(elParser.getExPression(context, begin, null).toString());
        String step = foreachDef.getStep();
        int stepValue = StringUtil.isEmpty(step) ? 1 : Integer.parseInt(elParser.getExPression(context, step, null).toString());
        String end = foreachDef.getEnd();
        int endValue = (int) Double.parseDouble(elParser.getExPression(context, end, null).toString());
        String scope = foreachDef.getScope();
        String condition = foreachDef.getCondition();
        boolean ignoreException = foreachDef.isIgnoreExption();
        if (StringUtil.isEmpty(scope))
            context.addVariable("step", contextKey + "_size", endValue);
        else
            context.addVariable(scope, contextKey + "_size", endValue);
        for (; beginValue < endValue; beginValue += stepValue) {
            boolean breakDone = context.getVariable("____breakDone") == null ? false : (Boolean) context.getVariable("____breakDone");
            if (!breakDone) {
                // breakDone = true;
                context.addVariable("transaction", "____breakDone", true);
                break;
            }
            // continueDone = true;
            context.addVariable("transaction", "____continueDone", true);
            if (StringUtil.isEmpty(scope))
                context.addVariable("step", contextKey, beginValue);
            else
                context.addVariable(scope, contextKey, beginValue);
            if (StringUtil.isEmpty(condition) || (!StringUtil.isEmpty(condition) && (Boolean) elParser.getExPression(context, condition, null))) {
                Object object = null;
                try {
                    // 执行foreach下的内容
                    for (int i = 0; i < list.size(); i++) {
                        object = list.get(i);
                        runStepSubDef(object);
                        // 执行成功
                    }
                } catch (Exception e)// 执行失败
                {
                    if (!ignoreException)
                        throw e;
                    else {
                        context.getResultMsg().setIgnoredException(true);
                        context.getResultMsg().setSuccess(false);
                        context.getResultMsg().setCauseException(e);
                        // TODO ignore异常的标记
                        context.addVariable("transaction", "cycle_exception", true);
                        // FIXME
                        // for循环里面的某个异常产生之后，先执行异常step，然后重置errorcode和errortext
                        handleExpcetion(e);
                        executeStep(EXCEPTION_STEP_SYMBOL, false);
                        context.getResource().setErrorcode(IResource.ERRORCODE_NORMAL);
                        context.getResource().setDescrption(null);
                        context.getResource().setDescription(null);
                        if (isExecutable(object)) {
                            IExecutableDef executableDef = (IExecutableDef) object;
                            logger.debug("[ignored exception]error occurs in " + executableDef.getId());
                        } else
                            logger.debug("[ignored exception]error occurs in " + object.getClass());
                    }
                }
            }
            context.addVariable("transaction", "cycle_exception", false);
            // 循环变量在处理过程中发生改动时，对循环变量重新赋值
            beginValue = (Integer) context.getVariable(contextKey);
            context.removeForeachMap(tag);
        }
        boolean breakD = (Boolean) context.getVariable("____breakDone");
        if (!breakD) {
            context.addVariable("transaction", "____breakDone", true);
        }
        boolean continuD = (Boolean) context.getVariable("____continueDone");
        if (!continuD) {
            context.addVariable("transaction", "____continueDone", true);
        }

        // TODO修复foreach缺陷
        context.remove("step", contextKey);

    }

    private boolean isExecutable(Object object) {
        if (object instanceof IExecutableDef)
            return true;
        return false;
    }

    private void runIfDef(IIfDef ifDef) throws Exception {
        Map<String, Object> map = ifDef.getConditionMap();
        Iterator<String> it = map.keySet().iterator();
        boolean ifValue = false;
        while (it.hasNext()) {
            String key = it.next();
            Object object = elParser.getExPression(context, key, null);
            if ((ifValue = ((Boolean) object).booleanValue()) == true) {
                Object value = map.get(key);
                List<?> instances = (List<?>) value;
                for (Object instance : instances) {
                    runStepSubDef(instance);
                }
                break;
            }
        }
        if (ifValue == false) {
            Object elseCondition = ifDef.getElseCondition();
            if (elseCondition == null)
                return;
            List<?> instances = (List<?>) ifDef.getElseCondition();
            for (Object instance : instances) {
                runStepSubDef(instance);
            }
        }
    }

    public void runExecutableInstance(IExecutableInstance executableInstance) throws Exception {
        executableInstance.execute(context);
        if (executableInstance instanceof AbsFilter) {
            AbsFilter filter = (AbsFilter) executableInstance;
            if (StringUtil.isEmpty(filter.getFilterDef().getScope()))
                filter.saveToContext("step", filter.getFilterDef().getTag(), context.getObject());
            else
                filter.saveToContext(filter.getFilterDef().getScope(), filter.getFilterDef().getTag(), context.getObject());
        }
    }

    private void runLogDef(ILogDef logDef) {
        Object msg = elParser.getExPression(context, logDef.getValue(), null);
        StringBuffer sb = context.getResource().getLogbuffer();
        if (sb != null)
            sb.append(msg);
    }

    private void runWhileDef(IWhileDef whileDef) throws Exception {
        String condition = whileDef.getCondition();
        while ((Boolean) elParser.getExPression(context, condition, null)) {
            boolean breakDone = context.getVariable("____breakDone") == null ? false : (Boolean) context.getVariable("____breakDone");
            if (!breakDone) {
                // breakDone = true;
                context.addVariable("transaction", "____breakDone", true);
                break;
            }
            context.addVariable("transaction", "____continueDone", true);
            List<IAbsDef> subList = whileDef.getSubList();
            for (IAbsDef sub : subList)
                runStepSubDef(sub);
        }
    }

    /**
     * 运行step子节点定义
     *
     * @param obj
     * @throws Exception
     */
    protected void runStepSubDef(Object obj) throws Exception {
        // logger.debug("running:" + obj);
        boolean breakDone = context.getVariable("____breakDone") == null ? true : (Boolean) context.getVariable("____breakDone");
        boolean continueDone = context.getVariable("____continueDone") == null ? true : (Boolean) context.getVariable("____continueDone");
        if (!breakDone || !continueDone)
            return;
        boolean subStepErrorPrinted = context.getVariable("____subStepErrorPrinted") == null ? false : (Boolean) context
                .getVariable("____subStepErrorPrinted");
        if (subStepErrorPrinted)
            // subStepErrorPrinted = false;
            context.addVariable("transaction", "____subStepErrorPrinted", false);
        try {
            if (isExecutable(obj)) {
                IExecutableDef executableDef = (IExecutableDef) obj;
                Class<?> executableClass = Class.forName(basePackage + "." + executableDef.getExecuteClass());
                IExecutableInstance executableInstance = (IExecutableInstance) executableClass.newInstance();
                context.addVariable(null, "__this", executableInstance);
                executableInstance.setIDef(executableDef);
                executableInstance.setContext(context);
                executableInstance.setElPaser(elParser);
                // executableInstance.setLogger(logger);
                /*
                 * context.addVariable("transaction","____subStepErrorPrinted",
                 * subStepErrorPrinted);
                 * context.addVariable("transaction","____breakDone",
                 * breakDone);
                 * context.addVariable("transaction","____continueDone",
                 * continueDone);
                 */
                int times = Integer.parseInt(executableDef.getTimes());
                boolean flag;
                while (times > 0) {
                    if (times > 1)
                        logger.debug(executableDef.getId() + " [time:" + times + "]");
                    flag = true;
                    try {
                        runExecutableInstance(executableInstance);
                    } catch (Exception e) {
                        flag = false;
                        if (times == 1) {
                            if (e instanceof InterfaceException) {
                                InterfaceException ex = (InterfaceException) e;
                                if (!ex.getId().equals("11111") && !StringUtil.isEmpty(executableDef.getErrorDescription())) {
                                    ex.setMessage(executableDef.getErrorDescription());
                                }
                            }
                            throw e;
                        }
                    }
                    if (flag)
                        times = 0;
                    else {
                        times--;
                    }
                }
                breakDone = true;
                continueDone = true;
                // 每执行完一个filter或connection之后清空当前transition内存
                context.getTransitionMap().clear();
            } else if (obj instanceof IForeachDef) {
                IForeachDef foreachDef = (IForeachDef) obj;
                runForeachDef(foreachDef);
            } else if (obj instanceof IWhileDef) {
                IWhileDef whileDef = (IWhileDef) obj;
                runWhileDef(whileDef);
            } else if (obj instanceof IIfDef) {
                IIfDef ifDef = (IIfDef) obj;
                runIfDef(ifDef);
            } else if (obj instanceof ILogDef) {
                ILogDef logDef = (ILogDef) obj;
                runLogDef(logDef);
            } else if (obj instanceof BreakDef) {
                breakDone = false;
                return;
            } else if (obj instanceof ContinueDef) {
                continueDone = false;
                return;
            }
        } catch (Exception e) {
            if (!subStepErrorPrinted) {
                logger.error("error occurs in:" + obj);
                subStepErrorPrinted = true;
            }
            if (e instanceof ClassNotFoundException) {
                throw new InterfaceException("00422", e);
            } else
                throw e;
        }

        // logger.debug("finished:" + obj);
    }

    /**
     * 当前Task的异常处理,并且将当前节点指向异常
     *
     * @param curTaskName
     * @param e
     */
    private void handleExpcetion(Exception e) {
        if (isCausedBySocketTimeout(e))
            context.addVariable("transaction", "socket_timeout_occurs", true);
        else
            context.addVariable("transaction", "socket_timeout_occurs", false);
        if ((e instanceof InterfaceException) && ((InterfaceException) e).getId().equals("11111")) {
            // validator异常，已经提前处理，这里不再处理
            return;
        }

        try {
            context.rollbackAllConnection();
        } catch (Exception e1) {
            logger.error("transaction rollback error:", e);
        }

        SysCode sysCode = ExceptionUtil.getErrorCode(context, e);
        String errorText = sysCode.getErrorText();
        String errorId = sysCode.getId();
        context.getResource().setErrorcode(errorId);

        if (!(e instanceof InterfaceException)) {
            String exceptionMsg = e.getMessage();
            if (!StringUtil.isEmpty(exceptionMsg)) {
                errorText = errorText + "[" + exceptionMsg + "]";
            }
        }

        context.getResource().setDescrption(errorText);
        context.getResource().setDescription(errorText);
        if (e instanceof InterfaceException && null != ((InterfaceException) e).getException())
            context.getResource().setException(((InterfaceException) e).getException());
        else
            context.getResource().setException(e);
        logger.error("[error:" + errorId + "]" + errorText, e);
    }

}
