package com.brilliance.eibs.core.service.instance.impl;

import com.brilliance.eibs.coder.impl.AESCoder;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.frame.ISocketProxy;
import com.brilliance.eibs.core.frame.instance.IInstance;
import com.brilliance.eibs.core.frame.instance.impl.CallInstance;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.core.model.impl.ContinueDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.IFilterInstance;
import com.brilliance.eibs.core.service.instance.plugin.JdbcExecutor;
import com.brilliance.eibs.core.service.instance.plugin.SocketProxy;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.el.ApacheJexl;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.factory.parser.XmlValidator;
import com.brilliance.eibs.sqlite.Sqlite_OP;
import com.brilliance.eibs.util.*;
import org.apache.commons.beanutils.ConstructorUtils;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.codec.binary.Base64;
import org.dom4j.DocumentException;
import org.slf4j.Logger;
import org.xml.sax.SAXException;

import javax.xml.bind.JAXBElement;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.sql.*;
import java.text.ParseException;
import java.util.Date;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * 抽象处理类
 *
 * @author xiaoyuanzhen
 */
public abstract class AbsFilter implements IFilterInstance {
    /**
     * 数字类型的补长使用的字符的属性名称
     */
    protected static final String NUMBER_FILLCHAR = "number_fillchar";
    protected static final String STRING_FILLCHAR = "string_fillchar"; // 字符串补长使用的字符的属性名称
    protected static final String FILL_NUMBER_STRATEGY = "fill_number_strategy"; // 数字类型的补长策略的属性名称
    protected static final String FILL_STRING_STRATEGY = "fill_string_strategy"; // 字符串补长策略的属性名称
    protected String numberFillChar = "0"; // 数字默认补0
    protected String numberFillStrategy = "left"; // 数字默认左边补
    protected String stringFillChar = " "; // 字符串默认补空格
    protected String stringFillStrategy = "right"; // 字符串默认右边补

    protected IFilterDef filterDef;
    protected Context context;
    protected Logger logger;
    protected AbstractEL elParser;
    protected boolean isinforeach = false;
    private static Object lock = new Object();
    protected boolean parametersSaved = false;
    public static final Map<String, Class<?>> typeMap = new HashMap<String, Class<?>>(); // 定义配置的类型到Class类型的对应
    public static Map<String, ExecutorService> executors = new HashMap<String, ExecutorService>();

    static {
        typeMap.put("byte", byte.class);
        typeMap.put("boolean", boolean.class);
        typeMap.put("short", short.class);
        typeMap.put("int", int.class);
        typeMap.put("long", long.class);
        typeMap.put("float", float.class);
        typeMap.put("double", double.class);
        typeMap.put("Byte", Byte.class);
        typeMap.put("Boolean", Boolean.class);
        typeMap.put("Short", Short.class);
        typeMap.put("Integer", Integer.class);
        typeMap.put("Long", Long.class);
        typeMap.put("Float", Float.class);
        typeMap.put("Double", Double.class);
        typeMap.put("string", String.class);
        typeMap.put("String", String.class);
        typeMap.put("object", Object.class);
        typeMap.put("Object", Object.class);
    }

    @Override
    public void setContext(Context context) {
        this.context = context;
        this.logger = LogUtil.getLogger(context);
    }

    /**
     * 执行filterDef
     */
    public void execute(Context context) {
        // context.addVariable("transaction", "__encoding",
        // elParser.getExPression(context, filterDef.getEncoding(),
        // null).toString());
        List<IAbsDef> subList = filterDef.getSubList();
        propertySaveToContext();
        this.context = context;
        for (IAbsDef sub : subList) {
            handleSubList(sub);
        }

    }

    /**
     * 获取Filter的属性配置
     *
     * @param name
     * @return
     */
    protected String getPropertyValue(String name) {
        String value = filterDef.getCommonsProperty(name);
        return (String) elParser.getExPression(context, value, null);
    }

    /**
     * 获取filter的必配属性
     *
     * @param name
     * @return
     */
    protected String getRequiredPropertyValue(String name) {
        String value = getPropertyValue(name);
        if (value == null) throw new InterfaceException("00430",
                "the required property [" + name + "] of " + filterDef.getRef() + " not exists.");
        return value;
    }

    /**
     * 递归处理filter的子节点
     *
     * @param sub
     */
    public void handleSubList(IAbsDef sub) {
        // logger.debug("running : " + sub);
        /*
         * if (!TransactionInstance.breakDone || !TransactionInstance.continueDone) return;
         */
        Object object = null;
        boolean breakD = true;
        boolean continueD = true;
        boolean subStepErrorP = false;
        object = context.getVariable("____subStepErrorPrinted");
        if (object != null) subStepErrorP = (Boolean) object;
        object = context.getVariable("____breakDone");
        if (object != null) breakD = (Boolean) object;
        object = context.getVariable("____continueDone");
        if (object != null) continueD = (Boolean) object;
        if (!breakD || !continueD) return;
        String iName = "";
        String tName = "";
        try {
            if (sub instanceof IFieldDef) {
                IFieldDef fieldDef = (IFieldDef) sub;
                if (fieldDef != null && fieldDef.getSubfields() != null && fieldDef.getSubfields().size() > 0) {
                    handleFieldAndSubfield(fieldDef);
                } else {
                    handleFieldDef(fieldDef);
                }

            } else if (sub instanceof IReturnDef) {
                IReturnDef returnDef = (IReturnDef) sub;
                handleReturnDef(returnDef);
            } else if (sub instanceof IForeachDef) {
                IForeachDef foreachDef = (IForeachDef) sub;
                handleForeachDef(foreachDef);
            } else if (sub instanceof IWhileDef) {
                IWhileDef whileDef = (IWhileDef) sub;
                handleWhileDef(whileDef);
            } else if (sub instanceof IConvertorDef) {
                IConvertorDef convertorDef = (IConvertorDef) sub;
                Object value = handleConvertorDef(convertorDef);
                saveToContext(convertorDef.getScope(), convertorDef.getTag(), value);
            } else if (sub instanceof IValidatorDef) {
                IValidatorDef validatorDef = (IValidatorDef) sub;
                if (!handleValidatorDef(validatorDef)) {
                    String errorCode = (String) elParser.getExPression(context,
                            validatorDef.getErrorCode(), null);
                    String description = (String) elParser.getExPression(context,
                            validatorDef.getDescription(), null);
                    context.getResource().setErrorcode(errorCode);
                    context.getResource().setDescrption(description);
                    context.getResource().setDescription(description);
                    logger.error("validator " + validatorDef.getId() + " of filter ["
                            + filterDef.getId() + "] error!");
                    throw new InterfaceException("11111");
                }
            } else if (sub instanceof IIfDef) {
                IIfDef ifDef = (IIfDef) sub;
                handleFilterIfDef(ifDef);
            } else if (sub instanceof ILogDef) {
                ILogDef logDef = (ILogDef) sub;
                Object msg = elParser.getExPression(context, logDef.getValue(), null);
                Object level = elParser.getExPression(context, logDef.getLevel(), null);

                if (level == null) {
                    logger.info(JsonUtil.toJson(msg));
                } else {

                    if (((String) level).trim().equals("")) {
                        logger.info(JsonUtil.toJson(msg));
                    } else if (level.equals("trace")) {
                        logger.trace(JsonUtil.toJson(msg));
                    } else if (level.equals("debug")) {
                        logger.debug(JsonUtil.toJson(msg));
                    } else if (level.equals("info")) {
                        logger.info(JsonUtil.toJson(msg));
                    } else if (level.equals("warn")) {
                        logger.warn(JsonUtil.toJson(msg));
                    } else if (level.equals("error")) {
                        logger.error(JsonUtil.toJson(msg));
                    }
                }

                // 死循环foreach中可能到处内存溢出
                // StringBuffer sb = context.getResource().getLogbuffer();
                // if (sb != null)
                // sb.append(msg);
            } else if (sub instanceof BreakDef) {
                context.addVariable("transaction", "____breakDone", false);
                return;
            } else if (sub instanceof ContinueDef) {
                context.addVariable("transaction", "____continueDone", false);
                return;
            } else if (sub instanceof ICallDef) {
                IInstance call = new CallInstance((ICallDef) sub, this);
                call.execute(context);
                call = null;
            }

        } catch (Exception e) {
            if (!subStepErrorP) {
                logger.error("error occurs in:" + iName + "/" + tName);
                context.addVariable("transaction", "____subStepErrorPrinted", true);
            }
            if (e instanceof InterfaceException) {
                InterfaceException ie = (InterfaceException) e;
                if (sub instanceof IFieldDef) {
                    String errstr = "maybe caused by:";
                    IFieldDef fieldDef = (IFieldDef) sub;
                    String value = fieldDef.getValue();
                    String etag = fieldDef.getEtag();
                    if (ie.getId().startsWith("001")) // 表达式调用异常
                    {
                        if (!value.equals("")) {
                            errstr += "[value=\"" + value + "\"]";
                        }
                        if (!etag.equals("")) {
                            errstr += "[etag=\"" + etag + "\"]";
                        }
                        logger.error(errstr);
                    } else {
                        if (!value.equals("")) {
                            if (value.contains("${") && value.contains("}")) {
                                Object elValue = context.getVariable(value);
                                errstr += "[value=\"" + value + "\"(" + elValue + " )]";
                            } else
                                errstr += "[value=\"" + value + "\"]";
                        }
                        if (!etag.equals("")) {
                            if (etag.contains("${") && etag.contains("}")) {
                                Object elEtag = elParser.dealElExpression(context, etag);
                                errstr += "[etag=\"" + etag + "\"(" + elEtag + ")]";
                            } else
                                errstr += "[etag=\"" + etag + "\"]";
                        }
                        logger.error(errstr);
                    }
                }
                throw ie;
            } else if (e instanceof RuntimeException) {
                if (sub instanceof IFieldDef) {
                    String errstr = "maybe caused by:";
                    IFieldDef fieldDef = (IFieldDef) sub;
                    String value = fieldDef.getValue();
                    String etag = fieldDef.getEtag();
                    if (!value.equals("")) {
                        if (value.contains("${") && value.contains("}")) {
                            Object elValue = context.getVariable(value);
                            errstr += "[value=\"" + value + "\"(" + elValue + " )]";
                        } else
                            errstr += "[value=\"" + value + "\"]";
                    }
                    if (!etag.equals("")) {
                        if (etag.contains("${") && etag.contains("}")) {
                            Object elEtag = context.getVariable(etag);
                            errstr += "[etag=\"" + etag + "\"(" + elEtag + " )]";
                        } else
                            errstr += "[etag=\"" + etag + "\"]";
                    }
                    logger.error(errstr);
                }
                throw (RuntimeException) e;
            } else {
                throw new RuntimeException(e);
            }
        }

    }

    protected void doCall(ICallDef def) {
        IInstance call = new CallInstance(def, this);
        call.execute(context);
        call = null;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    private void handleFieldAndSubfield(IFieldDef fieldDef) throws Exception {
        List<ISubfieldDef> subfieldDefs = fieldDef.getSubfields();
        String tag = fieldDef.getTag();
        String value = fieldDef.getValue();
        String scope = fieldDef.getScope();
        List<Map> newval = new ArrayList<Map>();
        Object[] objs = dealArguments(fieldDef.getAuguments());
        Object contextValue = elParser.getExPression(context, value, objs);
        if (contextValue instanceof List) {
            List<Map> caches = (List<Map>) contextValue;
            for (Map cache : caches) {
                Map<String, Object> ele = new HashMap<String, Object>();
                for (ISubfieldDef sbf : subfieldDefs) {
                    String key = sbf.getKey();
                    Object result = "";
                    if (StringUtil.isEmpty(key)) continue;
                    if (sbf.getSubfields().size() > 0 && !StringUtil.isEmpty(sbf.getSrckey())) {
                        result = handleSubfieldNesting(sbf, (List<Map>) cache.get(sbf.getSrckey()));
                    } else {
                        String srckey = sbf.getSrckey();

                        if (!StringUtil.isEmpty(srckey)) {
                            if (cache.containsKey(srckey)) {
                                result = cache.get(srckey);
                                sbf.setTag(key);
                                result = valFilter(result, sbf, context);
                            }
                        } else
                            result = valFilter(result, sbf, context);
                    }
                    ele.put(key, result);
                }
                newval.add(ele);
            }
            saveToContext(scope, tag, newval);
        } else
            // handleFieldDef(fieldDef);
            throw new RuntimeException("传入的值不符合规则");
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    private List<Map> handleSubfieldNesting(ISubfieldDef sbf, List<Map> caches) {
        List<ISubfieldDef> subfieldDefs = sbf.getSubfields();
        List<Map> newval = new ArrayList<Map>();
        for (Map cache : caches) {
            Map<String, Object> ele = new HashMap<String, Object>();
            for (ISubfieldDef subfieldDef : subfieldDefs) {
                String key = subfieldDef.getKey();
                Object result = "";
                if (StringUtil.isEmpty(key)) continue;
                if (subfieldDef.getSubfields().size() > 0
                        && !StringUtil.isEmpty(subfieldDef.getSrckey())) {
                    result = handleSubfieldNesting(subfieldDef,
                            (List<Map>) cache.get(subfieldDef.getSrckey()));
                } else {
                    String srckey = subfieldDef.getSrckey();
                    if (!StringUtil.isEmpty(srckey)) {
                        if (cache.containsKey(srckey)) {
                            result = cache.get(srckey);
                            subfieldDef.setTag(key);
                            result = valFilter(result, subfieldDef, context);
                        }
                    } else
                        result = valFilter(result, subfieldDef, context);
                }
                ele.put(key, result);
            }
            newval.add(ele);
        }
        return newval;
    }

    /**
     * 处理自定义函数的return节点定义
     *
     * @param returnDef
     * @return
     */
    protected Object handleReturnDef(IReturnDef returnDef) {
        return elParser.getExPression(context, returnDef.getValue(), null);
    }

    /**
     * 处理while循环
     *
     * @param whileDef
     */
    private void handleWhileDef(IWhileDef whileDef) {
        String condition = whileDef.getCondition();

        while ((Boolean) elParser.getExPression(context, condition, null)) {
            boolean breakD = (Boolean) context.getVariable("____breakDone");
            if (!breakD) {
                context.addVariable("transaction", "____breakDone", true);
                break;
            }
            context.addVariable("transaction", "____continueDone", true);

            List<IAbsDef> subList = whileDef.getSubList();
            for (IAbsDef sub : subList)
                handleSubList(sub);
        }
        // TODO gechengyang 20150911
        boolean breakD = (Boolean) context.getVariable("____breakDone");
        if (!breakD) {
            context.addVariable("transaction", "____breakDone", true);
        }

        boolean continuD = (Boolean) context.getVariable("____continueDone");
        if (!continuD) {
            context.addVariable("transaction", "____continueDone", true);
        }

        /*
         * System.out.println("breakvalue="+(Boolean) context.getVariable("____breakDone"));
         */
    }

    /**
     * 将filter的parameter定义都放到上下文
     */
    protected void propertySaveToContext() {
        if (!parametersSaved) {
            Map<String, String> propMap = filterDef.getCommonsPropertyMap();
            Iterator<String> iterator = propMap.keySet().iterator();
            while (iterator.hasNext()) {
                String key = iterator.next();
                String value = propMap.get(key);
                context.getTransitionMap().put(key, elParser.getExPression(context, value, null));
            }
            parametersSaved = true;
        }
    }

    /**
     * 组装时对每个field的操作
     *
     * @param fieldDef
     * @param fieldValue
     */
    protected void packField(IFieldDef fieldDef, Object fieldValue) {
    }

    /**
     * 对于符合"\\w+(\\[\\d+\\])+"的type进行创建数组
     *
     * @param type
     * @return
     */
    private Object createArrayByType(String type) {
        List<Integer> dimensionList = new ArrayList<Integer>();
        int startIndex = type.indexOf('[');
        String arrayType = type.substring(0, startIndex);
        int endIndex = 0;
        while (-1 != startIndex) {
            endIndex = type.indexOf(']', startIndex);
            int strSize = Integer.parseInt(type.substring(startIndex + 1, endIndex));
            dimensionList.add(strSize);
            startIndex = type.indexOf('[', endIndex);
        }
        int size = dimensionList.size();
        int[] dimensions = new int[size];
        for (int i = 0; i < size; i++) {
            dimensions[i] = dimensionList.get(i);
        }
        if (typeMap.containsKey(arrayType)) {
            Object array = Array.newInstance(typeMap.get(arrayType), dimensions);
            return array;
        }
        throw new InterfaceException("00401", "unsupport array type: [" + arrayType + "]");
    }

    /**
     * 处理field
     *
     * @param fieldDef
     * @throws Exception
     */
    protected void handleFieldDef(IFieldDef fieldDef) throws Exception {
        String tag = fieldDef.getTag();
        String etag = fieldDef.getEtag();
        String value = fieldDef.getValue();
        String type = fieldDef.getType();
        String scope = fieldDef.getScope();
        String ref = fieldDef.getRef();

        Object contextValue = null;

        if (!StringUtil.isEmpty(type)) {
            if (elParser instanceof ApacheJexl) {

                if (type.trim().matches("\\$\\{\\S+\\s*=new\\s+\\(\\S+\\)}")) {
                    // Object obj =
                    // elParser.el.parse(type).evaluate(elParser.jContext);
                    // elParser.jContext.set(tag, obj);
                    Object obj = elParser.dealTypeOfObj(type, context);
                    elParser.putObjtoConext(etag, obj);
                    saveToContext(scope, tag, obj);
                    return;
                } else if (type.trim().matches("\\$\\{new\\s+\\S+\\(\\S*\\)}")) {
                    // ${new w(a,b)}
                    type = type.trim();
                    // ///
                    String t11 = type.substring(0, 2);
                    // t2=new w(a,b)
                    String t2 = type.substring(2, type.length() - 1).trim();
                    String tt2[] = t2.split(" ");
                    // //// new
                    String tt21 = tt2[0].trim();
                    // // w(a,b)
                    String tt22 = tt2[1].trim();
                    tt22 = tt22.replace("(", ",");
                    tt22 = "(" + tt22;
                    tt22 = tt22.replace(",)", ")");
                    type = t11 + "xx=" + tt21 + " " + tt22 + "}";
                    /* System.out.println(type); */
                    Object obj = elParser.dealTypeOfObj(type, context);
                    elParser.putObjtoConext(etag, obj);
                    saveToContext(scope, tag, obj);
                    return;
                } else {
                    type = (String) elParser.dealElExpression(context, type);
                    type = type.trim();
                }
            } else {
                type = (String) elParser.getExPression(context, type, null);
                type = type.trim();
            }

        }
        if (type.toLowerCase().matches("\\w+(\\[\\d+\\])+")) {
            Object array = createArrayByType(type);
            saveToContext(scope, tag, array);
            return;
        } else if (type.toLowerCase().matches("arraylist")) {
            saveToContext(scope, tag, new ArrayList<Object>());
            return;
        } else if (type.toLowerCase().matches("map")) {
            saveToContext(scope, tag, new HashMap<Object, Object>());
            return;
        } else if (type.toLowerCase().matches("linkedmap")) {
            saveToContext(scope, tag, new LinkedHashMap<Object, Object>());
            return;
        }
        if (!StringUtil.isEmpty(etag) && getType().equals("in")) {
            contextValue = getFieldValue(fieldDef);
            // ------------added by weicong on 2016/3/14~15 -----------
            if (!StringUtil.isEmpty(value)) {
                saveToContext(scope, tag, contextValue);
                Object[] objs = dealArguments(fieldDef.getAuguments());
                contextValue = elParser.getExPression(context, value, objs);
            }
        }
        // ------------added by hujun on 2015/4/24 -----------
        else if (!StringUtil.isEmpty(ref)) {
            contextValue = elParser.getExPression(context, ref, null);
        }
        // ---------------------------------------------------
        else if (StringUtil.isEmpty(value)) {
            contextValue = getFieldValueBySubDefs(fieldDef);
        } else {
            // 直接获取值
            Object[] objs = dealArguments(fieldDef.getAuguments());
            // 如果类型是exceptioon，返回异常不抛出
            if (FIELD_EXCEPTION_TYPE.equalsIgnoreCase(type))
                contextValue = elParser.getExPression(context, value, objs, true);
            else
                contextValue = elParser.getExPression(context, value, objs);
        }
        if (filterDef != null) {
            if (filterDef.getType().equals("in")) {
                if (contextValue != null) contextValue = dealValueByType(contextValue, type);
                if (fieldDef.isRequired()
                        && (contextValue == null || StringUtil.isEmpty(contextValue.toString()))) {
                    throw new InterfaceException("00402",
                            "field [" + tag + "] value should not be empty");
                }
            } else {
                // 补充长度
                String fill = fieldDef.getFill();
                // String size = (String) elParser.getExPression(context,
                // fieldDef.getSize(), null);
                String size =
                        String.valueOf(elParser.getExPression(context, fieldDef.getSize(), null));

                if (!StringUtil.isEmpty(fill)) {
                    if (StringUtil.isEmpty(size) && contextValue != null) {
                        throw new InterfaceException("00431",
                                "attrubute [fill] and [size] should both exists.");
                    }
                    String[] fillArgs = fill.split(",");
                    int fillSize = Integer.valueOf(size);
                    String fixValue = getSizeFixedValue(contextValue.toString(), fillSize,
                            fillArgs[1], fillArgs[0]);
                    contextValue = fixValue;
                }
                packField(fieldDef, contextValue);
            }
            String regex = fieldDef.getValidator();
            if (!StringUtil.isEmpty(regex) && contextValue != null
                    && !contextValue.toString().matches(regex)) {
                throw new InterfaceException("00431",
                        "field value [" + contextValue + "] does not match regex [" + regex + "]");
            }
        } else {
            contextValue = dealValueByType(contextValue, type);
        }

        String codetable = (String) elParser.getExPression(context, fieldDef.getCodetable(), null);
        if (!StringUtil.isEmpty(codetable)) {
            contextValue = context.getCodeTableValue(codetable, contextValue.toString());
        }

        // add by gechengyang 20160515
        String direct = (String) elParser.getExPression(context, fieldDef.getDirect(), null);
        if (!StringUtil.isEmpty(direct)) {
            if (direct.equals("i"))
                context.getInputMap().put(tag, contextValue);
            else if (direct.equals("o")) context.getOutputMap().put(tag, contextValue);
        }
        saveToContext(scope, tag, contextValue);
    }

    /**
     * 补充长度
     *
     * @param value    待补充的字符串
     * @param size     补充后的长度
     * @param fillChar 补充时使用的字符
     * @param pos      补充的位置 left/right
     * @return
     */
    protected String getSizeFixedValue(String value, int size, String fillChar, String pos) {
        // 处理Byte时候用,贵金属需求
        if (fillChar.startsWith("__")) {
            size = getLen(value);
        }
        int addLen = size - getLen(value);
        if (addLen < 0)
            throw new InterfaceException("00432", "value length larger than fill length.");
        String addStr = StringUtil.repeat(fillChar, addLen);
        if (pos.equals("left"))
            return addStr + value;
        else if (pos.equals("right")) return value + addStr;
        return value;
    }

    protected byte[] fillWithBytes(String value, int size, String fillChar) {
        if (!fillChar.startsWith("__")) {
            return null;
        }
        int addLen = size - getLen(value);
        String flgfillChar = fillChar.substring(2);
        String str = "";
        for (int i = 0; i < addLen; i++) {
            str = str + flgfillChar;
        }
        if (addLen < 0)
            throw new InterfaceException("00432", "value length larger than fill length.");
        return ByteUtil.parseHexStr2Byte(str);
    }

    /**
     * 根据field的if/switch子节点获取值，并进行convertor/validator处理
     *
     * @param fieldDef
     * @return
     */
    private Object getFieldValueBySubDefs(IFieldDef fieldDef) {
        Object contextValue = fieldDef.getValue();
        // convertor和validator处理
        List<Object> cvList = fieldDef.getConvertorsAndValidators();
        if (cvList == null || cvList.size() == 0) {
            // contextValue = "";
        } else {
            for (Object cv : cvList) {
                if (cv instanceof IConvertorDef) {
                    contextValue = handleConvertorDef((IConvertorDef) cv);
                } else if (cv instanceof IValidatorDef) {
                    IValidatorDef validatorDef = (IValidatorDef) cv;
                    if (false == handleValidatorDef(validatorDef)) {
                        String errorCode = (String) elParser.getExPression(context,
                                validatorDef.getErrorCode(), null);
                        String description = (String) elParser.getExPression(context,
                                validatorDef.getDescription(), null);
                        context.getResource().setErrorcode(errorCode);
                        context.getResource().setDescrption(description);
                        context.getResource().setDescription(description);
                        logger.error("validator " + validatorDef.getId() + " of field ["
                                + fieldDef.getTag() + "] error!");
                        throw new InterfaceException("11111");
                    }
                }
            }
        }
        return contextValue;
    }

    /**
     * field的type是否是数字类型
     *
     * @param type
     * @return
     */
    protected boolean isNumberType(String type) {
        type = type.toLowerCase().trim();
        if (type.equals("int") || type.equals("long") || type.equals("short")
                || type.equals("float") || type.equals("double") || type.startsWith("number"))
            return true;
        return false;
    }

    /**
     * 根据type对value进行转化
     *
     * @param value
     * @param type
     * @return
     */
    public Object dealValueByType(Object value, String type) {
        Object obj = null;
        if (value == null) return null;
        if (!StringUtil.isEmpty(type)) // 根据type配置转化
        {
            type = type.toLowerCase().trim();
            String tmpValue = null;
            if (value instanceof byte[]) {
                try {
                    tmpValue = new String((byte[]) value, getFilterDef().getEncoding());
                } catch (UnsupportedEncodingException e) {
                    throw new InterfaceException("00403",
                            "unsupported encoding:" + getFilterDef().getEncoding(), e);
                }
            } else if (value instanceof Double || value instanceof Float)
                tmpValue = StringUtil.formatDouble((Double) value);
            else
                tmpValue = value.toString();
            if (type.equals("int"))
                obj = (int) Double.parseDouble(tmpValue);
            else if (type.equals("string"))
                obj = tmpValue;
            else if (type.equals("double"))
                obj = Double.parseDouble(tmpValue);
            else if (type.equals("boolean"))
                obj = Boolean.valueOf(tmpValue);
            else if (type.equals("long")) {
                obj = Long.parseLong(tmpValue);
            }
            // TODO
            // else if (type.equals("byte"));
            // obj= Byte.parseByte(value)
            else if (type.equals("short"))
                obj = Short.parseShort(tmpValue);
            else if (type.equals("float"))
                obj = Float.parseFloat(tmpValue);
            else if (type.startsWith("date")) {
                int startIndex = type.indexOf('(');
                int endIndex = type.indexOf(')', startIndex);
                if (-1 != startIndex && startIndex < endIndex) {
                    String pattern = type.substring(startIndex + 1, endIndex);
                    try {
                        obj = DateUtil.parseDate(tmpValue, pattern);
                    } catch (ParseException e) {
                        throw new InterfaceException("00404",
                                "parse [" + obj + "] to Date pattern [" + pattern + "]error", e);
                    }
                } else
                    obj = value;
            } else if (type.startsWith("number(") && type.endsWith(")")) {
                if (!type.contains(","))
                    obj = Integer.parseInt(tmpValue);
                else
                    obj = Double.parseDouble(tmpValue);
            } else
                obj = value;
            return obj;
        } else
            return value;
    }

    /**
     * 根据scope和tag将值存入上下文
     *
     * @param scope
     * @param tag
     * @param contextValue
     */
    public void saveToContext(String scope, String tag, Object contextValue) {
        if ("this".equals(scope)) {
            context.getResource().setObject(contextValue);
        }
        if (StringUtil.isEmpty(tag)) {
            return;
        }
        String tagKey = null;
        if (tag.matches("\\w+((\\[\\$\\{\\w+\\}\\])|(\\[\\d+\\]))+")) // 判断为数组，存入上下文数组
        {
            tagKey = tag.substring(0, tag.indexOf('['));
            Object objArray = context.getVariable(tagKey);
            String sizeTag = tag.replaceAll("\\}", "_size\\}");
            if (!tag.equals(sizeTag)) {
                String arrayType = (String) elParser.dealElExpression(context, sizeTag);
                if (arrayType.matches("\\w+(\\[\\d+\\])+")) // 确认表达式对中的变量为循环变量
                {
                    // 创建数组
                    Object newArray = createArrayByType(
                            "object" + arrayType.substring(arrayType.indexOf('[')));
                    if (objArray == null || !ArrayUtil.arraySizeEquals(objArray, newArray)) {
                        if (scope.equals("foreach"))
                            context.addForeachVariable(filterDef, tagKey, newArray);
                        else
                            context.addVariable(scope, tagKey, newArray);
                    }
                }
            }
            // 表达式解析tag
            String realTag = (String) elParser.dealElExpression(context, tag);
            int lastIndex = realTag.lastIndexOf('[');
            int num = Integer.parseInt(realTag.substring(lastIndex + 1, realTag.lastIndexOf(']')));
            // 定位到多维数组的最后一维
            Object arrayObject = elParser.dealElExpression(context,
                    "${" + realTag.substring(0, lastIndex) + "}");
            if (arrayObject instanceof Object[]) {
                Object[] object = (Object[]) arrayObject;
                object[num] = contextValue;
            } else if (arrayObject instanceof int[]) {
                int[] object = (int[]) arrayObject;
                object[num] = (Integer) contextValue;
            } else if (arrayObject instanceof double[]) {
                double[] object = (double[]) arrayObject;
                object[num] = (Double) contextValue;
            } else if (arrayObject instanceof boolean[]) {
                boolean[] object = (boolean[]) arrayObject;
                object[num] = (Boolean) contextValue;
            } else {
                throw new InterfaceException("00406",
                        "need a array,but type is " + arrayObject.getClass());
            }
        } else {
            Object tagObj = elParser.getExPression(context, tag, null);
            if (tagObj != null) {
                if (tagObj instanceof List<?>) {
                    if (contextValue instanceof List<?>)
                        ((List<Object>) tagObj).addAll((List<Object>) contextValue);
                    else
                        ((List<Object>) tagObj).add(contextValue);
                }
                tagKey = tagObj.toString();
                if ("foreach".equals(scope))
                    context.addForeachVariable(filterDef, tagKey, contextValue);
                else
                    context.addVariable(scope, tagKey, contextValue);
            }
        }

    }

    /**
     * 处理foreach循环节点
     *
     * @param foreachDef
     * @throws Exception
     */
    protected void handleForeachDef(IForeachDef foreachDef) {
        /* boolean b = true; */
        isinforeach = true;
        List<IAbsDef> subobjs = foreachDef.getSubList();
        String tag = foreachDef.getTag();
        String contextKey = (String) elParser.getExPression(context, tag, null);
        String begin = foreachDef.getBegin();
        int beginValue = StringUtil.isEmpty(begin)
                ? 0
                : Integer.parseInt(elParser.getExPression(context, begin, null).toString());
        String step = foreachDef.getStep();
        int stepValue = StringUtil.isEmpty(step)
                ? 1
                : Integer.parseInt(elParser.getExPression(context, step, null).toString());
        String end = foreachDef.getEnd();
        int endValue =
                (int) Double.parseDouble(elParser.getExPression(context, end, null).toString());
        String scope = foreachDef.getScope();
        String condition = foreachDef.getCondition();
        context.addVariable(scope, contextKey + "_size", endValue);
        for (; beginValue < endValue; beginValue += stepValue) {
            Object obj = context.getVariable("____breakDone");
            if (obj != null) {
                if (!(Boolean) context.getVariable("____breakDone")) {
                    context.addVariable("transaction", "____breakDone", true);
                    break;
                }
                context.addVariable("transaction", "____continueDone", true);
                if (beginValue >= endValue) break;
            }

            context.addVariable(scope, contextKey, beginValue);
            foreachDef.setCurrentCount(beginValue);
            // if (StringUtil.isEmpty(condition) ||
            // (!StringUtil.isEmpty(condition) && (Boolean)
            // elParser.getExPression(context, condition, null)))

            if (StringUtil.isEmpty(condition) || (!StringUtil.isEmpty(condition)
                    && (Boolean) elParser.getExPression(context, condition, null)))
                handleSingleCycle(subobjs, condition, scope, contextKey, beginValue, endValue);

            else
                break;

            // 循环变量在处理过程中发生改动时，对循环变量重新赋值
            beginValue = (Integer) context.getVariable(contextKey);
        }

        // modified by gechengyang 20150912 TODO
        boolean breakD = (Boolean) context.getVariable("____breakDone");
        if (!breakD) {
            context.addVariable("transaction", "____breakDone", true);
        }

        boolean continuD = (Boolean) context.getVariable("____continueDone");
        if (!continuD) {
            context.addVariable("transaction", "____continueDone", true);
        }
        isinforeach = false;
    }

    protected void handleSingleCycle(List<IAbsDef> subobjs, String condition, String scope,
                                     String contextKey, int beginValue, int endValue) {
        for (IAbsDef sub : subobjs) {
            handleSubList(sub);
        }
    }

    /**
     * 获取filter的输入来源
     *
     * @return
     */
    public void setContextVia() {
        String via = filterDef.getVia();
        if (!StringUtil.isEmpty(via)) {
            context.setObject(elParser.getExPression(context, via, null));
        }
    }

    /**
     * 处理filter下的if节点
     *
     * @param ifDef
     */
    private void handleFilterIfDef(IIfDef ifDef) {
        Map<String, Object> map = ifDef.getConditionMap();
        Iterator<String> it = map.keySet().iterator();
        boolean ifValue = false;
        while (it.hasNext()) {
            String key = it.next();
            Object obj = elParser.getExPression(context, key, null);
            if ((ifValue = ((Boolean) obj).booleanValue()) == true) {
                Object value = map.get(key);
                List<IAbsDef> ifsubs = (List<IAbsDef>) value;
                for (IAbsDef ifsub : ifsubs) {
                    handleSubList(ifsub);
                }
                break;
            }
        }
        if (ifValue == false) {
            Object elseCondition = ifDef.getElseCondition();
            if (elseCondition == null) return;
            List<IAbsDef> ifsubs = (List<IAbsDef>) ifDef.getElseCondition();
            for (IAbsDef ifsub : ifsubs) {
                handleSubList(ifsub);
            }
        }
    }

    /**
     * 处理convertor节点
     *
     * @param convertorDef
     * @return
     */
    private Object handleConvertorDef(IConvertorDef convertorDef) {
        String value = convertorDef.getValue();
        List<IArgumentDef> arguments = convertorDef.getArguments();
        Object[] objs = dealArguments(arguments);
        return elParser.getExPression(context, value, objs);

    }

    /**
     * 处理validator节点
     *
     * @param validatorDef
     * @return
     */
    private boolean handleValidatorDef(IValidatorDef validatorDef) {
        String condition = validatorDef.getCondition();
        List<IArgumentDef> arguments = validatorDef.getArguments();
        Object[] objs = dealArguments(arguments);
        Object object = elParser.getExPression(context, condition, objs);
        return ((Boolean) object).booleanValue();
    }

    /**
     * 处理参数，为反射函数调用准备参数
     *
     * @param argumentMap
     * @returnz
     */
    public Object[] dealArguments(List<IArgumentDef> argumentDefs) {
        int size = argumentDefs.size();
        Object[] obj = null;
        if (size > 0) {
            obj = new Object[size];
            for (int i = 0; i < size; i++) {
                IArgumentDef argumentDef = argumentDefs.get(i);
                Object value = elParser.getExPression(context, argumentDef.getValue(), null);
                obj[i] = dealValueByType(value, argumentDef.getType());
            }
        }
        return obj;
    }

    @Override
    public void setIDef(IExecutableDef executableDef) {
        this.filterDef = (IFilterDef) executableDef;
    }

    @Override
    public IExecutableDef getIDef() {
        return filterDef;
    }

    public IFilterDef getFilterDef() {
        return this.filterDef;
    }

    /**
     * 用于定长字符串处理，获取上下文resource的object中的String的长度
     *
     * @return
     * @throws UnsupportedEncodingException
     */
    public int getLen(Object obj) {
        try {
            if (obj instanceof String) {
                // System.out.println(filterDef.getEncoding()+"........."+((String)
                // obj).getBytes(filterDef.getEncoding()).length);
                return ((String) obj).getBytes(filterDef.getEncoding()).length;
            } else if (obj instanceof byte[]) {
                return ((byte[]) obj).length;
            }
        } catch (UnsupportedEncodingException e) {
            throw new InterfaceException("00403", "Unsupported Encoding Exception", e);
        }

        return 0;
    }

    /**
     * 返回数组或队列的长度
     *
     * @param obj
     * @return
     */
    public int getArrayLen(Object obj) {
        Class<?> clazz = obj.getClass();
        if (clazz.isArray())
            return Array.getLength(obj);
        else if (obj instanceof List) return ((List<?>) obj).size();
        return 0;
    }

    @Override
    public void setElPaser(AbstractEL elParser) {
        this.elParser = elParser;
    }

    public String getExternErrorDesc(String errorcodeid) {
        ResourceBundle resb =
                LocaleUtil.getLocaleByLanguage(context.getInterfaceDef().getLanguage());
        String interfaceid_errorcodeid = context.getInterfaceDef().getId() + "." + errorcodeid;
        return resb.getString(interfaceid_errorcodeid);
    }

    /**
     * 获取连接的type操作类型(小写)
     *
     * @return
     * @throws InterfaceException
     */
    protected String getType() throws InterfaceException {
        String type = filterDef.getType();
        return type.toLowerCase();
    }

    public String getAesBysid(String sid) throws Exception {
        String key = Base64.encodeBase64String(AESCoder.initKey());
        byte[] data = key.getBytes();
        return new String(data, "UTF-8");
    }

    /**
     * 获取一个多行的String的第一行，按照传入的分割符进行分割
     *
     * @return
     */
    public String[] getSplitLine(String regex) {
        Object obj = context.getObject();
        if (!(obj instanceof String))
            throw new InterfaceException("00407", "context object is not a String");
        String str = obj.toString();
        return str.split("\r\n")[0].split(regex);
    }

    /**
     * 获取Client调用传入的参数的长度
     *
     * @return
     */
    public int getArgSize() {
        Object[] objs = context.getResource().getTransArgs();
        if (objs == null)
            return 0;
        else
            return objs.length;
    }

    /**
     * 从tblName表中获取主键
     *
     * @param tblName 目标表 USR
     * @param cnt     CNT表 CNT
     * @param nam     NAM='USR'
     * @param val     VAR='val'
     * @param stp     STP='stp'
     * @return
     * @throws SQLException
     */
    public Object getCNTinr(String tblName, String cnt, String nam, String val, String stp) {
        synchronized (lock) {
            logger.info("Excute sqlConnection of dbCounter.");
            try {
                JdbcExecutor executor = null;
                tblName = tblName.toLowerCase();
                Connection c = context.getResource().getCntConnection();
                executor = new JdbcExecutor(context, c);
                // Map<String, Object> result =
                // executor.executeQueryForMap("SELECT * FROM " + cnt +
                // " WHERE " + nam + "='" + tblName + "' FOR UPDATE");
                Map<String, Object> result = executor.executeQueryForMap(
                        "SELECT * FROM " + cnt + " WHERE " + nam + "='" + tblName + "' FOR UPDATE");
                if (null == result || result.isEmpty()) {
                    executor.executeUpdate("INSERT INTO " + cnt + "(" + nam + "," + val + "," + stp
                            + ") VALUES('" + tblName + "',2,1)");
                    executor.commit();
                    return 1;
                }
                String produceName = executor.getProductName();
                produceName = produceName.toUpperCase();
                if (!produceName.equals("MYSQL") && !produceName.equals("POSTGRESQL")) {
                    val = val.toUpperCase();
                    stp = stp.toUpperCase();
                }
                int newval = Integer.valueOf(result.get(val).toString())
                        + Integer.valueOf(result.get(stp).toString());
                logger.info("newval=" + newval);
                String sql = "UPDATE  " + cnt + " SET  " + val + " = " + newval + " WHERE " + nam
                        + "='" + tblName + "'";
                logger.info("getCNT inr sql=" + sql);
                executor.executeUpdate(sql);
                executor.commit();
                logger.info("Cnt Instance:" + result.get(val).getClass());
                if (result.get(val) instanceof Integer) {
                    return result.get(val);
                } else {
                    return Integer.valueOf(result.get(val).toString());
                }
            } catch (SQLException e) {
                throw new InterfaceException("10011", "sql connection error:", e);
            }
        }
    }

    public Object getCNTinr1(String tblName, String cnt, String nam, String val, String stp) {
        synchronized (lock) {
            logger.info("Excute sqlConnection of dbCounter.");
            try {
                JdbcExecutor executor = null;
                tblName = tblName.toUpperCase();
                Connection c = context.getResource().getCntConnection();
                executor = new JdbcExecutor(context, c);
                // Map<String, Object> result =
                // executor.executeQueryForMap("SELECT * FROM " + cnt +
                // " WHERE " + nam + "='" + tblName + "' FOR UPDATE");
                Map<String, Object> result = executor.executeQueryForMap(
                        "SELECT * FROM " + cnt + " WHERE " + nam + "='" + tblName + "' FOR UPDATE");
                if (null == result || result.isEmpty()) {
                    executor.executeUpdate("INSERT INTO " + cnt + "(" + nam + "," + val + "," + stp
                            + ") VALUES('" + tblName + "',2,1)");
                    executor.commit();
                    return 1;
                }
                String produceName = executor.getProductName();
                produceName = produceName.toUpperCase();
                if (!produceName.equals("MYSQL")) {
                    val = val.toUpperCase();
                    stp = stp.toUpperCase();
                }
                int newval = Integer.valueOf(result.get(val).toString())
                        + Integer.valueOf(result.get(stp).toString());
                logger.info("newval=" + newval);
                String sql = "UPDATE  " + cnt + " SET  " + val + " = " + newval + " WHERE " + nam
                        + "='" + tblName + "'";
                logger.info("getCNT inr sql=" + sql);
                executor.executeUpdate(sql);
                executor.commit();
                logger.info("Cnt Instance:" + result.get(val).getClass());
                if (result.get(val) instanceof Integer) {
                    return result.get(val);
                } else {
                    return Integer.valueOf(result.get(val).toString());
                }
            } catch (SQLException e) {
                throw new InterfaceException("10011", "sql connection error:", e);
            }
        }
    }

    /**
     * 从CNT表中获取主键
     *
     * @param tblName
     * @param counter
     * @param nam
     * @param val
     * @param stp
     * @return Object
     */
    public Object getInnerCNTinr(String tblName) {
        synchronized (lock) {
            JdbcExecutor executor = null;
            tblName = tblName.toUpperCase();
            try {
                executor = new JdbcExecutor(context, new Sqlite_OP().getConnection(tblName));
                Map<String, Object> result = executor
                        .executeQueryForMap("SELECT * FROM CNT WHERE NAM='" + tblName + "'");
                if (null == result || result.isEmpty()) {
                    executor.executeUpdate(
                            "INSERT INTO CNT(NAM,VAL,STP) VALUES('" + tblName + "',2,1)");
                    executor.commit();
                    return "00000001";
                }
                int newval = Integer.valueOf(result.get("val").toString())
                        + Integer.valueOf(result.get("stp").toString());
                executor.executeUpdate(
                        "UPDATE CNT SET VAL = " + newval + " WHERE NAM='" + tblName + "'");
                executor.commit();
                return String.format("%08d", result.get("val"));
            } catch (SQLException e) {
                throw new InterfaceException("00408", "sql connection error:", e);
            }
        }
    }

    public Object counter(String tblName) throws SQLException {
        return counter(tblName, 8);
    }

    /**
     * 从CNT表中获取主键
     *
     * @param tblName
     * @param counter
     * @param nam
     * @param val
     * @param stp
     * @return Object
     * @throws SQLException
     */
    public Object counter(String tblName, int length) throws SQLException {

        synchronized (lock) {
            PreparedStatement ps = null;
            ResultSet rs = null;
            ICounterDef counterDef = null;
            int index = tblName.indexOf("*");
            if (index != -1) {
                counterDef = context.getCounterMap().get("*");
                tblName = tblName.substring(0, index) + tblName.substring(index, tblName.length());
            } else
                counterDef = context.getCounterMap().get(tblName);
            String sql = counterDef.getSql();
            String initSql = counterDef.getInitSql();
            long l = 0;
            Connection conn = context.getResource().getCntConnection();
            try {
                ps = conn.prepareStatement(sql, 1003, 1008);
                // Statement st =
                // conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_UPDATABLE);
                if (sql.indexOf("?") != -1) {
                    ps.setString(1, tblName);
                }
                rs = ps.executeQuery();
                if (rs.next()) {
                    l = rs.getLong(1);
                    ResultSetMetaData localResultSetMetaData = rs.getMetaData();
                    if (localResultSetMetaData.getColumnCount() > 1) {
                        int i = rs.getInt(2);
                        if (localResultSetMetaData.isWritable(1)) {
                            rs.updateLong(1, l + i);
                            rs.updateRow();
                            conn.commit();
                        }
                        boolean beforeStep = counterDef.getBeforeStep();
                        if (!beforeStep) l += i;
                    }
                } else {
                    ps = conn.prepareStatement(initSql, 1003, 1008);
                    if (sql.indexOf("?") != -1) {
                        ps.setString(1, tblName);
                    }
                    ps.executeUpdate();
                    conn.commit();
                    l = Long.parseLong(counterDef.getValue());
                }
            } catch (SQLException e) {
                // e.printStackTrace();
                ps = conn.prepareStatement(initSql, 1003, 1008);
                if (sql.indexOf("?") != -1) {
                    ps.setString(1, tblName);
                }
                ps.executeUpdate();
                conn.commit();
                l = Long.parseLong(counterDef.getValue());
            } finally {
                if (rs != null) rs.close();
                if (ps != null) ps.close();
            }

            return StringUtil.repeat("0", length - String.valueOf(l).length()) + l;
        }

    }

    /**
     * 校验字符串长度
     *
     * @param s
     * @param length
     * @return boolean
     */
    public boolean validateLen(String s, int length) {
        boolean isValidate = false;
        if (StringUtil.isEmpty(s)) return isValidate;
        if (s.length() == length) {
            isValidate = true;
        }
        return isValidate;
    }

    /**
     * 从COD表中获取错误码
     *
     * @param tblName
     * @param counter
     * @param nam
     * @param val
     * @param stp
     * @return Object
     */
    public Object getInnerCodDsp(String dbname, String system, String code) {
        // return "CI0221";
        logger.info("getInnerCod Param values=" + code);
        /*
         * if(code==null) { return null; }
         */
        JdbcExecutor executor = null;
        Map<String, Object> result = null;
        try {
            executor = new JdbcExecutor(context, new Sqlite_OP().getConnection(dbname));
            result = executor.executeQueryForMap("SELECT * FROM COD WHERE SYS ='" + system.trim()
                    + "' AND COD='" + code.trim() + "'");
            if (null == result || result.isEmpty()) {
                return null;
            }
        } catch (SQLException e) {
            throw new InterfaceException("00409", "sql commit error:", e);
        } finally {
            try {
                executor.close();
            } catch (SQLException e) {
                throw new InterfaceException("00410", "sql connection error:", e);
            }
        }
        return result.get("DSP");
    }

    /**
     * 按照光大IBS的规则获取最新的文件(根据文件名去判断，没有按照lastModifed去判断)
     *
     * @param base 上级目录
     * @param pre  前缀
     * @return
     */
    public static String getIBSLatestFileDate(String base, final String pre) {
        File baseFile = new File(base);
        File[] files = baseFile.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                return !pathname.isDirectory() && pathname.getName().startsWith(pre);
            }
        });

        List<Integer> dateList = new ArrayList<Integer>();

        for (File file : files) {
            String filename = file.getName();
            int indexUnderline = filename.indexOf(pre);
            int indexSuffix = filename.lastIndexOf(".");

            String date = filename.substring(indexUnderline + pre.length());
            if (-1 != indexSuffix)
                date = filename.substring(indexUnderline + pre.length(), indexSuffix);

            boolean isMatcher = date.matches("[0-9]{4}[0-9]{2}[0-9]{2}");
            if (isMatcher) dateList.add(Integer.parseInt(date));
        }

        int[] a = new int[dateList.size()];
        for (int i = 0; i < dateList.size(); i++) {
            a[i] = dateList.get(i);
        }

        int max = NumberUtil.getMax(a);
        if (max == -1)
            return "";
        else
            return String.valueOf(max);
    }

    /**
     * @param smrt 共享目录
     * @param arc  备份目录
     **/
    public String getLatestFileName(String smrt, String arc) {
        List<Integer> alldayTypeList = new ArrayList<Integer>();
        File smrtFile = new File(smrt);
        String str[] = smrtFile.list();
        for (String subFileName : str) {
            boolean isMatcher = subFileName.matches("[0-9]{4}[0-9]{2}[0-9]{2}");
            if (isMatcher && (!arc.equals(str))) alldayTypeList.add(Integer.parseInt(subFileName));
        }
        int a[] = new int[alldayTypeList.size()];
        for (int i = 0; i < alldayTypeList.size(); i++) {
            a[i] = alldayTypeList.get(i);
        }
        int max = NumberUtil.getMax(a);
        if (max == -1)
            return null;
        else
            return String.valueOf(max);
    }

    /**
     * 获取枚举类型
     *
     * @param classname 枚举类型
     * @param value     值
     */
    public Object getEnum(String classname, String value) {
        try {
            Class<?> clazz = Class.forName(classname);
            return MethodUtils.invokeStaticMethod(clazz, "valueOf", value);
        } catch (Exception e) {
            throw new InterfaceException("00421", "get enum [" + value + "] error", e);
        }
    }

    public String getXmlBindValue(JAXBElement<String> element) {
        return element.getValue();
    }

    /**
     * 反射调用静态方法
     *
     * @param classname
     * @param methodname
     * @param objects
     * @return
     */
    public Object invokeStaticMethod(String classname, String methodname, Object[] objects) {
        Class<?> clazz;
        try {
            clazz = Class.forName(classname);

            Object ret = MethodUtils.invokeStaticMethod(clazz, methodname, objects);
            return ret;
        } catch (IllegalAccessException e) {
            throw new InterfaceException("00424", "create instance error.", e);
        } catch (ClassNotFoundException e) {
            throw new InterfaceException("00422", "class [" + classname + "] not found.", e);
        } catch (NoSuchMethodException e) {
            throw new InterfaceException("00423",
                    "invoke method [" + methodname + "]error,no such method.", e);
        } catch (InvocationTargetException e) {
            throw new InterfaceException("00425", "invoke method error.", e);
        }
    }

    public Object getReflectField(Class<?> clazz, String name) {
        return getReflectField(clazz, name, null);
    }

    public Object getReflectField(Class<?> clazz, String name, Object o) {
        try {
            Field field = clazz.getDeclaredField(name);
            return field.get(o);
        } catch (SecurityException e) {
            throw new InterfaceException("00426", "field [" + name + "] is not accessable.", e);
        } catch (NoSuchFieldException e) {
            throw new InterfaceException("00427", "field [" + clazz + "] not found.", e);
        } catch (IllegalArgumentException e) {
            throw new InterfaceException("00428", "illegal argument.", e);
        } catch (IllegalAccessException e) {
            throw new InterfaceException("00426", "field [" + name + "] is not accessable.", e);
        }
    }

    public Object getReflectField(String classname, String name) {
        return getReflectField(classname, name, null);
    }

    public Object getReflectField(String classname, String name, Object o) {
        Class<?> clazz = null;
        try {
            clazz = Class.forName(classname);
            return getReflectField(clazz, name, o);
        } catch (ClassNotFoundException e) {
            throw new InterfaceException("00422", "class [" + classname + "] not found.", e);
        }
    }

    public Object getConstructInstance(String classname, Object[] objects) {
        try {
            Class<?> clazz = Class.forName(classname);
            return ConstructorUtils.invokeConstructor(clazz, objects);
            /*
             * Class<?>[] type = null; if (null != objects) { type = new Class[objects.length]; for
             * (int i = 0; i < objects.length; i++) { if(objects[i] instanceof java.util.List) {
             * type[i] = List.class; } else { type[i] = objects[i].getClass(); }
             *
             * } }
             *
             * Constructor<?> constructor = clazz.getConstructor(type); return
             * constructor.newInstance(objects); ConstructorUtils.invokeConstructor(arg0, arg1)
             */
        } catch (ClassNotFoundException e) {
            throw new InterfaceException("00422", "class [" + classname + "] not found.", e);
        } catch (Exception e) {
            throw new InterfaceException("00429", "deal constructor error.", e);
        }

    }

    /**
     * 反射调用非静态方法
     *
     * @param classname
     * @param methodname
     * @param objects
     * @return
     */
    public Object invokeMethod(String classname, String methodname) {
        return invokeMethod(classname, methodname, null);
    }

    public Object invokeMethod(Object instance, String methodname, Object[] objects) {
        try {
            return MethodUtils.invokeMethod(instance, methodname, objects);
        } catch (NoSuchMethodException e) {
            throw new InterfaceException("00423",
                    "method [" + methodname + "] not found in " + instance.getClass(), e);
        } catch (IllegalAccessException e) {
            throw new InterfaceException("00424", "method is not accessable.", e);
        } catch (InvocationTargetException e) {
            throw new InterfaceException("00425", "invoke method " + methodname + "error.", e);
        }
    }

    /**
     * 反射调用非静态方法
     *
     * @param classname
     * @param methodname
     * @param objects
     * @return
     */
    public Object invokeMethod(String classname, String methodname, Object[] objects) {
        Object classobj = null;
        try {
            classobj = Class.forName(classname).newInstance();
            logger.debug("create instace of [" + classname + "] success.");
            Object retObj = MethodUtils.invokeMethod(classobj, methodname, objects);
            logger.debug("invoke method [" + methodname + "] success.");
            return retObj;
        } catch (InstantiationException e) {
            throw new InterfaceException("00429", "create instance error.", e);
        } catch (IllegalAccessException e) {
            throw new InterfaceException("00424", "create instance error.", e);
        } catch (ClassNotFoundException e) {
            throw new InterfaceException("00422", "class [" + classname + "] not found.", e);
        } catch (NoSuchMethodException e) {
            throw new InterfaceException("00423",
                    "invoke method [" + methodname + "]error,no such method.", e);
        } catch (InvocationTargetException e) {
            throw new InterfaceException("00425", "invoke method error.", e);
        }
    }

    /**
     * 获取数组或队列的大小
     *
     * @param obj
     * @return
     */
    public int size(Object obj) {
        return getArrayLen(obj);
    }

    public int length(Object obj) {
        return getLen(obj);
    }

    /**
     * 如果Object[]里面的非空元素都是同一种类型，则把该数组转化为那种类型，否则返回原数组
     *
     * @param objects
     * @return
     */
    public Object[] fixArrayToType(Object[] objects) {
        Class<?> type = objects[0].getClass();
        boolean allTypeSame = true;
        for (int i = 1; i < objects.length; i++) {
            if (objects[i] != null && !objects[i].getClass().equals(type)) {
                allTypeSame = false;
                break;
            }
        }
        if (allTypeSame) {
            Object[] objs = (Object[]) Array.newInstance(type, objects.length);
            System.arraycopy(objects, 0, objs, 0, objects.length);
            return objs;
        }
        return objects;
    }

    /**
     * 去掉数组中重复的元素
     *
     * @param objects
     * @return
     */
    public Object[] deleteSameElement(Object[] objects) {
        Map<Object, String> map = new LinkedHashMap<Object, String>();
        for (int i = 0; i < objects.length; i++) {
            if (!map.containsKey(objects[i])) map.put(objects[i], "");
        }
        Iterator<Object> it = map.keySet().iterator();
        Object[] objs = new Object[map.size()];
        for (int i = 0; i < objects.length; i++) {
            if (it.hasNext()) objs[i] = it.next();
        }
        return objs;
    }

    /**
     * 一串开始日期和结束日期是否是连续的，精确到天
     *
     * @param begDates
     * @param endDates
     * @return
     */
    public boolean isDatesSequential(Date[] begDates, Date[] endDates) {
        if (begDates.length != endDates.length) return false;
        for (int i = 0; i < begDates.length; i++) {
            try {
                begDates[i] = DateUtil.formatAndParseDate(begDates[i], "yyyyMMdd");
                endDates[i] = DateUtil.formatAndParseDate(endDates[i], "yyyyMMdd");
            } catch (ParseException e) {
                throw new InterfaceException("00404", "parse date error", e);
            }
        }
        for (int i = 0; i < begDates.length - 1; i++) {
            long dayInteval = (begDates[i + 1].getTime() - endDates[i].getTime()) / 86400000;// 86400000
            // =
            // 1000*60*60*24
            if (dayInteval != 1) {
                return false;
            }
        }
        return true;
    }

    /**
     * 根据传入命名空间，追加XML报文，并根据xsd文件校验xml的正确性
     *
     * @param xmlstr   xml报文
     * @param rootName 根节点
     * @param ns       命名空间
     * @param xsdPath  xsd路径
     * @return
     * @throws DocumentException
     * @throws ParserConfigurationException
     * @throws SAXException
     */
    public String xmlvalidate(String xmlstr, String rootName, String ns, String xsdPath)
            throws DocumentException, ParserConfigurationException, SAXException {
        int index2 = xmlstr.indexOf(rootName) + rootName.length();
        StringBuilder sb = new StringBuilder(xmlstr.substring(0, index2));
        sb.append(" xmlns=\"" + ns
                + "\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\""
                + ns + " " + xsdPath + "\"  ");
        sb.append(xmlstr.substring(index2));
        XmlValidator xv = new XmlValidator();
        xv.validate(sb.toString(), xsdPath);
        return xv.validate(sb.toString(), xsdPath) ? "" : xv.getStrErrors();
    }

    /**
     * 解析时，根据etag配置得到解析串
     *
     * @param fieldDef
     * @return
     * @throws Exception
     */
    public abstract Object getFieldValue(IFieldDef fieldDef) throws Exception;

    /**
     * 将一个int转换成4个字节
     *
     * @param i
     * @return
     */
    public byte[] intToByteArray(int i) {
        return ByteUtil.intToBytes(i);
    }

    /**
     * 获取当前时间
     *
     * @return
     */
    public Date getDateNow() {
        return new Date();
    }

    /**
     * 获取系统换行符
     */
    public String getSystemLineSeparator() {
        return System.getProperty("line.separator");
    }

    public void sleep(long time) throws InterruptedException {
        Thread.sleep(time);
    }

    public void sleep(int time) throws InterruptedException {
        Thread.sleep(time);
    }

    public int compare(Date date1, Date date2) {
        int a = date1.compareTo(date2);
        return a;
    }

    // 根据流程名称Name返回流程定义ID
    public String getProcessDefineId(Object obj, String name) {
        List list = (List) obj;
        for (Object object : list) {
            Map map = (Map) object;
            // resourceName":"Test.bpmn20.xml
            String xmlName = name + ".bpmn20.xml";
            if (map.get("resourceName").equals(xmlName)) {
                return (String) map.get("id");
            }
            // System.out.println(object.getClass());
        }
        return null;
    }

    public ISocketProxy getSocketProxy() {
        return context.getResource().getSocketProxy();
    }

    public void setSocketProxy(ISocketProxy proxy) {
        context.getResource().setSocketProxy(proxy);
    }

    public static synchronized ExecutorService getExecutorService(String name) {
        if (!executors.containsKey(name)) {
            ExecutorService pool = Executors.newCachedThreadPool();
            executors.put(name, pool);
        }
        return executors.get(name);
    }

    public void closeExecutor(String nam) throws InterruptedException {
        ExecutorService pool = executors.get(nam);
        while (!pool.isTerminated()) {
            Thread.sleep(1000);
        }
        pool.shutdown();
    }

    /**
     * 获取不定长度的字节数组区域
     **/
    public int findNext(String moduid, byte[] bitmap) {
        int len = 0;
        IModuleDef moduleDef = context.getModulesMap().get(moduid);
        String bitInfo = "";
        for (int i = 0; i < bitmap.length; i++) {
            bitInfo += ByteUtil.byte2bits(bitmap[i]);
        }
        Map<String, IFieldDef> mapdef = moduleDef.getMap();
        for (int i = 0; i < bitInfo.length(); i++) {
            String t = String.valueOf(bitInfo.charAt(i));
            if (t.equals("1")) {
                IFieldDef dstField = mapdef.get(i + "");
                String type = dstField.getType();
                String size = dstField.getSize();
                if (type.equals("vrString") || type.equals("bit")) len += Integer.parseInt(size);
            }
        }
        return len % 8 == 0 ? len / 8 : (len / 8 + 1);
    }

    /***
     * 获取信息域的长度
     **/
    public int findNext(String moduid, byte[] bitmap, byte[] next) {
        int len = 0;
        IModuleDef moduleDef = context.getModulesMap().get(moduid);
        String bitInfo = "";
        String nextinfo = "";
        for (int i = 0; i < bitmap.length; i++) {
            bitInfo += ByteUtil.byte2bits(bitmap[i]);
        }

        for (int i = 0; i < next.length; i++) {
            nextinfo += ByteUtil.byte2bits(next[i]);
        }

        for (int i = 0; i < bitInfo.length(); i++) {
            String t = String.valueOf(bitInfo.charAt(i));
            if (t.equals("1")) {
                IFieldDef dstField = moduleDef.getMap().get(i + "");
                String type = dstField.getType();
                int size = Integer.parseInt(dstField.getSize());
                if (type.equals("bit")) {
                    nextinfo = nextinfo.substring(size);
                } else if (type.equals("vrString")) {
                    String temp = nextinfo.substring(0, size);
                    len += Integer.valueOf(temp, 2);
                    nextinfo = nextinfo.substring(size);
                } else if (type.equals("fxString")) {
                    len += size;
                }
            }
        }
        return len;
    }

    public Map<String, Object> getInfo(String moduid, byte[] bitmap, byte[] next, byte[] infos) {
        Map<String, Object> map = new HashMap<String, Object>();
        int offset = 0;

        IModuleDef moduleDef = context.getModulesMap().get(moduid);
        String bitInfo = "";
        String nextinfo = "";
        for (int i = 0; i < bitmap.length; i++) {
            bitInfo += ByteUtil.byte2bits(bitmap[i]);
        }

        for (int i = 0; i < next.length; i++) {
            nextinfo += ByteUtil.byte2bits(next[i]);
        }

        for (int i = 0; i < bitInfo.length(); i++) {
            String t = String.valueOf(bitInfo.charAt(i));
            if (t.equals("1")) {
                IFieldDef dstField = moduleDef.getMap().get(i + "");
                String type = dstField.getType();
                String etag = dstField.getEtag();
                int size = Integer.parseInt(dstField.getSize());
                if (type.equals("bit")) {
                    String temp = nextinfo.substring(0, size);
                    map.put(etag, Integer.valueOf(temp, 2));
                    nextinfo = nextinfo.substring(size);
                } else if (type.equals("vrString")) {
                    String temp = nextinfo.substring(0, size);
                    int len = Integer.valueOf(temp, 2);
                    byte[] buffer = new byte[len];
                    System.arraycopy(infos, offset, buffer, 0, len);
                    map.put(etag, new String(buffer));
                    offset += len;
                    nextinfo = nextinfo.substring(size);
                } else if (type.equals("fxString")) {
                    byte[] buffer = new byte[size];
                    System.arraycopy(infos, offset, buffer, 0, size);
                    map.put(etag, new String(buffer));
                    offset += size;
                }
            }
        }
        return map;
    }

    public boolean isFlag(byte[] bitmap) {
        byte b = bitmap[0];
        String s = ByteUtil.byte2bits(b);
        if (s.startsWith("1")) {
            return true;
        }
        return false;
    }

    public Date combineDate(XMLGregorianCalendar date, String pattern1, String time,
                            String pattern2) {
        if (StringUtil.isEmpty(time)) {
            return DateUtil.parseDate(date);
        }
        String date1 = formatDate(date, pattern1);
        return formatDate(date1 + time, pattern1 + pattern2);
    }

    /**
     * 将XMLGregorianCalendar按照pattern转成String
     *
     * @param xgc
     * @param pattern
     * @return
     */
    public String formatDate(XMLGregorianCalendar xgc, String pattern) {
        Date date = DateUtil.parseDate(xgc);
        return DateUtil.format(date, pattern);
    }

    /**
     * 将表示时间的字符串按照pattern转成Date
     *
     * @param date
     * @param pattern
     * @return
     */
    public Date formatDate(String date, String pattern) {
        try {
            return DateUtil.parseDate(date, pattern);
        } catch (ParseException e) {
            throw new InterfaceException("00004", "parse date error", e);
        }
    }

    public String createRandomNumStr(int num) {
        String str = String.format("%." + num + "f", Math.random());
        return str.substring(2);
    }

    public Object getFutureValue(Future<Object> future) {
        try {
            return future.get();
        } catch (Exception e) {
            throw new InterfaceException("00035", "get future value error.", e);
        }
    }

    public String[] splitString(String str, String regex) {
        // 特殊分隔字符处理
        int length = regex.length();
        String newRegex = "";
        for (int i = 0; i < length; i++) {
            char c = regex.charAt(i);
            if (c == '.' || c == '|' || c == '+' || c == '*' || (c == '\\' && i + 1 == length))
                newRegex += "\\";
            newRegex += c;
        }
        return StringUtil.split(newRegex, str);

    }

    /**
     * 对值进行二次处理，流程是：</br>
     * 1.检测value属性非空，非空则利用el表达式处理</br>
     * 2.检测type属性非空，非空则对值进行类型转换
     *
     * @param val
     * @param objdef
     * @param context
     * @return
     */
    public Object valFilter(Object val, IFieldDef objdef, Context context) {
        context.addVariable(null, objdef.getTag(), val);
        if (!StringUtil.isEmpty(objdef.getValue())) {
            Object[] objs = dealArguments(objdef.getAuguments());
            val = context.getElParser().getExPression(context, objdef.getValue(), objs);
        }
        // 如果有type属性，要对值转类型
        if (!StringUtil.isEmpty(objdef.getType())) {
            val = dealValueByType(val, objdef.getType());
        }
        return val;
    }

    public void setCallOutput(int seq) {
        Map<String, Object> out = context.getOutputMap();
        for (String key : out.keySet()) {
            context.addVariable("transaction", key + "_" + seq, out.get(key));
        }
        context.getOutputMap().clear();
    }

    /**
     * @return 获取context上下文中的变量map 此方法会暴露大量上下文内容 慎用
     */
    public Map<String, Object> getVariableMap() {
        return context.getVariableMap();
    }

    public void socketconnect(String ip, int port, int sleeptime) {
        ISocketProxy socketProxy = null;
        while (true) {
            try {
                // Socket s = new Socket(ip,port);
                socketProxy = new SocketProxy(new InetSocketAddress(ip, port), 0, 0,
                        getFilterDef().getEncoding(), context);
                if (socketProxy.isConnected()) {
                    context.getResource().setSocketProxy(socketProxy);
                    break;
                }
            } catch (UnknownHostException e) {
                try {
                    logger.error("socket connect failed ");
                    logger.error(e.getMessage(), e);
                    Thread.sleep(sleeptime);
                } catch (InterruptedException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                try {
                    logger.error("socket connect failed ", e);
                    Thread.sleep(1000);
                } catch (InterruptedException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
                e.printStackTrace();
            }
        }
    }

    /**
     * 需要考虑各种进制转换字符的请用这个方法
     *
     * @param rawstr
     * @return
     */
    public String exchangeStr(String rawstr) {
        if (rawstr.startsWith("0x")) {
            rawstr = new String(CommonFunctionUtils.parseHexStr2Byte(rawstr.replace("0x", "")));
        } else {
            rawstr = rawstr.replace("\\r", "\r").replace("\\n", "\n").replace("\\t", "\t");
        }
        return rawstr;
    }

}
