package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.core.model.IConnectionDef;
import org.enhydra.jdbc.standard.StandardXADataSource;
import org.objectweb.jotm.Jotm;
import org.objectweb.transaction.jta.TMService;

import javax.naming.NamingException;
import javax.sql.XAConnection;
import javax.transaction.*;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class JotmHelper
{
  private TMService                                jotm;
  private UserTransaction                          userTransaction;
  private static Map<String, StandardXADataSource> dataSourceMap = new HashMap<String, StandardXADataSource>();

  public Map<String, StandardXADataSource> getDataSourceMap()
  {
    return dataSourceMap;
  }

  /**
   * 启动事务管理服务
   * @throws NamingException 
   */
  public void startTMService()
      throws NamingException
  {
    jotm = new Jotm(true, false);
    userTransaction = jotm.getUserTransaction();
  }

  public synchronized static StandardXADataSource getStandardXADataSource(String key, IConnectionDef connectionDefintiion)
      throws SQLException
  {
    StandardXADataSource xads = dataSourceMap.get(key);
    if (xads == null)
    {
      xads = new StandardXADataSource();
      String username = connectionDefintiion.getProperty("username").getValue();
      String password = connectionDefintiion.getProperty("password").getValue();
      xads.setDriverName(connectionDefintiion.getProperty("driverClassName").getValue());
      xads.setUrl(connectionDefintiion.getProperty("url").getValue());
      xads.setUser(username);
      xads.setPassword(password);
      xads.setMaxCon(100);
      xads.setMinCon(20);
      xads.setLoginTimeout(2);
      System.out.println("xads counts="+xads.getAllConnections());
      dataSourceMap.put(key, xads);
    }
    return dataSourceMap.get(key);
  }

  /**
   * 取得数据库连接
   * 
   * @param db
   * @return
   * @throws Exception
   */
  public Connection getConnection(IConnectionDef connectionDefintiion)
      throws Exception
  {
    StandardXADataSource xads = new StandardXADataSource();
    XAConnection xaconn = null;
    xads.setTransactionManager(jotm.getTransactionManager());
    String username = connectionDefintiion.getProperty("username").getValue();
    String password = connectionDefintiion.getProperty("password").getValue();
    xads.setDriverName(connectionDefintiion.getProperty("driverClassName").getValue());
    xads.setUrl(connectionDefintiion.getProperty("url").getValue());
    xads.setTransactionManager(jotm.getTransactionManager());
    xads.setUser(username);
    xads.setPassword(password);
    xads.setMaxCon(1);
    xads.setMinCon(1);
    xads.setLoginTimeout(600);
    xaconn = xads.getXAConnection();
    Connection cnn = xaconn.getConnection();
    return cnn;
  }

  public void begin()
      throws NotSupportedException, SystemException
  {
    userTransaction.begin();
  }

  public void commit()
      throws SecurityException, IllegalStateException, RollbackException, HeuristicMixedException, HeuristicRollbackException,
      SystemException
  {
    userTransaction.commit();
  }

  public void rollback()
  {
    try
    {
      userTransaction.rollback();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * 停止事务管理服务
   */
  public void stopTMService()
  {
    jotm.stop();
    jotm = null;
  }
}
