package com.brilliance.eibs.core.service.instance.plugin;

import java.io.*;

public class LargeFileHelper {

    private File file;
    private int size;
    private int start;

    private int current;

    private String encoding = "utf-8";
    private int sz = 10 * 1024 * 1024;

    private IHandler handler;

    public LargeFileHelper(File file, int size, IHandler handler) {
        this.file = file;
        this.size = size;
        this.handler = handler;
    }

    public LargeFileHelper(File file, IHandler handler, int size, int start) {
        this.file = file;
        this.size = size;
        this.current = start;
        this.handler = handler;
    }

    public void setPageSize(int size) {
        this.size = size;
    }

    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    public void setSz(int sz) {
        this.sz = sz;
    }

    public void handle() throws IOException {
        BufferedReader in = null;
        try {
            BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file));
            in = new BufferedReader(new InputStreamReader(bis, encoding), sz);
            startForReady(in);
            while (in.ready()) {
                handler.handle(collectDataOnce(in));
            }
        } catch (IOException e) {
            throw new IOException("Large file handle exception,", e);
        } catch (Exception e) {
            throw new RuntimeException("Large file handle exception, ");
        } finally {
            if (null != in) in.close();
        }
    }



    private String collectDataOnce(BufferedReader in) throws IOException {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; in.ready() && i < size; i++) {
            sb.append(in.readLine());
            current++;
        }
        return sb.toString();
    }


    /**
     * 
     * 跳过多余行数，如果存在的话
     * 
     * @throws IOException
     */
    private void startForReady(BufferedReader in) throws IOException {
        if (null == in) throw new NullPointerException("Stream file does not exist");
        while (current < start && in.ready()) {
            in.readLine();
            // 丢弃不做处理
            current++;
        }
    }

    public interface IHandler {
        void handle(String s);
    }


}
