package com.brilliance.eibs.el;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IFuncDef;
import com.brilliance.eibs.core.model.IFunctionDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.core.service.instance.IExecutableInstance;
import com.brilliance.eibs.factory.BeanFactory.FunctionKey;
import com.brilliance.eibs.factory.FunctionInstanceFactory;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.main.client.CallableClient;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.el.ExpressionEvaluatorImpl;
import org.slf4j.Logger;

import javax.servlet.jsp.el.VariableResolver;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * 表达式解析器
 *
 * @author xiaoyuanzhen
 */
public class ApacheEL extends AbstractEL {

    Logger logger = LogUtil.getLogger(context);

    public ApacheEL(Context context) {
        super(context);

    }

    private Map<FunctionKey, IFunctionDef> functionMap;

    public static final String INTERFACE_UTIL = "com.brilliance.eibs.util";

    // 默认从com.brilliance.eibs.util中查找静态方法
    // private static Set<Class<?>> UtilClazz =
    // ClazzFactory.getClasses(INTERFACE_UTIL);

    public void setFunctionMap(Map<FunctionKey, IFunctionDef> functionMap) {
        this.functionMap = functionMap;
    }

    /**
     * MyExpression 例如：${totalAmout} objs :动态参数 当objs==null的时候，表示调用的函数为静态参数或者无参；
     * 当objs!=null的时候，objs表示从Argument获取参数
     **/
    public Object getExPression(Context context, String MyExpression, Object[] objs) {
        if (StringUtil.isEmpty(MyExpression)) {
            return MyExpression;
        }
        // 处理以#开头表达式
        if (MyExpression.matches("#\\{[\\w\\W]+"))
            return getSharpExpression(context, MyExpression);
        // 如果不是表达式类型，直接返回
        if (!IsContainelExpression(MyExpression)) {
            return MyExpression;
        }
        Object result = analyseExpression(context, MyExpression, objs);
        // String interfaceName = (String) context.getVariable("interfaceName");
        // if (interfaceName != null)
        // {
        // if (!interfaceName.toUpperCase().contains("BATCH"))
        // }

        if (MyExpression.contains("${")){
            String logLevel = (String)context.getVariable(Constants.Log_LEVEL);
            if(StringUtil.isNotEmpty(logLevel)){
                if("trace".equals(logLevel) || "debug".equals(logLevel)){
                    logger.debug("[" + MyExpression + "]=" + result);
                }
            }else{
                logger.debug("[" + MyExpression + "]=" + result);
            }
        }
        return result;
    }

    /**
     * 分析表达式串 ，进行解析
     *
     * @param context
     * @param exp
     * @param objs
     * @return
     */
    private Object analyseExpression(Context context, String exp, Object[] objs) {
        int len = exp.length();
        boolean inStr = false;// 当前字符在字符串里面
        boolean inExp = false;// 当前字符在表达式里面
        StringBuffer sb = new StringBuffer();
        List<Object> results = new ArrayList<Object>();
        for (int i = 0; i < len; i++) {
            char c = exp.charAt(i);
            if (c == '$') {
                if (!inStr && exp.charAt(i + 1) == '{') {
                    if (sb.length() > 0) {
                        results.add(sb.toString());
                        sb = new StringBuffer();
                    }
                    inExp = true;
                    sb.append(c);
                    sb.append('{');
                    i++;
                } else
                    sb.append(c);
            } else if (c == '\'') {
                if (inExp)
                    inStr = !inStr;
                sb.append(c);
            } else if (c == ' ') {
                if (!inExp || inStr)
                    sb.append(c);
            } else if (c == '}') {
                sb.append(c);
                if (!inStr && inExp) {
                    String s = sb.toString();
                    // 是函数表达式
                    if (s.matches("\\$\\{[\\s\\S]+\\([\\s\\S]*\\)\\}"))
                        results.add(dealReflectExpression(context, s.substring(2, s.length() - 1), objs));
                    else
                        results.add(dealElExpression(context, s));
                    sb = new StringBuffer();
                    inExp = false;
                }
            } else
                sb.append(c);
        }
        if (sb.length() > 0) {
            results.add(sb.toString());
        }
        if (results.size() == 1)
            return results.get(0);
        sb = new StringBuffer();
        for (Object result : results) {
            sb.append(result == null ? "" : result);
        }
        return sb.toString();
    }

    /**
     * 以"#"开头的字符串 解析为函数调用
     *
     * @param context
     * @param exp
     * @return
     */
    private Object getSharpExpression(Context context, String exp) {
        String[] strs = exp.substring(1).split(",");
        Object[] args = null;
        if (strs.length > 1) {
            args = new Object[strs.length - 1];
            for (int i = 0; i < strs.length - 1; i++) {
                args[i] = getExPression(context, strs[i + 1], null);
            }
        }
        return dealReflectExpression(context, strs[0] + "()", args);
    }

    /**
     * 处理函数调用
     *
     * @param context
     * @param m
     * @param objs
     * @return
     */
    private Object dealReflectExpression(Context context, String m, Object[] objs) {

        String ss[] = m.split("\\(");
        String ss0 = ss[0];
        String ss1 = ss[1].substring(0, ss[1].length() - 1).trim();
        Object[] obj = null;
        if (ss1.length() > 0) {
            String str[] = StringUtil.analyseArgs(ss1);
            obj = new Object[str.length];
            for (int i = 0; i < str.length; i++) {
                str[i] = str[i].trim();
                if (!str[i].contains("'"))// 参数不是字符串类型
                {
                    if (str[i].matches("\\d+(\\.\\d+)?"))// 参数为纯数字
                        obj[i] = Integer.parseInt(str[i]);
                    else
                        obj[i] = dealElExpression(context, "${" + str[i] + "}");
                } else {
                    obj[i] = str[i].substring(1, str[i].length() - 1);
                }
            }
        }
        if (objs == null && obj != null)
            objs = obj;
        Object o = null;
        try {
            if (ss0.equals("callEx")) {
                if (objs.length < 2) {
                    logger.error("arguments not enough");
                    throw new InterfaceException("00104");
                }
                Object[] args = new Object[objs.length - 2];
                System.arraycopy(objs, 2, args, 0, objs.length - 2);
                ResultMsg msg = new Client().call(objs[0].toString(), objs[1].toString(), args);
                return msg;
            }
            if (ss0.equals("tcall")) {
                if (objs.length < 2) {
                    logger.error("arguments not enough");
                    throw new InterfaceException("00104");
                }
                Object[] args = new Object[objs.length - 2];
                System.arraycopy(objs, 2, args, 0, objs.length - 2);
                ExecutorService exec = Executors.newCachedThreadPool();
                Future<ResultMsg> future = exec.submit(new CallableClient(objs[0].toString(), objs[1].toString(), args));
                exec.shutdown();
                return future;
            }

            CustomFunctionFinder finder = new CustomFunctionFinder(functionMap);
            if (finder.findFunction(ss0, objs)) {
                return FunctionInstanceFactory.createInstance(finder.getFunc(), context).execute(objs);
            }
            /*
             * for (Class<?> clazz : UtilClazz) ======= for (Class<?> clazz :
             * utilClazz) >>>>>>> .r86 { try { o =
             * MethodUtils.invokeStaticMethod(clazz, ss0, objs); return o; }
             * catch (Exception e) { //nothing to do //continue; } }
             */
            Object instance = context.getCurrentInstance();
            if (instance == null) {
                Class<?> clazz = Class.forName(IExecutableInstance.INNER_FILTER);
                instance = clazz.newInstance();
            }
            if (ss0.equals("getPropertyValue"))
                System.out.println(11);
            o = MethodUtils.invokeMethod(instance, ss0, objs);
        } catch (InterfaceException e) {
            throw e;
        } catch (Exception e) {
            Throwable throwable = e.getCause();
            if (throwable instanceof InterfaceException)
                throw (InterfaceException) throwable;
            else
                throw new InterfaceException("00102", "MethodUtils error,invoke [" + ss0 + "()] error:", e);
        }
        return o;
    }

    /**
     * 非函数调用的表达式处理，使用EL表达式解析
     *
     * @param context
     * @param MyExpression
     * @return
     */
    public Object dealElExpression(Context context, String MyExpression) {
        Map<String, Object> elMap = new HashMap<String, Object>();
        Map<String, Object> transitionMap = context.getTransitionMap();
        Map<String, Object> stepMap = context.getStepMap();
        Map<String, Object> transactionMap = context.getTransactionMap();
        Map<String, Object> globalMap = context.getGlobalMap();
        List<Map<String, Object>> foreachMaps = context.getCurrentInstanceForeachMaps();
        elMap.putAll(transactionMap);
        elMap.putAll(stepMap);
        if (foreachMaps != null) {
            for (int i = foreachMaps.size() - 1; i >= 0; i--) {
                elMap.putAll(foreachMaps.get(i));
            }
        }
        elMap.putAll(transitionMap);
        elMap.putAll(globalMap);
        Object title = ELObjectFind(context, elMap, MyExpression);
        return title;
    }

    private Object ELObjectFind(Context context, final Map<String, Object> elMap, String exp) {
        Object title = "";
        elMap.put("this", context.getResource().getObject());
        elMap.put("super", context.getResource());
        elMap.put("system", new Sys());
        ExpressionEvaluatorImpl expressionEvaluatorImpl = new ExpressionEvaluatorImpl();
        try {
            title = expressionEvaluatorImpl.evaluate(exp, Object.class, new VariableResolver() {
                public Object resolveVariable(String pName) {
                    Object pValue = elMap.get(pName);
                    return pValue;
                }

            }, null);
        } catch (Exception ex) {
            throw new InterfaceException("00101", "el error", ex);
        }
        return title;
    }

    /*
     * 是否含有EL表达式
     */
    private boolean IsContainelExpression(String s) {
        if (s.matches("([\\s\\S]*\\$\\{[\\s\\S]*\\w+[\\s\\S]*\\}[\\s\\S]*)+"))
            return true;
        else
            return false;
    }

    public int getArrayLen(Object obj) {
        if (obj instanceof Object[]) {
            return ((Object[]) obj).length;
        } else if (obj instanceof List) {
            return ((List<?>) obj).size();
        }
        return 0;
    }

    public Map<FunctionKey, IFuncDef> getFuncMap() {
        return null;
    }

    @Override
    public Object getExPression(Context context, String MyExpression, Object[] objs, boolean isException) {
        return getExPression(context, MyExpression, objs);
    }
}
