package com.brilliance.eibs.el;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IFuncDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.factory.BeanFactory.FunctionKey;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.jexl2.JexlContext;
import org.apache.commons.jexl2.JexlEngine;
import org.apache.commons.jexl2.JexlException.Variable;
import org.apache.commons.jexl2.MapContext;
import org.apache.commons.jexl2.UnifiedJEXL;
import org.slf4j.Logger;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ApacheJexl extends AbstractEL {
    static boolean flag = Boolean.parseBoolean(PropertyUtil.getProperty("sys.properties", "USE_EL"));
    static boolean ignore_elundefined_val = Boolean.parseBoolean(PropertyUtil.getProperty("sys.properties", "IGNORE_ELUNDEFINED_VAR"));
    Logger logger = LogUtil.getLogger(context);

    public ApacheJexl(Context context) {
        super(context);

    }

    public JexlContext jContext = null;
    public static JexlEngine engine = new JexlEngine();
    public static UnifiedJEXL el = new UnifiedJEXL(engine);
    private Map<FunctionKey, IFuncDef> funcMap;
    String STRINGREGEX = "^'(.*)'$";

    private Map<String, Object> funcs = new HashMap<String, Object>();

    public void setFunctionMap(Map<FunctionKey, IFuncDef> funcMap, Context context) {
        this.funcMap = funcMap;
        initFunction(context);
    }

    public void initFunction(Context context) {
        jContext = new MapContext(context.getVariableMap());
        try {
            // TODO 需要完善的代码
            // funcs.put(null,
            // Class.forName("com.brilliance.eibs.el.CommonFunctionUtils"));CommonFunctionUtils
            funcs.put(null, Thread.currentThread().getContextClassLoader().loadClass("com.brilliance.eibs.el.CommonFunctionUtils"));
            Iterator<FunctionKey> it = funcMap.keySet().iterator();
            while (it.hasNext()) {
                FunctionKey fk = (FunctionKey) it.next();
                IFuncDef iFunctionDef = funcMap.get(fk);
                // 静态函数处理
                if (iFunctionDef.getStc().equals("true")) {
                    String namepsace = iFunctionDef.getNamespace();
                    String clsnam = iFunctionDef.getCls();
                    // funcs.put(namepsace, Class.forName(clsnam));
                    funcs.put(namepsace, Thread.currentThread().getContextClassLoader().loadClass(clsnam));
                } else if (iFunctionDef.getStc().equals("false")) {
                    String name = iFunctionDef.getNamespace();
                    String classname = iFunctionDef.getCls();
                    try {
                        // jContext.set(name, Class.forName(classname));
                        jContext.set(name, Thread.currentThread().getContextClassLoader().loadClass(classname));
                    } catch (ClassNotFoundException e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public Object evaluateWithEl(Context context, String expression, boolean isException) {
        if (expression == null)
            return null; //
        // 这里必须每次执行，因为要兼容API调用
        initFunction(context);
        engine.setFunctions(funcs);
        engine.setStrict(true);

        if (!expression.contains("${"))
            return expression;
        Object obj = null;
        try {
            obj = el.parse(expression).evaluate(jContext);
            if (!isException && obj instanceof Exception) {
                Exception e = (Exception) obj;
                throw new InterfaceException("04001", e.getMessage(), e);
            }

        } catch (Exception e) {

            if (!ignore_elundefined_val) {
                throw new InterfaceException("04001", e.getMessage(), e);
            } else {
                // 兼容变量不存在的情况
                Throwable t = e.getCause();
                if (t == null) {
                    logger.warn("请检查表达式【" + expression + "】中的变量是否已经定义！");
                    return null;
                } else {
                    if (t instanceof Variable) {
                        Variable v = (Variable) t;
                        logger.warn("请检查表达式【{" + expression + "}】中的变量【" + v.getVariable() + "】是否已经定义！");
                        return null;
                    }

                }

            }

        }

        if (expression.contains("${")){
            String logLevel = (String)context.getVariable(Constants.Log_LEVEL);
            if(StringUtil.isNotEmpty(logLevel)){
                if("trace".equals(logLevel) || "debug".equals(logLevel)){
                    logger.debug("[" + expression + "]=" + obj);
                }
            }else{
                logger.debug("[" + expression + "]=" + obj);
            }
        }
        return obj;
    }

    @Override
    public Map<FunctionKey, IFuncDef> getFuncMap() {
        // TODO Auto-generated method stub
        return funcMap;
    }

    @Override
    public void setFuncMap(Map<FunctionKey, IFuncDef> funcMap) {
        // TODO Auto-generated method stub
        this.funcMap = funcMap;
    }

    @Override
    public Object getExPression(Context context, String MyExpression, Object[] objs) {
        // TODO Auto-generated method stub
        return evaluate(context, MyExpression, false);
    }

    public Object dealElExpression(Context context, String MyExpression) {
        return evaluate(context, MyExpression, false);
    }

    public Object dealTypeOfObj(String type, Context context) {
        // jContext = new MapContext(context.getVariableMap());
        initFunction(context);
        return el.parse(type).evaluate(jContext);
    }

    public void putObjToConext(String tag, Object obj) {
        jContext.set(tag, obj);
    }

    @Override
    public Object getExPression(Context context, String MyExpression, Object[] objs, boolean isException) {

        return evaluate(context, MyExpression, isException);
    }

    public Object evaluateWithNonEl(Context context, String MyExpression, boolean isException) {
        if (MyExpression == null)
            return null;
        if (MyExpression == "")
            return MyExpression;
        List<Object> result = new LinkedList<Object>();
        handleExpression(result, MyExpression, context);
        Object value = null;
        if (result.isEmpty()) {
            value = "";
        } else if (result.size() == 1) {
            value = result.get(0);
        } else {
            StringBuffer sb = new StringBuffer();
            for (Object obj : result) {
                sb.append(obj);
            }
            value = sb.toString();
            // engine.setStrict(true);
            // value = el.parse("${" + value + "}").evaluate(jContext);
        }
        if (MyExpression.contains("${")){
            String logLevel = (String)context.getVariable(Constants.Log_LEVEL);
            if(StringUtil.isNotEmpty(logLevel)){
                if("trace".equals(logLevel) || "debug".equals(logLevel)){
                    logger.debug("[" + MyExpression + "]=" + value);
                }
            }else{
                logger.debug("[" + MyExpression + "]=" + value);
            }
        }
        return value;
    }

    public void handleExpression(List<Object> value, String expression, Context context) {
        try {
            if (!expression.contains("${")) {
                value.add(expression);
            } else {
                List<String> exps = preprocessExpression(expression);
                if (exps.size() > 1) {
                    for (String exp : exps) {
                        handleExpression(value, exp, context);
                    }
                } else {
                    expression = expression.replaceAll("\\$\\{", "").replaceAll("\\}", "").trim();
                    handleExpressCore(value, expression, context);
                }
            }
        } catch (Exception e) {
            throw new InterfaceException("04001", e.getMessage(), e);
        }
    }

    private void handleExpressCore(List<Object> value, String expression, Context context) throws Exception {
        // 开头为__this的处理
        if (expression.startsWith("__this")) {
            invokeMethod(value, context.getCurrentInstance(), expression, context);
        }
        // 开头为this的处理
        else if (expression.startsWith("this")) {
            if (expression.matches("this(\\[\\d+\\])+")) {
                int index = Integer.parseInt(getMatchedPortion(expression, "this\\[(\\d+)\\]+", 1));
                value.add(((Object[]) context.getObject())[index]);
            } else {
                value.add(context.getObject());
            }
        }
        // 开头为system的处理
        else if (expression.startsWith("system")) {
            invokeStaticMethod(value, Sys.class, expression, context);
        }
        // 开头为super
        else if (expression.startsWith("super")) {
            invokeMethod(value, context.getResource(), expression, context);
        }
        // 静态函数的处理
        else if (expression.matches("[\\w]+:[\\w]+\\([\\w\\W]*\\)")) {
            String[] parts = expression.split(":");
            // 从静态函数查找
            Class<?> des = (Class<?>) funcs.get(parts[0]);
            invokeStaticMethod(value, des, expression, context);
        }
        // 通用函数的处理
        else if (expression.matches("[\\w]+\\([\\w\\W]*\\)")) {
            invokeStaticMethod(value, CommonFunctionUtils.class, expression, context);
        } else if (expression.matches("([\\w]+\\.[\\w]+)(\\([\\w\\W]*\\))")) {
            String objstr = getMatchedPortion(expression, "([\\w]+)\\.[\\w]+(\\([\\w\\W]*\\))", 1);
            invokeMethod(value, context.getVariable(objstr), expression, context);
        } else if (expression.matches("[^\\+]+(\\+[^\\+^\\d]*)+")) {
            String[] strs = expression.split("\\+");
            for (String str : strs) {
                str = str.trim();
                if (str.matches(STRINGREGEX)) {
                    value.add(getMatchedPortion(str, STRINGREGEX, 1));
                } else if (context.getVariable(str) != null) {
                    value.add(context.getVariable(str));
                } else {
                    handleExpressCore(value, str, context);
                }
            }
        } else {
            expression = expression.trim();
            if (expression.matches(STRINGREGEX)) {
                value.add(getMatchedPortion(expression, STRINGREGEX, 1));
            } else if (context.getVariable(expression) != null) {
                value.add(context.getVariable(expression));
            } else {
                // 最后还不行就交给EL处理
                jContext = new MapContext(context.getVariableMap());
                engine.setStrict(true);
                value.add(el.parse("${" + expression + "}").evaluate(jContext));
            }
        }
    }

    private String getMethodName(String expression) {
        String methodname = "";
        if (expression.matches("([\\w]+)(\\([\\w\\W]*\\))")) {
            // 形式为：方法名(参数...)
            methodname = getMatchedPortion(expression, "([\\w]+)(\\([\\w\\W]*\\))", 1);
        } else if (expression.matches("([\\w]+\\.[\\w]+)(\\([\\w\\W]*\\))")) {
            // 形式为：对象.方法名(参数...)
            methodname = getMatchedPortion(expression, "[\\w]+\\.([\\w]+)(\\([\\w\\W]*\\))", 1);
        } else if (expression.matches("[\\w]+:[\\w]+\\([\\w\\W]*\\)")) {
            methodname = getMatchedPortion(expression, "[\\w]+:([\\w]+)(\\([\\w\\W]*\\))", 1);
        }
        return methodname;
    }

    private void getMethodArguments(List<Object> arguments, String expression, Context context) throws Exception {
        String parameters = "";
        if (expression.matches("([\\w]+)(\\([\\w\\W]*\\))")) {
            // 形式为：方法名(参数...)
            parameters = getMatchedPortion(expression, "([\\w]+)(\\([\\w\\W]*\\))", 2);
        } else if (expression.matches("([\\w]+\\.[\\w]+)(\\([\\w\\W]*\\))")) {
            // 形式为：对象.方法名(参数...)
            parameters = getMatchedPortion(expression, "([\\w]+\\.[\\w]+)(\\([\\w\\W]*\\))", 2);
        } else if (expression.matches("[\\w]+:[\\w]+\\([\\w\\W]*\\)")) {
            parameters = getMatchedPortion(expression, "[\\w]+:[\\w]+(\\([\\w\\W]*\\))", 1);
        }

        String str[] = analyseArgs(parameters.substring(1, parameters.length() - 1));
        for (int i = 0; i < str.length; i++) {
            if (str[i].trim().matches(STRINGREGEX))// 参数是字符串类型
            {
                arguments.add(getMatchedPortion(str[i].trim(), STRINGREGEX, 1));
            } else {
                handleExpressCore(arguments, str[i], context);
            }
        }
    }

    private void invokeMethod(List<Object> value, Object obj, String expression, Context context) throws Exception {
        List<Object> arguments = new LinkedList<Object>();
        getMethodArguments(arguments, expression, context);
        value.add(MethodUtils.invokeMethod(obj, getMethodName(expression), arguments.toArray()));
    }

    private void invokeStaticMethod(List<Object> value, Class<?> clazname, String expression, Context context) throws Exception {
        List<Object> arguments = new LinkedList<Object>();
        getMethodArguments(arguments, expression, context);
        value.add(MethodUtils.invokeStaticMethod(clazname, getMethodName(expression), arguments.toArray()));
    }

    private List<String> preprocessExpression(String expression) {
        List<String> exps = new ArrayList<String>();
        String joinStr = "";
        while (expression.indexOf("${") != -1) {
            joinStr = expression.substring(0, expression.indexOf("${"));
            if (joinStr.trim().length() > 0) {
                exps.add(joinStr);
            }
            joinStr = expression.substring(expression.indexOf("${"), expression.indexOf("}") + 1);
            if (joinStr.trim().length() > 0) {
                exps.add(joinStr);
            }
            expression = expression.substring(expression.indexOf("}") + 1);
            if (StringUtil.isNotEmpty(expression) && expression.indexOf("${") == -1)
                exps.add(expression);
        }
        if (exps.isEmpty())
            exps.add(expression);
        return exps;
    }

    public static String getMatchedPortion(String expression, String regex, int index) {
        Matcher m = Pattern.compile(regex).matcher(expression);
        if (m.find()) {
            try {
                return m.group(index);
            } catch (Exception e) {
                // 静默处理
                return expression;
            }
        }
        return expression;
    }

    private String[] analyseArgs(String str) {
        int len = str.length();
        boolean inStr = false;
        List<String> args = new ArrayList<String>();
        StringBuffer tmpArg = new StringBuffer();
        int depth = 0;
        for (int i = 0; i < len; i++) {
            char c = str.charAt(i);
            if (c == ' ') {
                if (inStr)
                    tmpArg.append(c);
            } else if (c == '\'') {
                inStr = !inStr;
                tmpArg.append(c);
            } else if (c == '(') {
                depth++;
                tmpArg.append(c);
            } else if (c == ')') {
                depth--;
                tmpArg.append(c);
            } else if (c == ',') {
                if (inStr || depth != 0)
                    tmpArg.append(c);
                else {
                    args.add(tmpArg.toString());
                    tmpArg = new StringBuffer();
                }
            } else
                tmpArg.append(c);
        }
        String lastStr = tmpArg.toString();
        if (!lastStr.equals(""))
            args.add(lastStr);
        return args.toArray(new String[args.size()]);
    }

    public Map<String, Object> getFuncs() {
        return funcs;
    }

    public Object evaluate(Context context, String MyExpression, boolean isException) {
        if (flag)
            return evaluateWithEl(context, MyExpression, isException);
        else
            return evaluateWithNonEl(context, MyExpression, isException);
    }
}
