package com.brilliance.eibs.extension.mina.client;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.threadpool.NamedThreadFactory;
import com.brilliance.eibs.extension.core.IClient;
import com.brilliance.eibs.extension.core.IMinaService;
import com.brilliance.eibs.server.IServerInstance;
import com.brilliance.eibs.server.socket.impl.BeProtocolCodecFactoryWrapper;
import org.apache.mina.core.filterchain.IoFilter;
import org.apache.mina.core.future.ConnectFuture;
import org.apache.mina.core.service.IoHandler;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.executor.ExecutorFilter;
import org.apache.mina.transport.socket.nio.NioSocketConnector;

import java.net.InetSocketAddress;
import java.util.Map;
import java.util.concurrent.Executors;

/**
 * @author hujun
 * 
 * 
 *         apache mina nio 实现的socket客户端 
 * 201908 weicong 修改
 */
public class MinaNioClient implements IClient, IMinaService {

	protected String hostname;
	protected int port;
	protected NioSocketConnector connector;
	protected ConnectFuture cf;
	private int timeout = 3000;
	Map<String, String> props;

	public MinaNioClient(String hostname, int port) {
		this.hostname = hostname;
		this.port = port;
	}

	public MinaNioClient(String hostname, int port, Map<String, String> props) {
		this.hostname = hostname;
		this.port = port;
		this.props=props;
	}


	public void init() {
		try {
			connector = new NioSocketConnector(IServerInstance.DEFAULT_IO_THREADS);
			connector.setConnectTimeoutMillis(timeout);
			connector.getSessionConfig().setTcpNoDelay(true);
			connector.getSessionConfig().setKeepAlive(true);
			addFilter(
					"codec",
					new ProtocolCodecFilter(new BeProtocolCodecFactoryWrapper(this.props)));
			addFilter(
					"threadPool",
					new ExecutorFilter(Executors
							.newCachedThreadPool(new NamedThreadFactory(
									"MinaClientWorker", true))));
		} catch (Exception e) {
			throw new InterfaceException("02301", e);
		}
	}

	public void connect() {
		checkNullPointer();
		IoHandler handler = connector.getHandler();
		if (null == handler)
			throw new NullPointerException("IoHandler not set.");
		// 建立连接
		cf = connector.connect(new InetSocketAddress(hostname, port));
		// 等待连接创建完成
		cf.awaitUninterruptibly(timeout);
	}

	public IoSession getSession() {
		return cf.getSession();
	}

	public void close() {
		// 等待连接断开
		if (null != cf && cf.getException()==null)
			cf.getSession().getCloseFuture().awaitUninterruptibly(timeout);
		// 释放连接
		if (null != connector)
			connector.dispose();

	}

	public void setTimeout(int timeout) {
		checkNullPointer();
		connector.setConnectTimeoutMillis(timeout);
	}

	public void addFilter(String name, IoFilter filter) {
		checkNullPointer();
		connector.getFilterChain().addLast(name, filter);
	}

	public void loadPloyHandler(IoHandler handler) {
		checkNullPointer();
		connector.setHandler(handler);
	}

	private void checkNullPointer() {
		if (null == connector)
			throw new NullPointerException(
					"NioSocketConnector not initialized.");
	}

}
