package com.brilliance.eibs.extension.mina.filter;

import org.apache.mina.core.filterchain.IoFilterAdapter;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.core.write.WriteRequest;

import java.net.InetSocketAddress;

public class WhitelistFilter extends IoFilterAdapter {

    private String[] whitelist;

    public WhitelistFilter(String[] whitelist) {
        this.whitelist = whitelist;
    }


    @Override
    public void messageReceived(NextFilter nextFilter, IoSession session, Object message)
            throws Exception {
        if (isTrust(session))
            nextFilter.messageReceived(session, message);
        else
            blockSession(session);
    }


    private boolean isTrust(IoSession session) {
        InetSocketAddress isa = (InetSocketAddress) session.getRemoteAddress();
        String hostname = isa.getHostName();
        if (null == whitelist || whitelist.length == 0) return true;
        for (String ip : whitelist)
            if (hostname.equals(ip)) return true;
        return false;
    }

    @Override
    public void messageSent(NextFilter nextFilter, IoSession session, WriteRequest writeRequest)
            throws Exception {
        if (isTrust(session))
            nextFilter.messageSent(session, writeRequest);
        else
            blockSession(session);
    }


    private void blockSession(IoSession session) {
        session.closeNow();
    }
}
