package com.brilliance.eibs.extension.mina.server;

import com.brilliance.eibs.extension.core.IMinaService;
import com.brilliance.eibs.extension.core.IServer;
import com.brilliance.eibs.extension.mina.filter.WhitelistFilter;
import org.apache.mina.core.filterchain.IoFilter;
import org.apache.mina.core.service.IoHandler;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.transport.socket.nio.NioSocketAcceptor;

import java.io.IOException;
import java.net.InetSocketAddress;

/**
 * @author hujun
 * 
 *         apache mima nio 实现的 服务端
 *201908 weicong 修改
 */
public class MinaNioServer implements IServer, IMinaService {

	private int backlog;
	private int processorCount;
	private int port;
	private NioSocketAcceptor acceptor;
	private int readBufferSize;
	private int idleTime;

	private MinaNioServer(Builder builder) {
		this.backlog = builder.backlog;
		this.processorCount = builder.processorCount;
		this.port = builder.port;
		this.readBufferSize = builder.readBufferSize;
		this.idleTime = builder.idleTime;
	}

	public static class Builder {
		private int backlog;
		private int processorCount;
		private int port;

		private int readBufferSize = 2048;
		private int idleTime = 60;

		public Builder(int processorCount, int port) {
			this.processorCount = processorCount;
			this.port = port;
		}

		public Builder setBacklog(int backlog) {
			this.backlog = backlog;
			return this;
		}

		public Builder setReadBufferSize(int readBufferSize) {
			this.readBufferSize = readBufferSize;
			return this;
		}

		public Builder setIdleTimeMillis(int idleTime) {
			this.idleTime = idleTime / 100;
			return this;
		}

		public MinaNioServer build() {
			return new MinaNioServer(this);
		}

	}

	public void init() {
		acceptor = new NioSocketAcceptor(processorCount);
		acceptor.getSessionConfig().setReadBufferSize(readBufferSize);
		if (0 != backlog)
			acceptor.setBacklog(backlog);
		if (0 != idleTime)
			acceptor.getSessionConfig().setIdleTime(IdleStatus.BOTH_IDLE, idleTime);
	}

	public void start() throws IOException {
		checkNullPointer();
		IoHandler handler = acceptor.getHandler();
		if (null == handler)
			throw new NullPointerException("IoHandler not set.");
		acceptor.bind(new InetSocketAddress(port));
	}

	public void close() {
		if (null != acceptor)
			acceptor.dispose();
	}

	public void setTimeout(int timeout) {
		checkNullPointer();
		acceptor.getSessionConfig().setIdleTime(IdleStatus.BOTH_IDLE, timeout);
	}

	public void trusts(String[] whitelist) {
		checkNullPointer();
		acceptor.getFilterChain().addLast("__whitelist", new WhitelistFilter(whitelist));
	}

	public void addFilter(String name, IoFilter filter) {
		checkNullPointer();
		acceptor.getFilterChain().addLast(name, filter);
	}

	public void loadPloyHandler(IoHandler handler) {
		checkNullPointer();
		acceptor.setHandler(handler);
	}

	private void checkNullPointer() {
		if (null == acceptor)
			throw new NullPointerException("NioSocketAcceptor not initialized.");
	}

}
