package com.brilliance.eibs.factory.parser;

import com.brilliance.eibs.factory.parser.impl.XmlParser;
import com.brilliance.eibs.util.PropertyUtil;
import org.slf4j.Logger;

/**
 * @author hujun </br>------------modified by weicong on 2016/3/22
 *         -----------</br> 解析工场类，用于提供解析实例
 */
public class ParserFactory {
	//
	// private final static Map<ParserName, ConfigParser> cacheMap = new
	// HashMap<ParserName, ConfigParser>();

	private static String parserType = "";
	static {
		registerParser(ParserName.XMLPARSER, new XmlParser());
	}

	public static void setParserType(String parserType) {
		ParserFactory.parserType = parserType;
	}

	public static enum ParserName {
		XMLPARSER("XMLPARSER"), DBPARSER("DBPARSER");
		String code;

		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return code;
		}

		private ParserName(String i) {
			this.code = i;
		}
	}

	/**
	 * 开发人员注册实现的接口配置解析器，目前包括xmlParser和DbParser
	 * 
	 * @param parserName
	 * @param configParser
	 */
	private static void registerParser(ParserName parserName, ConfigParser configParser) throws NullPointerException {
		/*
		 * if (configParser != null) { cacheMap.put(parserName, configParser); }
		 * else { throw new NullPointerException(); }
		 */
	}

	/**
	 * 获取实际Parser实例
	 * 
	 * @param logger
	 * @return
	 */
	public static ConfigParser getInstance(Logger logger) {
		return getInstance(ParserName.valueOf(getParserType()), logger);
	}

	public static ConfigParser getInstance(Logger logger, String type) {
		return getInstance(ParserName.valueOf(getParserType(type)), logger);
	}

	/**
	 * 根据解析器名称，获取指定ConfigParser实例
	 * 
	 * @param parsername
	 *            目前名称包括<code>ParserName.XmlParser</code>和
	 *            <code>ParserName.DbParser</code>
	 * @param logger
	 * @return
	 */
	public static ConfigParser getInstance(ParserName parsername, Logger logger) {
		// parsername为空时，默认使用xmlParser
		/*
		 * if (parsername == null) { XmlParser xmlParser = (XmlParser)
		 * cacheMap.get(ParserName.XMLPARSER); xmlParser.setLogger(logger);
		 * return xmlParser; } else { ConfigParser configParser =
		 * cacheMap.get(parsername); if (configParser != null)
		 * configParser.setLogger(logger); return configParser; }
		 */
		XmlParser xmlParser = new XmlParser();
		xmlParser.setLogger(logger);
		return xmlParser;
	}

	/**
	 * 通过sys.properties的PARSERTYPE判断是否是数据库解析器(db)还是xml解析器(xml)
	 */
	public static String getParserType() {
		if (parserType == "")
			parserType = PropertyUtil.getProperty("sys.properties", "PARSERTYPE");
		return parserType.toUpperCase() + "PARSER";
	}

	public static String getParserType(String type) {
		return type.toUpperCase() + "PARSER";
	}

}
