package com.brilliance.eibs.factory.parser.impl;

import com.brilliance.eibs.coder.impl.*;
import com.brilliance.eibs.core.exception.*;
import org.apache.commons.codec.binary.Base64;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.InvalidParameterSpecException;
import java.util.Map;

/**
 * @author hujun
 * 
 *         用于动态密码的加密解密工作，支持多种加密算法， 但是由于存在单机
 *
 */
public class XPwdCoder {

	private static final String COMPANY_HOME_PAGE = "www.brilliance.com.cn";

	private String keyword = COMPANY_HOME_PAGE;
	private String salt = "qwerasdf";

	public static final String BASE64_ALGORITHM = "BASE64";
	public static final String AES_ALGORITHM = "AES";
	public static final String DES_ALGORITHM = "DES";
	public static final String DESEDE_ALGORITHM = "DESede";
	public static final String IDEA_ALGORITHM = "IDEA";
	public static final String PBE_ALGORITHM = "PBE";
	public static final String RSA_ALGORITHM = "RSA";

	public String encrypt(String s, String algorithm) {
		try {
			if (BASE64_ALGORITHM.equals(algorithm))
				return encodeBase64(s);
			if (AES_ALGORITHM.equals(algorithm))
				return encryptAES(s);
			if (DES_ALGORITHM.equals(algorithm))
				return encryptDES(s);
			if (DESEDE_ALGORITHM.equals(algorithm))
				return encryptDESede(s);
			if (IDEA_ALGORITHM.equals(algorithm))
				return encryptIDEA(s);
			if (PBE_ALGORITHM.equals(algorithm))
				return encryptPBE(s);
			if (RSA_ALGORITHM.equals(algorithm))
				return encryptRSA(s);
			throw new XPwdCoderUnsupportedAlgorithmException(algorithm);
		} catch (Exception e) {
			throw new XPwdCoderEncryptException(e);
		}
	}
	
	public String decrypt(String s, String algorithm) {
		try {
			if (BASE64_ALGORITHM.equals(algorithm))
				return decodeBase64(s);
			byte[] b = Base64.decodeBase64(s);
			if (AES_ALGORITHM.equals(algorithm))
				return decryptAES(b);
			if (DES_ALGORITHM.equals(algorithm))
				return decryptDES(b);
			if (DESEDE_ALGORITHM.equals(algorithm))
				return decryptDESede(b);
			if (IDEA_ALGORITHM.equals(algorithm))
				return decryptIDEA(b);
			if (PBE_ALGORITHM.equals(algorithm))
				return decryptPBE(b);
			if (RSA_ALGORITHM.equals(algorithm))
				return decryptRSA(b);
			throw new XPwdCoderUnsupportedAlgorithmException(algorithm);
		} catch (Exception e) {
			throw new XPwdCoderDecryptException(e);
		}
	}

	private String encodeBase64(String s) throws UnsupportedEncodingException {
		return Base64Coder.encode(s);
	}

	private String decodeBase64(String s) throws UnsupportedEncodingException {
		return Base64Coder.decode(s);
	}

	private String encryptAES(String s) throws UnsupportedEncodingException, InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException,
			NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException {
		return Base64.encodeBase64URLSafeString(AESCoder.encrypt((null == s ? "" : s).getBytes(), AESCoder.generateKey(keyword)));
	}

	private String decryptAES(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return new String(AESCoder.decrypt(data, AESCoder.generateKey(keyword)));
	}

	public String encryptDES(String s) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return Base64.encodeBase64URLSafeString(DESCoder.encrypt((null == s ? "" : s).getBytes(), DESCoder.generateKey(keyword)));
	}

	private String decryptDES(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return new String(DESCoder.decrypt(data, DESCoder.generateKey(keyword)));
	}

	private String encryptDESede(String s) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return Base64.encodeBase64URLSafeString(DESedeCoder.encrypt((null == s ? "" : s).getBytes(), DESedeCoder.generateKey(keyword)));
	}

	private String decryptDESede(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return new String(DESedeCoder.decrypt(data, DESedeCoder.generateKey(keyword)));
	}

	private String encryptIDEA(String s) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return Base64.encodeBase64URLSafeString(IDEACoder.encrypt((null == s ? "" : s).getBytes(), IDEACoder.generateKey(keyword)));
	}

	private String decryptIDEA(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		return new String(IDEACoder.decrypt(data, IDEACoder.generateKey(keyword)));
	}

	@Deprecated
	private String encryptELGamal(String s) throws NoSuchAlgorithmException, InvalidParameterSpecException, InvalidAlgorithmParameterException,
			InvalidKeyException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException {
		Map<String, Object> keyMap = ELGamalCoder.generateKey(keyword);
		return Base64.encodeBase64URLSafeString(ELGamalCoder.encryptByPublicKey((null == s ? "" : s).getBytes(), ELGamalCoder.getPublicKey(keyMap)));
	}

	@Deprecated
	private String decryptELGamal(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException, InvalidParameterSpecException, InvalidAlgorithmParameterException {
		Map<String, Object> keyMap = ELGamalCoder.generateKey(keyword);
		return new String(ELGamalCoder.decryptByPrivateKey(data, ELGamalCoder.getPrivateKey(keyMap)));
	}

	private String encryptPBE(String s) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException {
		return Base64.encodeBase64URLSafeString(PBECoder.encrypt((null == s ? "" : s).getBytes(), keyword, salt.getBytes()));
	}

	private String decryptPBE(byte[] data) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException {
		return new String(PBECoder.decrypt(data, keyword, salt.getBytes()));
	}

	private String encryptRSA(String s) throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		Map<String, Object> keyMap = RSACoder.generateKey(keyword);
		return Base64.encodeBase64URLSafeString(RSACoder.encryptByPublicKey((null == s ? "" : s).getBytes(), RSACoder.getPublicKey(keyMap)));
	}

	private String decryptRSA(byte[] data) throws NoSuchAlgorithmException, InvalidKeyException, InvalidKeySpecException, NoSuchPaddingException,
			IllegalBlockSizeException, BadPaddingException {
		Map<String, Object> keyMap = RSACoder.generateKey(keyword);
		return new String(RSACoder.decryptByPrivateKey(data, RSACoder.getPrivateKey(keyMap)));
	}

	public String getKeyword() {
		return keyword;
	}

	public void setKeyword(String keyword) {
		if (null == keyword)
			throw new XPwdCoderKeywordNullException();
		this.keyword = keyword;
	}

	public String getSalt() {
		return salt;
	}

	public void setSalt(String salt) {
		if (null == salt)
			throw new XPwdCoderSaltNullException();
		if (salt.getBytes().length != 8)
			throw new XPwdCoderSaltLengthException();
		this.salt = salt;
	}
}
