package com.brilliance.eibs.factory.parser.impl;

import com.brilliance.eibs.core.exception.ErrorCodes;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.exception.SysCode;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.core.model.impl.*;
import com.brilliance.eibs.core.model.impl.ConnectionDef.PoolSetting;
import com.brilliance.eibs.core.model.impl.ProcessDef.TaskDef;
import com.brilliance.eibs.core.model.impl.SwitchDef.CaseDef;
import com.brilliance.eibs.core.model.modelbeans.CommonsDef;
import com.brilliance.eibs.core.model.modelbeans.ConnectionsDef;
import com.brilliance.eibs.core.model.modelbeans.FiltersDef;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.BeanFactory.FunctionKey;
import com.brilliance.eibs.factory.parser.ConfigParser;
import com.brilliance.eibs.factory.parser.IHotDeployParser;
import com.brilliance.eibs.listener.HotDeployListener;
import com.brilliance.eibs.listener.helpers.Watchdog;
import com.brilliance.eibs.server.manager.BDHotDeployListener;
import com.brilliance.eibs.util.*;
import org.dom4j.*;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 配置解析器
 * 
 * @author xiaoyuanzhen
 * 
 */
@SuppressWarnings("unchecked")
public class XmlParser implements ConfigParser, IHotDeployParser {

    private static Map<String, Object> localRefFiles = new ConcurrentHashMap<String, Object>();

    private static final String XML_REF_FILE_PREFIX = "XML";
    private static final String PROP_REF_FILE_PREFIX = "PROP";

    protected Logger logger;
    private CommonsDef mainCommonsDef;
    public static final String MAIN_PATH = "cfg/main";
    public static final String COMMON_PATH = "cfg/etc/commons.xml";
    public static final String IMPORT_PATH = "cfg/imports";
    private Map<String, String> initMap = new HashMap<String, String>();
    private ErrorCodes errorCodes = new ErrorCodes();
    private Map<String, Map<String, String>> codetableMap =
            new HashMap<String, Map<String, String>>();
    private Map<String, String> dynamicCodetableMap = new HashMap<String, String>();
    private Map<FunctionKey, IFunctionDef> functionMap = new HashMap<FunctionKey, IFunctionDef>();
    private Map<FunctionKey, IFuncDef> funcMap = new HashMap<FunctionKey, IFuncDef>();
    private Map<String, IModuleDef> moduleMap = new HashMap<String, IModuleDef>();
    private Map<String, ICounterDef> counterMap = new HashMap<String, ICounterDef>();

    public XmlParser() {}

    public XmlParser(Logger logger) {
        this.logger = logger;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public Logger getLogger() {
        return logger;
    }

    public Map<String, String> getInitMap() {
        return initMap;
    }

    /**
     * @param filename 获取主接口的相对路径
     **/
    public static String getInterfaceRelativePath(String filename) {
        if (!filename.endsWith(".xml")) {
            filename += ".xml";
        }
        return MAIN_PATH + "/" + filename;
        /*
         * Collection<File> files = getAllFile(MAIN_PATH); Iterator<File> it = files.iterator();
         * while (it.hasNext()) {
         * 
         * File f = it.next(); String tmpfileName = f.getName(); if (tmpfileName.equals(filename)) {
         * String path = f.getPath(); int index = path.indexOf(MAIN_PATH); String relativePath =
         * null; if (index == -1) { index = path.indexOf("cfg\\main"); } relativePath =
         * path.substring(index); relativePath = relativePath.replace("\\", "/"); return
         * relativePath; } } return null;
         */
    }

    /**
     * 清除节点的所有子节点
     * 
     * @param parentNode
     */
    private void clearSubNodes(Element parentNode) {
        List<Element> subNodes = parentNode.elements();
        for (Element subNode : subNodes)
            parentNode.remove(subNode);
    }

    /**
     * 级联搜索import节点，进行处理
     * 
     * @param parentNode
     */
    private void doImport(Element parentNode) {
        List<Element> subNodes = parentNode.elements();
        if (subNodes.size() == 0) return;
        clearSubNodes(parentNode);
        for (Element subNode : subNodes) {
            if (subNode.getName().equals("import")) {
                String file = getAttribute(subNode, "file");
                String ref = getAttribute(subNode, "ref");
                List<Element> refNodes = getImportNodes(file, ref);
                if (subNode.elements().size() > 0) {
                    combineConfig(subNode.elements(), refNodes);
                }
                for (Element refNode : refNodes) {
                    parentNode.add(refNode.detach());
                }
            } else {
                parentNode.add(subNode);
                doImport(subNode);
            }
        }
    }

    /**
     * 合并主配置和import配置的相同配置项信息，以节点名称和tag配置为判断标准
     * 
     * @param srcNodes
     * @param desNodes
     */
    private void combineConfig(List<Element> srcNodes, List<Element> desNodes) {
        for (Element srcNode : srcNodes) {
            String tag = getAttribute(srcNode, "tag");
            for (Element desNode : desNodes) {
                if (srcNode.getName().equals(desNode.getName())
                        && getAttribute(desNode, "tag").equals(tag)) {
                    for (int i = 0; i < srcNode.attributeCount(); i++) {
                        Attribute attrNode = srcNode.attribute(i);
                        desNode.remove(attrNode);
                        desNode.add(attrNode.detach());
                    }
                    List<Element> srcChildren = srcNode.elements();
                    clearSubNodes(desNode);
                    for (Element child : srcChildren) {
                        desNode.add(child.detach());
                    }
                }
            }
        }
    }

    /**
     * import节点处理
     * 
     * @param file
     * @param ref
     * @return
     */
    private List<Element> getImportNodes(String file, String ref) {
        Document doc = XmlUtil.getDocument(file);
        if (StringUtil.isEmpty("ref")) return doc.getRootElement().elements();
        List<Element> refNodes = doc.getRootElement().elements("import");
        if (refNodes.size() > 0) {
            for (Element refNode : refNodes) {
                if (getAttribute(refNode, "id").equals(ref)) {
                    doImport(refNode);
                    return refNode.elements();
                }
            }
        }
        throw new InterfaceException("00021",
                "can not find import node in [" + file + "] with attribute [id] value is " + ref);
    }

    /**
     * 创建单独的不依赖其他文件的接口文件
     * 
     * @param interfaceid
     */
    public Document parseIndividualInterfaceDoc(Document doc) {
        Element root = doc.getRootElement();
        Element interfaceNode = root.element("interface");
        List<Element> importCommonsNodes = interfaceNode.selectNodes("./import");
        int size = importCommonsNodes.size();
        if (size > 0) {// 替换import的commons
            for (int i = 0; i < size; i++) {
                Element importNode = importCommonsNodes.get(i);
                interfaceNode.remove(importNode);
                String url = getRequiredAttribute(importNode, "file");
                String ref = getAttribute(importNode, "ref");
                Document importDoc = XmlUtil.getDocument(url);
                if (!StringUtil.isEmpty(ref)) {
                    String type = getRequiredAttribute(importNode, "type");
                    String path = "/root/" + type + "s/" + type + "/@id=" + ref;
                    Node node = importDoc.selectSingleNode(path);
                    if (node != null) {
                        Element commonsNode = interfaceNode.element(type + "s");
                        if (commonsNode == null) commonsNode = interfaceNode.addElement(type + "s");
                        commonsNode.add(node.getParent().detach());
                    } else {
                        throw new InterfaceException("00004",
                                "can not find import refrence [" + ref + "] in [" + url + "]");
                    }
                }
            }
        }
        // TODO 替换其他所有的import
        List<Element> importNodes = doc.selectNodes("//import");
        if (importNodes.size() > 0) {

        }
        return doc;
    }

    /**
     * 组装接口定义
     * 
     * @param interfaceid
     * @return
     */
    public IInterfaceDef getInterfaceDef(String interfaceid) {
        // interfaceId = interfaceid;
        Element rootNode = getInterfaceParentNode(interfaceid);
        List<Element> interfaces = rootNode.elements("interface");
        int index = interfaceid.lastIndexOf("/");
        String id = interfaceid;
        if (index != -1) id = interfaceid.substring(index + 1);
        for (Element interfaceNode : interfaces) {
            if (getAttribute(interfaceNode, "id").equals(id))
            // if (getAttribute(interfaceNode, "id").equals(interfaceid))
            {
                IInterfaceDef interfaceDef = getInterfaceDef(interfaceNode);
                // 初始化接口其他信息
                initInterfaceDef(interfaceDef, rootNode);
                return interfaceDef;
            }
        }
        throw new InterfaceException("00002", "interface [" + interfaceid + "] not exists");
    }

    private void initInterfaceDef(IInterfaceDef interfaceDef, Element rootNode) {
        // 初始化错误码
        Element errorCodesNode = rootNode.element("errorCodes");
        errorCodes.addSysCodes(getCommonSysCodes(interfaceDef.getId()));
        if (errorCodesNode != null) errorCodes.addSysCodes(getSysCodes(errorCodesNode));
        interfaceDef.setErrorCodes(errorCodes);
        // 初始化码表
        List<Element> codeTablesNodes = rootNode.elements("codetables");
        codetableMap.putAll(getCodeTables(codeTablesNodes));
        interfaceDef.setCodetableMap(codetableMap);
        interfaceDef.setDynamicCodetableMap(dynamicCodetableMap);
        // 初始化自定义函数
        List<Element> functionsNodes = rootNode.elements("functions");
        FunctionParser functionParser = new FunctionParser(this);
        functionMap.putAll(functionParser.getFunctionMap(functionsNodes));
        interfaceDef.setFunctionMap(functionMap);

        funcMap.putAll(functionParser.getFuncMap(functionsNodes));
        interfaceDef.setFuncMap(funcMap);

        // 初始化模型
        List<Element> modulesNodes = rootNode.elements("modules");
        moduleMap.putAll(getModules(modulesNodes));
        interfaceDef.setModuleMap(moduleMap);
        // 初始化计数器
        interfaceDef.setCountermap(counterMap);
    }

    public void prepareforInterface(Element rootNode) {
        Element publicNode = rootNode.element("public");
        if (publicNode != null) initMap.putAll(initEnvironment(publicNode));
        initMainCommonsDef(rootNode);
    }

    /**
     * 获取interface母节点，如果是接口主配置文件则为root节点，如果是services.xml文件，则为interfaces节点
     * 
     * @param interfaceId
     * @return
     */
    private Element getInterfaceParentNode(String interfaceId) {
        // URL url = getUrl(getPath(MAIN_PATH, interfaceId));
        URL url = XmlUtil.getUrl(getInterfaceRelativePath(interfaceId));
        Element rootNode = null;
        if (url != null) {
            Document doc = XmlUtil.getDocument(url);
            rootNode = doc.getRootElement();
            initMap.putAll(initEnvironment(doc));
            initMainCommonsDef(rootNode);
        } else
            throw new InterfaceException("00001", "[" + interfaceId + "] not exists.");
        return rootNode;
    }

    /**
     * @author hujun
     * @add added by hujun on 2014-6-17
     */
    /*
     * static class WriteXMLFlag { private boolean overwrite = false;
     * 
     * private void setOverwrite(boolean overwrite) { this.overwrite = this.overwrite || overwrite;
     * }
     * 
     * private void writeXML(URL path, Object doc) { XMLWriter writer = null; FileOutputStream fos =
     * null; File f = FileUtils.toFile(path); long l = f.lastModified(); try { // OutputFormat
     * xmlFormat = OutputFormat.createPrettyPrint(); // xmlFormat.setEncoding("utf-8"); fos = new
     * FileOutputStream(f); writer = new XMLWriter(fos); if (doc instanceof Document)
     * writer.write(doc); else if (doc instanceof Element) writer.write(((Element)
     * doc).getDocument()); else throw new
     * RuntimeException("dynamic encrypt, wrong parameter type."); } catch (IOException e) { throw
     * new RuntimeException("dynamic encrypt, rewriting file failed.", e); } finally { try { if
     * (writer != null) writer.close(); if (fos != null) fos.close(); } catch (IOException e) {
     * throw new RuntimeException("dynamic encrypt, rewriting file failed.", e); } }
     * f.setLastModified(l); } }
     */

    /**
     * 组装接口定义，只组装序列化相关信息，无交易信息
     * 
     * @param interfaceid
     * @return
     */
    public IInterfaceDef getOnlyInterfaceDef(String interfaceid) {
        Element rootNode = getInterfaceParentNode(interfaceid);
        List<Element> interfaceNodes = rootNode.elements("interface");
        for (Element interfaceNode : interfaceNodes) {
            if (interfaceNode.attributeValue("id").equals(interfaceid)) {
                loadCommonsFile(COMMON_PATH);
                return getOnlyInterfaceDef(interfaceNode);
            }
        }
        throw new InterfaceException("00002", "interface [" + interfaceid + "] not exists");
    }

    /**
     * 获取定时任务Task信息
     * 
     * @param filename
     * @return
     */
    @Override
    public List<ITaskDef> getTaskDefs(String filename) {
        Document doc = XmlUtil.getDocument(Constants.ETC_PATH, filename);
        List<ITaskDef> taskDefs = getTaskDefs(doc);
        if (Serializable2FileUtil.hotDeploy) Serializable2FileUtil.writeObject(taskDefs, "tasks");
        return taskDefs;
    }

    public List<ITaskDef> getTaskDefs(Document doc) {
        List<ITaskDef> list = new ArrayList<ITaskDef>();
        Element rootNode = doc.getRootElement();
        Element tasksNode = rootNode.element("tasks");
        if (tasksNode == null) return list;
        String state = getAttribute(tasksNode, "state");
        List<Element> taskNodes = tasksNode.elements("task");
        for (Element taskNode : taskNodes) {
            ITaskDef taskDef = new com.brilliance.eibs.core.model.impl.TaskDef();
            taskDef.setId(getAttribute(taskNode, "id"));
            taskDef.setDescription(getAttribute(taskNode, "description"));
            String taskState = getAttribute(taskNode, "state");
            String otherprocess = getAttribute(taskNode, "otherprocess");
			String depencency = getAttribute(taskNode, "depencency");
			String date = getAttribute(taskNode, "date");
			String continueExcute = getAttribute(taskNode, "continueExcute");
			taskDef.setContinueExcute(StringUtil.isEmpty(continueExcute) ? "true" : continueExcute);
			taskDef.setDate(StringUtil.isEmpty(date) ? "0" : date);
			taskDef.setDepencency(depencency);
            taskDef.setOtherprocess(otherprocess);
            if (state.equals("off") || taskState.equals("off"))
                taskDef.setState("off");
            else
                taskDef.setState("on");
            List<Element> propNodes = taskNode.elements("property");
            for (Element propNode : propNodes) {
                IPropertyDef propertyDef = getPropertyDef(propNode);
                taskDef.addProperty(propertyDef.getName(), propertyDef);
            }
            list.add(taskDef);
        }
        return list;
    }

    /**
     * 获取指定交易定义
     * 
     * @param interfaceid
     * @param transId
     * @return
     */
    @Override
    public ITransactionDef getTransactionDef(String interfaceid, String transid) {
        // interfaceId = interfaceid;
        IInterfaceDef interfaceDef = getInterfaceDef(interfaceid);
        return getTransactionDef(interfaceDef, transid);
    }

    /**
     * 从接口interfaceDef定义的所有交易中，获取与transid指定的id相同的交易
     * 
     * @param interfaceDef
     * @param transid
     * @return
     */
    @Override
    public ITransactionDef getTransactionDef(IInterfaceDef interfaceDef, String transid) {
        List<ITransactionDef> transactionDefs = interfaceDef.getTransactionDefs();
        for (ITransactionDef transactionDef : transactionDefs) {
            if (transactionDef.getId().equals(transid)) return transactionDef;
        }
        throw new InterfaceException("00003",
                "can not find transaction " + transid + " in interface " + interfaceDef.getId());
    }

    /**
     * 获取主配置中的connections、filters配置
     * 
     * @param rootNode
     * @return
     * 
     * @modify modified by hujun on 2014-6-17
     */
    private void initMainCommonsDef(Element rootNode) {
        loadCommonsFile(COMMON_PATH);
        WriteXMLFlag wxf = new WriteXMLFlag();
        if (rootNode.elements("connections").size() > 0) {
            Element connectionsNode = rootNode.element("connections");
            ConnectionsDef connectionsDef = getConnectionsDef(connectionsNode, wxf);
            List<IConnectionDef> connectionList = (List<IConnectionDef>) combineList(
                    mainCommonsDef.getConnectionsDef().getConnections(),
                    connectionsDef.getConnections());
            mainCommonsDef.getConnectionsDef().setConnections(connectionList);
        }
        if (rootNode.elements("filters").size() > 0) {
            Element filtersNode = rootNode.element("filters");
            FiltersDef filtersDef = getFiltersDef(filtersNode);
            List<IFilterDef> filterList =
                    (List<IFilterDef>) combineList(mainCommonsDef.getFiltersDef().getFilters(),
                            filtersDef.getFilters());
            mainCommonsDef.getFiltersDef().setFilters(filterList);
        }
        Element interf = rootNode.element("interface");
        if (interf != null) {
            if (wxf.overwrite)
                // wxf.writeXML(getUrl(getPath(MAIN_PATH, getAttribute(interf,
                // "id"))), rootNode);
                wxf.writeXML(XmlUtil.getUrl(getInterfaceRelativePath(getAttribute(interf, "id"))),
                        rootNode);
        }
    }

    /**
     * 接口级别的import
     * 
     * @param rootNode
     */
    // private void getImportFiles(Element rootNode)
    // {
    // if (rootNode.elements("import").size() > 0)
    // {
    // List<Element> importNodes = rootNode.elements("import");
    // for (Element importNode : importNodes)
    // {
    // String url = getAttribute(importNode, "file");
    // String ref = getAttribute(importNode, "ref");
    // String type = getAttribute(importNode, "type");
    // CommonsDef commonsDef = getCommonsDef(url);
    // if (StringUtil.isEmpty(ref))
    // {
    // List<IConnectionDef> connectionList = (List<IConnectionDef>)
    // combineList(mainCommonsDef.getConnectionsDef().getConnections(),
    // commonsDef.getConnectionsDef().getConnections());
    // mainCommonsDef.getConnectionsDef().setConnections(connectionList);
    // List<IFilterDef> filterList = (List<IFilterDef>)
    // combineList(mainCommonsDef.getFiltersDef().getFilters(),
    // commonsDef.getFiltersDef().getFilters());
    // mainCommonsDef.getFiltersDef().setFilters(filterList);
    // }
    // else
    // {
    // if (type.equals("connection"))
    // {
    // IConnectionDef connectionDef = getCommonsConnectionDef(commonsDef, ref);
    // if (connectionDef != null)
    // {
    // mainCommonsDef.getConnectionsDef().getConnections().add(connectionDef);
    // }
    // }
    // else if (type.equals("filter"))
    // {
    // IFilterDef filterDef = getCommonsFilterDef(commonsDef, ref);
    // if (filterDef != null)
    // {
    // mainCommonsDef.getFiltersDef().getFilters().add(filterDef);
    // }
    // }
    // }
    // }
    // }
    // }

    /**
     * 加载commons.xml的内容
     * 
     * @modify modified by hujun on 2014-6-17
     */
    private void loadCommonsFile(String path) {
        URL commonURL = XmlUtil.getUrl(path);
        if (commonURL != null) {
            Document doc = XmlUtil.getDocument(commonURL);
            // commons全局变量
            initMap.putAll(initEnvironment(doc));
            // modify modified by hujun on 2014-6-17
            WriteXMLFlag wxf = new WriteXMLFlag();
            mainCommonsDef = getCommonsDef(doc, wxf);
            Element rootNode = doc.getRootElement();
            List<Element> functionsNodes = rootNode.elements("functions");
            FunctionParser functionParser = new FunctionParser(this);
            functionMap.putAll(functionParser.getFunctionMap(functionsNodes));

            funcMap.putAll(functionParser.getFuncMap(functionsNodes));

            List<Element> codetablesNodes = rootNode.elements("codetables");
            codetableMap.putAll(getCodeTables(codetablesNodes));
            List<Element> modulesNodes = rootNode.elements("modules");
            moduleMap.putAll(getModules(modulesNodes));
            // 加载计数器
            List<Element> counters = rootNode.elements("counters");
            counterMap.putAll(getCounters(counters));
            // modify modified by hujun on 2014-6-17
            if (wxf.overwrite) wxf.writeXML(commonURL, doc);
        }
    }

    /**
     * 只组装接口相关信息
     * 
     * @param interfaceNode
     * @return
     */
    private IInterfaceDef getOnlyInterfaceDef(Element interfaceNode) {
        IInterfaceDef interfaceDef = new InterfaceDef();
        String id = getRequiredAttribute(interfaceNode, "id");
        interfaceDef.setId(id);
        // /
        // interfaceDef.setSingleton(Boolean.valueOf(interfaceNode.attributeValue("singleton")));
        interfaceDef.setBasePackage(getAttribute(interfaceNode, "basePackage"));
        interfaceDef.setDescription(interfaceNode.attributeValue("description"));
        String version = interfaceNode.attributeValue("version");
        String defaultScope = interfaceNode.attributeValue("defaultScope");
        String logLevel = interfaceNode.attributeValue("logLevel");
        String st = interfaceNode.attributeValue("singleton");
        boolean singleton = false;
        if (!StringUtil.isEmpty(st) && Boolean.valueOf(st)) {
            singleton = true;
        }
        if (StringUtil.isEmpty(defaultScope)) {
            defaultScope = "transition";
        }
        if (StringUtil.isEmpty(version)) {
            version = "1.0";
        }
        if (StringUtil.isEmpty(logLevel)) {
            logLevel = "debug";
        }
        interfaceDef.setSingleton(singleton);
        interfaceDef.setVersion(version);
        interfaceDef.setDefaultScope(defaultScope);
        interfaceDef.setLogLevel(logLevel);
        interfaceDef.setEncoding(interfaceNode.attributeValue("encoding"));
        interfaceDef.setLanguage(interfaceNode.attributeValue("language"));
        String type = interfaceNode.attributeValue("type");
        if (StringUtil.isEmpty(type)) type = "server";
        interfaceDef.setType(type);
        return interfaceDef;
    }

    // ThreadLocal<IInterfaceDef> interfaceDefth=new
    // ThreadLocal<IInterfaceDef>();
    /**
     * 组装接口定义
     * 
     * @param interfaceNode
     * @return
     */
    public IInterfaceDef getInterfaceDef(Element interfaceNode) {
        IInterfaceDef interfaceDef = getOnlyInterfaceDef(interfaceNode);
        // interfaceDefth.remove();
        // interfaceDefth.set(interfaceDef);
        interfaceDef.setTransactionDefs(getAllTransactions(interfaceDef, interfaceNode));
        return interfaceDef;
    }

    /**
     * 组装接口下所有交易
     * 
     * @param interfaceNode
     * @return
     */
    private List<ITransactionDef> getAllTransactions(IInterfaceDef interfaceDef,
            Element interfaceNode) {
        List<Element> transactions = interfaceNode.elements("transaction");
        List<ITransactionDef> list = new ArrayList<ITransactionDef>();
        for (Element transactionNode : transactions) {
            list.add(getTransactionDef(interfaceDef, transactionNode));
        }
        return list;
    }

    /**
     * 初始化交易定义
     * 
     * @param transactionNode
     * @return
     */
    private ITransactionDef getTransactionDef(IInterfaceDef interfaceDef, Element transactionNode) {
        doImport(transactionNode);
        checkInvalidNodes(transactionNode, new String[] {"process", "step"});
        ITransactionDef transactionDef = new TransactionDef();
        transactionDef.setInterfaceDef(interfaceDef);
        transactionDef.setId(getAttribute(transactionNode, "id"));
        transactionDef.setVersion(getAttribute(transactionNode, "version"));
        transactionDef.setLogLevel(getAttribute(transactionNode, "loglevel"));
        transactionDef.setDescription(getAttribute(transactionNode, "description"));
        transactionDef.setExpression(getAttribute(transactionNode, "expression"));
        transactionDef.setDetail(getAttribute(transactionNode, "detail"));
        String timeout = getAttribute(transactionNode, "timeout");
        if (!StringUtil.isEmpty(timeout)) transactionDef.setTimeout(Long.valueOf(timeout));
        String st = transactionNode.attributeValue("singleton");
        boolean singleton = false;
        if (!StringUtil.isEmpty(st) && Boolean.valueOf(st)) {
            singleton = true;
        }
        String rd = getAttribute(transactionNode, "reload");
        boolean reload = false;
        if (!StringUtil.isEmpty(rd) && Boolean.valueOf(rd)) {
            reload = true;
        }
        transactionDef.setReload(reload);
        transactionDef.setSingleton(singleton);
        transactionDef.setProcessDef(getProcessDef(transactionNode));
        transactionDef.setSteps(getAllSteps(transactionDef, transactionNode));
        return transactionDef;
    }

    /**
     * 组装process定义
     * 
     * @param transactionNode
     * @return
     */
    private IProcessDef getProcessDef(Element transactionNode) {
        Element processNode = transactionNode.element("process");
        if (processNode == null) return null;
        IProcessDef processDef = new ProcessDef();
        processDef.setId(getAttribute(processNode, "id"));
        Element exceptionNode = processNode.element("exception");
        if (exceptionNode != null) {
            processDef.setExceptionTaskName(getAttribute(exceptionNode, "id"));
            processNode.remove(exceptionNode);
        }
        processDef.setSubList(getProcessSubList(processNode));
        return processDef;
    }

    private List<IAbsDef> getProcessSubList(Element processNode) {
        List<IAbsDef> list = new ArrayList<IAbsDef>();
        List<Element> subNodes = processNode.elements();
        for (int i = 0; i < subNodes.size(); i++) {
            Element subNode = subNodes.get(i);
            String nodeName = subNode.getName();
            if (nodeName.equals("step")) {
                TaskDef taskDef = new ProcessDef().new TaskDef();
                taskDef.setName(getAttribute(subNode, "id"));
                taskDef.setValue(getAttribute(subNode, "nextstep"));
                list.add(taskDef);
            }
            // else if (nodeName.equals("foreach"))
            // {
            // list.add(getProcessForeachDef(subNode));
            // }
            else if (nodeName.equals("if")) {
                IfDef ifDef = new IfDef();
                String condition = getAttribute(subNode, "condition");
                ifDef.addConditionMap(condition, getProcessSubList(subNode));
                while (++i < subNodes.size()) {
                    Element conditionNode = subNodes.get(i);
                    condition = getAttribute(conditionNode, "condition");
                    if (conditionNode.getName().equals("elseif")) {
                        if (!ifDef.getConditionMap().containsKey(condition))
                            ifDef.addConditionMap(condition, getProcessSubList(conditionNode));
                    } else if (conditionNode.getName().equals("else")) {
                        ifDef.setElseCondition(getProcessSubList(conditionNode));
                    } else {
                        i--;
                        break;
                    }
                }
                list.add(ifDef);
            }
        }
        return list;
    }

    /**
     * 组装交易下所有步骤
     * 
     * @param transactionDef
     * @param transactionNode
     * @return
     */
    private List<IStepDef> getAllSteps(ITransactionDef transactionDef, Element transactionNode) {
        List<IStepDef> list = new ArrayList<IStepDef>();
        List<Element> steps = transactionNode.elements();
        for (Element stepNode : steps) {
            if (stepNode.getName().equals("step")) {
                list.add(getStepDef(transactionDef, stepNode));
            }
        }
        return list;
    }

    /**
     * 组装步骤定义
     * 
     * @param transactionDef
     * @param stepNode
     * @return
     */
    private IStepDef getStepDef(ITransactionDef transactionDef, Element stepNode) {
        IStepDef stepDef = new StepDef();
        stepDef.setId(getAttribute(stepNode, "id"));
        stepDef.setLevel(getAttribute(stepNode, "level"));
        stepDef.setTransaction(transactionDef);
        stepDef.setExecuteList(getExecuteList(stepDef, stepNode));
        return stepDef;
    }

    /**
     * 组装step下可执行的connection、filter
     * 
     * @param stepNode
     * @return
     */
    private List<IAbsDef> getExecuteList(IAbsDef parentDef, Element stepNode) {
        List<IAbsDef> executeList = new ArrayList<IAbsDef>();
        List<Element> list = stepNode.elements();
        for (int i = 0; i < list.size(); i++) {
            Element node = list.get(i);
            if (node.getName().equals("foreach")) {
                executeList.add(getStepForeachDef(parentDef, node));
            } else if (node.getName().equals("while")) {
                executeList.add(getStepWhileDef(parentDef, node));
            } else if (node.getName().equals("break")) {
                if (nodeInLoop(parentDef)) {
                    BreakDef breakDef = new BreakDef();
                    breakDef.setParentDef(parentDef);
                    executeList.add(breakDef);
                } else
                    throw new InterfaceException("00023",
                            "unexpected [break] node under [" + node.getName() + "].");
            } else if (node.getName().equals("continue")) {
                if (nodeInLoop(parentDef)) {
                    ContinueDef continueDef = new ContinueDef();
                    continueDef.setParentDef(parentDef);
                    executeList.add(continueDef);
                } else
                    throw new InterfaceException("00024",
                            "unexpected [continue] node under [" + node.getName() + "].");
            } else if (node.getName().equals("log")) {
                executeList.add(getLogDef(parentDef, node));
            } else if (node.getName().equals("if")) {
                IfDef ifDef = new IfDef();
                ifDef.setParentDef(parentDef);
                String condition = getAttribute(node, "condition");
                ifDef.addConditionMap(condition, getExecuteList(ifDef, node));
                while (++i < stepNode.elements().size()) {
                    Element conditionNode = (Element) stepNode.elements().get(i);
                    condition = getAttribute(conditionNode, "condition");
                    if (conditionNode.getName().equals("elseif")) {
                        if (!ifDef.getConditionMap().containsKey(condition))
                            ifDef.addConditionMap(condition, getExecuteList(ifDef, conditionNode));
                    } else if (conditionNode.getName().equals("else")) {
                        if (!ifDef.getConditionMap().containsKey(condition))
                            ifDef.setElseCondition(getExecuteList(ifDef, conditionNode));
                    } else {
                        i--;
                        break;
                    }
                }
                executeList.add(ifDef);
            } else if (node.getName().equals("connection")) {
                executeList.add(getConnectionDef(parentDef, node));
            } else if (node.getName().equals("filter")) {
                List<Element> subFilterNodes = node.elements("subfilter");
                if (subFilterNodes.size() > 0) {
                    for (Element subfilterNode : subFilterNodes) {
                        String ref = getRequiredAttribute(subfilterNode, "ref");
                        String[] classes = getCommonsFilterDef(mainCommonsDef, ref)
                                .getExecuteClass().split(",");
                        for (String clazz : classes) {
                            IFilterDef filterDef = getFilterDef(parentDef, subfilterNode);
                            filterDef.setExecuteClass(clazz);
                            executeList.add(filterDef);
                        }

                    }
                } else {
                    String ref = getRequiredAttribute(node, "ref");
                    String[] classes =
                            getCommonsFilterDef(mainCommonsDef, ref).getExecuteClass().split(",");
                    for (String clazz : classes) {
                        IFilterDef filterDef = getFilterDef(parentDef, node);
                        filterDef.setExecuteClass(clazz);
                        executeList.add(filterDef);
                    }
                }
            } else {
                logger.info(stepNode.asXML());
                throw new InterfaceException("00004", "not expected node [" + node.getName()
                        + "] under step node [" + stepNode.attributeValue("id") + "]");
            }
        }
        return executeList;
    }

    private IWhileDef getStepWhileDef(IAbsDef parentDef, Element node) {
        IWhileDef whileDef = new WhileDef();
        whileDef.setCondition(getAttribute(node, "condition"));
        whileDef.setParentDef(parentDef);
        whileDef.setSubList(getExecuteList(whileDef, node));
        return whileDef;
    }

    /**
     * 获取step下的foreach定义
     * 
     * @param foreachNode
     * @return
     */
    public IForeachDef getStepForeachDef(IAbsDef parentDef, Element foreachNode) {
        IForeachDef foreachDef = new ForeachDef();
        foreachDef.setBegin(getAttribute(foreachNode, "begin"));
        foreachDef.setEnd(getRequiredAttribute(foreachNode, "end"));
        foreachDef.setStep(getAttribute(foreachNode, "step"));
        foreachDef.setCondition(getAttribute(foreachNode, "condition"));
        foreachDef.setScope(getAttribute(foreachNode, "scope"));
        foreachDef.setTag(getRequiredAttribute(foreachNode, "tag"));
        foreachDef.setIgnoreExption(Boolean.valueOf(getAttribute(foreachNode, "ignoreException")));
        foreachDef.setParentDef(parentDef);
        foreachDef.setSubList(getExecuteList(foreachDef, foreachNode));
        return foreachDef;
    }

    /**
     * 添加连接定义
     * 
     * @param connectionNode
     * @return
     */
    private IConnectionDef getConnectionDef(IAbsDef parentDef, Element connectionNode) {
        String ref = getRequiredAttribute(connectionNode, "ref");
        IConnectionDef connectionDef = new ConnectionDef();
        connectionDef.combine(getCommonsConnectionDef(mainCommonsDef, ref));
        connectionDef.setRef(ref);
        connectionDef.setVia(getAttribute(connectionNode, "via"));
        connectionDef.setType(getAttribute(connectionNode, "type"));
        connectionDef.setEncoding(getAttribute(connectionNode, "encoding"));
        connectionDef.setTimes(getAttribute(connectionNode, "times"));
        connectionDef.setErrorDescription(getAttribute(connectionNode, "errdsp"));
        connectionDef.setJoinedid(getAttribute(connectionNode, "joinedid"));
        connectionDef.setParentDef(parentDef);
        connectionDef.setSubList(getConnectionSubList(connectionDef, connectionNode));

        /**
         * moified by weicong 2016-04-05
         */
        // interfaceDefth.get().addConnection(connectionDef);
        return connectionDef;
    }

    /**
     * 获取connection的子节点
     * 
     * @param connectionDef
     * @param connectionNode
     * @return
     */
    private List<Object> getConnectionSubList(IConnectionDef connectionDef,
            Element connectionNode) {
        List<Element> list = connectionNode.elements();
        List<Object> subList = connectionDef.getSubList();
        for (Element node : list) {
            String nodeName = node.getName();
            if (nodeName.equals("property")) {
                connectionDef.addProperty(getAttribute(node, "name"), getPropertyDef(node));
            } else if (nodeName.equals("foreach")) {
                subList.add(getConnectionForeachDef(node));
            } else if (nodeName.equals("switch")) {
                subList.add(getSwitchDef(connectionNode));
            } else if (nodeName.equals("log")) {
                subList.add(getLogDef(connectionDef, node));
            } else
                throw new InterfaceException("00022",
                        "config error,unexpected node [" + nodeName + "] under node [connection]");
        }
        return subList;
    }

    /**
     * 连接的属性子节点
     * 
     * @param propertyNode
     * @return
     */
    private IPropertyDef getPropertyDef(Element propertyNode) {
        IPropertyDef propertyDef = new PropertyDef();
        String name = getRequiredAttribute(propertyNode, "name");
        propertyDef.setName(name);
        String value = getAttribute(propertyNode, "value");
        String ref = getAttribute(propertyNode, "ref");
        value = StringUtil.isEmpty(value) && !StringUtil.isEmpty(ref)
                ? parseRefFileVal(ref)
                : value;
        // value = StringUtil.isEmpty(value) && !StringUtil.isEmpty(ref) ?ref :
        // value;
        propertyDef.setFlg(StringUtil.isEmpty(value) && !StringUtil.isEmpty(ref) ? "0" : "1");
        propertyDef.setValue(value);
        propertyDef.setArguments(getArguments(propertyNode));
        return propertyDef;
    }

    /**
     * 组装连接的foreach子节点
     * 
     * @param node
     * @return
     */
    private IForeachDef getConnectionForeachDef(Element node) {
        IForeachDef foreachDef = new ForeachDef();
        foreachDef.setTag(getAttribute(node, "tag"));
        foreachDef.setBegin(getAttribute(node, "begin"));
        foreachDef.setEnd(getAttribute(node, "end"));
        foreachDef.setScope(getAttribute(node, "scope"));
        foreachDef.setStep(getAttribute(node, "step"));
        foreachDef.setCondition(getAttribute(node, "condition"));
        List<Element> propertyNodes = node.elements("property");
        for (Element propertyNode : propertyNodes) {
            foreachDef.addPropertyDef(getAttribute(propertyNode, "name"),
                    getPropertyDef(propertyNode));
        }
        return foreachDef;
    }

    /**
     * 添加filter定义
     * 
     * @param filterNode
     * @return
     */
    private IFilterDef getFilterDef(IAbsDef parentDef, Element filterNode) {
        String ref = getRequiredAttribute(filterNode, "ref");
        IFilterDef filterDef = new FilterDef();
        filterDef.combine(getCommonsFilterDef(mainCommonsDef, ref));
        filterDef.setRef(ref);
        String type = getRequiredAttribute(filterNode, "type");
        filterDef.setType(type);
        String encoding = getAttribute(filterNode, "encoding");
        String level = getAttribute(filterNode, "level");
        if (level == null || level == "") {
            level = "debug";
        }
        filterDef.setLevel(level);
        filterDef.setEncoding(encoding);
        filterDef.setTimes(getAttribute(filterNode, "times"));
        filterDef.setScope(getAttribute(filterNode, "scope"));
        filterDef.setTag(getAttribute(filterNode, "tag"));
        filterDef.setVia(getAttribute(filterNode, "via"));
        filterDef.setErrorDescription(getAttribute(filterNode, "errdsp"));
        filterDef.setParentDef(parentDef);
        filterDef.setSubList(getFilterSubList(filterDef, filterNode));

        /**
         * moified by weicong 2016-04-05
         */
        // interfaceDefth.get().addFilters(filterDef);
        return filterDef;
    }

    private boolean nodeInLoop(IAbsDef parentDef) {
        return parentDef instanceof IForeachDef || parentDef instanceof IWhileDef
                || parentDef.subNodeOf(ForeachDef.class) || parentDef.subNodeOf(WhileDef.class);
    }

    /**
     * 添加filter子节点field、foreach、if、convertor、validator
     * 
     * @param filterNode
     * @return
     */
    protected List<IAbsDef> getFilterSubList(IAbsDef parentDef, Element filterNode) {
        List<Element> subList = filterNode.elements();
        List<IAbsDef> list = new ArrayList<IAbsDef>();
        for (int i = 0; i < subList.size(); i++) {
            Element subNode = subList.get(i);
            if (subNode.getName().equals("foreach")) {
                list.add(getFilterForeachDef(parentDef, subNode));
            } else if (subNode.getName().equals("break")) {
                if (nodeInLoop(parentDef)) {
                    BreakDef breakDef = new BreakDef();
                    breakDef.setParentDef(parentDef);
                    list.add(breakDef);
                } else
                    throw new InterfaceException("00023",
                            "unexpected [break] node under [" + filterNode.getName() + "].");
            } else if (subNode.getName().equals("continue")) {
                if (nodeInLoop(parentDef)) {
                    ContinueDef continueDef = new ContinueDef();
                    continueDef.setParentDef(parentDef);
                    list.add(continueDef);
                } else
                    throw new InterfaceException("00024",
                            "unexpected [continue] node under [" + filterNode.getName() + "].");
            } else if (subNode.getName().equals("return")) {
                list.add(getReturnDef(parentDef, subNode));
            } else if (subNode.getName().equals("while")) {
                list.add(getFilterWhileDef(parentDef, subNode));
            } else if (subNode.getName().equals("field")) {
                list.add(getFieldDef(parentDef, subNode));
            } else if (subNode.getName().equals("call")) {
                list.add(getCallDef(parentDef, subNode));
            } else if (subNode.getName().equals("convertor")) {
                list.add(getConvertorDef(parentDef, subNode));
            } else if (subNode.getName().equals("validator")) {
                list.add(getValidatorDef(parentDef, subNode));
            } else if (subNode.getName().equals("log")) {
                list.add(getLogDef(parentDef, subNode));
            } else if (subNode.getName().equals("if")) {
                IfDef ifDef = new IfDef();
                ifDef.setParentDef(parentDef);
                String condition = getAttribute(subNode, "condition");
                ifDef.addConditionMap(condition, getFilterSubList(ifDef, subNode));
                while (++i < subList.size()) {
                    Element conditionNode = (Element) filterNode.elements().get(i);
                    condition = getAttribute(conditionNode, "condition");
                    if (conditionNode.getName().equals("elseif")) {
                        if (!ifDef.getConditionMap().containsKey(condition)) ifDef
                                .addConditionMap(condition, getFilterSubList(ifDef, conditionNode));
                    } else if (conditionNode.getName().equals("else")) {
                        ifDef.setElseCondition(getFilterSubList(ifDef, conditionNode));
                    } else {
                        i--;
                        break;
                    }
                }
                list.add(ifDef);
            } else {
                throw new InterfaceException("00007",
                        "config error,not expected node [" + subNode.getName() + "]");
            }
            list.get(list.size() - 1).setLabel(subNode.getName());
        }
        return list;
    }

    private String getInterfaceId(Element node) {
        Element root = node.getDocument().getRootElement();
        Element interfaceNode = root.element("interface");
        return getRequiredAttribute(interfaceNode, "id");
    }

    private IAbsDef getCallDef(IAbsDef parentDef, Element callNode) {
        CallDef callDef = new CallDef();
        callDef.setTag(getAttribute(callNode, "tag"));
        callDef.setScope(getAttribute(callNode, "scope"));
        callDef.setInterfaceName(getRequiredAttribute(callNode, "interfaceName"));
        callDef.setTransactionName(getRequiredAttribute(callNode, "transactionName"));
        callDef.setType(getAttribute(callNode, "type"));
        callDef.setMode(getAttribute(callNode, "mode"));
        callDef.setPre(getAttribute(callNode, "pre"));
        callDef.setMerge(getAttribute(callNode, "merge"));
        callDef.setArguments(getArguments(callNode));

        // ------------added by weicong on 2016/3/21 -----------
        callDef.setParentDef(parentDef);
        String xpath = callNode.getUniquePath();
        callDef.setXpath(xpath);
        callDef.load2DefFactory(xpath, getInterfaceId(callNode));
        return callDef;
    }

    private IReturnDef getReturnDef(IAbsDef parentDef, Element defNode) {
        IReturnDef returnDef = new ReturnDef();
        returnDef.setValue(getAttribute(defNode, "value"));
        returnDef.setParentDef(parentDef);
        return returnDef;
    }

    /**
     * 组装field节点
     * 
     * @param parentDef
     * @param fieldNode
     * @return
     */
    private IFieldDef getFieldDef(IAbsDef parentDef, Element fieldNode) {
        IFieldDef fieldDef = new FieldDef();
        // ------------added by weicong on 2016/3/18 -----------
        fieldDef.setSplit((getAttribute(fieldNode, "split")));
        // ------------added by hujun on 2015/4/24 -----------
        fieldDef.setRef(getAttribute(fieldNode, "ref"));
        // ---------------------------------------------------
        fieldDef.setId(getAttribute(fieldNode, "id"));
        fieldDef.setTag(getAttribute(fieldNode, "tag"));
        fieldDef.setEtag(getAttribute(fieldNode, "etag"));
        fieldDef.setType(getAttribute(fieldNode, "type"));
        fieldDef.setFill(getAttribute(fieldNode, "fill"));
        fieldDef.setDirect(getAttribute(fieldNode, "direct"));
        // modified by hujun on 20160304
        fieldDef.setValue(parseRefFileVal(getAttribute(fieldNode, "value")));
        fieldDef.setRequired(Boolean.valueOf(getAttribute(fieldNode, "required")));
        fieldDef.setSize(getAttribute(fieldNode, "size"));
        fieldDef.setVarSize(getAttribute(fieldNode, "varsize"));
        fieldDef.setScope(getAttribute(fieldNode, "scope"));
        fieldDef.setDescription(getAttribute(fieldNode, "description"));
        fieldDef.setCodetable(getAttribute(fieldNode, "codetable"));
        fieldDef.setValidator(getAttribute(fieldNode, "validator"));
        fieldDef.setParentDef(parentDef);
        fieldDef.setConvertorsAndValidators(getConvertorAndValidators(fieldDef, fieldNode));
        fieldDef.setArguments(getArguments(fieldNode));
        fieldDef.setSubfields(getSubfields(fieldNode, fieldDef));
        List<Element> elements = fieldNode.elements();
        for (Element element : elements) {
            String name = element.getName();
            if (name.equals("convertor") || name.equals("validator") || name.equals("argument")
                    || name.equals("if") || name.equals("elseif") || name.equals("else")
                    || name.equals("switch") || name.equals("subfield")) {} else {
                throw new InterfaceException("00004", "unexpected node [" + name + "] under field");
            }
        }
        return fieldDef;
    }

    /**
     * 组装field旗下的子节点
     * 
     * @param parentNode field节点
     * @param fieldDef
     * @return
     */
    private List<ISubfieldDef> getSubfields(Element parentNode, IFieldDef fieldDef) {
        List<Element> nodes = parentNode.elements();
        List<ISubfieldDef> subfields = new LinkedList<ISubfieldDef>();
        for (Element node : nodes) {
            if (node.getName().equals("subfield")) {
                ISubfieldDef subfieldDer = new SubfieldDef();
                subfieldDer.setId(getAttribute(node, "id"));
                subfieldDer.setType(getAttribute(node, "type"));
                subfieldDer.setValue(parseRefFileVal(getAttribute(node, "value")));
                subfieldDer.setEtag(getAttribute(node, "etag"));
                subfieldDer.setKey(getAttribute(node, "key"));
                subfieldDer.setTag(getAttribute(node, "tag"));
                subfieldDer.setSize(getAttribute(node, "size"));
                subfieldDer.setParentDef(fieldDef);
                subfieldDer.setSplit(getAttribute(node, "split"));
                subfieldDer.setSrckey(getAttribute(node, "srckey"));
                subfieldDer.setDirect(getAttribute(node, "direct"));
                subfieldDer.setFill(getAttribute(node, "fill"));
                subfieldDer.setVarSize(getAttribute(node, "varsize"));
                subfields.add(subfieldDer);
                if (node.elements().size() > 0) {
                    subfieldDer.setSubfields(getSubfields(node, subfieldDer));
                }
            } else if (node.getName().equals("argument")) {

            } else
                throw new InterfaceException("00025",
                        "unexpected node [" + node.getName() + "] under " + parentNode.getName());
        }
        return subfields;
    }

    private IWhileDef getFilterWhileDef(IAbsDef parentDef, Element node) {
        IWhileDef whileDef = new WhileDef();
        whileDef.setCondition(getAttribute(node, "condition"));
        whileDef.setParentDef(parentDef);
        whileDef.setSubList(getFilterSubList(whileDef, node));
        return whileDef;
    }

    private ILogDef getLogDef(IAbsDef parentDef, Element logNode) {
        ILogDef logDef = new LogDef();
        String value = getAttribute(logNode, "value");
        String level = getAttribute(logNode, "level");
        logDef.setValue(value);
        logDef.setLevel(level);
        logDef.setParentDef(parentDef);

        return logDef;
    }

    /**
     * 组装filter的foreach子节点
     * 
     * @param foreachNode
     * @return
     */
    private IForeachDef getFilterForeachDef(IAbsDef parentDef, Element foreachNode) {
        IForeachDef foreachDef = new ForeachDef();
        foreachDef.setTag(getRequiredAttribute(foreachNode, "tag"));
        foreachDef.setBegin(getAttribute(foreachNode, "begin"));
        foreachDef.setEnd(getRequiredAttribute(foreachNode, "end"));
        foreachDef.setScope(getAttribute(foreachNode, "scope"));
        foreachDef.setStep(getAttribute(foreachNode, "step"));
        foreachDef.setCondition(getAttribute(foreachNode, "condition"));
        foreachDef.setIgnoreExption(Boolean.valueOf(getAttribute(foreachNode, "ignoreException")));
        foreachDef.setParentDef(parentDef);
        foreachDef.setSubList(getFilterSubList(foreachDef, foreachNode));
        return foreachDef;
    }

    /**
     * 组装field下的convertor和validator子节点
     * 
     * @param fieldNode
     * @return
     */
    private List<Object> getConvertorAndValidators(IFieldDef fieldDef, Element fieldNode) {
        List<Element> nodes = fieldNode.elements();
        List<Object> list = new ArrayList<Object>();
        for (Element node : nodes) {
            if (node.getName().equals("convertor")) {
                list.add(getConvertorDef(fieldDef, node));
            } else if (node.getName().equals("validator")) {
                list.add(getValidatorDef(fieldDef, node));
            }
        }
        if (list.size() == 0) return null;
        return list;
    }

    /**
     * 组装参数
     * 
     * @param fieldNode
     * @return
     */
    private List<IArgumentDef> getArguments(Element parentNode) {
        List<Element> nodes = parentNode.elements();
        List<IArgumentDef> arguments = new ArrayList<IArgumentDef>();
        for (Element node : nodes) {
            if (node.getName().equals("argument")) {
                IArgumentDef argumentDef = new ArgumentDef();
                argumentDef.setId(getAttribute(node, "id"));
                argumentDef.setValue(getAttribute(node, "value"));
                argumentDef.setType(getAttribute(node, "type"));
                argumentDef.setDirection(getAttribute(node, "direction"));
                argumentDef.setDescription(getAttribute(node, "description"));
                arguments.add(argumentDef);
            } else if (node.getName().equals("subfield")) {} else
                throw new InterfaceException("00025",
                        "unexpected node [" + node.getName() + "] under " + parentNode.getName());
        }
        return arguments;
    }

    /**
     * 组装convertor定义
     * 
     * @param fieldNode
     * @return
     */
    private IConvertorDef getConvertorDef(IAbsDef parentDef, Element convertorNode) {
        IConvertorDef convertorDef = new ConvertorDef();
        convertorDef.setTag(getAttribute(convertorNode, "tag"));
        convertorDef.setScope(getAttribute(convertorNode, "scope"));
        convertorDef.setValue(getAttribute(convertorNode, "value"));
        convertorDef.setErrorCode(getAttribute(convertorNode, "errorcode"));
        convertorDef.setDescription(getAttribute(convertorNode, "description"));
        convertorDef.setParentDef(parentDef);
        convertorDef.setArguments(getArguments(convertorNode));
        return convertorDef;
    }

    /**
     * 组装field下的switch定义
     * 
     * @param fieldNode
     * @return
     */
    private ISwitchDef getSwitchDef(Element fieldNode) {
        Element switchNode = fieldNode.element("switch");
        if (switchNode == null) return null;
        ISwitchDef switchDef = new SwitchDef();
        switchDef.setCondition(getAttribute(switchNode, "condition"));
        List<Element> caseList = switchNode.elements("case");
        for (Element caseNode : caseList) {
            CaseDef caseDef = new SwitchDef().new CaseDef();
            String condition = getAttribute(caseNode, "condition");
            String value = getAttribute(caseNode, "value");
            String type = getAttribute(caseNode, "type");
            caseDef.setCondition(condition);
            caseDef.setValue(value);
            caseDef.setType(type);
            caseDef.setPropertyDef(getPropertyDef(caseNode.element("property")));
            switchDef.addCaseMap(condition, caseDef);
        }
        return switchDef;
    }

    /**
     * 组装field下的if定义
     * 
     * @param fieldNode
     * @return
     */
    @Deprecated
    private IfDef getIfDef(IAbsDef parentDef, Element fieldNode) {
        Element ifNode = fieldNode.element("if");
        if (ifNode == null) return null;
        IfDef ifDef = new IfDef();
        ifDef.setParentDef(parentDef);
        String condition = getAttribute(ifNode, "condition");
        String value = getAttribute(ifNode, "value");
        ifDef.addConditionMap(condition, value);

        List<Element> elseifNodes = fieldNode.elements("elseif");
        for (int i = 0; i < elseifNodes.size(); i++) {
            Element elseifNode = elseifNodes.get(i);
            condition = getAttribute(elseifNode, "condition");
            value = getAttribute(elseifNode, "value");
            if (!ifDef.getConditionMap().containsKey(condition))
                ifDef.addConditionMap(condition, value);
        }
        Element elseNode = fieldNode.element("else");
        if (elseNode != null) {
            value = getAttribute(elseNode, "value");
            ifDef.addConditionMap(condition, value);
        }
        return ifDef;
    }

    /**
     * 组装validator定义
     * 
     * @param validatorNode
     * @return
     */
    private IValidatorDef getValidatorDef(IAbsDef parentDef, Element validatorNode) {
        IValidatorDef validatorDef = new ValidatorDef();
        validatorDef.setId(getAttribute(validatorNode, "id"));
        validatorDef.setCondition(getAttribute(validatorNode, "condition"));
        validatorDef.setErrorCode(getAttribute(validatorNode, "errorcode"));
        validatorDef.setDescription(getAttribute(validatorNode, "description"));
        validatorDef.setParentDef(parentDef);
        validatorDef.setArguments(getArguments(validatorNode));
        return validatorDef;
    }

    /**
     * 获取Commons定义
     * 
     * @param doc
     * @return
     * 
     * @modify modified by hujun on 2014-6-17
     */
    public CommonsDef getCommonsDef(Document doc, WriteXMLFlag wxf) {
        CommonsDef commonsDef = new CommonsDef();
        Element rootNode = doc.getRootElement();
        Element connectionsNode = rootNode.element("connections");
        Element filtersNode = rootNode.element("filters");
        if (connectionsNode != null)
            commonsDef.setConnectionsDef(getConnectionsDef(connectionsNode, wxf));
        if (filtersNode != null) commonsDef.setFiltersDef(getFiltersDef(filtersNode));
        return commonsDef;
    }

    /**
     * 组装connections定义
     * 
     * @param connectionsNode
     * @return
     * 
     * @modify modified by hujun on 2014-6-17
     */
    private ConnectionsDef getConnectionsDef(Element connectionsNode, WriteXMLFlag wxf) {
        ConnectionsDef connectionsDef = new ConnectionsDef();
        List<Element> connectionNodes = connectionsNode.elements("connection");
        List<IConnectionDef> connectionDefs = new ArrayList<IConnectionDef>();
        for (Element connectionNode : connectionNodes) {
            connectionDefs.add(getCommonsConnectionDef(connectionNode, wxf));
        }
        connectionsDef.setConnections(connectionDefs);
        return connectionsDef;
    }

    /**
     * 组装filters定义
     * 
     * @param filtersNode
     * @return
     */
    private FiltersDef getFiltersDef(Element filtersNode) {
        FiltersDef filtersDef = new FiltersDef();
        List<Element> filterNodes = filtersNode.elements("filter");
        List<IFilterDef> filterDefs = new ArrayList<IFilterDef>();
        for (Element filterNode : filterNodes) {
            filterDefs.add(getCommonsFilterDef(filterNode));
        }
        filtersDef.setFilters(filterDefs);
        return filtersDef;
    }

    /**
     * 用于解析本地外部文件配置
     * 
     * add by hujun on 2015/4/24
     * 
     * @param s
     * @return
     */
    public String parseRefFileVal(String s) {

        if (s == null || !(s.startsWith(XML_REF_FILE_PREFIX + ":")
                || s.startsWith(PROP_REF_FILE_PREFIX + ":")))
            return s;
        String head = s.split(":")[0];
        String[] opers = s.substring(s.indexOf(head) + head.length() + 1).split("\\|");
        if (opers.length != 2) throw new IllegalArgumentException("The operand is not correct.");
        String path = opers[0];
        String value = opers[1];

        Object o = localRefFiles.get(path);

        if (head.equalsIgnoreCase(XML_REF_FILE_PREFIX)) {
            Document doc = null == o ? null : (Document) o;
            if (doc == null) {
                try {
                    doc = XmlUtil.getDocument(new File(path).toURI().toURL());
                } catch (MalformedURLException e) {
                    throw new IllegalArgumentException("[" + path + "], xml can't be found");
                }
                localRefFiles.put(path, doc);
            }
            if (value.startsWith("/")) {
                Node node = doc.selectSingleNode(value);
                if (node == null) return "";
                return node.getText();
            } else
                return XmlUtil.getXmlNodeValue(doc, value);
        }

        if (head.equalsIgnoreCase(PROP_REF_FILE_PREFIX)) {
            Properties properties = null == o ? new Properties() : (Properties) o;
            if (o == null) {
                try {
                    properties.load(new FileInputStream(new File(path)));
                } catch (IOException e) {
                    throw new IllegalArgumentException("[" + path + "], properies can't be found");
                }
                localRefFiles.put(path, properties);
            }
            return properties.getProperty(value);
        }

        throw new IllegalArgumentException(
                "The protocol is not correct, must begin with XML or PROP.");
    }

    /**
     * 组装commons的连接定义
     * 
     * @param connectionNode
     * @return
     * 
     * @modify modified by hujun on 2014-6-17
     */
    private IConnectionDef getCommonsConnectionDef(Element connectionNode, WriteXMLFlag wxf) {
        IConnectionDef connectionDef = new ConnectionDef();
        connectionDef.setId(getAttribute(connectionNode, "id"));
        connectionDef.setExecuteClass(getAttribute(connectionNode, "class"));
        connectionDef.setPooled(Boolean.valueOf(getAttribute(connectionNode, "pooled")));
        connectionDef.setPoolSetting(getPoolSetting(connectionNode));
        List<Element> propertyNodes = connectionNode.elements("property");
        for (Element propertyNode : propertyNodes) {
            String propName = getAttribute(propertyNode, "name");
            String propValue = getAttribute(propertyNode, "value");
            String scope = getAttribute(propertyNode, "scope");

            String encrypt = getAttribute(propertyNode, "encrypt");
            // ------------added by hujun on 2015/5/24 -----------
            String ref = getAttribute(propertyNode, "ref");
            if (StringUtil.isEmpty(propValue) && !StringUtil.isEmpty(ref))
                propValue = parseRefFileVal(ref);
            // ---------------------------------------------------
            else {
                // ------------added by hujun on 2014/6/17 -----------
                if (propName.equals("password") && StringUtil.isEmpty(encrypt))
                    encrypt = XPwdCoder.DES_ALGORITHM;
                if (!StringUtil.isEmpty(encrypt)) {
                    if (!propName.startsWith(IPropertyDef.ENCRYPT_PRE)) {
                        connectionDef.addEncryptProperty(propName,
                                IPropertyDef.ENCRYPT_PRE + propName);
                        propName = IPropertyDef.ENCRYPT_PRE + propName;
                        propValue = new XPwdCoder().encrypt(propValue, encrypt);
                        propertyNode.addAttribute("name", propName);
                        propertyNode.addAttribute("value", propValue);
                        wxf.setOverwrite(true);
                    } else {
                        connectionDef.addEncryptProperty(
                                propName.substring(IPropertyDef.ENCRYPT_PRE.length()), propName);
                    }
                }
            }
            // ------------------------- ----------- -----------
            IPropertyDef propertyDef = new PropertyDef();
            propertyDef.setEncrypt(encrypt);
            propertyDef.setName(propName);
            propertyDef.setValue(propValue);
            propertyDef.setScope(scope);
            connectionDef.addProperty(propName, propertyDef);
        }
        return connectionDef;
    }

    /**
     * 获取连接池定义
     * 
     * @param connectionNode
     * @return
     */
    private PoolSetting getPoolSetting(Element connectionNode) {
        PoolSetting poolSetting = new ConnectionDef().new PoolSetting();
        Element poolNode = connectionNode.element("poolSetting");
        if (poolNode == null) return null;
        List<Element> propNodes = poolNode.elements("property");
        for (Element prop : propNodes) {
            String name = getAttribute(prop, "name");
            String value = getAttribute(prop, "value");
            String ref = getAttribute(prop, "ref");
            value = StringUtil.isEmpty(value) && !StringUtil.isEmpty(ref)
                    ? parseRefFileVal(ref)
                    : value;
            poolSetting.addProperty(name, value);
        }

        return poolSetting;
    }

    /**
     * 根据connectionId匹配commonsDef中的IConnectionDef
     * 
     * @param commonsDef
     * @param connectionId
     * @return
     */
    private IConnectionDef getCommonsConnectionDef(CommonsDef commonsDef, String connectionId) {
        ConnectionsDef connectionsDef = commonsDef.getConnectionsDef();
        if (connectionsDef == null) {
            throw new InterfaceException("00009", "commons connections node not exist!");
        }
        List<IConnectionDef> connectionDefs = connectionsDef.getConnections();
        for (IConnectionDef connectionDef : connectionDefs) {
            if (connectionDef.getId().equals(connectionId)) {
                return connectionDef;
            }
        }
        throw new InterfaceException("00010",
                "commons connection [" + connectionId + "] not exist!");
    }

    /**
     * 组装commons的filter定义
     * 
     * @param filterNode
     * @return
     */
    private IFilterDef getCommonsFilterDef(Element filterNode) {
        IFilterDef filterDef = new FilterDef();
        filterDef.setId(getAttribute(filterNode, "id"));
        String executeClass = getAttribute(filterNode, "class");
        filterDef.setExecuteClass(executeClass);
        List<Element> propertyNodes = filterNode.elements("parameter");
        for (Element propertyNode : propertyNodes) {
            String propId = getAttribute(propertyNode, "name");
            String propValue = getAttribute(propertyNode, "value");
            String ref = getAttribute(propertyNode, "ref");
            propValue = StringUtil.isEmpty(propValue) && !StringUtil.isEmpty(ref)
                    ? parseRefFileVal(ref)
                    : propValue;
            filterDef.addCommonsProperty(propId, propValue);
        }
        return filterDef;
    }

    /**
     * 根据filterId匹配commonsDef中的IFilterDef
     * 
     * @param commonsDef
     * @param filterId
     * @return
     */
    private IFilterDef getCommonsFilterDef(CommonsDef commonsDef, String filterId) {
        FiltersDef filtersDef = commonsDef.getFiltersDef();
        if (filtersDef == null) {
            throw new InterfaceException("00011", "commons filters node not exist!");
        }
        List<IFilterDef> filterDefs = filtersDef.getFilters();
        for (IFilterDef filterDef : filterDefs) {
            if (filterDef.getId().equals(filterId)) {
                return filterDef;
            }
        }
        throw new InterfaceException("00012", "commons filter [" + filterId + "] not exist!");
    }

    /**
     * 批量设置错误码对应值
     * 
     * @param interfaceId
     * @param sysCodeList
     */
    public void setSysCodesRef(String interfaceId, List<SysCode> sysCodeList) {
        for (SysCode sysCode : sysCodeList) {
            sysCode.setRef(getErrorCodeRef(interfaceId, sysCode.getRef()));
        }
    }

    /**
     * 根据ref配置获取对应的接口错误码
     * 
     * @param interfaceId
     * @param ref
     * @return
     */
    public String getErrorCodeRef(String interfaceId, String ref) {
        if (StringUtil.isEmpty(ref) || StringUtil.isEmpty(interfaceId)) return "";
        int startIndex = ref.indexOf(interfaceId);
        if (-1 != startIndex) {
            int endIndex = ref.indexOf(',', startIndex);
            if (endIndex == -1) endIndex = ref.length();
            String interfaceDotCode = ref.substring(startIndex, endIndex);
            return interfaceDotCode.split("\\.")[1];
        }
        return "";
    }

    /**
     * 解析主配置的所有codeatable节点
     * 
     * @param rootNodes
     * @return
     */
    private Map<String, Map<String, String>> getCodeTables(List<Element> rootNodes) {
        Map<String, Map<String, String>> codetableMap = new HashMap<String, Map<String, String>>();
        for (Element rootNode : rootNodes) {
            String ref = getAttribute(rootNode, "ref");
            if (!StringUtil.isEmpty(ref)) {
                codetableMap.putAll(getCodeTables(XmlUtil.getDocument(ref).getRootElement()));
            } else
                codetableMap.putAll(getCodeTables(rootNode));
        }
        return codetableMap;
    }

    private Map<String, Map<String, String>> getCodeTables(Element rootNode) {
        Map<String, Map<String, String>> codetableMap = new HashMap<String, Map<String, String>>();
        List<Element> codetableNodes = rootNode.elements();
        for (Element codetableNode : codetableNodes) {
            if (codetableNode.getName().equals("codetable")) {
                String id = getAttribute(codetableNode, "id");
                String sql = getAttribute(codetableNode, "sql");
                // 静态码表
                if (StringUtil.isEmpty(sql)) {
                    codetableMap.put(id, getCodeTable(codetableNode));
                } else {
                    // 动态码表
                    dynamicCodetableMap.put(id, sql);
                }
            } else
                throw new InterfaceException("00013",
                        "unexpected node [" + codetableNode.getName() + "] in codetables.xml");
        }
        return codetableMap;
    }

    private Map<String, String> getCodeTable(Element codetableNode) {
        Map<String, String> codeMap = new HashMap<String, String>();
        List<Element> codeNodes = codetableNode.elements();
        for (Element codeNode : codeNodes) {
            if (codeNode.getName().equals("code")) {
                String id = getAttribute(codeNode, "id");
                String value = getAttribute(codeNode, "value");
                codeMap.put(id, value);
            } else
                throw new InterfaceException("00014", "unexpected node [" + codeNode.getName()
                        + "] under " + codetableNode.getName());
        }
        return codeMap;
    }

    /**
     * 解析主配置
     * 
     * @param rootNodes
     * @return
     */
    private Map<String, ICounterDef> getCounters(List<Element> rootNodes) {
        Map<String, ICounterDef> counterMap = new HashMap<String, ICounterDef>();
        for (Element rootNode : rootNodes) {
            List<Element> e = rootNode.elements("counter");
            for (Element rootNode1 : e) {
                ICounterDef counterDef = new CounterDef();
                String tblnam = getAttribute(rootNode1, "tblnam");
                String source = getAttribute(rootNode1, "source");
                String sql = getAttribute(rootNode1, "sql");
                String initSql = getAttribute(rootNode1, "initSql");
                String value = getAttribute(rootNode1, "value");
                value = StringUtil.isEmpty(value) == true ? "0" : value;
                String beforeStep = getAttribute(rootNode1, "beforeStep");
                boolean b =
                        StringUtil.isEmpty(beforeStep) == true ? true : Boolean.valueOf(beforeStep);
                counterDef.setTblnam(tblnam);
                counterDef.setSource(source);
                counterDef.setSql(sql);
                counterDef.setInitSql(initSql);
                counterDef.setValue(value);
                counterDef.setBeforeStep(b);
                counterMap.put(tblnam, counterDef);
            }
        }
        return counterMap;
    }

    /**
     * 
     * 获取所有Module对象
     **/
    public Map<String, IModuleDef> getModules(List<Element> rootNodes) {
        Map<String, IModuleDef> moduleMap = new HashMap<String, IModuleDef>();
        for (Element rootNode : rootNodes) {
            String ref = getAttribute(rootNode, "ref");
            if (!StringUtil.isEmpty(ref)) {
                rootNode = XmlUtil.getDocument(ref).getRootElement();
            }
            List<Element> modulesNodes = rootNode.elements();
            for (Element moduleNode : modulesNodes) {
                if (moduleNode.getName().equals("module")) {
                    String moduleid = getAttribute(moduleNode, "id");
                    moduleMap.put(moduleid, getModule(moduleNode));
                } else
                    throw new InterfaceException("00013",
                            "unexpected node [" + moduleNode.getName() + "] in modules.xml");
            }
        }
        return moduleMap;
    }

    /**
     * 获取一个Module模型对象
     * 
     **/
    private IModuleDef getModule(Element moduleNode) {
        IModuleDef moduleDef = new ModuleDef();
        String moduleid = getAttribute(moduleNode, "id");
        String moduletype = getAttribute(moduleNode, "type");
        moduleDef.setId(moduleid);
        moduleDef.setType(moduletype);

        List<Element> fieldNodes = moduleNode.elements();
        for (Element fieldNode : fieldNodes) {
            if (fieldNode.getName().equals("field")) {
                String tag = getAttribute(fieldNode, "tag");
                String etag = getAttribute(fieldNode, "etag");
                String type = getAttribute(fieldNode, "type");
                String size = getAttribute(fieldNode, "size");
                String varsize = getAttribute(fieldNode, "varsize");
                String fill = getAttribute(fieldNode, "fill");
                String direct = getAttribute(fieldNode, "direct");
                IFieldDef fieldDef = new FieldDef();
                fieldDef.setTag(tag);
                fieldDef.setEtag(etag);
                fieldDef.setType(type);
                fieldDef.setSize(size);
                fieldDef.setVarSize(varsize);
                fieldDef.setFill(fill);
                fieldDef.setDirect(direct);
                fieldDef.setSubfields(getSubfields(fieldNode, fieldDef));
                moduleDef.getMap().put(etag, fieldDef);
            } else
                throw new InterfaceException("00014", "unexpected node [" + fieldNode.getName()
                        + "] under " + moduleNode.getName());
        }
        return moduleDef;
    }

    /**
     * 从errorCodes.xml获取全局通用错误码,并根据当前接口进行ref值的对应处理
     */
    public List<SysCode> getCommonSysCodes(String interfaceId) {
        List<SysCode> sysCodes = new ArrayList<SysCode>();
        URL url = XmlUtil.getUrl(Constants.ETC_PATH + "/" + "errorCodes.xml");
        if (url != null) {
            Element rootNode = XmlUtil.getDocument(url).getRootElement();
            sysCodes = getSysCodes(rootNode);
            setSysCodesRef(interfaceId, sysCodes);
        }
        return sysCodes;
    }

    public List<SysCode> getSysCodes(Element errorCodesNode) {
        List<Element> syscodeNodes = errorCodesNode.elements("sysCode");
        List<SysCode> sysCodes = new ArrayList<SysCode>();
        for (Element sysCodeNode : syscodeNodes) {
            sysCodes.add(getSysCode(sysCodeNode));
        }
        return sysCodes;
    }

    /**
     * 组装SysCode定义
     * 
     * @param sysCodeNode
     * @return
     */
    private SysCode getSysCode(Element sysCodeNode) {
        SysCode sysCode = new SysCode();
        sysCode.setId(getAttribute(sysCodeNode, "id"));
        sysCode.setRef(getAttribute(sysCodeNode, "ref"));
        return sysCode;
    }

    /**
     * 获取服务配置定义
     * 
     * @param filename
     * @return
     */
    @Override
    public List<IServiceDef> getServiceDefs(String filename) {
        Document doc = XmlUtil.getDocument(Constants.ETC_PATH, filename);
        List<IServiceDef> serviceDefs = getServiceDefs(doc);
        if (Serializable2FileUtil.hotDeploy)
            Serializable2FileUtil.writeObject(serviceDefs, filename);
        // context.getBeanFactory().setServiceDefs(serviceDefs);
        return serviceDefs;
    }

    /**
     * 返回配置文件所有路径
     * 
     * @return
     */
    public List<String> getCfgPath() {
        List<String> list = new ArrayList<String>();
        list.add(Constants.BEANS_PATH);
        list.add(Constants.ETC_PATH);
        list.add(MAIN_PATH);
        list.add(COMMON_PATH);
        list.add(IMPORT_PATH);
        return list;
    }

    /**
     * 提供热部署状态
     * 
     * @return
     * 
     */
    @Override
    public String getHotDeployState() {
        Document doc = XmlUtil.getDocument(Constants.ETC_PATH, "services.xml");
        return getAttribute(doc.getRootElement().element("services"), "hotDeploy");
    }

    /**
     * 获取服务配置定义
     * 
     * @param doc
     * @return
     */
    public List<IServiceDef> getServiceDefs(Document doc) {
        initMap.putAll(initEnvironment(doc));
        Element rootNode = doc.getRootElement();
        Element servicesNode = rootNode.element("services");
        String hotDeploy = getAttribute(servicesNode, "hotDeploy");
        if (hotDeploy.equals("on"))
            Serializable2FileUtil.hotDeploy = true;
        else
            Serializable2FileUtil.hotDeploy = false;
        initMainCommonsDef(rootNode);
        return getServiceDefs(servicesNode);
    }

    public IServiceDef getServiceDef(String id) {
        Document doc = XmlUtil.getDocument(Constants.ETC_PATH + "/services.xml");
        List<IServiceDef> serviceDefs = getServiceDefs(doc);
        for (IServiceDef serviceDef : serviceDefs) {
            if (serviceDef.getId().equals(id)) {
                return serviceDef;
            }
        }
        return null;
    }

    /**
     * 获取服务配置定义
     * 
     * @param servicesNode
     * @return
     */
    public List<IServiceDef> getServiceDefs(Element servicesNode) {
        List<IServiceDef> serviceDefs = new ArrayList<IServiceDef>();
        String language = getAttribute(servicesNode, "language");
        String state = getAttribute(servicesNode, "state");
        if (StringUtil.isEmpty(state)) state = "on";
        String encoding = getAttribute(servicesNode, "encoding");
        if (servicesNode != null) {
            List<Element> serviceList = servicesNode.elements("service");
            for (Element service : serviceList) {
                IServiceDef serviceDef = new ServiceDef();
                String serviceid = getAttribute(service, "id");
                serviceDef.setId(serviceid);
                serviceDef.setLanguage(language);
                serviceDef.setEncoding(encoding);
                serviceDef.setType(getRequiredAttribute(service, "type"));
                serviceDef.setExpression(getAttribute(service, "expression"));
                serviceDef.setDescription(getAttribute(service, "description"));
                String serviceState = getAttribute(service, "state");
                if (StringUtil.isEmpty(serviceState)) serviceState = "on";
                if (state.equals("on") && serviceState.equals("on"))
                    serviceDef.setState("on");
                else
                    serviceDef.setState("off");
                List<Element> propertyNodes = service.elements();
                for (Element propertyNode : propertyNodes) {
                    String propName = getRequiredAttribute(propertyNode, "name");
                    String propValue = getRequiredAttribute(propertyNode, "value");
                    serviceDef.addProperty(propName, propValue);
                    List<Element> argumentNodes=propertyNode.elements();
                    if(argumentNodes != null && argumentNodes.size()>0){
                    	for(Element argumentNode : argumentNodes){
                    		String tag=getAttribute(argumentNode, "tag");
                    		String value=getAttribute(argumentNode, "value");
                    		if(StringUtil.isEmpty(tag) || StringUtil.isEmpty(value)){
                    			continue;
                    		}
                    		((ServiceDef)serviceDef).addPropertyArgument(propName, tag, value);
                    	}
                    }
                }
                serviceDefs.add(serviceDef);
            }
        }
        return serviceDefs;
    }

    public String getExtValue(Object obj, String value) {
        if (obj instanceof Document) {
            Document doc = (Document) obj;
            if (value.startsWith("/")) {
                Node node = doc.selectSingleNode(value);
                if (node == null) return "";
                return node.getText();
            } else
                return XmlUtil.getXmlNodeValue(doc, value);
        } else if (obj instanceof Properties) {
            Properties properties = (Properties) obj;
            return properties.getProperty(value);
        }
        return null;
    }

    public void addExtEnvironment(URL url, Element parentNode) {
        Object object = null;
        String type = getRequiredAttribute(parentNode, "type");
        if (type.endsWith("xml")) {
            object = XmlUtil.getDocument(url);
        } else if (type.endsWith("properties")) {
            Properties properties = new Properties();
            try {
                properties.load(new FileInputStream(new File(url.getFile())));
            } catch (Exception e) {
                throw new InterfaceException("00022", "load file [" + url.getFile() + "] error.");
            }
            object = properties;
        }
        if (object != null) {
            List<Element> sysNodes = parentNode.elements();
            for (Element sysNode : sysNodes) {
                if (sysNode.getName().equals("sys")) {
                    String name = getRequiredAttribute(sysNode, "name");
                    String value = getRequiredAttribute(sysNode, "value");
                    initMap.put(name, getExtValue(object, value));
                }
            }
        }
    }

    public Map<String, String> initEnvironment(Element envNode) {
        Map<String, String> envMap = new HashMap<String, String>();
        List<Element> sysList = envNode.elements();
        for (Element sys : sysList) {
            if (sys.getName().equals("sys")) {
                String name = getAttribute(sys, "name");
                String value = getAttribute(sys, "value");
                envMap.put(name, value);
                initMap.put(name, value);
            } else if (sys.getName().equals("extsys")) {
                String ref = getRequiredAttribute(sys, "ref");
                URL url = XmlUtil.getUrl(ref);
                if (url == null) {
                    if (new File(ref).exists()) {
                        try {
                            if (!ref.startsWith("/"))
                                url = new URL("file:/" + ref);
                            else
                                url = new URL("file:" + ref);
                        } catch (Exception e) {
                            throw new InterfaceException("00006",
                                    "can not find resource [" + ref + "].");
                        }
                    } else {
                        throw new InterfaceException("00023", "file [" + ref + "] not exists.");
                    }
                }
                if (url != null) {
                    addExtEnvironment(url, sys);
                }
            } else {
                throw new InterfaceException("00015", "unexpected node [" + sys.getName()
                        + "] under [" + envNode.getName() + "]");
            }
        }
        return envMap;
    }

    public Map<String, String> initEnvironment(Document doc) {
        Map<String, String> envMap = new HashMap<String, String>();
        Element publicNode = doc.getRootElement().element("public");
        if (publicNode != null) envMap = initEnvironment(publicNode);
        return envMap;
    }

    /**
     * 获取必须配置的属性值，如果不存在，则会抛出异常
     * 
     * @param node
     * @param name
     * @return
     */
    protected String getRequiredAttribute(Element node, String name) {
        String str = getAttribute(node, name);
        if (StringUtil.isEmpty(str)) throw new InterfaceException("00005",
                "attribute [" + name + "] of node [" + node.getPath() + "] should not be empty.");
        return str;
    }

    /**
     * 获取指定节点的指定属性值
     * 
     * @param node
     * @param name
     * @return
     */
    protected String getAttribute(Element node, String name) {
        String value = node.attributeValue(name);
        if (value == null)
            return "";
        // 确定为表达式且不含函数调用，则尝试进行解析，是否为全局变量
        else if (value.matches("[\\s\\S]*\\#\\{\\w+\\}[\\s\\S]*")) {
            String test1 = getGlobalValue(value);
            return test1;
        }
        String nodeName = node.getName();
        // if (!nodeName.equals("property") &&
        // !nodeName.equals("parameter")&&!nodeName.equals("argument") &&
        // !nodeName.equals("field"))
        // return value.trim();
        if (nodeName.equals("filter") || nodeName.equals("connection")
                || nodeName.equals("interface") || nodeName.equals("transaction")
                || nodeName.equals("step"))
            return value.trim();
        return value;
    }

    /**
     * 获取全局变量值
     * 
     * @param value
     * @return
     */
    private String getGlobalValue(String MyExpression) {
        // 多表达式处理
        int a = 0;
        int b = 0;
        while (-1 != (a = MyExpression.indexOf("#{", b))) {
            b = MyExpression.indexOf("}", a);
            if (-1 != b) // 存在表达式对
            {
                String m = MyExpression.substring(a + 2, b);
                String value = initMap.get(m);
                if (value == null) throw new InterfaceException("00020",
                        "environment value [" + m + "]  not exists.");
                MyExpression = MyExpression.replace(MyExpression.substring(a, b + 1), value);
                b += value.length() - (b - a);
            }
        }
        return MyExpression;
    }

    /**
     * 合并两个list，并去重根据id
     * 
     * @param originalList 原始list，保留其中所有的元素
     * @param joinList 加入的list，只加入originalList不存在的
     * @return
     */
    private List<? extends IDefinition> combineList(List<? extends IDefinition> originalList,
            List<? extends IDefinition> joinList) {
        if (originalList.size() == 0) return joinList;
        List<IDefinition> copyOriginalList = new ArrayList<IDefinition>(originalList);
        IDefinition join = null;
        String originalId = null;
        for (int i = 0, joinSize = joinList.size(); i < joinSize; i++) {
            join = joinList.get(i);
            int j = 0, originalSize = originalList.size();
            for (; j < originalSize; j++) {
                originalId = copyOriginalList.get(j).getId();
                if (originalId.equals(join.getId())) {
                    copyOriginalList.remove(j);
                    copyOriginalList.add(j, join);
                    break;
                }
            }
            if (j == originalSize) copyOriginalList.add(join);
        }
        return copyOriginalList;
    }

    /**
     * 检查指定父节点是否存在非法子节点
     * 
     * @param pNode
     * @param validNodes
     */
    public void checkInvalidNodes(Element pNode, String[] validNodes) {
        List<Element> subNodes = pNode.elements();
        Map<String, String> validMap = new HashMap<String, String>();
        for (String validNode : validNodes)
            validMap.put(validNode, "");
        for (Element subNode : subNodes) {
            if (!validMap.containsKey(subNode.getName())) throw new InterfaceException("00025",
                    "invalid node [" + subNode.getName() + "] under " + pNode.getName());
        }
    }

    // public FixedModel getFixedModel(Element modelNode)
    // {
    // List<Element> fields = modelNode.elements();
    // int fieldCnt = fields.size();
    // FixedModel fixedModel = new FixedModel(fieldCnt);
    // for (int i = 0; i < fieldCnt; i++)
    // {
    // Element fieldNode = fields.get(i);
    // int len = Integer.parseInt(getRequiredAttribute(fieldNode, "length"));
    // fixedModel.setField(i, len);
    // }
    // return fixedModel;
    // }

    public Map<String, ICounterDef> getCounterMap() {
        return counterMap;
    }

    public void setCounterMap(Map<String, ICounterDef> counterMap) {
        this.counterMap = counterMap;
    }

    /**
     * 主配置解析到内存
     * 
     * @param beanFactory
     */
    @Override
    public boolean mainFileToMemo(BeanFactory beanFactory) {
        Collection<File> mainFiles = XmlUtil.getAllFile(MAIN_PATH);
        for (File mainFile : mainFiles) {
            if (mainFile.isFile()) {
                IInterfaceDef interfaceDef = null;
                String filename = FileUtil.getFileName(mainFile.getName());
                // 新增TODO
                filename = XmlUtil.getRelativeName(mainFile, MAIN_PATH);
                try {
                    String path = getInterfaceRelativePath(filename);
                    Document doc = XmlUtil.getDocument(path);
                    Element interfaceNode = doc.getRootElement().element("interface");
                    if (interfaceNode != null) {
                        String type = interfaceNode.attributeValue("type");
                        if (!"client".equals(type)) {
                            interfaceDef = getInterfaceDef(filename);
                        }
                    }
                } catch (Exception e) {
                    logger.info("init interface [" + filename + "] failed.", e);
                    continue;
                }
                beanFactory.addInterface(filename, interfaceDef);

                logger.info("init interface [" + filename + "] success.");
            }
        }
        return true;
    }

    /**
     * 指定主配置解析到内存
     * 
     * @param beanFactory
     */
    @Override
    public boolean mainFileToMemo(BeanFactory beanFactory, String interfaceid,
            String transactionid) {
        Collection<File> mainFiles = XmlUtil.getAllFile(MAIN_PATH);
        for (File mainFile : mainFiles) {
            if (mainFile.isFile()) {
                IInterfaceDef interfaceDef = null;
                String filename = FileUtil.getFileName(mainFile.getName());
                // 新增TODO
                filename = XmlUtil.getRelativeName(mainFile, MAIN_PATH);
                if (!filename.equals(interfaceid)) continue;
                try {
                    String path = getInterfaceRelativePath(filename);
                    Document doc = XmlUtil.getDocument(path);
                    Element interfaceNode = doc.getRootElement().element("interface");
                    if (interfaceNode != null) {
                        String type = interfaceNode.attributeValue("type");
                        if (!"client".equals(type)) {
                            interfaceDef = getInterfaceDef(filename);
                        }
                    }
                } catch (Exception e) {
                    logger.info("init interface [" + filename + "] failed.", e);
                    continue;
                }
                beanFactory.addInterface(filename, interfaceDef);
                logger.info("init interface [" + filename + "] success.");
            }
        }
        return true;
    }

    public void serviceMainToMemo(BeanFactory beanFactory) {
        Document doc = XmlUtil.getDocument(Constants.ETC_PATH, "services.xml");
        Element rootNode = doc.getRootElement();
        Element interfacesNode = rootNode.element("interfaces");
        if (interfacesNode != null) {
            List<Element> interfaceNodes = interfacesNode.elements();
            for (Element interfaceNode : interfaceNodes) {
                IInterfaceDef interfaceDef = getInterfaceDef(interfaceNode);
                beanFactory.addInterface(interfaceDef.getId(), interfaceDef);
                logger.info("init interface [" + interfaceNode.attributeValue("id") + "] success.");
            }
            Serializable2FileUtil.writeObject(beanFactory.getInterfaceMap(), "interfaces");
        }
    }

    @Override
    public Watchdog createHotDeployListener() {
        return new HotDeployListener(this);
    }

    @Override
    public Watchdog createBDHotDeployListener() {
        return new BDHotDeployListener(this);
    }

    public IInterfaceDef xml2InterfaceDef(String xml) {
        try {
            Element rootNode = DocumentHelper.parseText(xml).getRootElement();
            prepareforInterface(rootNode);
            Element interfaceNode = rootNode.element("interface");
            IInterfaceDef interfaceDef = getInterfaceDef(interfaceNode);
            // 初始化接口其他信息
            initInterfaceDef(interfaceDef, rootNode);
            return interfaceDef;
        } catch (DocumentException e) {
            e.printStackTrace();
            return null;
        }
    }
}
