package com.brilliance.eibs.listener.helpers.impl;

import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.factory.parser.impl.XmlParser;
import com.brilliance.eibs.listener.helpers.Watchdog;
import com.brilliance.eibs.server.manager.EibsServiceHandle;
import com.brilliance.eibs.util.XmlUtil;
import org.slf4j.Logger;

import java.io.File;
import java.net.URL;
import java.util.*;

/**
 * 文件监控线程类
 * 
 * @author hujun
 *
 */
public abstract class FilesWatchdog extends Watchdog {

    protected List<String> pathnames;

    boolean interrupted = false;
    protected Map<String, Long> lastModifMap = new HashMap<String, Long>();

    protected EibsServiceHandle eibsServiceHandle;

    protected FilesWatchdog(List<String> pathnames, Logger log) {
        super(log);
        this.pathnames = pathnames;
        // setDaemon(true);
        // updateMap(pathnames);
        checkAndConfigure();
    }

    /**
     * 更新配置文件的修改时间
     * 
     * @param pathnames
     */
    protected void updateMap(List<String> pathnames) {
        URL url = null;
        File file = null;
        // 遍历所有监控文件或目录
        for (String pathname : pathnames) {
            // url = this.getClass().getClassLoader().getResource(pathname);
            url = CommonFunctionUtils.getCommonURL(pathname);
            if (url != null) {
                file = new File(CommonFunctionUtils.decodeURL(url.getFile()));
                long l;
                String filename;
                if (file.isDirectory()) {
                    // 非主配置文件目录
                    if (!pathname.equals(XmlParser.MAIN_PATH)) {
                        for (File f : file.listFiles()) {
                            if (f.getAbsolutePath().endsWith(".xml")) {
                                l = f.lastModified();
                                filename = f.getName();
                                addToLastModifMap(filename, l, f);
                            }
                        }
                    }
                    // 主配置文件目录
                    else {
                        Collection<File> mainFiles = XmlUtil.getAllFile(XmlParser.MAIN_PATH);
                        for (File f : mainFiles) {
                            if (f.getAbsolutePath().endsWith(".xml")) {
                                l = f.lastModified();
                                filename = XmlUtil.getRelativeName(f, XmlParser.MAIN_PATH);
                                addToLastModifMap(filename, l, f);
                            }
                        }
                    }
                }
                if (file.isFile() && file.getAbsolutePath().endsWith(".xml")) {
                    l = file.lastModified();
                    filename = file.getName();
                    lastModifMap.put(filename, l);
                }
            }
        }
    }


    public void addToLastModifMap(String filename, long l, File f) {
        if (!lastModifMap.containsKey(filename)) {
            log.info("[hot deploy]added: " + filename);
            lastModifMap.put(filename, l);
        } else if (isModified(f)) {
            log.info("[hot deploy]updated: " + filename);
            lastModifMap.put(filename, l);
        }
    }

    /**
     * 抽象定义文件改变后需要执行的具体操作
     * 
     * @param changedFileMap
     */
    protected abstract void doOnChange(Map<String, List<String>> changedFileMap);

    /**
     * 检查文件是否存在修改，以最后的修改时间作为判断依据
     */
    @Override
    public void checkAndConfigure() {
        Map<String, List<String>> changedFileMap = new HashMap<String, List<String>>();
        for (String pathname : pathnames) {
            // URL url = this.getClass().getClassLoader().getResource(pathname);
            URL url = CommonFunctionUtils.getCommonURL(pathname);
            if (url != null) {
                File file = new File(CommonFunctionUtils.decodeURL(url.getFile()));
                if (file.isDirectory()) {
                    // 非主配置文件目录
                    if (!pathname.equals(XmlParser.MAIN_PATH)) {
                        for (File f : file.listFiles()) {
                            if (f.getAbsolutePath().endsWith(".xml") && isModified(f)) {
                                addChangedFile(changedFileMap, pathname, f.getName());
                            }
                        }
                    }
                    // 主配置文件目录
                    else {
                        Collection<File> mainFiles = XmlUtil.getAllFile(XmlParser.MAIN_PATH);
                        for (File f : mainFiles) {
                            if (f.getAbsolutePath().endsWith(".xml") && isModified(f)) {
                                // addChangedFile(changedFileMap, pathname, f.getName());
                                addChangedFile(changedFileMap, pathname,
                                        XmlUtil.getRelativeName(f, XmlParser.MAIN_PATH));
                            }
                        }
                    }
                } else if (file.isFile())
                    if (file.getAbsolutePath().endsWith(".xml") && isModified(file)) {
                    addChangedFile(changedFileMap, pathname, file.getName());
                    }
            }
        }
        if (!changedFileMap.isEmpty()) doOnChange(changedFileMap);
    }

    /**
     * 添加修改过的文件记录到Map中
     * 
     * @param changedMap
     * @param path
     * @param filename
     */
    public void addChangedFile(Map<String, List<String>> changedMap, String path, String filename) {
        if (!changedMap.containsKey(path)) {
            List<String> fileNames = new ArrayList<String>();
            fileNames.add(filename);
            changedMap.put(path, fileNames);
        } else
            changedMap.get(path).add(filename);
    }

    /**
     * 判断文件是否修改，以文件最后修改时间做准
     * 
     * @param file
     * @return
     */
    protected boolean isModified(File file) {
        long l = file.lastModified();
        String filename = file.getName();
        // 新增子文件目录处理 TODO
        filename = XmlUtil.getRelativeName(file, XmlParser.MAIN_PATH);
        if (!lastModifMap.containsKey(filename)) {
            return true;
        }
        if (l > lastModifMap.get(filename)) {
            return true;
        } else
            return false;
    }


    public EibsServiceHandle getEibsServiceHandle() {
        return eibsServiceHandle;
    }

    public void setEibsServiceHandle(EibsServiceHandle eibsServiceHandle) {
        this.eibsServiceHandle = eibsServiceHandle;
    }
}
