package com.brilliance.eibs.main;

import com.brilliance.eibs.core.debug.DebugMode;
import com.brilliance.eibs.core.exception.SysCode;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.model.ITaskDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.FactoryManager;
import com.brilliance.eibs.factory.parser.ConfigParser;
import com.brilliance.eibs.factory.parser.IHotDeployParser;
import com.brilliance.eibs.factory.parser.ParserFactory;
import com.brilliance.eibs.factory.parser.impl.XmlParser;
import com.brilliance.eibs.listener.helpers.Watchdog;
import com.brilliance.eibs.monitor.StreamGobbler;
import com.brilliance.eibs.quartz.CronTask;
import com.brilliance.eibs.server.AbsServer;
import com.brilliance.eibs.server.IServerInstance;
import com.brilliance.eibs.util.*;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;

import java.io.*;
import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.lang.reflect.Constructor;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 服务端启动类
 *
 * @author xiaoyuanzhen
 */
public class Server {
    private static Logger log = LogUtil.getLogger("services");

    private static Context context = new Context();
    private static ExecutorService executorService = Executors.newCachedThreadPool();

    private static Logger logger;

    static {
        context.addVariable("transaction", Constants.Log_VAR, log);
        logger = LogUtil.getLogger(context);

    }

    public static Map<String, CronTask> taskMap = new HashMap<String, CronTask>();
    public static Watchdog listener; // 热部署监听器
    public static Watchdog bdlistener; // 热部署监听器
    public static final String PID_FILE_NAME = "cfg/pid.properties";
    public static final String MONITER = "cfg/monitor.properties";

    public static boolean startError = false;

    public static Map<String, IServerInstance> servers = new HashMap<String, IServerInstance>();

    private static Map<String, ServerSocket> innerServer = new HashMap<String, ServerSocket>();

    // public static Map<String, StopServerHandle> stopServermap = new
    // HashMap<String, StopServerHandle>();

    /**
     * 处理热部署 热部署开关打开，则进行启动
     * 热部署不启动，只调用热部署进行内存的初始化。热部署监听器构造时会默认检测一次文件，由于所有文件均不在内存中，
     * 所以会导致服务重新启动（服务处于关闭状态，即为启动所有服务）和内存初始化。
     */
    public static void dealHotDeploy(String[] args) {
        IHotDeployParser parser = checksupportedDeploy(ParserFactory.getInstance(log));// 配置解析器
        Serializable2FileUtil.hotDeploy = "on".equals(parser.getHotDeployState()) ? true : false;
        // 热部署开关打开，则进行启动

        if (Serializable2FileUtil.hotDeploy) {
            try {
                startHotDeploy(parser);
            } catch (InterruptedException e) {
                logger.info("hot deploy abort.");
            }
        } else {// 热部署不启动，只调用热部署进行内存的初始化。热部署监听器构造时会默认检测一次文件，由于所有文件均不在内存中，所以会导致服务重新启动（服务处于关闭状态，即为启动所有服务）。
            if (parser instanceof XmlParser)
                if (listener == null)
                    listener = createHotDeloDeployListener(parser);
                else
                    listener.checkAndConfigure();
            /*
             * else if (parser instanceof DbParser) startServersFromDb(args);
             */
        }
    }

    private static IHotDeployParser checksupportedDeploy(ConfigParser parser) {
        if (!(parser instanceof IHotDeployParser))
            throw new UnsupportedOperationException(parser.getClass() + " does not support hot deployment.");
        return (IHotDeployParser) parser;
    }

    public static void esbHotDeploy() {
        IHotDeployParser parser = checksupportedDeploy(ParserFactory.getInstance(log));// 配置解析器
        try {
            createHotDeloDeployListener(parser).esbrun();
        } catch (Exception e) {
            logger.info("hot deploy abort.");
        }
    }

    public static void dealBDHotDeploy() {
        IHotDeployParser parser = checksupportedDeploy(ParserFactory.getInstance(log));// 配置解析器
        Serializable2FileUtil.hotDeploy = "on".equals(parser.getHotDeployState()) ? true : false;
        // 热部署开关打开，则进行启动
        if (Serializable2FileUtil.hotDeploy) {
            try {
                bdlisntenerSart(parser);
            } catch (Exception e) {
                logger.error("start error:", e);
                logger.info("hot deploy abort.");
            }
        }
    }

    /**
     * 初始化内存
     *
     * @param parser
     * @param beanFactory
     */
    public static void initBeanFactoryMemory(ConfigParser parser, BeanFactory beanFactory) {
        parser.mainFileToMemo(beanFactory);
    }

    /*
     * public static void initBeanFactoryMemory(ConfigParser parser,ServiceDef
     * serviceDef, BeanFactory beanFactory) {
     * parser.mainFileToMemo(beanFactory); }
     */

    /**
     * 开启热部署监听
     *
     * @param parser
     * @throws InterruptedException
     */
    public static void startHotDeploy(IHotDeployParser parser) throws InterruptedException {
        runListeners(parser);
    }

    /**
     * 开启定时任务
     *
     * @param taskDef
     */
    public static void startTask(ITaskDef taskDef) {

        CronTask cronTask = new CronTask(taskDef);
        if (taskDef.getId().startsWith("_")) {
            executorService.execute(cronTask);
        } else
            cronTask.start();
        taskMap.put(taskDef.getId(), cronTask);
        logger.info("start task [" + taskDef.getId() + "]");
    }

    public static void startTask(ITaskDef taskDef, Object[] args) {
        CronTask cronTask = new CronTask(taskDef, args);
        cronTask.start();
        taskMap.put(taskDef.getId(), cronTask);
        logger.info("start task [" + taskDef.getId() + "]");
    }

    /**
     * 根据id关闭指定定时任务
     *
     * @param id
     */
    public static void closeTask(String id) {
        CronTask cronTask = taskMap.get(id);
        if (cronTask != null) {
            cronTask.interrupt();
            System.out.println("close task [" + id + "]");
        }
        taskMap.remove(id);
    }

    private synchronized static String dealMsg(String str, String hostname) {
        String desp = "failed exception ,Please check !";
        String line = System.getProperty("line.separator");
        Context context = new Context();
        // context.setBeanFactory(FactoryManager.getBeanFactory());
        // context.setLogger(logger);
        context.addVariable("transaction", Constants.Log_VAR, log);
        ConfigParser parser = null;
        parser = ParserFactory.getInstance(log);

        logger.info("message info=" + str);
        String args[] = str.split(" ");
        for (int i = 0; i < args.length; i++) {
            args[i] = args[i].trim();
        }

        if (args[0].equals("startall")) {
            /*
             * //利用热部署进行服务的启动和内存的初始化 dealHotDeploy();
             */
            List<IServiceDef> serviceDefs = parser.getServiceDefs("services");
            for (IServiceDef serviceDef : serviceDefs) {
                startServer(serviceDef);
            }
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            for (ITaskDef taskDef : taskDefs) {
                startTask(taskDef);
            }
            desp = "services start up success";
        } else if (args[0].equals("stopall")) {
            List<IServiceDef> serviceDefs = parser.getServiceDefs("services");
            for (IServiceDef serviceDef : serviceDefs) {
                String id = serviceDef.getId();
                if (servers.containsKey(id)) {
                    closeServer(id);
                }
            }
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            for (ITaskDef taskDef : taskDefs) {
                String id = taskDef.getId();
                closeTask(id);
            }
            desp = "services stop success";
        } else if (args[0].equals("serverlist") || str.equals("sl")) {
            List<IServiceDef> serviceDefs = parser.getServiceDefs("services");
            StringBuffer sb = new StringBuffer();
            sb.append(String.format("%-30s%-10s%-40s", "service", "state", "description") + line);
            sb.append("--------------------------------------------------------------------------------" + line);
            for (IServiceDef serviceDef : serviceDefs) {
                String id = serviceDef.getId();
                sb.append(String.format("%-30s", id));
                if (servers.containsKey(id))
                    sb.append(String.format("%-10s", "on"));
                else
                    sb.append(String.format("%-10s", "off"));
                sb.append(serviceDef.getDescription() + line);
            }
            desp = sb.toString();
        } else if (args[0].equals("start")) {
            String id = args[1];
            IServiceDef serviceDef = parser.getServiceDef(id);
            if (serviceDef != null) {
                startServer(serviceDef);
                desp = ("service [" + id + "] start success.");
            } else {
                desp = ("service [" + id + "] not exist.");
            }
        } else if (args[0].equals("stop")) {
            String id = args[1];
            IServiceDef serviceDef = parser.getServiceDef(id);
            if (serviceDef != null) {
                closeServer(id);
                desp = ("service [" + id + "] stop success.");
            } else {
                desp = ("service [" + id + "] not exist.");
            }
        } else if (args[0].equals("starttask")) {
            String id = args[1];
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            boolean start = false;
            for (ITaskDef taskDef : taskDefs) {
                if (taskDef.getId().equals(id)) {
                    startTask(taskDef);
                    start = true;
                    break;
                }
            }
            if (start == false) {
                desp = "task [" + id + "] not exists";
            } else {
                desp = "task [" + id + "] started";
            }
        } else if (args[0].equals("stoptask")) {
            String id = args[1];
            if (taskMap.containsKey(id)) {
                closeTask(id);
                desp = "task [" + id + "] closed successfully";
            } else
                desp = "task [" + id + "] not started";
        } else if (args[0].equals("tasklist") || args[0].equals("tl")) {
            StringBuffer sb = new StringBuffer();
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            sb.append(String.format("%-30s%-10s%-40s", "tasks", "state", "description") + line);
            sb.append("--------------------------------------------------------------------------------" + line);
            for (ITaskDef taskDef : taskDefs) {
                String id = taskDef.getId();
                sb.append(String.format("%-30s", id));
                if (taskMap.containsKey(id))
                    sb.append(String.format("%-10s", "on"));
                else
                    sb.append(String.format("%-10s", "off"));
                sb.append(taskDef.getDescription() + line);
            }
            desp = sb.toString();
        } else if (args[0].equals("exit")) {
            desp = "exit Interface Platform success.";
            // System.exit(0);
        } else if (args[0].equals("init")) {
            if (!Serializable2FileUtil.hotDeploy) {
                IHotDeployParser hotDeployParser = checksupportedDeploy(ParserFactory.getInstance(log));// 配置解析器
                createHotDeloDeployListener(hotDeployParser).checkAndConfigure();
                desp = "init memory:" + FactoryManager.curFactoryNum;
                if (Serializable2FileUtil.hotDeploy)
                    runListeners(hotDeployParser);
            } else {
                desp = "this action is not activated.";
            }
        } else if (args[0].equals("-help")) {
            StringBuffer sb = new StringBuffer();
            sb.append("----------------------------新晨接口平台服务控制台中心--------------------------" + line);
            sb.append(String.format("%-40s%-40s", "command（命令）", "description(描述)") + line);
            sb.append("--------------------------------------------------------------------------------" + line);
            sb.append(String.format("%-40s", "startall")).append(String.format("%-40s", "启动所有服务(包括定时服务)") + line);
            sb.append(String.format("%-40s", "stopall")).append(String.format("%-40s", "关闭所有服务(包括定时服务)") + line);
            sb.append(String.format("%-40s", "start xxx")).append(String.format("%-40s", "启动某个服务(xxx为服务id)") + line);
            sb.append(String.format("%-40s", "stop xxx")).append(String.format("%-40s", "关闭某个服务(xxx为服务id)") + line);
            sb.append(String.format("%-40s", "serverlist(sl)")).append(String.format("%-40s", "查看服务列表") + line);
            sb.append(String.format("%-40s", "starttask xxx")).append(String.format("%-40s", "启动某个定时服务(xxx为定时服务id)") + line);
            sb.append(String.format("%-40s", "stoptask xxx")).append(String.format("%-40s", "关闭某个定时服务(xxx为定时服务id)") + line);
            sb.append(String.format("%-40s", "tasklist(tl)")).append(String.format("%-40s", "查看定时服务列表") + line);
            sb.append(String.format("%-40s", "exit")).append(String.format("%-40s", "关闭接口平台服务进程") + line);
            sb.append(String.format("%-40s", "init")).append(String.format("%-40s", "内存切换(热部署)") + line);
            sb.append(String.format("%-40s", "-help")).append(String.format("%-40s", "查看相关命令解释") + line);
            sb.append(String.format("%-40s", "quit")).append(String.format("%-40s", "退出接口平台服务控制台中心") + line);
            sb.append(String.format("%-40s", "switch all")).append(String.format("%-40s", "更新内存中所有服务") + line);
            sb.append(String.format("%-40s", "switch interfaceid transactionid")).append(String.format("%-40s", "更新内存中指定服务") + line);
            sb.append("--------------------------------------------------------------------------------");
            desp = sb.toString();
        } else if (args[0].equals("monitor")) {
            desp = callShell(args);
        } else if (args[0].equals("debug")) {

        } else if (args[0].startsWith("switch")) {
            BeanFactory beanFactory = FactoryManager.getNextBeanFactory();
            if (args[1].equals("all")) {
                if (!parser.mainFileToMemo(beanFactory))
                    return desp = "switch all fail";
                FactoryManager.switchNext();
                desp = "switch all Successful implementation.";
            } else if (args.length == 3) {
                if (!parser.mainFileToMemo(beanFactory, args[1], args[2]))
                    return desp = "switch " + args[1] + " " + args[2] + " fail";
                FactoryManager.switchNext();
                desp = "switch " + args[1] + " " + args[2] + " Successful implementation";
            } else {
                desp = "No such command,please check your command !";
            }
        } else {
            desp = "No such command,please check your command !";
        }
        return desp;
    }

    public static void main(String[] args) {
        String same_log = PropertyUtil.getProperty("sys.properties", "SAME_LOG");
        if (!StringUtil.isEmpty(same_log) && Boolean.valueOf(same_log)) {
            DateUtil.CURRENT_DATE = DateUtil.formatNow("yyyyMMdd");
        }
        // 检测是否debug模式
        checkDebugMode();
        dealHotDeploy(args);
        // startServers();
        // 当前服务进程号写入到文件中，用于监控系统使用
        if (!startError) {
            writeCurrentPidToFile();
            dealInnerServer(args);
        }
        printSysInfo();
    }

    private static void printSysInfo() {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PrintStream ps = new PrintStream(baos, true);
        System.getProperties().list(ps);
        logger.info("--打印jvm默认系统参数--");
        logger.info(baos.toString());
        try {
            ps.close();
            baos.close();
        } catch (IOException e) {
            logger.error("关闭IO流异常", e);
        }
        //返回java虚拟机中的内存总量
        long totalMemory = Runtime.getRuntime().totalMemory();
        //返回java虚拟机试图使用的最大内存量
        long maxMemory = Runtime.getRuntime().maxMemory();
        logger.info("Total_Memory(-Xms ) =  " + totalMemory + " 字节  " + (totalMemory / (double) 1024 / 1024) + "MB");
        logger.info("Max_Memory(-Xmx ) =  " + maxMemory + " 字节  " + (maxMemory / (double) 1024 / 1024) + "MB");
    }

    /**
     * 通过sys.properties的DEBUG判断是否是debug模式 如果是需要在DebugMode中注册
     */
    private static void checkDebugMode() {
        boolean isDebug = Boolean.valueOf(PropertyUtil.getProperty("sys.properties", "DEBUG"));
        if (isDebug)
            DebugMode.singleInstance().register(DebugMode.DEBUG_STATUS);
    }

    public static void start() {
        String port = PropertyUtil.getProperty("cfg/beainner.properties", "port");
        if (StringUtil.isEmpty(port)) {
            logger.error("port is null,please check cfg/beainner.properties");
            return;
        }
        main(new String[]{port});
    }

    public static void dealInnerServer(String args[]) {
        String usr_port_flg = PropertyUtil.getProperty("sys.properties", "USR_SVR_PORT");
        if (StringUtil.isEmpty(usr_port_flg)) {
            usr_port_flg = "true";
        }
        boolean usr_svr_flg = Boolean.valueOf(usr_port_flg);
        if (!usr_svr_flg)
            return;

        ServerSocket serverSocket = null;
        if (args == null || args.length == 0) {
            logger.error("Please give port of ServerManager!");
            return;
        }
        String strport = args[0];
        int port = Integer.parseInt(strport);
        try {
            serverSocket = new ServerSocket(port);
            innerServer.put(strport, serverSocket);
            Socket socket = null;
            while (true) {
                socket = serverSocket.accept();
                byte[] buffer = new byte[4];

                IOUtils.readFully(socket.getInputStream(), buffer);
                int len = Integer.parseInt(new String(buffer, "GBK"));
                logger.info("message len is" + len);
                buffer = new byte[len];
                IOUtils.readFully(socket.getInputStream(), buffer);

                String desp = "";
                String str = new String(buffer).trim();
                try {
                    desp = dealMsg(str, socket.getInetAddress().getHostAddress());
                    buffer = desp.getBytes();
                    if (buffer.length != 0) {
                        IOUtils.write(String.format("%04d", buffer.length), socket.getOutputStream());
                        IOUtils.write(buffer, socket.getOutputStream());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    String exceptionStr = "Warning:if the command contains whitespace ,please check whether \" contained !";
                    buffer = exceptionStr.getBytes();
                    IOUtils.write(String.format("%04d", buffer.length), socket.getOutputStream());
                    IOUtils.write(buffer, socket.getOutputStream());
                } finally {
                    IOUtils.closeQuietly(socket);
                    logger.info("socket is closed");
                    if ("exit Interface Platform success.".equals(desp)) {
                        System.exit(0);
                    }
                }
            }
        } catch (Exception e) {
            if ((e instanceof SocketException) && e.getMessage().equals("socket closed"))
                ;
            else
                e.printStackTrace();
        } finally {

        }
    }

    /**
     * 创建热部署文件监听器
     *
     * @param xmlParser
     * @return
     */
    private static Watchdog createHotDeloDeployListener(IHotDeployParser parser) {
        if (listener == null) {
            listener = parser.createHotDeployListener();
        }
        return listener;
    }

    private static void runListeners(IHotDeployParser parser) {
        createHotDeloDeployListener(parser).start();
    }

    private synchronized static void bdlisntenerSart(IHotDeployParser parser) {
        if (bdlistener == null) {
            bdlistener = parser.createBDHotDeployListener();
        }
        if (bdlistener.getState().equals(Thread.State.NEW))
            bdlistener.start();
    }

    /**
     * Java获取当前服务进程PID
     **/
    public static String getServersPid() {
        String pidName = null;
        RuntimeMXBean runtime = ManagementFactory.getRuntimeMXBean();
        if (runtime != null) {
            pidName = runtime.getName();
        }
        return pidName;
    }

    /**
     * 将当前进程号写入文件中
     **/
    public static void writeCurrentPidToFile() {
        FileOutputStream fos = null;
        try {
            String pidname = getServersPid();
            int index = pidname.lastIndexOf("@");
            if (index != -1) {
                pidname = pidname.substring(0, index);
            }
            if (!StringUtil.isEmpty(pidname)) {
                // String path =
                // Thread.currentThread().getContextClassLoader().getResource("").getPath()
                // + "/" +
                // PID_FILE_NAME;
                String path = CommonFunctionUtils.getCommonURL("").getPath() + "/" + PID_FILE_NAME;
                path = CommonFunctionUtils.decodeURL(path);
                File file = new File(path);
                if (!file.getParentFile().exists()) {
                    file.getParentFile().mkdirs();
                }
                if (!file.exists())
                    file.createNewFile();
                fos = new FileOutputStream(file);
                IOUtils.write("pid=" + pidname, fos);
            }
        } catch (IOException e) {
            logger.error("interfacePlatform Write PID error :", e);
        } finally {
            IOUtils.closeQuietly(fos);
        }
    }

    /**
     * 调用脚本
     *
     * @param args
     * @return
     */
    public static String callShell(String[] args) {
        String desp = null;
        String shellname = "";
        if (args[1].equals("start")) {
            shellname = PropertyUtil.getProperty(MONITER, "start");
        } else if (args[1].equals("stop")) {
            shellname = PropertyUtil.getProperty(MONITER, "stop");
        }
        int lindex = shellname.lastIndexOf(".");
        String laststr = shellname.substring(lindex + 1, shellname.length()).trim();
        if ("sh".equals(laststr)) {
            args[0] = "sh";
            args[1] = shellname;
            desp = excuteShell(args);
        } else {
            String cmdargs[] = new String[args.length + 3];
            cmdargs[0] = "cmd";
            cmdargs[1] = "/c";
            cmdargs[2] = "start";
            for (int i = 3; i < cmdargs.length; i++) {
                cmdargs[i] = args[i - 2];
            }
            desp = excuteShell(cmdargs);
        }
        return desp;
    }

    /**
     * 脚本管理服务
     *
     * @param cmmand
     * @return
     */
    public static String excuteShell(String cmmand[]) {
        String desp = "monitor startup failed";
        InputStream in = null;
        InputStream errorin = null;
        StringBuffer stringBuffer = new StringBuffer();
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss ");
        try {
            stringBuffer.append(dateFormat.format(new Date())).append("准备执行Shell命令 ");
            for (String str : cmmand) {
                logger.info("cmand parameter:" + str);
                stringBuffer.append(str + " ");
            }
            stringBuffer.append("\r\n");
            Process proc = null;
            Runtime runtime = Runtime.getRuntime();
            proc = runtime.exec(cmmand);
            // 执行Shell命令
            if (proc != null) {
                stringBuffer.append("进程号：").append(proc.toString()).append("\r\n");
                StreamGobbler errorGobbler = new StreamGobbler(proc.getErrorStream(), "ERROR", log);
                StreamGobbler outputGobbler = new StreamGobbler(proc.getErrorStream(), "ERROR", log);
                errorGobbler.start();
                outputGobbler.start();
                int a = proc.waitFor();
                logger.info("pid flag=" + a);
                if (a == 0) {
                    desp = "monitor startup successed";
                }
            } else {
                stringBuffer.append("没有pid\r\n");
            }
        } catch (Exception ioe) {
            logger.info("ioeerror", ioe);
            stringBuffer.append("执行Shell命令时发生异常：\r\n").append(ioe.getMessage()).append("\r\n");
        } finally {
            logger.info("shell mssage:" + stringBuffer.toString());
            IOUtils.closeQuietly(in);
            IOUtils.closeQuietly(errorin);
        }
        return desp;
    }

    /**
     * 关闭单个服务，根据id区分
     *
     * @param id
     */
    public synchronized static void closeServer(String id) {
        AbsServer server = (AbsServer) servers.get(id);

        if (server != null) {
            server.close();
            servers.remove(id);
        }
    }

    /**
     * 启动所有服务（包括定时任务）
     */
    public static void startServers() {
        final ConfigParser parser = ParserFactory.getInstance(log);// 配置解析器
        List<IServiceDef> serviceDefs = parser.getServiceDefs("services");// 获取所有服务定义
        Serializable2FileUtil.writeObject(serviceDefs, "services");// 保存为序列化文件，用于热部署
        try {
            for (IServiceDef serviceDef : serviceDefs)// 启动services
            {
                if (serviceDef.getState().equals("on") && !servers.containsKey(serviceDef.getId()))
                    startServer(serviceDef);
            }
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            Serializable2FileUtil.writeObject(taskDefs, "tasks");
            for (ITaskDef taskDef : taskDefs)// 启动tasks
            {
                if (taskDef.getState().equals("on") && !taskMap.containsKey(taskDef.getId())) {
                    startTask(taskDef);
                }
            }

            initBeanFactoryMemory(parser, FactoryManager.getBeanFactory());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 启动一个服务
     *
     * @param serviceDef
     */
    public static synchronized void startServer(IServiceDef serviceDef, Object... args) {
        ApacheEL elParser = new ApacheEL(context);
        AbsServer server = null;
        try {
            String type = serviceDef.getType();
            String id = serviceDef.getId();
            if ("webservice".equals(type)) {
                logger.info("[starting] webservice " + id);
                server = startServer("com.brilliance.eibs.server.WSServer", "WebService '" + id + "' Service Thread", serviceDef, elParser, args);
                logger.info("[started] webservice " + id);
            } else if ("socket".equals(type)) {
                logger.info("[starting] socketservice " + id);
                server = startServer("com.brilliance.eibs.server.SocketServer", "Socket '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] socketservice " + id);
            } else if ("shortSocket".equals(type)) {
                logger.info("[starting] socketservice " + id);
                server = startServer("com.brilliance.eibs.server.ShortSocketServer", "Socket '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] socketservice " + id);
            } else if ("http".equals(type)) {
                logger.info("[starting] HttpServer " + id);
                server = startServer("com.brilliance.eibs.server.HttpServer", "HttpServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] HttpServer " + id);
            } else if ("https".equals(type)) {
                logger.info("[starting] HttpsService " + id);
                server = startServer("com.brilliance.eibs.server.HttpsService", "HttpsService '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] HttpsService " + id);
            } else if ("udp".equals(type)) {
                logger.info("[starting] UdpServer " + id);
                server = startServer("com.brilliance.eibs.server.UdpServer", "UdpServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] UdpServer " + id);
            } else if ("init".equals(type)) {
                logger.info("[starting] InitServer " + id);
                server = startServer("com.brilliance.eibs.server.InitServer", "InitServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] InitServer " + id);
            } else if ("link".equals(type)) {
                logger.info("[starting] LinkServer " + id);
                server = startServer("com.brilliance.eibs.server.LinkServer", "LinkServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] LinkServer " + id);
            } else if ("snmp".equals(type)) {
                logger.info("[starting] SnmpServer " + id);
                server = startServer("com.brilliance.eibs.server.SnmpServer", "SnmpServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] SnmpServer " + id);
            } else if ("channel".equals(type)) {
                logger.info("[starting] ChannelServer " + id);
                server = startServer("com.brilliance.eibs.server.ChannelServer", "ChannelServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] ChannelServer " + id);
            } else if ("terminal".equals(type)) {
                logger.info("[starting] TerminalServer " + id);
                server = startServer("com.brilliance.eibs.server.TerminalServer", "TerminalServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] TerminalServer " + id);
            } else if ("tibco".equals(type)) {
                logger.info("[starting] tibco " + id);
                server = startServer("com.brilliance.eibs.server.TibcoJmsServer", "tibco '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] TibcoServer " + id);
            } else if ("rocketmq".equals(type)) {
                logger.info("[starting] rocketmq " + id);
                server = startServer("com.brilliance.eibs.server.RocketMQServer", "RocketMQServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] RocketMQServer " + id);
            } else if ("rocketmqons".equals(type)) {
                logger.info("[starting] rocketmqons " + id);
                server = startServer("com.brilliance.eibs.server.RocketMQServerOns", "RocketMQServerOns '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] RocketMQServer " + id);
            } else if ("jms".equals(type)) {
                logger.info("[starting] jms listener " + id);
                server = startServer("com.brilliance.eibs.server.JmsListeningServer", "JmsListeningServer '" + id + "' Server Thread", serviceDef, elParser, args);
                logger.info("[started] jms listener " + id);
            }
            servers.put(id, server);
        } catch (Exception e) {
            SysCode sysid = ExceptionUtil.getErrorCode(context, e);
            String id = sysid.getId();
            String errortext = sysid.getErrorText();
            logger.error(errortext, e);
            context.getResource().setDescrption(id);
            context.getResource().setDescription(id);
        }
    }

    /**
     * 关闭所有服务
     */
    public static synchronized void closeAllServers() {
        Iterator<String> it = servers.keySet().iterator();
        while (it.hasNext()) {
            IServerInstance instance = servers.get(it.next());
            instance.close();
        }
        servers.clear();

        // modified by gechengyang 20150915
        Iterator<String> ittask = taskMap.keySet().iterator();
        while (ittask.hasNext()) {
            String key = ittask.next();
            closeTask(key);
        }
        taskMap.clear();

        StaticInfomation.clear();

    }

    /**
     * 关闭所有服务
     */
    public static synchronized void closeAllInnerServers() {
        Iterator<String> it = innerServer.keySet().iterator();
        while (it.hasNext()) {
            ServerSocket instance = innerServer.get(it.next());
            try {
                if (instance != null)
                    instance.close();
            } catch (IOException e) {
                logger.error("close inner socket error:", e);
            }
        }
        innerServer.clear();

    }

    /**
     * 启动服务
     *
     * @param serverClassName
     * @param threadName
     * @param serviceDef
     * @param elParser
     * @param args
     * @return
     */
    private static AbsServer startServer(String serverClassName, String threadName, IServiceDef serviceDef, ApacheEL elParser, Object... args) {
        AbsServer server = null;
        try {
            Constructor<?> constructor = Class.forName(serverClassName).getConstructor(context.getClass(), IServiceDef.class, elParser.getClass());
            server = (AbsServer) constructor.newInstance(context, serviceDef, elParser);
            server.setName(threadName);
            server.setArgs(args);
            server.start();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return server;
    }
}
