package com.brilliance.eibs.main.client;

import com.brilliance.eibs.core.debug.model.DebugIdentity;
import com.brilliance.eibs.core.frame.ISocketProxy;
import com.brilliance.eibs.core.frame.ISocketProxyBuilder;
import com.brilliance.eibs.core.frame.SocketProxyFactry;
import com.brilliance.eibs.core.model.ITransactionDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.core.service.instance.plugin.SocketProxy;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.*;

import java.io.IOException;
import java.net.Socket;
import java.util.concurrent.Callable;

/**
 * 提供给Socket服务端调用的Client.call
 *
 * @author xiaoyuanzhen ------------modified by weicong on 2016/4/01 -----------
 * 兼容sslsocketserver
 */
public class SocketClient extends Client implements Callable<ResultMsg> {
    private String interfaceName;
    private String transactionName;
    private Socket socket;
    private Object[] args;

    public SocketClient(String interfaceName, String transactionName, Socket socket, Object... args) {
        this.interfaceName = interfaceName;
        this.transactionName = transactionName;
        this.socket = socket;
        this.args = args;
    }

    @Override
    public void execute(ITransactionDef transDef, final Context context) {
        ISocketProxy socketProxy = null;
        try {
            socketProxy = SocketProxyFactry.getProxy(new ISocketProxyBuilder() {
                @Override
                public ISocketProxy buildSocketProxy() throws IOException {
                    return new SocketProxy(socket, null, context);
                }
            });
        } catch (IOException e) {
            LogUtil.getLogger(context).error(e.getMessage(), e);
        }
        context.setIdentity(new DebugIdentity(socket.getInetAddress().getHostAddress(), interfaceName, transactionName));
        context.getResource().setSocketProxy(socketProxy);
        super.execute(transDef, context);
    }

    @Override
    public ResultMsg call() throws Exception {
        String same_log = PropertyUtil.getProperty("sys.properties", "SAME_LOG");
        if (!StringUtil.isEmpty(same_log) && Boolean.valueOf(same_log)) {
            Context subcontext = new Context();
            subcontext.addVariable("transaction", Constants.Log_VAR, LogUtil.getTrnLog(DateUtil.CURRENT_DATE, interfaceName, transactionName));
            return call(subcontext, interfaceName, transactionName, args);
        } else
            return call(interfaceName, transactionName, args);
    }

}
