package com.brilliance.eibs.server;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.main.client.SocketClient;
import org.apache.commons.io.IOUtils;

import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class AliveSocketServer extends AbsServer {
	private boolean isOpen = true;
	private static int connCount = 0;
	private ServerSocket serverSocket;
	List<String> trustips = new ArrayList<String>();

	public AliveSocketServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
		super(context, serviceDef, elParser);
	}

	public synchronized void incConnectionCount() {
		connCount++;
		logger.info( "Connection count: " + connCount);
	}

	public synchronized void decConnectionCount() {
		connCount--;
		logger.info( "Connection count: " + connCount);
	}

	@Override
	public void run() {
		logger.info( "Running SocketClientConnection.");
		try {
			serverSocket = (ServerSocket) context.getObject();
			if (serverSocket != null && !serverSocket.isClosed()) {
				// 获取上下文中的线程池大小
				int size = Integer.valueOf(((String) context.getVariable(THREAD_POOL_SIZE_SYMBOL)));
				String trustips = (String) context.getVariable(TRUST_IP_SYMBOL);
				ExecutorService executorService = Executors.newFixedThreadPool(size);
				Socket socket = null;
				int timeout = (Integer) context.getVariable(RECEIVE_TIMEOUT_SYMBOL);
				String interfaceName = getRequiredPropertyValue("interfaceName");
				String transName = getRequiredPropertyValue("transactionName");
				getTrustIp(trustips);
				while (isOpen) {
					socket = serverSocket.accept();
					socket.setSoTimeout(timeout);
					String ip = socket.getInetAddress().getHostAddress();
					logger.debug( "Client socket from " + socket.getInetAddress() + " : " + socket.getPort() + ", timeout=" + timeout);
					if ("*".equals(trustips) || trustips.contains(ip)) {
						Future<ResultMsg> future = executorService.submit(new SocketClient(interfaceName, transName, socket));
						incConnectionCount();
					}

					else {
						logger.debug( "[" + ip + "] is not trusted ip. ");
						IOUtils.closeQuietly(socket);
					}
				}
			}
		} catch (Exception e) {
			throw new InterfaceException("00017", "SocketServerFilter exception occurs.", e);
		} finally {
			if (!serverSocket.isClosed())
				close();
		}
	}

	/**
	 * 获取允许
	 * 
	 * @param trustips
	 */
	private void getTrustIp(String trustips) {
		String[] trusts = trustips.split(",");
		for (String trust : trusts)
			this.trustips.add(trust);
	}

	/**
	 * 设置超时时间，包括连接超时，返回超时
	 */
	protected void setTimeOut() {
		setConnectTimeOut();
	}

	/**
	 * 获取连接超时时间，如果为空，则使用默认时间
	 */
	protected void setConnectTimeOut() {
		String connect_timeout = getPropertyValue(CONNECT_TIMEOUT_SYMBOL);
		if (connect_timeout != null) {
			int connectTimeout = Integer.valueOf(connect_timeout);
			if (connectTimeout != 0) {
				this.connectTimeout = connectTimeout;
			}
			logger.debug( "connect timeout = " + this.connectTimeout);
		}
	}

	@Override
	public void close() {
		this.isOpen = false;
		try {
			IOUtils.closeQuietly(serverSocket);
		} catch (Exception e) {
			logger.error( "Close ServerSocket exception occurs.", e);
		}
	}

}
