package com.brilliance.eibs.server.channel.model.impl;

import com.brilliance.eibs.server.channel.model.Channel;
import com.brilliance.eibs.server.channel.server.ChannelServer;
import com.brilliance.eibs.server.connection.Connection;
import com.brilliance.eibs.server.connection.Detect;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class AsyncChannel
    implements Channel
{

  private Connection    in;
  private Connection    out;
  private ChannelServer server;

  private Detect send;
  private Detect receive;

  public AsyncChannel( Connection in, Connection out, ChannelServer server )
  {
    this.in = in;
    this.out = out;
    this.server = server;
  }

  @Override
  public void transfer()
  {
    send = new AsyncDetect(Direction.REQUEST, in, out);
    receive = new AsyncDetect(Direction.RESPONSE, out, in);
    Thread t = new Thread(send);
    t.setName("Asynchronous " + Thread.currentThread().getName() + " Channel '" + hashCode() + "' Send Detect");
    t.setDaemon(true);
    t.start();
    t = new Thread(receive);
    t.setName("Asynchronous " + Thread.currentThread().getName() + " Channel '" + hashCode() + "' Receive Detect");
    t.setDaemon(true);
    t.start();
  }

  protected void detailTransfer(Connection in, Connection out, Direction direction)
      throws IOException
  {
    InputStream is = in.getInputStream();
    OutputStream os = out.getOutputStream();
    byte[] b = new byte[1];
    IOUtils.readFully(is, b);
    IOUtils.write(b, os);
    int count = is.available();
    if (count > 0)
    {
      b = new byte[count];
      IOUtils.readFully(is, b);
      IOUtils.write(b, os);
    }

  }

  protected static enum Direction
  {
    REQUEST, RESPONSE
  }

  private class AsyncDetect
      extends Detect
  {
    private Direction  direction;
    private Connection in;
    private Connection out;

    public AsyncDetect( Direction direction, Connection in, Connection out )
    {
      this.direction = direction;
      this.in = in;
      this.out = out;
    }

    @Override
    protected void handle()
        throws IOException
    {
      detailTransfer(in, out, direction);
    }

    @Override
    protected void close()
    {
      in.close();
      out.close();
      server.getPool().remove(AsyncChannel.this);
    }

    @Override
    protected void exception()
    {
      interrupted();
    }

  }

  @Override
  public void shutdown()
  {
    send.interrupted();
    receive.interrupted();
    server.getPool().remove(this);
  }

  @Override
  public boolean equals(Object obj)
  {
    return EqualsBuilder.reflectionEquals(this, obj);
  }

  @Override
  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

}
