package com.brilliance.eibs.server.channel.server.impl;

import com.brilliance.eibs.server.channel.model.Channel;
import com.brilliance.eibs.server.channel.model.impl.AsyncChannel;
import com.brilliance.eibs.server.channel.server.ChannelServer;
import com.brilliance.eibs.server.connection.Connection;
import com.brilliance.eibs.server.connection.impl.DuplexConnection;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AsyncChannelServer
    implements ChannelServer
{

  private int    port;
  private int    destPort;
  private String destIP;

  private List<Channel> pool = new ArrayList<Channel>();;

  private boolean isInterrupted = false;

  public AsyncChannelServer( int port, int destPort, String destIP )
  {
    this.port = port;
    this.destPort = destPort;
    this.destIP = destIP;
  }

  protected Channel createChannel(Connection in, Connection out)
  {
    return new AsyncChannel(in, out, this);
  }

  @Override
  public void startup()
      throws IOException
  {
    ServerSocket server = new ServerSocket();
    server.setReuseAddress(true);
    server.bind(new InetSocketAddress(port));

    while (!isInterrupted)
    {
      Socket receive = server.accept();
      Connection in = new DuplexConnection(receive);
      Socket send = new Socket();
      send.connect(new InetSocketAddress(destIP, destPort));
      Connection out = new DuplexConnection(send);
      Channel channel = createChannel(in, out);
      channel.transfer();
      pool.add(channel);
    }
  }

  @Override
  public List<Channel> getPool()
  {
    return pool;
  }

  @Override
  public void close()
  {
    isInterrupted = true;
    for (Iterator<Channel> it = pool.iterator(); it.hasNext();)
    {
      Channel c = it.next();
      c.shutdown();
    }
  }

}
