package com.brilliance.eibs.server.link_v2_0;

import org.apache.commons.lang.ArrayUtils;

import java.io.IOException;
import java.util.Arrays;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.Lock;

public class LinkDispatch
    implements Runnable
{
  private boolean isInterrupted = false;
  private long    interval      = 5000;

  private Link link;

  private int    headTagLen;
  private byte[] fin;

  private String  interfaceName;
  private String  transcationId;
  private boolean isReturn;

  public int getHeadTagLen()
  {
    return headTagLen;
  }

  public String getInterfaceName()
  {
    return interfaceName;
  }

  public String getTranscationId()
  {
    return transcationId;
  }

  @SuppressWarnings("unused")
  private LinkDispatch()
  {}

  public LinkDispatch( Link link, String interfaceName, String transcationId )
  {
    this(link, interfaceName, transcationId, true);
  }

  public LinkDispatch( Link link, String interfaceName, String transcationId, boolean isReturn )
  {
    this.link = link;
    this.interfaceName = interfaceName;
    this.transcationId = transcationId;
    this.isReturn = isReturn;
  }

  public Link getLink()
  {
    return link;
  }

  public void setHeadTagLen(int headTagLen)
  {
    this.headTagLen = headTagLen;
  }

  public void setFin(byte[] fin)
  {
    this.fin = fin;
  }

  public void buildLink()
  {
    try
    {
      link.build();
    }
    catch (IOException e)
    {
      interrupted();
    }
    catch (InterruptedException e)
    {
      interrupted();
    }
  }

  public boolean isReturn()
  {
    return isReturn;
  }

  private void sendFin()
  {
    if (link != null && link.isReady() && fin != null)
    {
      int len = fin.length;
      String s = String.format("%0" + headTagLen + "d", len);
      byte[] result = ArrayUtils.addAll(s.getBytes(), fin);
      try
      {
        link.write(result);
      }
      catch (IOException e)
      {}
      finally
      {
        interrupted();
      }
    } //fin 不计算长度域
  }

  @Override
  public void run()
  {
    buildLink();

    ExecutorService executorService = Executors.newCachedThreadPool();

    while (!isInterrupted)
    {
      if (!link.isReady())
      {
        try
        {
          Thread.sleep(interval);
        }
        catch (InterruptedException e)
        {
          interrupted();
        }
        continue;
      }

      try
      {
        Detect receiveDetect = link.getReceiveDetect();
        Heartbeat heartbeat = link.getHeartbeat();
        byte[] b = link.read(headTagLen);
        if (null != receiveDetect)
          receiveDetect.setNextExecutionTime(System.currentTimeMillis());
        byte[] hbeat = null;
        if (null != heartbeat)
          hbeat = heartbeat.getHeartbeat();
        if (Arrays.equals(b, hbeat))
          continue;
        int len = Integer.valueOf(new String(b));
        byte[] request = link.read(len);
        if (Arrays.equals(request, fin))
        {
          sendFin();
          interrupted();
        }
        executorService.submit(new LinkClient(this, request));
      }
      catch (IOException e)
      {
        rebuild();
      }
    }
    executorService.shutdown();
  }

  private void rebuild()
  {
    if (!isInterrupted)
      try
      {
        link.rebuild();
      }
      catch (IOException e1)
      {
        interrupted();
      }
      catch (InterruptedException e1)
      {
        interrupted();
      }
  }

  public void response(byte[] content)
  {
    Lock lock = link.getLock();
    lock.lock();
    Detect heartbeat = link.getHeartbeat();
    String len = String.format("%0" + headTagLen + "d", content.length);
    byte[] response = ArrayUtils.addAll(len.getBytes(), content);
    try
    {
      link.write(response);
      heartbeat.setNextExecutionTime(System.currentTimeMillis());
    }
    catch (IOException e)
    {
      rebuild();
    }
    finally
    {
      lock.unlock();
    }

  }

  public void interrupted()
  {
    isInterrupted = true;
    link.shutdown();
  }

}