package com.brilliance.eibs.server.socket.impl;

import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.PropertyUtil;
import org.apache.mina.core.buffer.IoBuffer;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.CumulativeProtocolDecoder;
import org.apache.mina.filter.codec.ProtocolDecoderOutput;

import java.nio.charset.Charset;

public class CustomDecoder extends CumulativeProtocolDecoder {
	private int headlength;
	private int errorheadlength;

	public CustomDecoder(Charset charset, int headlength) {
		this.headlength = headlength;
	}

	@Override
	protected boolean doDecode(IoSession session, IoBuffer in, ProtocolDecoderOutput out) throws Exception {
		if (in.remaining() > 0) {// 有数据时，读取前headlength字节判断消息长度
			byte[] sizeBytes = new byte[headlength];
			in.mark();// 标记当前位置，以便reset
			try {
				in.get(sizeBytes, 0, headlength);
			} catch (Exception e) {
				in.reset();
				errorheadlength = in.remaining();
				sizeBytes = new byte[errorheadlength];
				in.get(sizeBytes, 0, errorheadlength);
				LogUtil.getLogger("error").error(
						"渠道接收报文头长度不足[" + headlength + "位],已接受到的报文["
								+ new String((byte[]) sizeBytes, PropertyUtil.getProperty("cfg/config.properties", "SYS_ENCODING")) + "]");
				session.closeOnFlush();
				throw e;
			}
			int size = Integer.parseInt(new String(sizeBytes).trim());
			int totalsize = size + headlength;
			in.reset();
			int remaintSize = in.remaining();
			if (totalsize > remaintSize) {
				in.mark();// 标记当前位置，以便reset
				byte[] bytes = new byte[remaintSize];
				in.get(bytes, 0, remaintSize);
				in.reset();
				session.setAttribute(session.getId(), bytes);
				return false;
			} else {

				// int totalsize = size;
				byte[] bytes = new byte[totalsize];
				in.get(bytes, 0, totalsize);
				out.write(IoBuffer.wrap(bytes));
				// 如果读取内容后还粘了包，就让父类再重读 一次，进行下一次解析
				if (in.remaining() > 0) {
					return true;
				}
			}
		}
		return false;
	}

}
