package com.brilliance.eibs.server.socket.impl;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.frame.SocketProxyFactry;
import com.brilliance.eibs.core.threadpool.ThreadPoolFactory;
import com.brilliance.eibs.main.client.ShortSocketHandle;
import com.brilliance.eibs.main.client.SocketClient;
import com.brilliance.eibs.server.IServerInstance;
import com.brilliance.eibs.server.socket.IServerMode;
import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;

import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;

/**
 * @author hujun
 * <p>
 * socket原生模式 ------------modified by weicong on 2016/4/01 -----------
 * 兼容sslsocketserver
 */
public class SocketServerMode implements IServerMode {

    private List<String> trustlist = new ArrayList<String>(); // 信任的IP列表
    private boolean isOpen = true; // socket服务是否打开
    private ServerSocket serverSocket; // socket服务对象
    private Socket socket;
    private String trustips; // 信任的IP列表
    private int port;
    private int queueLen;
    private int size;
    private String delayTime;
    private String interfaceName;
    private String transName;
    private int timeout;
    private String expression;
    private Object[] args;
    private String serverId;
    private Logger logger;

    private String mode = "";

    private boolean has_head = true;// 是否有报文头
    private String head_len_type = "10";// 报文头长度是二进制还是十进制
    private int head_len = 0;// 报文头长度
    private boolean is_contain_head_len = false;// 报文头长度是否包含报文头
    private int fill_len = 0;// 默认为0
    private boolean is_contain_fill_len = false;// 报文头长度是否包含报文头和报文体之间数据的长度 true
    // false
    private int body_offset = 0;// 报文体位移量 默认为0, -1为减去1个长度，+1为增加1个长度
    private String encoding = "UTF-8";

    public SocketServerMode(String mode, String trustips, int port, int queueLen, int size, String delayTime, String interfaceName, String transName,
                            int timeout, String expression, Object[] args, String serverId, Logger logger, boolean has_head, String head_len_type, int head_len,
                            boolean is_contain_head_len, int fill_len, boolean is_contain_fill_len, int body_offset, String encoding) {
        this.mode = mode;
        this.trustips = trustips;
        this.port = port;
        this.queueLen = queueLen;
        this.size = size;
        this.delayTime = delayTime;
        this.interfaceName = interfaceName;
        this.transName = transName;
        this.timeout = timeout;
        this.expression = expression;
        this.args = args;
        this.serverId = serverId;
        this.logger = logger;
        this.has_head = has_head;
        this.head_len_type = head_len_type;
        this.head_len = head_len;
        this.is_contain_head_len = is_contain_head_len;
        this.fill_len = fill_len;
        this.is_contain_fill_len = is_contain_fill_len;
        this.body_offset = body_offset;
        this.encoding = encoding;
    }

    public SocketServerMode(String mode, String trustips, int port, int queueLen, int size, String delayTime, String interfaceName, String transName,
                            int timeout, String expression, Object[] args, String serverId, Logger logger) {
        this.mode = mode;
        this.trustips = trustips;
        this.port = port;
        this.queueLen = queueLen;
        this.size = size;
        this.delayTime = delayTime;
        this.interfaceName = interfaceName;
        this.transName = transName;
        this.timeout = timeout;
        this.expression = expression;
        this.args = args;
        this.serverId = serverId;
        this.logger = logger;
    }


    @Override
    public void startup() {

        try {
            serverSocket = new ServerSocket(port, queueLen);
            logger.info(IServerInstance.LOG_FLAG + "SocketServer started ." + IServerInstance.LOG_FLAG);
            // 获取上下文中的线程池大小
            String name = "socketServer_" + interfaceName + "_" + transName;
            ExecutorService executorService = ThreadPoolFactory.getFixedExecutor(size, name);
            logger.info("server start at port: " + port);
            while (isOpen) {
                socket = serverSocket.accept();
                socket.setSoTimeout(timeout);
                String ip = socket.getInetAddress().getHostAddress();
                logger.debug("Client socket from " + socket.getInetAddress() + " : " + socket.getPort() + ", timeout=" + timeout);
                if (!StringUtil.isEmpty(delayTime)) {
                    Thread.sleep(Long.parseLong(delayTime));
                }
                getTrustIp(trustips);
                if ("*".equals(trustips) || trustlist.contains(ip)) {
                    if (mode.equals(SocketProxyFactry.ORG_SOCKET_TYPE)) {
                        if (DateUtil.isDateValid(expression)) {
                            executorService.submit(new SocketClient(interfaceName, transName, socket, args));
                        } else {
                            logger.debug("time is invalid.");
                            IOUtils.closeQuietly(socket);
                        }
                    } else {
                        executorService.submit(new ShortSocketHandle(socket, interfaceName, transName, has_head, head_len_type, head_len,
                                is_contain_head_len, fill_len, is_contain_fill_len, body_offset, args, encoding, logger));
                    }
                } else {
                    logger.debug("[" + ip + "] is not trusted ip. ");
                    IOUtils.closeQuietly(socket);
                }
            }
            close();
        } catch (Exception e) {
            if ((e instanceof SocketException) && e.getMessage().equals("socket closed"))
                ;
            else {
                throw new InterfaceException("02301", e);
            }
        } finally {
            if (socket != null && !socket.isClosed())
                IOUtils.closeQuietly(socket);
        }

    }

    @Override
    public void close() {
        this.isOpen = false;
        try {
            if (serverSocket != null && !serverSocket.isClosed()) {
                IOUtils.closeQuietly(serverSocket);
                logger.info("SocketServer[id:" + serverId + "] is closed.");
            }
        } catch (Exception e) {
            throw new InterfaceException("02303", e);
        }
    }

    /**
     * 获取允许
     *
     * @param trustips
     */
    private void getTrustIp(String trustips) {
        String[] trusts = trustips.split(",");
        for (String trust : trusts)
            this.trustlist.add(trust);
    }

}
