package com.brilliance.eibs.server.terminal.impl;

import com.brilliance.eibs.server.connection.Connection;
import com.brilliance.eibs.server.connection.FilterChain;
import com.brilliance.eibs.server.connection.impl.DuplexConnection;
import com.brilliance.eibs.server.terminal.Terminal;
import org.apache.commons.io.IOUtils;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

public class AsyncFixedSocketTerminal
    implements Terminal
{

  private String ip;
  private int    port;
  private int    length;

  private FilterChain chain;
  private Connection  conn;

  private long    interval      = 5000;
  private boolean isInterrupted = false;

  public AsyncFixedSocketTerminal( String ip, int port, int length, FilterChain chain )
  {
    this.ip = ip;
    this.port = port;
    this.length = length;
    this.chain = chain;
  }

  private void reconnect()
  {
    try
    {
      createConnection();
    }
    catch (IOException e)
    {
      sleep();
      reconnect();
    }
  }

  private void sleep()
  {
    try
    {
      Thread.sleep(interval);
    }
    catch (InterruptedException e1)
    {
      Thread.interrupted();
    }
  }

  private synchronized void createConnection()
      throws IOException
  {
    if (conn == null || conn.isClosed())
    {
      isInterrupted = false;
      Socket s = new Socket();
      s.connect(new InetSocketAddress(ip, port));
      conn = new DuplexConnection(s);
    }
  }

  @Override
  public void startup()
  {
    try
    {
      createConnection();
      while (!isInterrupted)
      {
        InputStream is = conn.getInputStream();
        byte[] b = new byte[length];
        IOUtils.readFully(is, b);
        b = new byte[Integer.valueOf(new String(b))];
        IOUtils.readFully(is, b);
        chain.doResponseFilterChain(b);
      }
    }
    catch (IOException e)
    {
      close();
      sleep();
      startup();
    }
  }

  @Override
  public void close()
  {
    isInterrupted = true;
    if (conn != null)
      conn.close();
  }

  @Override
  public boolean send(byte[] b)
  {
    if (!isConnected())
      return false;
    b = chain.doRequestFilterChain(b);
    int len = b.length;
    String strLen = String.format("%0" + length + "d", len);
    OutputStream os;
    try
    {
      os = conn.getOutputStream();
      IOUtils.write(strLen, os);
      IOUtils.write(b, os);
    }
    catch (IOException e)
    {
      close();
      Thread t = new Thread(new Runnable()
      {
        @Override
        public void run()
        {
          reconnect();
        }
      });
      t.start();
      t = null;
      return false;
    }
    return true;
  }

  @Override
  public boolean isConnected()
  {
    return !(null == conn || conn.isClosed());
  }

}
