package com.brilliance.eibs.swift;

import com.brilliance.eibs.sqlite.Sqlite_OP;
import com.brilliance.eibs.util.IniUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import com.google.gson.Gson;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;

import java.io.*;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class MessageReader
{
  final static Logger                  log     = LogUtil.getLogger(MessageReader.class);
  private static List<String>          content = null;
  private static Map<Long, Integer>    CurPos  = new HashMap<Long, Integer>();
  private static Map<String, Object[]> Buff    = Collections.checkedMap(new HashMap<String, Object[]>(), String.class, Object[].class);
  /* ???????????????????????????TagPtn???????????? */
  private static Map<String, String[]> TagPtn  = Collections.checkedMap(new HashMap<String, String[]>(), String.class, String[].class);

  /* ?????????????TagPtn???????????????????ICallback tagCb????? */
  private static String[] getTagPatternViaCb(String tagNam)
  {

    String[] tagPtn = null;
    if (tagNam == null)
      return null;
    if (TagPtn.containsKey(tagNam))
      return TagPtn.get(tagNam);

    tagPtn = getTagPattern(tagNam);
    if (tagPtn != null)
      TagPtn.put(tagNam, tagPtn);
    return tagPtn;

  }

  /* ??????????????????????? */
  public static String[] getTagPattern(String tagNam)
  {
    if (tagNam == null)
      return null;
    if (TagPtn.containsKey(tagNam))
      return TagPtn.get(tagNam);
    try
    {
      Connection conn = new Sqlite_OP().getConnection(null);
      //      Connection conn = DBConnection.getConnection();
      String sql = "select ptn,adj,desptn,edtptn from tfi where tag = ? ";
      PreparedStatement pstmt = conn.prepareStatement(sql);
      pstmt.setString(1, tagNam);
      ResultSet rs = pstmt.executeQuery();
      if (rs.next())
      {
        String ptn = rs.getString("ptn");
        ptn = ptn.replace("\\r\\n", "\r\n");
        ptn = ptn.replace("e", "x");

        String adj = rs.getString("adj");
        String desptn = rs.getString("desptn");
        String edtptn = rs.getString("edtptn");
        String[] ptnsArr = new String[]{ptn, adj, desptn, edtptn};
        TagPtn.put(tagNam, ptnsArr);
        if (null != rs)
          rs.close();
        rs = null;
        if (null != pstmt)
          pstmt.close();
        pstmt = null;
        if (null != conn)
          conn.close();
        conn = null;
        return ptnsArr;
      }
      else
        return null;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return null;
    }

  }

  /* ????????????????????? */
  public static List<Map<String, String>> getTagsInfo(String msgtyp)
  {
    if (!msgtyp.startsWith("MT"))
      msgtyp = "MT " + msgtyp;
    try
    {
      Connection conn = new Sqlite_OP().getConnection("swift");
      PreparedStatement pstmt = conn.prepareStatement("select * from mfd where mty=? order by tno asc");
      pstmt.setString(1, msgtyp);
      ResultSet rs = pstmt.executeQuery();
      List<Map<String, String>> list = new ArrayList<Map<String, String>>();
      while (rs.next())
      {
        Map<String, String> tag = new HashMap<String, String>();
        tag.put("inr", rs.getString("inr"));
        tag.put("tag", rs.getString("tag"));
        tag.put("fldnam", rs.getString("fldnam"));
        tag.put("sta", rs.getString("sta"));
        tag.put("cyc", rs.getString("cyc"));
        tag.put("mty", rs.getString("mty"));
        tag.put("tno", rs.getString("tno"));
        tag.put("midnam", rs.getString("midnam"));
        tag.put("msgdsp", rs.getString("msgdsp"));
        list.add(tag);
      }
      if (null != rs)
        rs.close();
      rs = null;
      if (null != pstmt)
        pstmt.close();
      pstmt = null;
      if (null != conn)
        conn.close();
      conn = null;
      return list;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return null;
    }
  }

  /* ????????????????????????????????? */
  public static String[] getAllSwfMsg(String pth)
  {
    BufferedReader br = null;
    try
    {
      br = new BufferedReader(new FileReader(pth));
      StringBuilder sb = new StringBuilder();
      char[] buf = new char[1024];
      int len = -1;
      while ((len = br.read(buf)) > 0)
      {
        sb.append(buf, 0, len);
      }
      return sb.toString().split(new String(new char[]{(char) 1}));// ???????????
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return new String[]{};
    }
    finally
    {
      IOUtils.closeQuietly(br);
    }
  }

  /* ??????????????????? */
  public static String getSwfMsg(String pth)
  {
    BufferedReader br = null;
    try
    {
      br = new BufferedReader(new FileReader(pth));
      StringBuilder sb = new StringBuilder();
      char[] buf = new char[1024];
      int len = -1;
      while ((len = br.read(buf)) > 0)
      {
        sb.append(buf, 0, len);
      }
      return sb.toString();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return null;
    }
    finally
    {
      IOUtils.closeQuietly(br);
    }
  }

  /*
   * @param msg:?????? ???????????????????? ??ACK/NCK???????????????????????????????KEY??????KEY??????
   * ???????S????????????????Acsii????KEY?
   */
  public static Map<Integer, String> anlyMsgHead(String msg)
  {
    Map<Integer, String> headSet = new HashMap<Integer, String>();
    String reg = "\\{[\\d|S]:";
    Matcher m = Pattern.compile(reg).matcher(msg);
    int lstEnd = -1;
    int lstHead = -1;
    while (m.find())
    {
      if (lstEnd > 0 && lstHead > 0)
      {
        String val = msg.substring(lstEnd, m.start() - 1);
        if (!headSet.containsKey(lstHead))
        {
          headSet.put(lstHead, val);
        }
        else
        {
          headSet.put(-lstHead, val);
        }
      }
      String g = m.group();
      char c = g.charAt(1);
      lstHead = c;
      if (c >= '0' && c <= '9') // ????????????KEY????Acsii????
        lstHead -= '0';
      lstEnd = m.end();
    }
    if (lstEnd > 0 && lstHead > 0) // ?????????Map
    {
      int lstidx = msg.lastIndexOf('}');
      if (lstidx <= lstEnd)
        return headSet;
      if (!headSet.containsKey(lstHead))
        headSet.put(lstHead, msg.substring(lstEnd, lstidx));
      else
        headSet.put(-lstHead, msg.substring(lstEnd, lstidx));
    }
    return headSet;
  }

  /*
   * @param mt:???block 4?????????????????????:????
   */
  public static List<String[]> msgTxtAnly(String mt)
  {
    String[] tags = mt.split("((\\r\\n)|(\\n\\r)|(\\r)|(\\n)):");
    List<String[]> list = new ArrayList<String[]>();
    for (String tv : tags)
    {
      if (tv.trim().length() > 0)
      {
        int i = tv.indexOf(':');
        if (i > 0)
        {
          String[] arr = new String[2];
          arr[0] = tv.substring(0, i);
          arr[1] = tv.substring(i + 1);
          list.add(arr);
        }
      }
    }
    if (list.size() > 0)
    {
      String[] lstOne = list.get(list.size() - 1);
      int i = lstOne[1].lastIndexOf("\r\n");
      if (i < 0)
      {
        i = lstOne[1].lastIndexOf("\r");
      }
      if (i < 0)
      {
        i = lstOne[1].lastIndexOf("\n");
      }
      if (i > -1)
      {
        lstOne[1] = lstOne[1].substring(0, i);
        list.set(list.size() - 1, lstOne);
      }
    }
    return list;
  }

  /* ??????????????????????????????????????????????????????? */
  private int CurrentPos = 0;

  /*
   * startR?endR????R????????????startS?endS??????S???????????? Tools???????????????? S?????? R????????????
   * L???R????R???????? circLst???????????List C???????List tagLst???List????????? dataSet??????????
   */
  @SuppressWarnings("unchecked")
  private void msgSplit(int startR, int endR, int startS, int endS, Map<String, Object> Tools)
  {
    String S = (String) Tools.get("S");
    String R = (String) Tools.get("R");
    String L = (String) Tools.get("L");
    List<Object[]> circLst = (List<Object[]>) Tools.get("circLst");
    List<String> C = (List<String>) Tools.get("C");
    List<Map<String, String>> tagLst = (List<Map<String, String>>) Tools.get("tagLst");
    List<String[]> dataSet = (List<String[]>) Tools.get("dataSet");
    int a = 0;
    int leftseek = startR;
    int rightseek = 0;
    int currentPos = 0;
    while (leftseek <= endR)
    {
      if (R.charAt(leftseek) != '(')
      {
        leftseek++;
        continue;
      }
      else
        leftseek++;
      rightseek = getRightCommaPos(leftseek, endR, R);// ????????????
      if (rightseek > 0)// ?????
      {
        a++;
        boolean canRep = false;// ???? ???? ???S??????
        if (R.length() > rightseek + 1 && (R.charAt(rightseek + 1) == '+' || R.charAt(rightseek + 1) == '*'))
          canRep = true;
        String subRNew = R.substring(leftseek, rightseek);
        Matcher m = Pattern.compile(subRNew).matcher(S.substring(startS, endS + 1));
        while (m.find())
        {
          if (m.end() == m.start())// ???"?"?????????????continue;
            continue;
          currentPos = m.start() + startS;
          if (currentPos != CurrentPos)// ???????????CurrentPos????????continue
          {
            continue;
          }
          String inrNew = "";
          int i = leftseek;
          while (i < rightseek) // ?L????????TNO
          {
            char c = L.charAt(i);
            if (c >= '0' && c <= '9')
              inrNew += c;
            else
              break;
            i++;
          }
          if (!"".equals(inrNew))
          {
            String h = getHead(inrNew, tagLst);// ??tno??????cyc??????????????????cyc?????null
            if (h != null) // ??????
            {
              int in = isKeyInLst(h, circLst); // ??????List?????????????????List????????-1
              if (in >= 0)
              {
                Object[] objs = circLst.get(in);
                int rt = (Integer) objs[1]; // ??????1
                rt++;
                objs[1] = rt;
                while (circLst.size() > in + 1) // ??????????????
                {
                  circLst.remove(in + 1);
                }
              }
              else
              {
                circLst.add(new Object[]{h, 1}); // ???????????????
              }
            }
          }
          msgSplit(leftseek, rightseek - 1, m.start() + startS, m.end() + startS - 1, Tools);// ????????
          if (!canRep)
            break;
        }
        leftseek = rightseek;// ?????????leftseek???rightseek
      }
    }
    if (a == 0)// ?????????????????????????
    {
      Matcher comaMch = Pattern.compile(",").matcher(S.substring(0, startS));
      int index = 0;
      while (comaMch.find())
      {
        index++; // ???,??? ?
      }
      String val = C.get(index);// ?????
      String inr = String.valueOf(Integer.parseInt(L.substring(startR, endR)));// ?????000???tno
      String tagNam = S.substring(startS, endS);// ??????
      int size = tagLst.size(); //
      String belong = null;
      /* ????tno????????midnam */
      for (int i = 0; i < size; i++)
      {
        Map<String, String> tag = tagLst.get(i);
        if (inr.equals(tag.get("tno")) && tagNam.equals(tag.get("tag")))
        {
          belong = tag.get("midnam");
          break;
        }
      }
      if (belong != null) // ????????
      {
        Matcher m = Pattern.compile("(\\w+|#)\\d*").matcher(belong);
        String key = ""; // ?????
        while (m.find())
        {
          String ckey = belong.substring(0, m.end());
          int times = getTimesInLst(ckey, circLst); // ??????????????????????
          key = key + m.group() + "," + times + ",";// ??????
        }
        key = key + tagNam;
        dataSet.add(new String[]{key, val}); // ???????????dataSet?
      }
      else
        dataSet.add(new String[]{tagNam, val}); // ?????????????????????dataSet?
      CurrentPos = CurrentPos + tagNam.length() + 1; // ?CurrentPos??????????????
      Tools.put("CurrentPos", CurrentPos);
    }
  }

  /**/
  private static int getTimesInLst(String key, List<Object[]> lst)
  {
    int in = isKeyInLst(key, lst);
    if (in >= 0)
    {
      Integer times = (Integer) lst.get(in)[1];
      return times == null ? 1 : times;
    }
    else
      return 1;
  }

  private static int isKeyInLst(String key, List<Object[]> lst)
  {
    if (lst == null || lst.size() == 0)
      return -1;
    int size = lst.size();
    for (int i = 0; i < size; i++)
    {
      Object[] objs = lst.get(i);
      if (objs[0].equals(key))
        return i;
    }
    return -1;
  }

  /* ????????????????? */
  public static int getRightCommaPos(int leftseek, int endR, String R)
  {
    int rightseek = leftseek;
    int m = -1;
    while (rightseek <= endR)
    {
      char c = R.charAt(rightseek);
      if (c == ')')
        m++;
      else if (c == '(')
        m--;
      if (m == 0)
        break;
      rightseek++;
    }
    if (m != 0)//
      return -1;
    return rightseek;
  }

  /* ??tno???????????????????????????????????cyc?????null */
  private static String getHead(String inr, List<Map<String, String>> tagLst)
  {
    int size = tagLst.size();
    inr = String.valueOf(Integer.parseInt(inr));
    for (int i = 0; i < size; i++)
    {
      Map<String, String> tag = tagLst.get(i);
      if (inr.equals(tag.get("tno")))
      {
        String cyc = tag.get("cyc");
        if (cyc != null && cyc.length() > 1 && cyc.endsWith("@"))
        {
          return tag.get("midnam");
        }
      }
    }
    return null;
  }

  /* ?????????????????tno????0???????L? */
  public static String getL(String R)
  {
    String r = "\\d{2}[A-Z\\[\\]\\?]*";
    Matcher m = Pattern.compile(r).matcher(R);
    StringBuilder rSb = new StringBuilder(R);
    int i = 1;
    while (m.find())
    {
      String inr = String.valueOf(i);
      int iLen = inr.length();
      int gLen = m.group().length();
      while (iLen < gLen)
      {
        inr = "0" + inr;
        iLen++;
      }
      rSb.replace(m.start(), m.end(), inr);
      i++;
    }
    return rSb.toString();
  }

  /*
   * @param path ???? ?????????DBConnection ?????????????
   */
  public static List<String[]> splitViaFile(String path)
  {
    String msg = MessageReader.getSwfMsg(path);
    return splitViaStr(msg);
  }

  /*
   * @param msg ????? ??????????DBConnection ?????????????
   */
  @SuppressWarnings("unchecked")
  public static List<String[]> splitViaStr(String msg)
  {
    if (msg != null && msg.trim().length() > 0)
    {
      try
      {
        CurPos.put(Thread.currentThread().getId(), 0);
        Map<String, Object> Tools = new HashMap<String, Object>();
        List<String> C = new ArrayList<String>();
        Map<Integer, String> map = anlyMsgHead(msg);
        String msgTyp = map.get(2).substring(1, 4);
        if (map.get(3) != null && map.get(3).contains("COV"))
          msgTyp = msgTyp + "COV";
        List<String[]> lst = msgTxtAnly(map.get(4));// ???4??????
        Iterator<String[]> it = lst.iterator();
        StringBuilder buf = new StringBuilder();
        while (it.hasNext())
        {
          String[] kv = it.next();
          buf.append(kv[0] + ",");
          C.add(kv[1]);// ???????????List?
        }
        String S = buf.toString();// ????????,????????S?
        Tools.put("S", S);// ??S?
        Tools.put("circLst", new ArrayList<Object[]>());
        Tools.put("C", C);
        Tools.put("dataSet", new ArrayList<String[]>());
        if (!Buff.containsKey(msgTyp))
        {
          List<Map<String, String>> tagLst = MessageReader.getTagsInfo(msgTyp);
          String R = generReg(tagLst);
          if (R == null)
            return new ArrayList<String[]>();// ????????????????List
          Tools.put("R", R);
          // ??L?????
          Tools.put("L", getL(R));
          Tools.put("tagLst", tagLst);
          Tools.put("CurrentPos", 0);
          Object[] bufInf = new Object[]{Tools.get("R"), Tools.get("L"), Tools.get("tagLst")};
          Buff.put(msgTyp, bufInf);
        }
        else
        {
          Object[] bufInf = Buff.get(msgTyp);
          Tools.put("R", bufInf[0]);
          Tools.put("L", bufInf[1]);
          Tools.put("tagLst", bufInf[2]);
        }
        new MessageReader().msgSplit(0, Tools.get("R").toString().length() - 1, 0, S.length() - 1, Tools);
        return (ArrayList<String[]>) Tools.get("dataSet");
      }
      catch (Exception e)
      {
        e.printStackTrace();
        return new ArrayList<String[]>();
      }
    }
    return new ArrayList<String[]>();
  }

  /*
   * @param msg ????
   *
   * @param tagLst ???? ?????????????????
   */
  @SuppressWarnings("unchecked")
  public static List<String[]> splitViaStr(String msg, List<Map<String, String>> tagLst)
  {
    if (msg != null && msg.trim().length() > 0)
    {
      try
      {
        Map<String, Object> Tools = new HashMap<String, Object>();
        List<String> C = new ArrayList<String>();
        Map<Integer, String> map = anlyMsgHead(msg);
        List<String[]> lst = msgTxtAnly(map.get(4));// ???4????????
        Iterator<String[]> it = lst.iterator();
        StringBuilder buf = new StringBuilder();
        while (it.hasNext())
        {
          String[] kv = it.next();
          buf.append(kv[0] + ",");
          C.add(kv[1]);// ??????????????List
        }
        String S = buf.toString();// ??????","???????????
        Tools.put("S", S);
        Tools.put("circLst", new ArrayList<Object[]>());
        Tools.put("C", C);
        Tools.put("dataSet", new ArrayList<String[]>());
        String R = generReg(tagLst);
        if (R == null)
          return new ArrayList<String[]>();// ?????????????List
        Tools.put("R", R);
        Tools.put("L", getL(R));
        Tools.put("tagLst", tagLst);
        Tools.put("CurrentPos", 0);
        new MessageReader().msgSplit(0, R.length() - 1, 0, S.length() - 1, Tools);
        return (ArrayList<String[]>) Tools.get("dataSet");
      }
      catch (Exception e)
      {
        e.printStackTrace();
        return new ArrayList<String[]>();
      }
    }
    return new ArrayList<String[]>();
  }

  /*
   * @param pth?????
   *
   * @param tagLst????? ??????????????????
   */
  public static List<String[]> splitViaFile(String pth, List<Map<String, String>> tagLst)
  {
    String msg = getSwfMsg(pth);
    return splitViaStr(msg, tagLst);
  }

  /*
   * @param lst????????List? ????? ?????????????????????
   */
  public static String generReg(List<Map<String, String>> lst)
  {
    int size = lst.size();
    int maxSeq = Integer.parseInt(lst.get(size - 1).get("tno"));
    List<Map<String, String>> STBuf = new ArrayList<Map<String, String>>();// ????TNO??????
    int pos = 0; // ??????
    StringBuilder ds = new StringBuilder();// ?????
    for (int i = 1; i <= maxSeq; i++)// ???TNO???? i ????????TNO
    {
      String midNam = null;
      String cyc = null;
      while (pos < size)
      {
        Map<String, String> st = lst.get(pos);
        if (Integer.parseInt(st.get("tno")) == i)// ??????? TNO?
        {
          cyc = st.get("cyc");
          if (STBuf.size() == 0 && cyc != null && cyc.length() > 1 && cyc.endsWith("@"))// ???????cyc?????????2???H??????H????????(?
          {
            ds.append("(");
          }
          midNam = st.get("midnam");
          STBuf.add(st);
          pos++;
        }
        else
          break; // TNO?i??? ????????????????
      }
      if (STBuf.size() == 1)// ?????????TAG
      {
        String d = "(";
        Map<String, String> st = STBuf.get(0);
        d += st.get("tag") + "," + ")";
        if ("O".equals(st.get("sta")))
        {
          d += "?";
        }
        STBuf.clear();
        ds.append(d);
      }
      else if (STBuf.size() > 1)// ????????TAG ??????a????
      {
        StringBuilder d = new StringBuilder("("); // ???????????
        StringBuilder suffix = new StringBuilder("[]");// ???
        int bufSiz = STBuf.size();
        for (int j = 0; j < bufSiz; j++)
        {
          String cod = STBuf.get(j).get("tag");
          if (cod.length() == 3)// ????????????[]?
            suffix.insert(1, cod.charAt(2));
          else
            // ????????[]????
            suffix.insert(suffix.length(), '?');
        }
        Map<String, String> fstTg = STBuf.get(0);
        d.append(fstTg.get("tag").substring(0, 2));// ?????????
        d.append(suffix);
        d.append(',');
        d.append(")");
        if (fstTg.get("sta").equals("O"))
          d.append('?');
        STBuf.clear();
        ds.append(d);
      }
      else if (STBuf.size() == 0)
      {
        System.out.println("NO: " + i);
        System.out.println("????");
        return null;// ????
      }
      if (midNam == null) //????midnam???null?continue
        continue;
      String nxtMid = null;
      if (pos < size)// ????????
      {
        nxtMid = lst.get(pos).get("midnam");// ????????MIDNAM
      }
      //????midnam?????????
      int difpos = 0;
      String[] midArr = midNam.split(",");
      if (nxtMid == null)
      {
        String rightCs = generDiffCom(midArr, difpos);
        ds.append(rightCs);
        continue;
      }
      if (nxtMid.startsWith(midNam)) //??nxtmid?????midnam????????????????????????????
        continue;
      String[] nxtArr = nxtMid.split(",");
      for (; difpos < midArr.length; difpos++) //??midnam?????????????????
      {
        if (difpos >= nxtArr.length)
          break;
        if (!midArr[difpos].equals(nxtArr[difpos]))
          break;
      }
      String rightCs = generDiffCom(midArr, difpos);
      ds.append(rightCs);
    }
    return ds.toString();
  }

  private static String generDiffCom(String[] midArr, int difpos)
  {
    StringBuilder rhtbuf = new StringBuilder();
    for (; difpos < midArr.length; difpos++)
    {
      rhtbuf.append(")");
      if (midArr[difpos].matches("#\\d+")) //?????
        rhtbuf.append("+");
      else
        rhtbuf.append("*");
    }
    return rhtbuf.toString();
  }

  /* ????????????????????? */
  private static int getLinePos(String str)
  {
    if (str == null)
      return -1;
    int idxr = str.indexOf("\r");
    int idxn = str.indexOf("\n");
    if (idxr > -1 && idxn > -1)// \r\n?????
    {
      return idxr < idxn ? idxr : idxn;
    }
    else
    {
      return idxr <= idxn ? idxn : idxr;// ??\r??\n?????????
    }
  }

  /*
   * @param msg?????? ??????????????NCK/ACK???mty?NCK/ACK?????????????????emty??
   */
  public static Map<String, String> getSwfMsgInfoViaMsg(String msg)
  {
    Map<String, String> infMap = new HashMap<String, String>();
    if (msg != null && msg.trim().length() > 0)
    {
      Map<Integer, String> heads = MessageReader.anlyMsgHead(msg);
      String hd4 = heads.get(4);
      boolean isNACK = false; // ???ACK/NCK??
      if (hd4 != null)
      {
        if (hd4.contains("{451:0}") || hd4.contains("{451:1}"))
        {
          isNACK = true;
          if (hd4.contains("{451:0}"))
            infMap.put("mty", "ACK");
          else
            infMap.put("mty", "NCK");
          // infMap.put("ehf",heads.get(-4) );
          int errIdx = hd4.indexOf("{405:");
          if (errIdx > -1)
          {
            int idx2 = hd4.indexOf("}", errIdx);// ???????
            if (idx2 > -1)
              infMap.put("errcod", hd4.substring(errIdx + 5, idx2));// ???????
          }
        }
        else
        // ?ACK ?NCK??
        {

        }
      }
      /* ????????????????? */
      String hd2 = heads.containsKey(-2) ? heads.get(-2) : heads.get(2);
      String hd1 = heads.containsKey(-1) ? heads.get(-1) : heads.get(1);
      String hd3 = heads.containsKey(-3) ? heads.get(-3) : heads.get(3);
      hd4 = heads.containsKey(-4) ? heads.get(-4) : heads.get(4);
      String hd5 = heads.containsKey(-5) ? heads.get(-5) : heads.get(5);
      if (hd5 != null)
        infMap.put("trl", hd5);
      String msgTyp = null; // ????????NCK/ACK???????????????????
      String direct = null; // ??????
      if (hd2 != null) // ??2??
      {
        // ??2?????,?????????????
        if (hd2.length() >= 4)
        {
          direct = hd2.substring(0, 1);
          infMap.put("direct", direct);// ??????
          msgTyp = hd2.substring(1, 4);
          if ("I".equals(direct)) // ????
          {
            if (hd2.length() >= 16)
            {
              String rcvbic = hd2.substring(4, 16); // ?????
              infMap.put("rcvbic", rcvbic);
            }
            if (hd2.length() >= 17)
              infMap.put("prt", hd2.substring(16, 17));// ?????
            if (hd2.length() >= 18)
              infMap.put("tns", hd2.substring(17, 18));
            if (hd2.length() >= 21)
              infMap.put("dyt", hd2.substring(18, 21));
          }
          else if ("O".equals(direct))
          {
            if (hd2.length() >= 26)
            {
              String sndbic = hd2.substring(14, 26);
              infMap.put("sndbic", sndbic); // ?????
              if (hd2.length() >= 36)
              {
                infMap.put("mir", hd2.substring(8, 14) + " " + hd2.substring(14, 36));// ??MIR
                if (hd2.length() == 47)
                  infMap.put("prt", hd2.substring(46));// ?????
              }
            }
          }
        }
      }
      if (hd1 != null) // ??1??
      {
        if (hd1.length() >= 15)
        {
          String bnk1 = hd1.substring(3, 15);
          if ("O".equals(direct))
          {
            infMap.put("rcvbic", bnk1);
          }
          else if ("I".equals(direct))
          {
            infMap.put("sndbic", bnk1);
          }
          if (hd2 != null && hd2.length() >= 14)
          {
            infMap.put("mor", hd2.substring(8, 14) + " " + hd1.substring(3));// ??MOR
          }
        }
      }
      if (hd3 != null)
      {
        if (hd3.contains("{119:COV}"))
        {
          msgTyp = msgTyp + "COV";
        }
        if (hd3.contains("{FMT:"))
        {
          String fmt = hd3.substring(1, hd3.length() - 1);
          String args[] = fmt.split(":");
          infMap.put("fmttype", args[0]);
          infMap.put("fmtstd", args[1]);
          String strarry[] = hd3.split("\\}");
          for (String substr : strarry)
          {
            substr = substr.substring(1);
            String sp[] = substr.split(":");
            if (sp.length == 2)
            {
              infMap.put(sp[0], sp[1]);
            }
            else
            {
              infMap.put(sp[0], null);
            }
          }
        }
        // else if(hd3.contains("{119:STP}"))
        // msgTyp = msgTyp + "+";
        Matcher m = Pattern.compile("\\{108:\\w+\\}").matcher(hd3);
        if (m.find())
        {
          String gp = m.group();
          int idx = gp.indexOf(":");
          infMap.put("sndnum", gp.substring(idx + 1, gp.length() - 1)); /* ?????? */
        }
      }
      infMap.put("std", "MT");
      if (!isNACK)
        infMap.put("mty", msgTyp);// ???????????Map?
      else
        infMap.put("emty", msgTyp);// ?????????
      if (hd4 != null) // ??4?????
      {
        List<String[]> kv = msgTxtAnly(hd4);
        for (String[] strArr : kv)
        {
          if (strArr[0].endsWith("20"))// ????
          {
            if (!infMap.containsKey("ref")) // ??????20????20???????????????nXX???????????
            {
              if (strArr[1].length() > 16)
                infMap.put("ref", strArr[1].substring(0, 16).trim());
              else
                infMap.put("ref", strArr[1].trim());
            }
          }
          if (strArr[0].endsWith("21"))// ????
          {
            if (!infMap.containsKey("tscref")) // ??????21????21???????????????nXX???????????
            {
              if (strArr[1].length() > 16)
                infMap.put("tscref", strArr[1].substring(0, 16));
              else
                infMap.put("tscref", strArr[1]);
            }
          }
          if (strArr[0].endsWith("22C"))// 300???COMMON REFFERENCE
          {

          }
          if (strArr[0].endsWith("32A"))// ???????
          {
            if (strArr[1].matches("\\d{6}[A-Z]{3}\\d{1,15},\\d*"))
            {
              String valDat = strArr[1].substring(0, 6);
              String cur = strArr[1].substring(6, 9);
              String amt = strArr[1].substring(9);
              infMap.put("vludat", valDat);
              infMap.put("amt", amt.replace(",", "."));
              infMap.put("cur", cur);
            }
          }

          if (strArr[0].matches("62[FM]"))
          {
            // 950/940???????????
            if (strArr[1] != null && strArr[1].matches("[DC]\\d{6}[A-Z]{3}\\d{1,15},\\d*"))
            {
              String valDat = strArr[1].substring(1, 7);
              String cur = strArr[1].substring(7, 10);
              String amt = strArr[1].substring(10).replace(",", ".");
              infMap.put("vludat", valDat);
              infMap.put("amt", amt.replace(",", "."));
              infMap.put("cur", cur);
            }
            else
              return infMap;
          }
          if (strArr[0].matches("25")) // ??940/950?????
          {
            String act = strArr[1];
            infMap.put("act", act);
          }
          if ("O".equals(infMap.get("direct")) && strArr[0].matches("54A") && strArr[1] != null)
          {
            String actbic = strArr[1];
            int secondLine = getLinePos(actbic);
            if (secondLine > -1)
            {
              actbic = actbic.substring(secondLine);
              actbic = actbic.replaceAll("\\r|\\n", "");// ?????\r\n
            }
            infMap.put("actbic", actbic);
          }
          if ("O".equals(infMap.get("direct")) && strArr[0].matches("53A") && strArr[1] != null)
          {
            String actbic = strArr[1];
            int secondLine = getLinePos(actbic);
            if (secondLine > -1)
            {
              actbic = actbic.substring(secondLine);
              actbic = actbic.replaceAll("\\r|\\n", "");// ?????\r\n
            }
            infMap.put("t53bic", actbic);
          }
          if ("I".equals(direct) && strArr[0].matches("53A") && strArr[1] != null)
          {
            String actbic = strArr[1];
            int secondLine = getLinePos(actbic);
            if (secondLine > -1)
            {
              actbic = actbic.substring(secondLine);
              actbic = actbic.replaceAll("\\r|\\n", "");// ?????\r\n
            }
            infMap.put("actbic", actbic);
          }
          if (strArr[0].matches("50[AK]") && strArr[1] != null)
          {
            int index = getLinePos(strArr[1]);
            if (index > -1 && strArr[1].startsWith("/"))
              infMap.put("act", strArr[1].substring(1, index));
            if (strArr[0].endsWith("K"))
            {
              if (index > -1)
              {
                String nam = strArr[1].substring(index).replaceAll("^[\\r\\n]{1,2}", "");
                infMap.put("ordnamadr", nam);
              }
              else
              {
                infMap.put("ordnamadr", strArr[1]);
              }
            }
          }
          if (strArr[0].matches("59") && strArr[1] != null)
          {
            int index = getLinePos(strArr[1]);
            String nam = strArr[1];
            if (index > -1 && strArr[1].startsWith("/"))
            {
              String act = strArr[1].substring(1, index);// ???????
              infMap.put("act", act);// ??59??????????
              infMap.put("benact", act);// ??????????50????
              nam = strArr[1].substring(index).replaceAll("^[\\r\\n]{1,2}", "");
            }
            infMap.put("bennamadr", nam);
          }
          // ?????MT300???
          if (strArr[0].endsWith("22C"))
          {
            if (!infMap.containsKey("comref")) // ??????22C?
            {
              if (strArr[1].length() > 16)
                infMap.put("comref", strArr[1].substring(0, 16));
              else
                infMap.put("comref", strArr[1]);
            }
          }
          if (strArr[0].endsWith("33B"))
          {
            String cur = strArr[1].substring(0, 3);
            String amt = strArr[1].substring(3);
            infMap.put("sellamt", amt.replace(",", "."));
            infMap.put("sellcur", cur);

          }

          // ??????MT320???
          if (strArr[0].endsWith("17R"))
          {
            if (!infMap.containsKey("BorL"))
              infMap.put("BorL", strArr[1]);
          }
          if (strArr[0].endsWith("34E"))
          {
            String tagN = strArr[1].substring(0, 1);
            String interest = "";
            if ("N".equals(tagN))
              interest = strArr[1].substring(4);
            else
              interest = strArr[1].substring(3);
            infMap.put("interest", interest.replace(",", "."));
          }
          // ????
          if (strArr[0].endsWith("30V"))
          {
            if (!infMap.containsKey("m3xValDate"))
              infMap.put("m3xValDate", strArr[1]);
          }

          if (strArr[0].endsWith("32B"))
          {
            String cur = strArr[1].substring(0, 3);
            String amt = strArr[1].substring(3);
            infMap.put("buyamt", amt.replace(",", "."));
            infMap.put("buycur", cur);

          }
        }
      }
    }
    return infMap;
  }

  /*
   * @param pth ???? ????????????????
   */
  public static Map<String, String> getSwfMsgInfo(String pth) // ????????????
  {
    String msg = MessageReader.getSwfMsg(pth);
    return getSwfMsgInfoViaMsg(msg);
  }

  /* ????????????????? */
  public static List<String[]> getAllTagsViaStr(String msg)
  {
    if (msg == null || msg.length() > 0)
    {
      Map<Integer, String> head = anlyMsgHead(msg);
      String head4 = head.containsKey(-4) ? head.get(-4) : head.get(4);
      if (head4 != null)
      {
        List<String[]> alst = msgTxtAnly(head4);
        return alst;
      }
      else
        return null;
    }
    else
      return null;
  }

  /* ?????????????? */
  public static List<String[]> getAllTags(String pth)
  {
    String msg = getSwfMsg(pth);
    return getAllTagsViaStr(msg);
  }

  public static List<String[]> simpleSplit(String pth)// ??????????????????
  {
    List<String[]> rs = getAllTags(pth);
    if (rs != null)
      return rs;
    else
      return new ArrayList<String[]>();
  }

  //  public static List<Map<String, String>> simpleSplit(String)

  /* ????????????????? */
  /*
   * @param msgpth ?????
   *
   * @param tagNames ???????????
   *
   * @param needSeq ????????????
   */
  public static List<String[]> getTagsViaStr(String msgStr, List<String> tagNames, boolean needSeq)
  {
    List<String[]> datas = new ArrayList<String[]>();
    if (tagNames == null)
      return datas;
    List<String[]> rs = null;
    if (needSeq)
      rs = splitViaStr(msgStr);// ????????????Tag?
    else
      rs = getAllTagsViaStr(msgStr);// ?????????????????
    if (rs == null)
      return datas;
    for (String tagName : tagNames)
    {
      for (String[] nv : rs)
      {
        if (nv[0].endsWith(tagName))
          datas.add(nv);
      }
    }
    return datas;
  }

  /* ???????????????? */
  /*
   * @param msgpth ????
   *
   * @param tagNames ???????????
   *
   * @param needSeq ????????????
   */
  public static List<String[]> getTags(String msgPth, List<String> tagNames, boolean needSeq)
  {
    String msg = getSwfMsg(msgPth);
    return getTagsViaStr(msg, tagNames, needSeq);
  }

  /* ??????????Text?,???????? */
  public static String getTextRemViaStr(String msg, List<String> removedTags)
  {
    List<String[]> rs = getAllTagsViaStr(msg);
    if (rs == null)
      return null;
    StringBuilder buf = new StringBuilder();
    if (removedTags != null)
    {
      for (String rt : removedTags)
      {
        Iterator<String[]> it = rs.listIterator();
        while (it.hasNext())
        {
          String nv[] = it.next();
          if (nv[0].equals(rt))
            it.remove();
        }
      }
    }
    for (String[] nv : rs)
    {
      buf.append(":");
      buf.append(nv[0]);
      buf.append(":");
      buf.append(nv[1]);
      buf.append("\r\n");
    }
    return buf.toString();
  }

  /*
   * ??????????4?????
   *
   * @param pth ????
   *
   * @param removedTags ????????
   */
  public static String getTextRem(String pth, List<String> removedTags)
  {
    String msg = getSwfMsg(pth);
    return getTextRemViaStr(msg, removedTags);
  }

  /*
   * ?????Swift?????????????????????????????????????????????validateTag???
   *
   * @param pattern ??? swift??
   *
   * @param spt? key?????????????????????????? ?value????
   */
  public static String generTagReg(String pattern, Map<String, String> spt)
  {
    //pattern = pattern.replaceAll("\r\n\\[", "(\r\n").replaceAll("\\]", ")?");
    pattern = pattern.replaceAll("\\[", "(").replaceAll("\\]", ")?");
    String nail = "\\d+(\\*\\d+)?!?[nacxd]";
    Matcher m = Pattern.compile(nail).matcher(pattern);
    int offset = 0;
    StringBuilder buf = new StringBuilder(pattern);
    int pos = 0;
    while (m.find())
    {
      StringBuilder oneR = new StringBuilder();
      String grp = m.group();
      int grplen = grp.length();
      pos++;
      String adjReg = null;
      if (spt != null)
      {
        adjReg = spt.get(String.valueOf(pos));// ??????????
      }
      if (adjReg != null)
      {
        /* ???????? */
        oneR.append("(" + adjReg + ")");
      }
      else
      {
        char c = grp.charAt(grplen - 1);// ??????
        boolean isFixed = grp.charAt(grplen - 2) == '!';// ??????
        int lines = 1;
        int len = 0;
        int sidx = grp.indexOf('*');
        if (sidx == -1)// ???????
        {
          if (isFixed)
            len = Integer.parseInt(grp.substring(0, grplen - 2));
          else
            len = Integer.parseInt(grp.substring(0, grplen - 1));
        }
        else
        // ????
        {
          lines = Integer.parseInt(grp.substring(0, sidx));
          if (isFixed)
            len = Integer.parseInt(grp.substring(sidx + 1, grplen - 2));
          else
            len = Integer.parseInt(grp.substring(sidx + 1, grplen - 1));
        }
        switch (c)
        {
          case 'a':
            oneR.append("[A-Za-z]");
            break;
          case 'n':
            oneR.append("\\d");
            break;
          case 'c':
            oneR.append("\\w");
            break;
          case 'x':
            oneR.append("[\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]");// SWIFT ???
            break;
          case 'd':
            oneR.append("\\d{1," + len + "},\\d*");
            break;
        }
        if (isFixed)
        {
          oneR.append("{");
          oneR.append(len);
          oneR.append("}");
        }
        else
        {
          if (c != 'd')
          {
            oneR.append("{");
            oneR.append(1 + "," + len);
            oneR.append("}");
          }
        }
        if (lines > 1)
        {
          String reg1 = oneR.toString();
          oneR.append("([\\r\\n]{1,2}" + reg1 + "){0," + (lines - 1) + "}");
        }
        oneR.insert(0, "(");
        oneR.append(")");
      }
      buf.replace(m.start() + offset, m.end() + offset, oneR.toString());
      offset = offset + oneR.length() - grplen;
    }
    /* ??????? */
    /*  for (int i = 0; i < buf.length();)
      {
        if (buf.charAt(i) == '(')
        {
          if (buf.charAt(i + 1) == '(')
          {// ???????????
            int n = -1;
            for (int j = i + 1; j < buf.length(); j++)
            {
              if (buf.charAt(j) == ')')
                n++;
              else if (buf.charAt(j) == '(')
                n--;
              if (n == 0)
              {
                if (buf.charAt(j) == ')')
                {
                  buf.delete(i, i + 1);
                  buf.delete(j - 1, j);
                }
                i = j + 1;
                break;
              }
            }
          }
          else
            i++;
        }
        else
          i++;
      }*/
    String reg = buf.toString();
    /* ?????? */
    if (spt != null)
    {
      for (Map.Entry<String, String> en : spt.entrySet())
      {
        String key = en.getKey();
        if (key.matches("[A-Z]{1,}"))
        {
          reg = reg.replaceAll(key, en.getValue());
        }
      }
    }
    return reg;
  }

  /*
   * ??????????????????????????????????????????? ??????????????????????????????????????????? ?????????????
   */
  public static String generValidateTagReg(String tagNam)
  {
    if ("61".equals(tagNam))
    {
      return "(\\d{6})(\\d{4})?(RC|RD|D|C)([A-Za-z]{1})?(\\d{1,15},\\d*)([A-Za-z]{1})(\\w{3})((([\\w-\\?:\\(\\)\\.,'+\\{\\} ])|((?<!/)/(?!/))){1,16})(//([\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]{1,16}))?([\r\n]{1,2})?([\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]{1,34})?";
    }
    String[] tagPtns = getTagPatternViaCb(tagNam);
    if (tagPtns == null)
      return null;
    @SuppressWarnings("unchecked")
    Map<String, String> adjMap = new Gson().fromJson(tagPtns[1], Map.class);
    String reg = generTagReg(tagPtns[0], adjMap);
    String rnreg = "\\r\\n|\\n\\r|\\r|\\n";
    StringBuilder rsb = new StringBuilder(reg);
    Matcher m = Pattern.compile(rnreg).matcher(reg);
    int offset = 0;
    while (m.find())
    {
      int pos = 0;
      int ed = m.end();
      int bst = m.start();
      if (bst > 1 && reg.charAt(bst - 1) == '?' && reg.charAt(bst - 2) == ')')
      {
        rsb.delete(m.start() + offset, m.end() + offset);
        rsb.insert(bst - 2, "[\\r\\n]{1,2}");
        offset = offset - m.group().length() + 11;
      }
      else if (reg.charAt(ed) == '(')
      {
        int com = -1;
        for (int i = ed + 1; i < reg.length(); i++)
        {
          if (reg.charAt(i) == '(')
            com--;
          else if (reg.charAt(i) == ')')
            com++;
          if (com == 0)
          {
            pos = i;
            break;
          }
        }
        if (reg.length() > pos + 1 && reg.charAt(pos + 1) == '?')// ??\r\n????????????\r\n????????
        {
          rsb.insert(m.end() + 1 + offset, "[\\r\\n]{1,2}");
          rsb.delete(m.start() + offset, m.end() + offset);
          offset = offset - m.group().length() + 11;
        }
        else
        {
          rsb.replace(m.start() + offset, m.end() + offset, "[\\r\\n]{1,2}");
          offset = offset - m.group().length() + 11;
        }
      }
      else
      {

        rsb.replace(m.start() + offset, m.end() + offset, "[\\r\\n]{1,2}");
        offset = offset - m.group().length() + 11;
      }
    }
    return rsb.toString();

  }

  public static boolean validateTag(String tagNam, String tagVal)
  {
    String reg = generValidateTagReg(tagNam);
    if (reg == null)
      return false;
    if (tagVal == null)
      return false;
    return tagVal.matches(reg);
  }

  /*
   * @tagVal ???
   */
  /*
   * @tagVal ???
   */
  public static Map<Integer, String> splitTag(String tagVal, String pattern, Map<String, String> adjMap)
  {
    Map<Integer, String> dataSet = new LinkedHashMap<Integer, String>();
    if (tagVal == null || pattern == null)
      return dataSet;
    String reg = generTagReg(pattern, adjMap);
    //System.out.println("reg:"+reg);
    if (reg != null)
    {
      int regLen = reg.length();
      int pos = 0;
      int regpos = 0;
      int seq = 0;
      for (int i = 0; i < regLen; i++)
      {
        if (reg.charAt(i) == '(')
        {
          int rtCom = -1;
          seq++;
          for (int j = i + 1; j < regLen; j++)
          {
            if (reg.charAt(j) == '(')
            {
              rtCom--;
            }
            else if (reg.charAt(j) == ')')
              rtCom++;
            if (rtCom == 0)
            {
              String subreg = reg.substring(i + 1, j);
              String regsp = reg.substring(regpos, i).replaceAll("\\(.*\\)\\?", "");
              // System.out.println("regsp:"+regsp);

              boolean isOpt = reg.length() > j + 1 && reg.charAt(j + 1) == '?';
              Matcher m = Pattern.compile(subreg).matcher(tagVal);
              if (m.find(pos))
              {
                if (isOpt)
                  regpos = j + 2;
                else
                  regpos = j + 1;
                String sp = tagVal.substring(pos, m.start());
                //      System.out.println("sp:"+sp);
                if (m.start() == pos || (i >= 1 && reg.substring(i - 1, i).matches("\\r|\\n") && sp.matches("\\r\\n|\\n\\r|\\r|\\n"))
                        || sp.equals(regsp))
                {
                  pos = m.end();
                  String val = m.group();
                  if (sp.length() > 0 && !sp.equals(regsp))
                    val += sp;
                  if (val.matches("\\d+,\\d*")) // ??????????????
                  {
                    if (val.endsWith(","))
                      val += "00";
                    val = val.replace(",", ".");
                    int staPos = val.indexOf(".");
                    StringBuilder valBuf = new StringBuilder(val);
                    int numCnt = 0;
                    for (int p = staPos - 1; p >= 0; p--)
                    {
                      if (numCnt == 3)
                      {
                        valBuf.insert(p + 1, ',');
                        numCnt = 0;
                      }
                      numCnt++;
                    }
                    val = "#" + valBuf.toString() + "#";
                  }
                  if (subreg.startsWith("//"))
                    val = val.replaceAll("^//", "");
                  dataSet.put(seq, val); // ????????MAP
                }
                else if (!isOpt) // ?????
                {
                  return dataSet;
                  // throw new Exception("??????"+des);
                }
                i = j;
                break;
              }
              else if (!isOpt)
              {
                return dataSet;
              }
              i = j;
              break;
            }
          }
        }
      }
    }
    return dataSet;
  }

  /*
   * ???????????
   *
   * @param tagNam????
   *
   * @param tagVal???
   */
  @SuppressWarnings("unchecked")
  public static Map<Integer, String> splitTag(String tagNam, String tagVal)
  {
    String[] ptnsArr = getTagPattern(tagNam);
    String pattern = null;
    Map<String, String> adj = null;
    if (ptnsArr != null && ptnsArr.length >= 3)
    {
      pattern = ptnsArr[0];
      if (ptnsArr[1] != null)
        adj = new Gson().fromJson(ptnsArr[1], Map.class);
    }
    return splitTag(tagVal, pattern, adj);
  }

  /* ???????????????? */
  public static int caculateTagWidthByTagNam(String tagNam)
  {
    String[] tagPtns = getTagPatternViaCb(tagNam);
    if (tagPtns == null)
      return 0;
    return caculateTagWidth(tagPtns[0]);
  }

  /* ????????????????????? */
  public static int caculateTagWidth(String pattern)
  {
    if (pattern == null)
      return 0;
    String nail = "\\d+(\\*\\d+)?!?[nacxd]";
    int lstLen = 0;
    int len = 0;
    int lstIdx = 0;
    pattern = pattern.replaceAll("\\[|\\]", "");
    Matcher m = Pattern.compile(nail).matcher(pattern);
    while (m.find())
    {
      int subLen = m.start() - lstIdx;
      if (subLen != 0)
      {
        String subVal = pattern.substring(lstIdx, m.start());
        if (subVal.matches("\\r\\n|\\n\\r|\\r|\\n"))
        {
          if (lstLen < len)
          {
            lstLen = len;
          }
          len = 0;// ??
          lstIdx += subLen;
        }
        else
        {
          lstIdx += subLen;
          len += subLen;
        }
      }
      String subp = m.group();
      lstIdx += subp.length();
      Matcher subm = Pattern.compile("\\d+(?=[!]?[nacxd])").matcher(subp);
      if (subm.find())
      {
        int sublen = Integer.parseInt(subm.group());
        len += sublen;
      }
    }
    return lstLen > len ? lstLen : len;
  }

  /* ???????????????????? */
  public static String[] combineTag(Map<Integer, String> map, String pattern)
  {
    if (map == null || map.size() == 0)
      return null;
    if (pattern == null || !pattern.matches("(\\d+(\\^\\d+)*:\\d+:.+(;|$))+"))
    {
      return null;
    }
    String[] pArr = pattern.split(";");
    StringBuilder desBuf = new StringBuilder();
    List<List<String>> valLst = new ArrayList<List<String>>();
    List<String> oneLst = new ArrayList<String>();
    for (int i = 0; i < pArr.length; i++)
    {
      String oneDes = pArr[i];
      String[] desArr = oneDes.split(":");
      Matcher m = Pattern.compile("\\d+").matcher(desArr[0]);
      StringBuilder oneBuf = new StringBuilder(desArr[0]);
      int offset = 0;
      while (m.find())
      {
        String pos = m.group();
        int posInt = Integer.parseInt(pos);
        String subVal = map.get(posInt);
        subVal = subVal == null ? "" : subVal;
        oneBuf.replace(m.start() + offset, m.end() + offset, subVal);
        offset = offset + subVal.length() - pos.length();
      }
      desBuf.append(String.format("%-" + desArr[1] + "s", desArr[2]));
      String oneStr = oneBuf.toString().replaceAll("\\^", "");
      if (oneStr.contains("~"))
      {
        String[] oneArr = oneStr.split("~");
        oneLst.add(String.format("%-" + desArr[1] + "s", oneArr[0]));
        for (int j = 1; j < oneArr.length; j++)
        {
          if (oneArr[j].startsWith("\r\n"))
            oneArr[j] = oneArr[j].substring(2);
          boolean isExist = valLst.size() > j + 1;
          List<String> newLine = null;
          if (isExist)
            newLine = valLst.get(j + 1);
          else
          {
            newLine = new ArrayList<String>();
            valLst.add(newLine);
          }
          int fillCnt = oneLst.size() - 1;
          if (newLine.size() < fillCnt + 1)
          {
            for (int p = newLine.size(); p < fillCnt; p = newLine.size())
            {
              int pos = p;
              if (pos > -1)
                newLine.add(String.format("%-" + oneLst.get(pos).length() + "s", "")); // ?????????????????????
            }
            newLine.add(String.format("%-" + desArr[1] + "s", oneArr[j]));
          }
          else
          {
            newLine.set(fillCnt, String.format("%-" + desArr + "s", oneArr[j]));
          }
        }
      }
      else
        oneLst.add(String.format("%-" + desArr[1] + "s", oneStr));
    }
    StringBuilder valBuf = new StringBuilder();
    valLst.add(0, oneLst);
    for (List<String> lst : valLst)
    {
      for (String str : lst)
      {
        valBuf.append(str);
      }
      valBuf.append("\r\n");
    }
    return new String[]{desBuf.toString(), valBuf.toString()};
  }

  @SuppressWarnings("unused")
  private static boolean verfyMTPtn(String msgTyp)
  {
    List<Map<String, String>> lst = getTagsInfo(msgTyp);
    String reg = generReg(lst);
    int rcm = 0;
    int lcm = 0;
    for (int i = 0; i < reg.length(); i++)
    {
      if (reg.charAt(i) == '(')
        rcm++;
      else if (reg.charAt(i) == ')')
        lcm++;
    }
    System.out.println("?????" + lcm);
    System.out.println("?????" + rcm);
    return lcm == rcm;
  }

  private static String toUpper(String str)
  {
    if (str != null)
      return str.toUpperCase();
    return str;
  }

  private static String changeAddr(Object adr)
  {
    if (adr == null)
      return "";
    int pos = 35;
    StringBuilder buf = new StringBuilder((String) adr);
    while (pos < buf.length())
    {
      buf.insert(pos, "\r\n");
      pos = pos + 35 + 2;
    }
    return buf.toString().replaceAll("[ ]+(?=\\r\\n)", "");
  }

  @SuppressWarnings("rawtypes")
  private static boolean isInJsonMap(Map msgMap, Object key)
  {
    Object val = msgMap.get(key);
    if (val != null && val.toString().length() > 0)
      return true;
    else
      return false;
  }

  public static String getLogicalTerminalAddress(String bic)
  {
    if (bic == null)
      return null;
    if (bic.length() == 11)
    {
      return bic.substring(0, 8) + "X" + bic.substring(8, 11);
    }
    else if (bic.length() == 8)
    {
      return bic.substring(0, 8) + "XXXX";
    }
    else
    {
      return null;
    }
  }

  public static String get21Ref(String orgseq)
  {
    if (orgseq == null || "".equals(orgseq))
      return "NONREF";
    else if ("ZJDB^DCDB^NBFK".contains(orgseq))
      return "NONREF";
    return orgseq;
  }

  public static void toNACKMessage(String folder)
          throws FileNotFoundException, IOException
  {
    File fod = new File(folder);
    for (File file : fod.listFiles())
    {
      if (!file.isFile())
        continue;
      if (!file.getName().endsWith("sf2"))
        continue;
      int typ = new Random().nextInt(2);
      String typStr = "";
      if (typ == 0)
        typStr += "ACK";
      else if (typ == 1)
        typStr += "NCK";
      else
        continue;
      String newNam = typStr + file.getName();
    }
  }

  public static void fenjian(String path)
          throws IOException
  {
    File f = new File(path);
    File f103 = new File(f, "i103");
    if (!f103.exists())
      f103.mkdir();
    File fo = new File(f, "o");
    if (!fo.exists())
      fo.mkdir();
    for (File file : f.listFiles())
    {
      if (file.isFile())
      {
        try
        {
          Map<String, String> map = getSwfMsgInfo(file.getAbsolutePath());
          if (map.get("mty").equals("103") && map.get("direct").equals("I"))
          {
            String nam = file.getName();
            File nfile = new File(f103, nam);
            if (!nfile.exists())
              nfile.createNewFile();
            IOUtils.copy(new FileInputStream(file), new FileOutputStream(nfile));

          }
          else if (map.get("direct").equals("O"))
          {
            String nam = file.getName();
            File nfile = new File(fo, nam);
            if (!nfile.exists())
              nfile.createNewFile();
            IOUtils.copy(new FileInputStream(file), new FileOutputStream(nfile));
          }
        }
        catch (Exception e)
        {
          e.printStackTrace();
          System.out.println(file.getName());

        }
      }
    }
  }

  @SuppressWarnings("unchecked")
  public static List<String> unCompressZipFile(String zipPth, String bufFolder)
          throws IOException
  {
    ZipFile zf = new ZipFile(zipPth);
    List<String> zipSubFileList = new ArrayList<String>();
    Enumeration<ZipEntry> em = (Enumeration<ZipEntry>) zf.entries();
    while (em.hasMoreElements())
    {
      ZipEntry ze = em.nextElement();
      if (ze.isDirectory())
      {
        new File(ze.getName()).mkdirs();
      }
      else
      {
        File nFile = new File(bufFolder, ze.getName());
        zipSubFileList.add(ze.getName());
        if (!nFile.getParentFile().exists())
        {
          nFile.getParentFile().mkdirs();
          nFile.createNewFile();
        }
        else if (!nFile.exists())
          nFile.createNewFile();
        FileOutputStream fos = new FileOutputStream(nFile);
        IOUtils.copy(zf.getInputStream(ze), fos);
        fos.flush();
        fos.close();
      }
    }
    zf.close();
    return zipSubFileList;
  }

  /***
   * ??str?\r\n?????????
   * */
  public static String getRightPart(String str)
  {
    if (str.endsWith("\r\n"))
    {
      str = str.substring(0, str.length() - 2);
      return getRightPart(str);
    }
    else
    {
      return str;
    }
  }

  public static String fmtToswift(String fmt)
          throws FileNotFoundException, IOException
  {
    String swiftmsg = fmt;
    swiftmsg = swiftmsg.replaceAll("(?<=(\r\n)|:)\\/(([A-Z]\\d{2})|(\\d{1,4}))\\/", "");

    return swiftmsg;
  }

  /**
   * ??Tag?Value
   * **/

  public static Map<String, String> splitTagValue(String message)
  {
    String sign = "\r\n";
    if (!message.startsWith(sign))
    {
      message = sign + message;
    }
    Map<String, String> map = new LinkedHashMap<String, String>();
    String lastKey = null;
    int lastHead = -1;
    String reg = "(?<=(\r\n))\\/([A-Z]\\d{2})\\/";
    Matcher m = Pattern.compile(reg).matcher(message);
    while (m.find())
    {
      if (lastKey != null)
      {
        map.put(lastKey, message.substring(lastHead, m.start() - 2));
      }
      int head = -1;
      int end = -1;
      String key = m.group();
      head = m.end();
      boolean hasNext = m.find();
      if (hasNext)
      {
        end = m.start() - 2;
        if (end > head)
        {
          map.put(key, message.substring(head, end));
        }
        lastKey = m.group();
        lastHead = m.end();
      }
      else
      {
        lastKey = key;
        lastHead = head;
      }
    }
    if (lastKey != null)
      map.put(lastKey, message.substring(lastHead, message.length()));
    System.out.println("map=" + map);
    return map;

    /*  for (String key : map.keySet())
      {
        System.out.println("....key=" + key + "......");
      }*/
  }

  /**
   * @param val ?????
   * @fmttype FMT????
   * @inipth   INI????
   *
   * **/
  public static List<List<Map<String, String>>> splitDtl(String val, String fmttype, String inipth)
          throws IOException
  {
    List<List<Map<String, String>>> list = new ArrayList<List<Map<String, String>>>();
    if (content == null)
    {
      FileInputStream fis = new FileInputStream(new File(inipth));
      content = IOUtils.readLines(fis);
      fis.close();
    }
    Map<String, Map<String, Object>> iniMap = IniUtil.initINIFileX(content);
    Map<String, Object> dtlKeyMap = iniMap.get(fmttype);
    // System.out.println(dtlKeyMap);
    List<String> liststr = splitDtlValue(val);
    //???????
    for (String str : liststr)
    {
      List<Map<String, String>> sublist = new ArrayList<Map<String, String>>();
      String substrs[] = str.split("\r\n");
      //??????
      for (String substr : substrs)
      {
        if (!StringUtil.isEmpty(substr))
        {
          String key = null;
          if (substr.startsWith("/1/"))
          {
            key = dtlKeyMap.get("1").toString();
          }
          else if (substr.startsWith("/2/"))
          {
            key = dtlKeyMap.get("2").toString();
          }
          String keys[] = key.split("\\/");
          substr = substr.replace("\r\n", "").replace("\n", "").replace("\r", "");
          String values[] = substr.substring(1).split("\\/");
          Map<String, String> cMap = new LinkedHashMap<String, String>();
          //??????
          for (int i = 0; i < keys.length; i++)
          {
            if (i < values.length)
              cMap.put(keys[i], values[i]);
            else
              cMap.put(keys[i], null);
          }
          sublist.add(cMap);
        }
      }
      list.add(sublist);
    }
    return list;
  }

  /***
   * ???????Value
   * **/
  public static List<String> splitDtlValue(String message)
  {
    String sign = "\r\n";
    if (!message.startsWith(sign))
    {
      message = sign + message;
    }
    List<String> list = new ArrayList<String>();
    String lastKey = null;
    int lastHead = -1;
    String reg = "(?<=(\r\n))\\/([1])\\/";
    Matcher m = Pattern.compile(reg).matcher(message);
    while (m.find())
    {
      if (lastKey != null)
      {
        list.add(lastKey + message.substring(lastHead, m.start() - 1));
      }
      int head = -1;
      int end = -1;
      String key = m.group();
      head = m.end();
      boolean hasNext = m.find();
      if (hasNext)
      {
        end = m.start() - 1;
        if (end > head)
        {
          list.add(key + message.substring(head, end));
        }
        lastKey = m.group();
        lastHead = m.end();
      }
      else
      {
        lastKey = key;
        lastHead = head;
      }
    }
    if (lastKey != null)
      list.add(lastKey + message.substring(lastHead, message.length()));
    return list;
  }

  public static void main(String[] args)
          throws FileNotFoundException, IOException
  {
    // boolean b= MessageReader.validateTag("12E", "test1111111111111111");
    // System.out.println(b);
    // String swiftmsg = IOUtils.toString(new FileInputStream(new File("g:/fmt100.txt")));
    String swiftmsg = IOUtils.toString(new FileInputStream(new File("g:/login.txt")));
    //swiftmsg=swiftmsg.replaceAll("\r\n\\/\\w+\\/", "\r\n");
    //swiftmsg = swiftmsg.replaceAll("\r\n\\/\\w+\\/", "\r\n");
    swiftmsg = swiftmsg.replaceAll("(?<=(\r\n)|:)\\/(([A-Z]\\d{2})|(\\d{1,2}))\\/", "");
    /*  Pattern p = Pattern.compile("\r\n\\:\\w+\\:\\/\\w+\\/"); // ?????
      Matcher m = p.matcher(swiftmsg); // ??????
      StringBuffer sb = new StringBuffer();

      while (m.find())
      {
        // System.out.println(m.group());
        String txt = m.group();
        txt = txt.replaceAll("\\/\\w+\\/", "");
        m.appendReplacement(sb, txt);
        m.appendTail(sb);
      }
      swiftmsg = sb.toString();*/
    String s = "??";
    System.out.println(s.getBytes("GB18030").length);
    System.out.println(new String(s.getBytes("GB18030"), "GB18030"));
    //  System.out.println(swiftmsg);
    // List<String[]> list = MessageReader.splitViaStr(swiftmsg);
    //    for (String[] strings : list)
    //    {
    //      //System.out.println(strings[0] + "------->" + strings[1]);
    //    }

    Map<String, String> message = MessageReader.getSwfMsgInfoViaMsg(swiftmsg);
    for (String b : message.keySet())
    {
      // System.out.println(b + ":" + message.get(b));
    }
    List<String[]> ll = simpleSplit("g:/login.txt");
    for (String[] strs : ll)
    {
      System.out.println(strs[0] + "----------->" + strs[1] + ".....");
    }
    Map<Integer, String> map = anlyMsgHead(swiftmsg);
    System.out.println(map);
  }
}
