package com.brilliance.eibs.swift;

import org.apache.commons.io.IOUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author gechengyang
 * SWIFT报文的栏位顺序校验，栏位值校验，以及栏位互斥/同时出现等校验
 * **/
public class SwiftValidate
{

  private StringBuffer         erError       = new StringBuffer();
  private String               pth           = "";
  private String               swiftmsg      = "";
  private String               mty           = "";
  //获取报文的基本信息
  private Map<String, String>  basemap       = new HashMap<String, String>();
  //获取报文每个模块
  private Map<Integer, String> swiftblockMap = new HashMap<Integer, String>();
  //获取SWIFT报文的第四域的信息   
  private List<String[]>       listPartfour  = new ArrayList<String[]>();
  public static String         splitLine     = System.getProperty("line.separator");

  public SwiftValidate( String pth )
  {
    this.pth = pth;
    //根据报文路径和；获取报文信息
    this.swiftmsg = MessageReader.getSwfMsg(pth);
    //获取报文的基本信息
    this.basemap = MessageReader.getSwfMsgInfoViaMsg(swiftmsg);
    this.mty = basemap.get("mty");
    this.swiftblockMap = MessageReader.anlyMsgHead(swiftmsg);
    this.listPartfour = MessageReader.msgTxtAnly(swiftblockMap.get(4));
  }

  public SwiftValidate()
  {
  }

  /**
   * @param msg
   * 
   * add by hujun 2014/10/28
   */
  public void setData(String msg)
  {
    this.swiftmsg = msg;
    //获取报文的基本信息
    this.basemap = MessageReader.getSwfMsgInfoViaMsg(swiftmsg);
    this.mty = basemap.get("mty");
    this.swiftblockMap = MessageReader.anlyMsgHead(swiftmsg);
    this.listPartfour = MessageReader.msgTxtAnly(swiftblockMap.get(4));
  }

  /**
   * 获取详细的错误校验信息
   * **/
  public String getErError()
  {
    return erError.toString();
  }

  /**
   * @param pth 报文路径
   * 校验SWIFT报文所有的栏位顺序，校验成功则返回true
   * **/
  public boolean checkAllTagSequnce()
  {
    boolean check = true;
    StringBuffer sb = new StringBuffer();
    for (String[] strings : listPartfour)
    {
      sb.append(strings[0] + ",");
    }
    //得到栏位值的顺序
    String str = sb.toString();
    List<Map<String, String>> lst = MessageReader.getTagsInfo(mty);
    //得到栏位的正则
    String tagsreg = MessageReader.generReg(lst);
    tagsreg = tagsreg.replace("null", "[\\s\\S]*");
    if (!(str.matches(tagsreg)))
    {
      check = false;
      erError.append("栏位顺序校验失败[+" + pth + "]" + splitLine);
      erError.append("栏位字符串为：" + str + splitLine);
      erError.append("栏位正则为：" + tagsreg + splitLine);
    }
    return check;
  }

  /**
   * @param pth报文路径
   * 校验报文所有栏位的值
   * **/
  public boolean checkAllTagsValue()
  {
    boolean check = true;
    for (String[] strings : listPartfour)
    {
      boolean b = MessageReader.validateTag(strings[0], MessageReader.getRightPart(strings[1]));
      if (!b)
      {
        check = false;
        erError.append("[错误警告][+" + pth + "]栏位[" + strings[0] + "]的值为:" + splitLine + strings[1] + splitLine);
        erError.append("SWIFT报文格式为：" + splitLine + (MessageReader.getTagPattern(strings[0]))[0] + splitLine);
        erError.append("栏位自动生成的正则为:" + splitLine + MessageReader.generValidateTagReg(strings[0]) + splitLine);
      }
    }
    return check;
  }

  public static void main(String[] args) throws FileNotFoundException, IOException
  {
    String pth = "G:/USO120510000555.sf2";
    SwiftValidate sv = new SwiftValidate(pth);
    System.out.println(sv.checkAllTagsValue());
    System.out.println(sv.getErError());
    String ss=IOUtils.toString(new FileInputStream(new File("g:/1.txt")));
    System.out.println("....."+ss+"...");
    String regex="(/([\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]{1,34}))?([\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]{1,35}([\r\n]{1,2}[\\w/\\-\\?:\\(\\)\\.,'+\\{\\} ]{1,35}){0,3})";
    System.out.println(ss.matches(regex));
  }
}
