package com.brilliance.eibs.util;

import java.io.UnsupportedEncodingException;
import java.nio.ByteOrder;

/**
 * byte数组操作类
 * 
 * @author xyz
 *
 */
public class ByteUtil {

	/**
	 * 注释：字节数组到short的转换！
	 * 
	 * @param b
	 * @return
	 */
	public static int byteToShort(byte[] b) {
		short s = 0;
		short s0 = (short) (b[0] & 0xff);// 最低位
		short s1 = (short) (b[1] & 0xff);
		s1 <<= 8;
		s = (short) (s0 | s1);
		return s;
	}

	public static int byteToShort(byte[] b, int flg) {
		if (flg == Constants.LITTLE_ENDIAN)
			return byteToShort(b);
		if (flg == Constants.BIG_ENDIAN) {
			short s = 0;
			short s0 = (short) (b[1] & 0xff);// 最低位
			short s1 = (short) (b[0] & 0xff);
			s1 <<= 8;
			s = (short) (s0 | s1);
			return s;
		}
		return 0;
	}

	public static int byteToShortC(byte[] b) {
		short s = 0;
		short s0 = (short) (b[0] & 0xff);// 最低位
		short s1 = (short) (b[1] & 0xff);
		if (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN)
			s0 <<= 8;
		else
			s1 <<= 8;
		s = (short) (s0 | s1);
		return s;
	}

	/**
	 * 注释：short到字节数组的转换！
	 * 
	 * @param s
	 * @return
	 */
	public static byte[] shortToByte(int number) {
		int temp = number;
		byte[] b = new byte[2];
		for (int i = 0; i < b.length; i++) {
			b[i] = new Integer(temp & 0xff).byteValue();// 将最低位保存在最低位
			temp = temp >> 8; // 向右移8位
		}
		return b;
	}

	public static byte[] shortToByte(int number, int flg) {
		if (flg == Constants.LITTLE_ENDIAN) {
			return shortToByte(number);
		}
		if (flg == Constants.BIG_ENDIAN) {
			int temp = number;
			byte[] b = new byte[2];
			for (int i = (b.length - 1); i >= 0; i--) {
				b[i] = new Integer(temp & 0xff).byteValue();// 将最低位保存在最低位
				temp = temp >> 8; // 向右移8位
			}
			return b;
		}
		return null;
	}

	public static byte[] shortToByteC(int number) {
		if (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN) {
			int temp = number;
			byte[] b = new byte[2];
			for (int i = b.length - 1; i >= 0; i--) {
				b[i] = new Integer(temp & 0xff).byteValue();// 将最低位保存在最低位
				temp = temp >> 8; // 向右移8位
			}
			return b;
		} else {
			int temp = number;
			byte[] b = new byte[2];
			for (int i = 0; i < b.length; i++) {
				b[i] = new Integer(temp & 0xff).byteValue();// 将最低位保存在最低位
				temp = temp >> 8; // 向右移8位
			}
			return b;
		}
	}

	/**
	 * 将一个int转换成4个字节
	 * 
	 * @param i
	 * @return
	 */
	public static byte[] intToBytes(int i) {
		byte[] result = new byte[4];
		result[0] = (byte) ((i >> 24) & 0xFF);
		result[1] = (byte) ((i >> 16) & 0xFF);
		result[2] = (byte) ((i >> 8) & 0xFF);
		result[3] = (byte) (i & 0xFF);
		return result;
	}

	public static byte[] intToBytes(int i, int flg) {
		if (flg == Constants.BIG_ENDIAN)
			return intToBytes(i);
		if (flg == Constants.LITTLE_ENDIAN) {
			byte[] result = new byte[4];
			result[3] = (byte) ((i >> 24) & 0xFF);
			result[2] = (byte) ((i >> 16) & 0xFF);
			result[1] = (byte) ((i >> 8) & 0xFF);
			result[0] = (byte) (i & 0xFF);
			return result;
		}
		return null;
	}

	public static byte[] intToBytesC(int i) {
		byte[] result = new byte[4];
		if (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN) {
			result[0] = (byte) ((i >> 24) & 0xFF);
			result[1] = (byte) ((i >> 16) & 0xFF);
			result[2] = (byte) ((i >> 8) & 0xFF);
			result[3] = (byte) (i & 0xFF);
		} else {
			result[3] = (byte) ((i >> 24) & 0xFF);
			result[2] = (byte) ((i >> 16) & 0xFF);
			result[1] = (byte) ((i >> 8) & 0xFF);
			result[0] = (byte) (i & 0xFF);
		}
		return result;
	}

	/**
	 * byte[]转int
	 * 
	 * @param bytes
	 * @return
	 */
	public static int bytesToInt(byte[] bytes) {
		if (bytes.length > 4)
			throw new RuntimeException("00004");
		int des = 0;
		for (int i = 0; i < bytes.length; i++) {
			des += (bytes[bytes.length - i - 1] & 0xff) << (i * 8);
		}
		return des;
	}

	// 1--> 将byte数组bRefArr转为一个整数,字节数组的低位是整型的低字节位

	public static int bytesToInt(byte[] bytes, int flg) {
		if (flg == Constants.BIG_ENDIAN) {
			return bytesToInt(bytes);
		}
		if (flg == Constants.LITTLE_ENDIAN) {
			int iOutcome = 0;
			byte bLoop;

			for (int i = 0; i < bytes.length; i++) {
				bLoop = bytes[i];
				iOutcome += (bLoop & 0xFF) << (8 * i);
			}
			return iOutcome;
		}
		return 0;
	}

	public static int bytesToIntC(byte[] bytes) {
		if (bytes.length > 4)
			throw new RuntimeException("00004");
		int des = 0;
		for (int i = 0; i < bytes.length; i++)
			if (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN)
				des += (bytes[bytes.length - i - 1] & 0xff) << (i * 8);
			else
				des += (bytes[i] & 0xff) << (i * 8);
		return des;
	}

	public static String parseByte2HexStr(byte buf[]) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < buf.length; i++) {
			String hex = Integer.toHexString(buf[i] & 0xFF);
			if (hex.length() == 1) {
				hex = '0' + hex;
			}
			sb.append(hex.toUpperCase());
		}
		return sb.toString();
	}

	/**
	 * 将16进制转换为二进制
	 * 
	 * @param hexStr
	 * @return
	 */
	public static byte[] parseHexStr2Byte(String hexStr) {
		if (hexStr.length() < 1)
			return null;
		byte[] result = new byte[hexStr.length() / 2];
		for (int i = 0; i < hexStr.length() / 2; i++) {
			int high = Integer.parseInt(hexStr.substring(i * 2, i * 2 + 1), 16);
			int low = Integer.parseInt(hexStr.substring(i * 2 + 1, i * 2 + 2), 16);
			result[i] = (byte) (high * 16 + low);
		}
		return result;
	}

	public static int bytes2int(byte[] bytes, int offset, int n) {
		int value = 0;
		for (int i = 0; i < n; i++) {
			int shift = (n - 1 - i) * 8;
			value += (bytes[i + offset] & 0x000000FF) << shift;
		}
		return value;
	}

	public static byte[] int2bytes(int i, int n) {

		byte[] value = new byte[n];
		for (int j = 0; j < n; j++) {
			value[j] = (byte) (i >> (n - j - 1) * 8 & 0xFF);
		}
		return value;
	}

	public static byte bit2byte(String bString) {
		byte result = 0;
		for (int i = bString.length() - 1, j = 0; i >= 0; i--, j++) {
			result += (Byte.parseByte(bString.charAt(i) + "") * Math.pow(2, j));
		}
		return result;
	}

	public static String byte2bits(byte b) {
		int z = b;
		z |= 256;
		String str = Integer.toBinaryString(z);
		int len = str.length();
		return str.substring(len - 8, len);
	}

	public static String bytes2Str(byte[] buffer, String encoding) throws UnsupportedEncodingException {
		if (StringUtil.isEmpty(encoding))
			return bytes2Str(buffer);
		else
			return new String(buffer, encoding);
	}

	public static String bytes2Str(byte[] buffer) throws UnsupportedEncodingException {
		return new String(buffer, "UTF-8");
	}

}
