package com.brilliance.eibs.util;

import com.brilliance.eibs.bean.ParseConfig;
import com.brilliance.eibs.el.CommonFunctionUtils;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * 用以动态见天classpath路径，是利用反射调用URLClassLoader的addURL方法
 * 
 * @author hujun
 *
 */
public class ClassPathUpdater {

    private static final Class<?>[] PARAMETERS = new Class[] {URL.class};

    private static final Class<?> CLASS_LOADER = URLClassLoader.class;

    public static void add(String s)
            throws SecurityException, IllegalArgumentException, MalformedURLException,
            NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        add(new File(s));
    }

    public static void add(File f)
            throws SecurityException, IllegalArgumentException, MalformedURLException,
            NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        f = f.isDirectory() ? f : f.getParentFile();
        add(f.toURI().toURL());
    }

    public static void add(URL url) throws SecurityException, NoSuchMethodException,
            IllegalArgumentException, IllegalAccessException, InvocationTargetException {
        Method method = CLASS_LOADER.getDeclaredMethod("addURL", PARAMETERS);
        method.setAccessible(true);
        method.invoke(getClassLoader(), new Object[] {url});
    }

    private static URLClassLoader getClassLoader() {
        return (URLClassLoader) ClassLoader.getSystemClassLoader();
        // return (URLClassLoader)
        // Thread.currentThread().getContextClassLoader();
    }

    /**
     * 动态加载class，动态路径目前有ParseConfig中的常量bulidpath提供
     * 
     * @param classname
     * @return
     * @throws SecurityException
     * @throws IllegalArgumentException
     * @throws MalformedURLException
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws ClassNotFoundException
     */
    public static Class<?> dynamicLoadClass(String classname) throws SecurityException,
            IllegalArgumentException, MalformedURLException, NoSuchMethodException,
            IllegalAccessException, InvocationTargetException, ClassNotFoundException {
        add(ParseConfig.buildpath);
        // return new
        // MyClassLoader(MyClassLoader.class.getClassLoader()).loadClass(ParseConfig.buildpath,
        // classname);
        return Class.forName(classname);
    }

    /**
     * 为file前加上classpath路径
     * 
     * @param file
     * @return
     */
    public static String addClasspath(String file) {

        // return
        // URLDecoder.decode(Thread.currentThread().getContextClassLoader().getResource("").toURI().getPath()
        // + file, "utf-8");
        return CommonFunctionUtils.decodeURL(CommonFunctionUtils.getCommonURL("").getPath() + file);
        /*
         * catch (URISyntaxException e) { throw new RuntimeException("URI error.", e); }
         */

    }

}
