package com.brilliance.eibs.util;

import com.brilliance.eibs.core.exception.ErrorCodes;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.exception.SysCode;
import com.brilliance.eibs.core.model.IInterfaceDef;
import com.brilliance.eibs.core.service.Context;
import org.slf4j.Logger;

import java.util.List;

/**
 * 异常处理工具类
 *
 * @author xiaoyuanzhen
 */
public class ExceptionUtil {

    /**
     * 获取产生错误的错误码对象
     *
     * @param context
     * @param e
     * @return
     */
    public static SysCode getErrorCode(Context context, Exception e) {
        Logger logger = LogUtil.getLogger(context);
        String ref = null;
        String codeId = null;
        String message = null;
        if (e instanceof InterfaceException) {
            InterfaceException iException = (InterfaceException) e;
            codeId = iException.getId();
            message = iException.getMessage();
            if (iException.getException() == null) {
                logger.error(iException.getMessage(), iException);
            } else {
                logger.error(iException.getMessage(), iException.getException());
            }
        } else {
            logger.error("error", e);
            codeId = "10000";
        }
        SysCode sysCode = new SysCode();
        ErrorCodes errorCodes = context.getErrorCodes();
        if (errorCodes == null) {
            sysCode.setId(codeId);
            sysCode.setErrorText(message);
            return sysCode;
        }
        List<SysCode> sysCodes = errorCodes.getSysCodes();
        IInterfaceDef interfaceDef = context.getInterfaceDef();
        if (interfaceDef != null) {
            if ("client".equals(interfaceDef.getType())) {
                String description = (String) context.getVariable(codeId);
                sysCode.setId(codeId);
                if (!StringUtil.isEmpty(description))
                    sysCode.setErrorText(description);
                else
                    sysCode.setErrorText(message);
                return sysCode;
            }
        }
        for (SysCode innerCode : sysCodes) {
            if (innerCode.getId().equals(codeId)) {
                ref = innerCode.getRef();
                sysCode.setId(ref);
                /*
                 * String id = context.getInterfaceDef().getId() + "." + ref;
                 * System.out.println(id); sysCode.setErrorText((String)
                 * context.getVariable(id));
                 */
                break;
            }
        }
        if (StringUtil.isEmpty(sysCode.getId())) {
            sysCode.setId(codeId);
        }
        String description = (String) context.getVariable(codeId);
        if (!StringUtil.isEmpty(description))
            sysCode.setErrorText(description);
        else
            sysCode.setErrorText(message);

        if (!StringUtil.isEmpty(ref)) {
            String id = context.getInterfaceDef().getId() + "." + ref;
            sysCode.setErrorText((String) context.getVariable(id));
        }
        return sysCode;
    }
}
