package com.brilliance.eibs.util;

import org.apache.log4j.Appender;
import org.apache.log4j.DailyRollingFileAppender;
import org.apache.log4j.spi.LoggingEvent;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 重新自定义log4j中的DailyRollingFileAppender,利用多个Append完成多并发的日志输出
 *
 * @author hujun
 */
public class MyDailyRollingFileAppender extends DailyRollingFileAppender {

    private Map<String, Appender> appenderCache = new HashMap<String, Appender>();

    @Override
    public synchronized void doAppend(LoggingEvent event) {
        String id = (String) event.getMDC(LogUtil.LOG_FILE_NAME_SYMBOL);
        if (LogUtil.SERVICE.equals(id)) {
            id = id + "_" + DateUtil.format(new Date(event.getTimeStamp()), "yyyyMMdd");
        } else {
            id = DateUtil.format(new Date(event.getTimeStamp()), "yyyyMMdd") + "/" + id;
        }
        Appender appender = appenderCache.get(id);
        if (appender == null) {
            try {
                appender = createAppender(id);
            } catch (IOException e) {
                // 因为该类属于log4j，顾不知道如何处理异常，直接抛出
                throw new RuntimeException(e);
            }
            appenderCache.put(id, appender);
        }
        appender.doAppend(event);
    }

    @Override
    public synchronized void setFile(String fileName, boolean append, boolean bufferedIO, int bufferSize) throws IOException {
        // 为了能够使用log4j.appender.FILE.File属性，同时能够不生成文件，该方法置空
        this.fileName = fileName;
        this.fileAppend = append;
        this.bufferedIO = bufferedIO;
        this.bufferSize = bufferSize;
    }

    private Appender createAppender(String name) throws IOException {
        String directory = StringUtil.isEmpty(getFile()) ? ClassPathUpdater.addClasspath(LogUtil.LOG_PATH) : getFile();
        DailyRollingFileAppender result = new DailyRollingFileAppender(getLayout(), directory + File.separator + name + ".log", getDatePattern());
        result.setName(getName());
        result.setLayout(getLayout());
        result.setAppend(getAppend());
        result.setBufferedIO(getBufferedIO());
        result.setBufferSize(getBufferSize());
        result.setEncoding(getEncoding());
        result.setErrorHandler(getErrorHandler());
        result.setImmediateFlush(getImmediateFlush());
        result.setThreshold(getThreshold());
        result.setFile(directory + File.separator + name + ".log");
        return result;

    }

}
