package com.brilliance.eibs.util;

import com.brilliance.eibs.core.exception.InterfaceException;
import org.apache.commons.io.FileUtils;

import java.io.*;

/**
 * 提供对象序列化的操作
 * 
 * @author hujun
 *
 */
public class Serializable2FileUtil
{
  private static final String serialExt = ".obj";                              //序列化文件后缀名
  private static final String serialDic = ClassPathUpdater.addClasspath("obj"); //序列化文件路径
  public static boolean       hotDeploy = false;                               //是否开启热部署

  /**
   * 初始序列化文件，避免重复序列化
   * 
   * @param object
   * @param filename
   */
  public synchronized static void initObject(Object object, String filename)
  {
    filename = getSerialReadPath(filename);
    File file = new File(filename);

    if (file.exists())
      return;
    directWriteObject(object, filename);
  }

  /**
   * 删除所有序列化文件
   */
  public static void deleteSerialFiles()
  {
    File file = new File(serialDic);
    if (file.exists())
    {
      File[] files = file.listFiles();
      for (int i = 0; i < files.length; i++)
      {
        FileUtils.deleteQuietly(files[i]);
      }
    }
  }

  /**
   * 删除指定序列化文件
   * @param filename
   */
  public static void deleteSerialFile(String filename)
  {
    File file = new File(serialDic);
    if (file.exists())
    {
      File[] files = file.listFiles();

      for (int i = 0; i < files.length; i++)
      {
        if (filename.equals(FileUtil.getFileName(files[i].getName())))
        {
          FileUtils.deleteQuietly(files[i]);
          break;
        }
      }
    }
  }

  /**
   * 将一个对象序列化到制定的文件中
   * 
   * @param object 需要序列化的对象
   * @param filename 文件路径
   * @throws IOException
   * @throws InterruptedException
   */
  public static void writeObject(Object object, String filename)
  {
    filename = getSerialReadPath(filename);
    directWriteObject(object, filename);
  }

  /**
   * 写入序列化文件到临时生成的temp目录
   * @param object
   * @param filename
   */
  public static void writeTempObject(Object object, String filename)
  {
    filename = getSerialWritePath(filename);
    directWriteObject(object, filename);
  }

  /**
   * 写入序列化文件，如果已存在，会进行覆盖
   * @param object
   * @param filename
   */
  public static void directWriteObject(Object object, String filename)
  {
    ObjectOutputStream objectOutputStream = null;
    File file = null;
    RandomAccessFile raf = null;
    FileOutputStream fos = null;
    BufferedOutputStream bos = null;
    try
    {
      file = new File(filename);
      File pFile = file.getParentFile();
      if (null != pFile && !pFile.exists())
      {
        pFile.mkdirs();
      }
      raf = new RandomAccessFile(file, "rw");
      fos = new FileOutputStream(raf.getFD());
      bos = new BufferedOutputStream(fos);
      objectOutputStream = new ObjectOutputStream(bos);
      objectOutputStream.writeObject(object);
    }
    catch (IOException e)
    {
      throw new InterfaceException("00010", "write file" + filename + "error");
    }
    finally
    {
      if (objectOutputStream != null)
        try
        {
          objectOutputStream.close();
          bos.close();
          raf.close();
        }
        catch (IOException e)
        {
          throw new InterfaceException("00010", "close outputstream error.");
        }
    }
  }

  /**
   * 从制定的文件从反序列化一个对象
   * 
   * @param filename 文件路径
   * @return
   * @throws IOException
   * @throws ClassNotFoundException
   */
  public static Object readObject(String filename)
  {
    Object object = null;
    filename = getSerialReadPath(filename);
    ObjectInputStream objectInputStream = null;
    RandomAccessFile raf = null;
    FileInputStream fis = null;
    BufferedInputStream bis = null;
    try
    {
      if (isFileExist(filename))
      {
        raf = new RandomAccessFile(filename, "r");
        fis = new FileInputStream(raf.getFD());
        bis = new BufferedInputStream(fis);
        objectInputStream = new ObjectInputStream(bis);
        object = objectInputStream.readObject();
      }
    }
    catch (FileNotFoundException e)
    {
      throw new InterfaceException("00010", "file" + filename + " not found");
    }
    catch (IOException e)
    {
      object = null;
    }
    catch (ClassNotFoundException e)
    {
      throw new InterfaceException("00010", e.getMessage());
    }
    finally
    {
      try
      {
        if (objectInputStream != null)
        {
          objectInputStream.close();
        }
        if (bis != null)
        {
          bis.close();
        }
        if (fis != null)
        {
          fis.close();
        }
        if (raf != null)
        {
          raf.close();
        }

      }
      catch (IOException e)
      {
        throw new InterfaceException("00010", "inputstream close error");
      }
    }
    return object;
  }

  private static boolean isFileExist(String filename)
  {
    File file = new File(filename);
    if (file.exists() && file.isFile())
    {
      return true;
    }
    return false;
  }

  public static String getSerialReadPath(String name)
  {
    name = FileUtil.getFileName(name);
    return serialDic + "/" + name + serialExt;
  }

  public static String getSerialWritePath(String name)
  {
    name = FileUtil.getFileName(name);
    return serialDic + "/temp/" + name + serialExt;
  }

  public static void copySerialFiles()
  {
    try
    {
      File tmpFile = new File(serialDic + "/temp");
      FileUtils.copyDirectory(tmpFile, new File(serialDic));
      FileUtils.deleteDirectory(tmpFile);
    }
    catch (IOException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }
}
