package com.brilliance.eibs.util;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.ISubfieldDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.impl.AbsFilter;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.factory.parser.XmlValidator;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.DirectoryFileFilter;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.dom4j.*;
import org.dom4j.io.SAXReader;

import java.io.File;
import java.net.URL;
import java.util.*;

public class XmlUtil {

    public static String getRelativeName(File f, String s) {
        String MAIN = "cfg\\main";
        if (!f.getPath().contains(s) && !f.getPath().contains(MAIN)) {
            return f.getName();
        }
        String path = f.getPath();
        int index = path.indexOf(s);
        String relativePath = null;
        if (index == -1) {
            index = path.indexOf(MAIN);
        }
        relativePath = path.substring(index);
        relativePath = relativePath.replace("\\", "/");
        relativePath = relativePath.substring(s.length() + 1);
        if (relativePath.endsWith(".xml"))
            relativePath = relativePath.substring(0, relativePath.length() - 4);
        return relativePath;
    }

    /**
     * @param path 获取某个文件目录下的所有XML文件
     **/
    public static Collection<File> getAllFile(String path) {
        URL url = XmlUtil.getUrl(path);
        File srcfile = new File(CommonFunctionUtils.decodeURL(url.getPath()));
        Collection<File> files = FileUtils.listFiles(srcfile, FileFilterUtils.suffixFileFilter("xml"), DirectoryFileFilter.INSTANCE);
        return files;
    }

    public static String getXmlNode(String xmlstr, String path) throws DocumentException {
        return getXmlNodeValue(DocumentHelper.parseText(xmlstr), path);
    }

    /**
     * 获取文档对象的指定路径的值
     *
     * @param doc
     * @param path
     * @return
     */
    @SuppressWarnings("unchecked")
    public static String getXmlNodeValue(Document doc, String path) {
        String[] paths = path.split("@");
        String[] layers = paths[0].split("\\.");
        Element rootElement = doc.getRootElement();
        Element curElement = rootElement;
        if (layers[0].equals(rootElement.getName())) {
            for (int i = 1; i < layers.length; i++) {
                int startIndex = layers[i].indexOf('(');
                if (-1 != startIndex) {
                    int endIndex = layers[i].indexOf(')');
                    int num = Integer.parseInt(layers[i].substring(startIndex + 1, endIndex));
                    layers[i] = layers[i].substring(0, startIndex);
                    List<Element> subElements = curElement.elements(layers[i]);
                    Element subElement = subElements.get(num);
                    curElement = subElement;
                } else {
                    Element subElement = curElement.element(layers[i]);
                    if (subElement == null) {
                        return null;
                    }
                    curElement = subElement;
                }
            }
            if (paths.length > 1)
                return curElement.attributeValue(paths[paths.length - 1]);
            return curElement.getText();
        }
        return null;
    }

    public static String getXmlValue(String xmlStr, String etag) {
        try {
            Document document = DocumentHelper.parseText(xmlStr);
            return getXmlNodeValue(document, etag);
        } catch (DocumentException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取配置文件Document对象
     *
     * @param filepath
     * @return
     */
    public static Document getDocument(String filepath) {
        URL url = getUrl(filepath);
        if (url == null)
            throw new InterfaceException("00006", "can not find resource path: " + filepath);
        return getDocument(url);
    }

    /**
     * 获得资源位置
     *
     * @param filepath
     * @return
     */
    public static URL getUrl(String filepath) {
        return CommonFunctionUtils.getCommonURL(filepath);
    }

    /**
     * 获取配置文件Document对象
     *
     * @param url
     * @return
     */
    public static Document getDocument(URL url) {
        SAXReader saxReader = new SAXReader();
        try {
            Document doc = saxReader.read(url);
            Element rootNode = doc.getRootElement();
            if (rootNode.getNamespace() != null && rootNode.attribute("schemaLocation") != null)
                validateDoc(doc);
            return doc;
        } catch (DocumentException e) {
            throw new InterfaceException("00008", "get document [" + url + "] error.", e);
        }
    }

    /**
     * 获取配置文件Document对象
     *
     * @param path
     * @param filename
     * @return
     */
    public static Document getDocument(String path, String filename) {
        return XmlUtil.getDocument(getPath(path, filename));
    }

    /**
     * 由路径和文件名拼出资源路径
     *
     * @param path
     * @param filename
     * @return
     */
    public static String getPath(String path, String filename) {
        if (!filename.endsWith(".xml")) {
            filename += ".xml";
        }
        return path + "/" + filename;
    }

    private static void validateDoc(Document doc) {
        XmlValidator validator = new XmlValidator();
        try {
            if (!validator.validate(doc)) {
                throw new InterfaceException("00026", validator.getStrErrors());
            }
        } catch (Exception e) {
            throw new InterfaceException("00026", e);
        }

    }

    /**
     * 获取指定节点的子节点组，并以<code>List</code>的形式返回
     *
     * @param doc      Xml文档
     * @param etag     符合接口配置规则的指定节点
     * @param fieldDef
     * @param context
     * @return 以<code>List</code>的形式返回
     */
    public static Object getElementsOfListformat(Document doc, String etag, IFieldDef fieldDef, Context context) {
        // 校验指定的报文元素是否存在
        if (getNodesByXpath(doc, etag, fieldDef.getType()) == null)
            return null;

        // 开始组装
        return convertElement(doc, fieldDef, etag, context);
    }

    public static boolean isXpath(String type) {
        return (!StringUtil.isEmpty(type) && type.equalsIgnoreCase("xpath")) ? true : false;
    }

    /**
     * @param doc
     * @param fieldDef
     * @param prefixetag 搜索的前缀，后续形式为：'.'+subfield.getetag()
     * @param context
     * @return
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    private static Object convertElement(Object doc, IFieldDef fieldDef, String prefixetag, Context context) {
        List<Map<String, Object>> rs = new LinkedList<Map<String, Object>>();
        List<ISubfieldDef> subfieldDefs = fieldDef.getSubfields();

        if (subfieldDefs != null && subfieldDefs.size() > 0) {
            List<Element> fieldsubnodes = getNodesByXpath((Branch) doc, prefixetag, fieldDef.getType());
            // List<Element> fieldsubnodes = ((Branch)
            // doc).selectNodes(convertEtag(prefixetag));
            for (Element subnode : fieldsubnodes) {
                Map<String, Object> rsmap = new HashMap<String, Object>();
                for (ISubfieldDef sd : subfieldDefs) {
                    if (StringUtil.isEmpty(sd.getTag()))
                        continue;
                    Node ems = null;
                    if (!sd.getSubfields().isEmpty()) {
                        List<Map> subs = null;
                        if (isXpath(sd.getType()))
                            subs = (List<Map>) convertElement(subnode, sd, sd.getEtag(), context);
                        else
                            subs = (List<Map>) convertElement(subnode, sd, "./" + sd.getEtag(), context);
                        rsmap.put(sd.getTag(), subs);
                    } else {
                        Object val = null;
                        if (!StringUtil.isEmpty(sd.getEtag())) {
                            if (isXpath(sd.getType()))
                                ems = subnode.selectSingleNode(sd.getEtag());
                            else
                                ems = subnode.selectSingleNode(convertEtag("./" + sd.getEtag()));
                            if (ems != null) {
                                val = ems.getText();
                            }
                        }
                        val = ((AbsFilter) context.getCurrentInstance()).valFilter(val, sd, context);
                        rsmap.put(sd.getTag(), val);
                    }
                }
                rs.add(rsmap);
            }
        }
        return rs;
    }

    /**
     * 转换接口中etag格式为xpath格式
     *
     * @param etag
     * @return
     */
    public static String convertEtag(String etag) {
        if (!etag.startsWith("./") && !etag.startsWith("//")) {
            etag = "//" + etag;
            etag = etag.replaceAll("\\.", "/");
        }

        return etag;
    }


    @SuppressWarnings("unchecked")
    public static List<Element> getNodesByXpath(Branch document, String path, String type) {
        if (isXpath(type))
            return document.selectNodes(path);
        else
            return document.selectNodes(convertEtag(path));
    }

    public static int getChildrenCount(Branch document, String path, String type) {
        if (isXpath(type))
            return document.selectNodes(path).size();
        else {
            String[] paths = path.split("@");
            String[] layers = paths[0].split("\\.");
            Element rootElement = ((Document) document).getRootElement();
            Element curElement = rootElement;
            if (layers[0].equals(rootElement.getName())) {
                for (int i = 1; i < layers.length - 1; i++) {
                    int startIndex = layers[i].indexOf('(');
                    if (-1 != startIndex) {
                        int endIndex = layers[i].indexOf(')');
                        int num = Integer.parseInt(layers[i].substring(startIndex + 1, endIndex));
                        String mutiLayer = layers[i].substring(0, startIndex);
                        List<Element> subElements = curElement.elements(mutiLayer);
                        Element subElement = null;
                        int size = subElements.size();
                        if (num >= size) {
                            subElement = curElement.addElement(mutiLayer);
                        } else {
                            subElement = subElements.get(num);
                        }
                        curElement = subElement;
                    } else {
                        Element subElement = null;
                        if (layers[i].contains(":"))
                            subElement = curElement.element(layers[i].substring(layers[i].indexOf(':') + 1));
                        else
                            subElement = curElement.element(layers[i]);
                        if (subElement == null) {
                            subElement = curElement.addElement(layers[i]);
                        }
                        curElement = subElement;
                    }
                }
            }
            return curElement.elements(layers[layers.length - 1]).size();
        }

    }

    public static String getXmlNodeValueByXpath(Document document, String path) {
        String[] paths = path.split("#");
        String xpath = paths[0];
        List<Element> lst = getNodesByXpath(document, xpath, "xpath");
        if(lst.isEmpty()){
            return null;
        }
        Element curElement = lst.get(0);
        if (paths.length > 1){
            return curElement.attributeValue(paths[paths.length - 1]);
        }
        return curElement.getText();
    }


}
