package com.brilliance.eibs.util.logging.log4j2;

import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.FileUtils;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginElement;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.layout.PatternLayout;
import org.slf4j.MDC;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;

/**
 * 自定义appender:实现按日期和交易种类两个维度归档日志
 */
@Plugin(name = "Log4j2Appender", category = "Core", elementType = "appender", printObject = true)
public class Log4j2Appender extends AbstractAppender {
    /**
     * 日志归档根路径
     */
    private String rootDir;

    public Log4j2Appender(String name, Filter filter, Layout<? extends Serializable> layout, boolean ignoreExceptions, String rootDir) {
        super(name, filter, layout, ignoreExceptions);
        this.rootDir = rootDir;
    }

    @PluginFactory
    public static Log4j2Appender createAppender(@PluginAttribute("name") String name,
                                                @PluginAttribute("rootDir") String rootDir,
                                                @PluginElement("Filter") final Filter filter,
                                                @PluginElement("Layout") Layout<? extends Serializable> layout,
                                                @PluginAttribute("ignoreExceptions") boolean ignoreExceptions) {
        if (name == null) {
            LOGGER.error("no name defined in conf.");
            return null;
        }
        if (layout == null) {
            layout = PatternLayout.createDefaultLayout();
        }

        return new Log4j2Appender(name, filter, layout, ignoreExceptions, rootDir);
    }

    @Override
    public void append(LogEvent event) {
        String loggerName = MDC.getCopyOfContextMap().get(LogUtil.LOG_FILE_NAME_SYMBOL);
        if (StringUtil.isEmpty(loggerName)) {
            loggerName = LogUtil.SERVICE;
            MDC.put(LogUtil.LOG_FILE_NAME_SYMBOL, loggerName);
        }
        String date = DateUtil.format(new Date(event.getTimeMillis()), "yyyyMMdd");

        if (LogUtil.SERVICE.equals(loggerName)) {
            loggerName = LogUtil.SERVICE + "_" + date;
        } else {
            loggerName = date + "/" + loggerName;
        }
        final byte[] bytes = getLayout().toByteArray(event);
        try {
            String filePath = loggerName + LogUtil.LOG_SUFFIX;
            FileUtils.writeByteArrayToFile(new File(this.rootDir, filePath), bytes, true);
        } catch (IOException e) {
            LOGGER.error("Failed to output log to [" + loggerName + "] file, cause by " + e.getLocalizedMessage(), e);
        }
    }

}