package com.brilliance.eibs.util.soap;

import com.brilliance.eibs.util.ClassPathUpdater;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.w3c.dom.Document;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.io.File;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;


public class SoapMessageHandler implements SOAPHandler<SOAPMessageContext> {

    private final Logger log;

    private String serviceName = "";

    private String logFileName = ClassPathUpdater.addClasspath("data");

    private String fileName = "";

    public SoapMessageHandler(String serviceName, Logger log) {
        this.log = log;
        this.serviceName = serviceName;
        try {
            if (logFileName.endsWith("/"))
                logFileName += this.serviceName + "/";
            else
                logFileName += "/" + this.serviceName + "/";

            logFileName += new SimpleDateFormat("yyyyMM").format(new Date()) + "/";
            logFileName += new SimpleDateFormat("yyyyMMdd").format(new Date()) + "/";

            File dir = new File(logFileName);
            if (!dir.exists()) dir.mkdirs();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            e.printStackTrace();
        }
    }

    /*
     * Soap报文回调,正常报文收发都会触发一次 如果返回的报文是异常报文,则会触发handleFault
     */
    @Override
    public boolean handleMessage(SOAPMessageContext soapMessageContext) {
        try {
            SOAPMessage message = soapMessageContext.getMessage();
            String soapMessage = this.getSoapMessageString(message);

            Boolean out =
                    (Boolean) soapMessageContext.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

            if (out) {
                String soapType = message.getSOAPBody().getChildNodes().item(0).getNodeName();
                if (soapType.contains(":")) {
                    String[] soaps = soapType.split("\\:");
                    soapType = soaps[soaps.length - 1];
                }

                if ("".equals(fileName)) {
                    fileName = soapType + "_"
                            + new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + ".log";
                    this.logFileName += fileName;
                }

                soapMessage += "\r\n\r\n\r\n";
            }

            this.writeSoapMessage(soapMessage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            e.printStackTrace();
        }
        return true;
    }

    @Override
    public void close(MessageContext messageContext) {

    }

    @Override
    public boolean handleFault(SOAPMessageContext soapMessageContext) {
        try {
            SOAPMessage message = soapMessageContext.getMessage();
            String soapMessage = this.getSoapMessageString(message);
            this.writeSoapMessage(soapMessage);
        } catch (Exception e) {
            log.error("读取SOAP报文异常.");
        }
        return true;
    }

    @Override
    public Set<QName> getHeaders() {
        return null;
    }

    public String getSoapMessageString(SOAPMessage message) {
        String soapMessage = "";
        try {
            Document doc = message.getSOAPPart().getEnvelope().getOwnerDocument();
            StringWriter output = new StringWriter();
            Transformer t = TransformerFactory.newInstance().newTransformer();
            t.setOutputProperty(OutputKeys.INDENT, "yes");
            t.transform(new DOMSource(doc), new StreamResult(output));
            soapMessage = output.toString();
        } catch (Exception e) {
            log.error("读取SOAP报文异常.");
        }
        return soapMessage;
    }

    public void writeSoapMessage(String soapMessage) {
        try {
            log.info("报文写入文件:" + logFileName);
            File logFile = new File(this.logFileName);
            if (!logFile.exists()) logFile.createNewFile();
            FileUtils.writeStringToFile(logFile, soapMessage, true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            e.printStackTrace();
        }
    }
}
